/* special-folders.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __SRC_CLIENT_PLUGIN_SPECIAL_FOLDERS_SPECIAL_FOLDERS_H__
#define __SRC_CLIENT_PLUGIN_SPECIAL_FOLDERS_SPECIAL_FOLDERS_H__

#include <glib-object.h>
#include "geary-client-46.0.h"
#include <gio/gio.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define PLUGIN_TYPE_SPECIAL_FOLDERS (plugin_special_folders_get_type ())
#define PLUGIN_SPECIAL_FOLDERS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_SPECIAL_FOLDERS, PluginSpecialFolders))
#define PLUGIN_SPECIAL_FOLDERS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_SPECIAL_FOLDERS, PluginSpecialFoldersClass))
#define PLUGIN_IS_SPECIAL_FOLDERS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_SPECIAL_FOLDERS))
#define PLUGIN_IS_SPECIAL_FOLDERS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_SPECIAL_FOLDERS))
#define PLUGIN_SPECIAL_FOLDERS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_SPECIAL_FOLDERS, PluginSpecialFoldersClass))

typedef struct _PluginSpecialFolders PluginSpecialFolders;
typedef struct _PluginSpecialFoldersClass PluginSpecialFoldersClass;
typedef struct _PluginSpecialFoldersPrivate PluginSpecialFoldersPrivate;

struct _PluginSpecialFolders {
	PluginPluginBase parent_instance;
	PluginSpecialFoldersPrivate * priv;
};

struct _PluginSpecialFoldersClass {
	PluginPluginBaseClass parent_class;
};

VALA_EXTERN void peas_register_types (GTypeModule* module);
GType plugin_special_folders_register_type (GTypeModule * module);
VALA_EXTERN GType plugin_special_folders_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginSpecialFolders, g_object_unref)
VALA_EXTERN PluginSpecialFolders* plugin_special_folders_new (void);
VALA_EXTERN PluginSpecialFolders* plugin_special_folders_construct (GType object_type);

G_END_DECLS

#endif
