/* imap-mailbox-attribute.c generated by valac 0.56.19, the Vala compiler
 * generated from imap-mailbox-attribute.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	GEARY_IMAP_MAILBOX_ATTRIBUTE_0_PROPERTY,
	GEARY_IMAP_MAILBOX_ATTRIBUTE_NUM_PROPERTIES
};
static GParamSpec* geary_imap_mailbox_attribute_properties[GEARY_IMAP_MAILBOX_ATTRIBUTE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

static gpointer geary_imap_mailbox_attribute_parent_class = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__no_inferiors;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__no_inferiors = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__nonexistent;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__nonexistent = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__no_select;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__no_select = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__marked;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__marked = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__unmarked;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__unmarked = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__has_no_children;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__has_no_children = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__has_children;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__has_children = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__allows_new;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__allows_new = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_all;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_all = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_archive;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_archive = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_drafts;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_drafts = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_flagged;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_flagged = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_important;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_important = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_junk;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_junk = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_sent;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_sent = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_trash;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__special_use_trash = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_inbox;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_inbox = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_all_mail;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_all_mail = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_spam;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_spam = NULL;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_starred;
static GearyImapMailboxAttribute* geary_imap_mailbox_attribute__xlist_starred = NULL;

VALA_EXTERN void geary_imap_mailbox_attribute_init (void);
static void geary_imap_mailbox_attribute_finalize (GObject * obj);
static GType geary_imap_mailbox_attribute_get_type_once (void);
static void _vala_geary_imap_mailbox_attribute_get_property (GObject * object,
                                                      guint property_id,
                                                      GValue * value,
                                                      GParamSpec * pspec);

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_NO_INFERIORS (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__no_inferiors;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\noinferiors");
		_g_object_unref0 (geary_imap_mailbox_attribute__no_inferiors);
		geary_imap_mailbox_attribute__no_inferiors = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__no_inferiors;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_NONEXISTENT (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__nonexistent;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__nonexistent;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\NonExistent");
		_g_object_unref0 (geary_imap_mailbox_attribute__nonexistent);
		geary_imap_mailbox_attribute__nonexistent = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__nonexistent;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_NO_SELECT (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__no_select;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\noselect");
		_g_object_unref0 (geary_imap_mailbox_attribute__no_select);
		geary_imap_mailbox_attribute__no_select = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__no_select;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_MARKED (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__marked;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\marked");
		_g_object_unref0 (geary_imap_mailbox_attribute__marked);
		geary_imap_mailbox_attribute__marked = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__marked;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_UNMARKED (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__unmarked;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\unmarked");
		_g_object_unref0 (geary_imap_mailbox_attribute__unmarked);
		geary_imap_mailbox_attribute__unmarked = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__unmarked;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_HAS_NO_CHILDREN (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__has_no_children;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\hasnochildren");
		_g_object_unref0 (geary_imap_mailbox_attribute__has_no_children);
		geary_imap_mailbox_attribute__has_no_children = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__has_no_children;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_HAS_CHILDREN (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__has_children;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\haschildren");
		_g_object_unref0 (geary_imap_mailbox_attribute__has_children);
		geary_imap_mailbox_attribute__has_children = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__has_children;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_ALLOWS_NEW (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__allows_new;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\*");
		_g_object_unref0 (geary_imap_mailbox_attribute__allows_new);
		geary_imap_mailbox_attribute__allows_new = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__allows_new;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_ALL (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_all;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_all;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\All");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_all);
		geary_imap_mailbox_attribute__special_use_all = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_all;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_ARCHIVE (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_archive;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_archive;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Archive");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_archive);
		geary_imap_mailbox_attribute__special_use_archive = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_archive;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_DRAFTS (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_drafts;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_drafts;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Drafts");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_drafts);
		geary_imap_mailbox_attribute__special_use_drafts = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_drafts;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_FLAGGED (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_flagged;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_flagged;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Flagged");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_flagged);
		geary_imap_mailbox_attribute__special_use_flagged = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_flagged;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_IMPORTANT (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_important;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_important;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Important");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_important);
		geary_imap_mailbox_attribute__special_use_important = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_important;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_JUNK (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_junk;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_junk;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Junk");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_junk);
		geary_imap_mailbox_attribute__special_use_junk = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_junk;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_SENT (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_sent;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_sent;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Sent");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_sent);
		geary_imap_mailbox_attribute__special_use_sent = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_sent;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_TRASH (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_ = NULL;
	GearyImapMailboxAttribute* _tmp1_;
	_tmp1_ = geary_imap_mailbox_attribute__special_use_trash;
	if (_tmp1_ != NULL) {
		GearyImapMailboxAttribute* _tmp2_;
		_tmp2_ = geary_imap_mailbox_attribute__special_use_trash;
		_tmp0_ = _tmp2_;
	} else {
		GearyImapMailboxAttribute* _tmp3_;
		GearyImapMailboxAttribute* _tmp4_;
		_tmp3_ = geary_imap_mailbox_attribute_new ("\\Trash");
		_g_object_unref0 (geary_imap_mailbox_attribute__special_use_trash);
		geary_imap_mailbox_attribute__special_use_trash = _tmp3_;
		_tmp4_ = geary_imap_mailbox_attribute__special_use_trash;
		_tmp0_ = _tmp4_;
	}
	result = _tmp0_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_XLIST_INBOX (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__xlist_inbox;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\Inbox");
		_g_object_unref0 (geary_imap_mailbox_attribute__xlist_inbox);
		geary_imap_mailbox_attribute__xlist_inbox = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__xlist_inbox;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_XLIST_ALL_MAIL (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__xlist_all_mail;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\AllMail");
		_g_object_unref0 (geary_imap_mailbox_attribute__xlist_all_mail);
		geary_imap_mailbox_attribute__xlist_all_mail = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__xlist_all_mail;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_XLIST_SPAM (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__xlist_spam;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\Spam");
		_g_object_unref0 (geary_imap_mailbox_attribute__xlist_spam);
		geary_imap_mailbox_attribute__xlist_spam = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__xlist_spam;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_get_XLIST_STARRED (void)
{
	GearyImapMailboxAttribute* result;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp2_;
	_tmp0_ = geary_imap_mailbox_attribute__xlist_starred;
	if (_tmp0_ == NULL) {
		GearyImapMailboxAttribute* _tmp1_;
		_tmp1_ = geary_imap_mailbox_attribute_new ("\\Starred");
		_g_object_unref0 (geary_imap_mailbox_attribute__xlist_starred);
		geary_imap_mailbox_attribute__xlist_starred = _tmp1_;
	}
	_tmp2_ = geary_imap_mailbox_attribute__xlist_starred;
	result = _tmp2_;
	return result;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_construct (GType object_type,
                                        const gchar* value)
{
	GearyImapMailboxAttribute * self = NULL;
	g_return_val_if_fail (value != NULL, NULL);
	self = (GearyImapMailboxAttribute*) geary_imap_flag_construct (object_type, value);
	return self;
}

GearyImapMailboxAttribute*
geary_imap_mailbox_attribute_new (const gchar* value)
{
	return geary_imap_mailbox_attribute_construct (GEARY_IMAP_TYPE_MAILBOX_ATTRIBUTE, value);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
geary_imap_mailbox_attribute_init (void)
{
	GearyImapMailboxAttribute* to_init = NULL;
	GearyImapMailboxAttribute* _tmp0_;
	GearyImapMailboxAttribute* _tmp1_;
	GearyImapMailboxAttribute* _tmp2_;
	GearyImapMailboxAttribute* _tmp3_;
	GearyImapMailboxAttribute* _tmp4_;
	GearyImapMailboxAttribute* _tmp5_;
	GearyImapMailboxAttribute* _tmp6_;
	GearyImapMailboxAttribute* _tmp7_;
	GearyImapMailboxAttribute* _tmp8_;
	GearyImapMailboxAttribute* _tmp9_;
	GearyImapMailboxAttribute* _tmp10_;
	GearyImapMailboxAttribute* _tmp11_;
	GearyImapMailboxAttribute* _tmp12_;
	GearyImapMailboxAttribute* _tmp13_;
	GearyImapMailboxAttribute* _tmp14_;
	GearyImapMailboxAttribute* _tmp15_;
	GearyImapMailboxAttribute* _tmp16_;
	GearyImapMailboxAttribute* _tmp17_;
	GearyImapMailboxAttribute* _tmp18_;
	GearyImapMailboxAttribute* _tmp19_;
	GearyImapMailboxAttribute* _tmp20_;
	GearyImapMailboxAttribute* _tmp21_;
	GearyImapMailboxAttribute* _tmp22_;
	GearyImapMailboxAttribute* _tmp23_;
	GearyImapMailboxAttribute* _tmp24_;
	GearyImapMailboxAttribute* _tmp25_;
	GearyImapMailboxAttribute* _tmp26_;
	GearyImapMailboxAttribute* _tmp27_;
	GearyImapMailboxAttribute* _tmp28_;
	GearyImapMailboxAttribute* _tmp29_;
	GearyImapMailboxAttribute* _tmp30_;
	GearyImapMailboxAttribute* _tmp31_;
	GearyImapMailboxAttribute* _tmp32_;
	GearyImapMailboxAttribute* _tmp33_;
	GearyImapMailboxAttribute* _tmp34_;
	GearyImapMailboxAttribute* _tmp35_;
	GearyImapMailboxAttribute* _tmp36_;
	GearyImapMailboxAttribute* _tmp37_;
	GearyImapMailboxAttribute* _tmp38_;
	GearyImapMailboxAttribute* _tmp39_;
	GearyImapMailboxAttribute* _tmp40_;
	GearyImapMailboxAttribute* _tmp41_;
	GearyImapMailboxAttribute* _tmp42_;
	GearyImapMailboxAttribute* _tmp43_;
	GearyImapMailboxAttribute* _tmp44_;
	GearyImapMailboxAttribute* _tmp45_;
	GearyImapMailboxAttribute* _tmp46_;
	GearyImapMailboxAttribute* _tmp47_;
	GearyImapMailboxAttribute* _tmp48_;
	GearyImapMailboxAttribute* _tmp49_;
	GearyImapMailboxAttribute* _tmp50_;
	GearyImapMailboxAttribute* _tmp51_;
	GearyImapMailboxAttribute* _tmp52_;
	GearyImapMailboxAttribute* _tmp53_;
	GearyImapMailboxAttribute* _tmp54_;
	GearyImapMailboxAttribute* _tmp55_;
	GearyImapMailboxAttribute* _tmp56_;
	GearyImapMailboxAttribute* _tmp57_;
	GearyImapMailboxAttribute* _tmp58_;
	GearyImapMailboxAttribute* _tmp59_;
	_tmp0_ = geary_imap_mailbox_attribute_get_NO_INFERIORS ();
	_tmp1_ = _tmp0_;
	_tmp2_ = _g_object_ref0 (_tmp1_);
	to_init = _tmp2_;
	_tmp3_ = geary_imap_mailbox_attribute_get_NONEXISTENT ();
	_tmp4_ = _tmp3_;
	_tmp5_ = _g_object_ref0 (_tmp4_);
	_g_object_unref0 (to_init);
	to_init = _tmp5_;
	_tmp6_ = geary_imap_mailbox_attribute_get_NO_SELECT ();
	_tmp7_ = _tmp6_;
	_tmp8_ = _g_object_ref0 (_tmp7_);
	_g_object_unref0 (to_init);
	to_init = _tmp8_;
	_tmp9_ = geary_imap_mailbox_attribute_get_MARKED ();
	_tmp10_ = _tmp9_;
	_tmp11_ = _g_object_ref0 (_tmp10_);
	_g_object_unref0 (to_init);
	to_init = _tmp11_;
	_tmp12_ = geary_imap_mailbox_attribute_get_UNMARKED ();
	_tmp13_ = _tmp12_;
	_tmp14_ = _g_object_ref0 (_tmp13_);
	_g_object_unref0 (to_init);
	to_init = _tmp14_;
	_tmp15_ = geary_imap_mailbox_attribute_get_HAS_NO_CHILDREN ();
	_tmp16_ = _tmp15_;
	_tmp17_ = _g_object_ref0 (_tmp16_);
	_g_object_unref0 (to_init);
	to_init = _tmp17_;
	_tmp18_ = geary_imap_mailbox_attribute_get_HAS_CHILDREN ();
	_tmp19_ = _tmp18_;
	_tmp20_ = _g_object_ref0 (_tmp19_);
	_g_object_unref0 (to_init);
	to_init = _tmp20_;
	_tmp21_ = geary_imap_mailbox_attribute_get_ALLOWS_NEW ();
	_tmp22_ = _tmp21_;
	_tmp23_ = _g_object_ref0 (_tmp22_);
	_g_object_unref0 (to_init);
	to_init = _tmp23_;
	_tmp24_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_ALL ();
	_tmp25_ = _tmp24_;
	_tmp26_ = _g_object_ref0 (_tmp25_);
	_g_object_unref0 (to_init);
	to_init = _tmp26_;
	_tmp27_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_ARCHIVE ();
	_tmp28_ = _tmp27_;
	_tmp29_ = _g_object_ref0 (_tmp28_);
	_g_object_unref0 (to_init);
	to_init = _tmp29_;
	_tmp30_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_DRAFTS ();
	_tmp31_ = _tmp30_;
	_tmp32_ = _g_object_ref0 (_tmp31_);
	_g_object_unref0 (to_init);
	to_init = _tmp32_;
	_tmp33_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_FLAGGED ();
	_tmp34_ = _tmp33_;
	_tmp35_ = _g_object_ref0 (_tmp34_);
	_g_object_unref0 (to_init);
	to_init = _tmp35_;
	_tmp36_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_IMPORTANT ();
	_tmp37_ = _tmp36_;
	_tmp38_ = _g_object_ref0 (_tmp37_);
	_g_object_unref0 (to_init);
	to_init = _tmp38_;
	_tmp39_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_JUNK ();
	_tmp40_ = _tmp39_;
	_tmp41_ = _g_object_ref0 (_tmp40_);
	_g_object_unref0 (to_init);
	to_init = _tmp41_;
	_tmp42_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_SENT ();
	_tmp43_ = _tmp42_;
	_tmp44_ = _g_object_ref0 (_tmp43_);
	_g_object_unref0 (to_init);
	to_init = _tmp44_;
	_tmp45_ = geary_imap_mailbox_attribute_get_SPECIAL_FOLDER_TRASH ();
	_tmp46_ = _tmp45_;
	_tmp47_ = _g_object_ref0 (_tmp46_);
	_g_object_unref0 (to_init);
	to_init = _tmp47_;
	_tmp48_ = geary_imap_mailbox_attribute_get_XLIST_ALL_MAIL ();
	_tmp49_ = _tmp48_;
	_tmp50_ = _g_object_ref0 (_tmp49_);
	_g_object_unref0 (to_init);
	to_init = _tmp50_;
	_tmp51_ = geary_imap_mailbox_attribute_get_XLIST_INBOX ();
	_tmp52_ = _tmp51_;
	_tmp53_ = _g_object_ref0 (_tmp52_);
	_g_object_unref0 (to_init);
	to_init = _tmp53_;
	_tmp54_ = geary_imap_mailbox_attribute_get_XLIST_SPAM ();
	_tmp55_ = _tmp54_;
	_tmp56_ = _g_object_ref0 (_tmp55_);
	_g_object_unref0 (to_init);
	to_init = _tmp56_;
	_tmp57_ = geary_imap_mailbox_attribute_get_XLIST_STARRED ();
	_tmp58_ = _tmp57_;
	_tmp59_ = _g_object_ref0 (_tmp58_);
	_g_object_unref0 (to_init);
	to_init = _tmp59_;
	_g_object_unref0 (to_init);
}

static void
geary_imap_mailbox_attribute_class_init (GearyImapMailboxAttributeClass * klass,
                                         gpointer klass_data)
{
	geary_imap_mailbox_attribute_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = _vala_geary_imap_mailbox_attribute_get_property;
	G_OBJECT_CLASS (klass)->finalize = geary_imap_mailbox_attribute_finalize;
}

static void
geary_imap_mailbox_attribute_instance_init (GearyImapMailboxAttribute * self,
                                            gpointer klass)
{
}

static void
geary_imap_mailbox_attribute_finalize (GObject * obj)
{
	GearyImapMailboxAttribute * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEARY_IMAP_TYPE_MAILBOX_ATTRIBUTE, GearyImapMailboxAttribute);
	G_OBJECT_CLASS (geary_imap_mailbox_attribute_parent_class)->finalize (obj);
}

/**
 * An IMAP mailbox attribute (flag).
 *
 * See [[http://tools.ietf.org/html/rfc3501#section-7.2.2]]
 *
 * @see ListCommand
 * @see MailboxInformation
 */
 G_GNUC_NO_INLINE static GType
geary_imap_mailbox_attribute_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyImapMailboxAttributeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_imap_mailbox_attribute_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyImapMailboxAttribute), 0, (GInstanceInitFunc) geary_imap_mailbox_attribute_instance_init, NULL };
	GType geary_imap_mailbox_attribute_type_id;
	geary_imap_mailbox_attribute_type_id = g_type_register_static (GEARY_IMAP_TYPE_FLAG, "GearyImapMailboxAttribute", &g_define_type_info, 0);
	return geary_imap_mailbox_attribute_type_id;
}

GType
geary_imap_mailbox_attribute_get_type (void)
{
	static gsize geary_imap_mailbox_attribute_type_id__once = 0;
	if (g_once_init_enter (&geary_imap_mailbox_attribute_type_id__once)) {
		GType geary_imap_mailbox_attribute_type_id;
		geary_imap_mailbox_attribute_type_id = geary_imap_mailbox_attribute_get_type_once ();
		g_once_init_leave (&geary_imap_mailbox_attribute_type_id__once, geary_imap_mailbox_attribute_type_id);
	}
	return geary_imap_mailbox_attribute_type_id__once;
}

static void
_vala_geary_imap_mailbox_attribute_get_property (GObject * object,
                                                 guint property_id,
                                                 GValue * value,
                                                 GParamSpec * pspec)
{
	GearyImapMailboxAttribute * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEARY_IMAP_TYPE_MAILBOX_ATTRIBUTE, GearyImapMailboxAttribute);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

