/* geary-engine-error.c generated by valac 0.56.19, the Vala compiler
 * generated from geary-engine-error.vala, do not modify */

/*
 * Copyright 2016 Software Freedom Conservancy Inc.
 * Copyright 2018 Michael Gratton <mike@vee.net>
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

GQuark
geary_engine_error_quark (void)
{
	return g_quark_from_static_string ("geary-engine-error-quark");
}

 G_GNUC_NO_INLINE static GType
geary_engine_error_get_type_once (void)
{
	static const GEnumValue values[] = {{GEARY_ENGINE_ERROR_OPEN_REQUIRED, "GEARY_ENGINE_ERROR_OPEN_REQUIRED", "open-required"}, {GEARY_ENGINE_ERROR_ALREADY_OPEN, "GEARY_ENGINE_ERROR_ALREADY_OPEN", "already-open"}, {GEARY_ENGINE_ERROR_ALREADY_EXISTS, "GEARY_ENGINE_ERROR_ALREADY_EXISTS", "already-exists"}, {GEARY_ENGINE_ERROR_ALREADY_CLOSED, "GEARY_ENGINE_ERROR_ALREADY_CLOSED", "already-closed"}, {GEARY_ENGINE_ERROR_CLOSE_REQUIRED, "GEARY_ENGINE_ERROR_CLOSE_REQUIRED", "close-required"}, {GEARY_ENGINE_ERROR_NOT_FOUND, "GEARY_ENGINE_ERROR_NOT_FOUND", "not-found"}, {GEARY_ENGINE_ERROR_BAD_PARAMETERS, "GEARY_ENGINE_ERROR_BAD_PARAMETERS", "bad-parameters"}, {GEARY_ENGINE_ERROR_INCOMPLETE_MESSAGE, "GEARY_ENGINE_ERROR_INCOMPLETE_MESSAGE", "incomplete-message"}, {GEARY_ENGINE_ERROR_SERVER_UNAVAILABLE, "GEARY_ENGINE_ERROR_SERVER_UNAVAILABLE", "server-unavailable"}, {GEARY_ENGINE_ERROR_CORRUPT, "GEARY_ENGINE_ERROR_CORRUPT", "corrupt"}, {GEARY_ENGINE_ERROR_PERMISSIONS, "GEARY_ENGINE_ERROR_PERMISSIONS", "permissions"}, {GEARY_ENGINE_ERROR_VERSION, "GEARY_ENGINE_ERROR_VERSION", "version"}, {GEARY_ENGINE_ERROR_UNSUPPORTED, "GEARY_ENGINE_ERROR_UNSUPPORTED", "unsupported"}, {0, NULL, NULL}};
	GType geary_engine_error_type_id;
	geary_engine_error_type_id = g_enum_register_static ("GearyEngineError", values);
	return geary_engine_error_type_id;
}

GType
geary_engine_error_get_type (void)
{
	static gsize geary_engine_error_type_id__once = 0;
	if (g_once_init_enter (&geary_engine_error_type_id__once)) {
		GType geary_engine_error_type_id;
		geary_engine_error_type_id = geary_engine_error_get_type_once ();
		g_once_init_leave (&geary_engine_error_type_id__once, geary_engine_error_type_id);
	}
	return geary_engine_error_type_id__once;
}

