/* imap.c generated by valac 0.56.19, the Vala compiler
 * generated from imap.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include <glib.h>
#include "geary-engine.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

VALA_EXTERN gint geary_imap_init_count;
gint geary_imap_init_count = 0;

VALA_EXTERN void geary_imap_init (void);
VALA_EXTERN void geary_imap_message_flag_init (void);
VALA_EXTERN void geary_imap_mailbox_attribute_init (void);
VALA_EXTERN void geary_imap_tag_init (void);

void
geary_imap_init (void)
{
	gint _tmp0_;
	_tmp0_ = geary_imap_init_count;
	geary_imap_init_count = _tmp0_ + 1;
	if (_tmp0_ != 0) {
		return;
	}
	geary_imap_message_flag_init ();
	geary_imap_mailbox_attribute_init ();
	geary_imap_tag_init ();
}

