/* desktop-notifications.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __SRC_CLIENT_PLUGIN_DESKTOP_NOTIFICATIONS_DESKTOP_NOTIFICATIONS_H__
#define __SRC_CLIENT_PLUGIN_DESKTOP_NOTIFICATIONS_DESKTOP_NOTIFICATIONS_H__

#include <glib-object.h>
#include "geary-client-46.0.h"
#include <gio/gio.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define PLUGIN_TYPE_DESKTOP_NOTIFICATIONS (plugin_desktop_notifications_get_type ())
#define PLUGIN_DESKTOP_NOTIFICATIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_DESKTOP_NOTIFICATIONS, PluginDesktopNotifications))
#define PLUGIN_DESKTOP_NOTIFICATIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_DESKTOP_NOTIFICATIONS, PluginDesktopNotificationsClass))
#define PLUGIN_IS_DESKTOP_NOTIFICATIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_DESKTOP_NOTIFICATIONS))
#define PLUGIN_IS_DESKTOP_NOTIFICATIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_DESKTOP_NOTIFICATIONS))
#define PLUGIN_DESKTOP_NOTIFICATIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_DESKTOP_NOTIFICATIONS, PluginDesktopNotificationsClass))

typedef struct _PluginDesktopNotifications PluginDesktopNotifications;
typedef struct _PluginDesktopNotificationsClass PluginDesktopNotificationsClass;
typedef struct _PluginDesktopNotificationsPrivate PluginDesktopNotificationsPrivate;

struct _PluginDesktopNotifications {
	PluginPluginBase parent_instance;
	PluginDesktopNotificationsPrivate * priv;
};

struct _PluginDesktopNotificationsClass {
	PluginPluginBaseClass parent_class;
};

VALA_EXTERN void peas_register_types (GTypeModule* module);
GType plugin_desktop_notifications_register_type (GTypeModule * module);
VALA_EXTERN GType plugin_desktop_notifications_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginDesktopNotifications, g_object_unref)
VALA_EXTERN PluginDesktopNotifications* plugin_desktop_notifications_new (void);
VALA_EXTERN PluginDesktopNotifications* plugin_desktop_notifications_construct (GType object_type);

G_END_DECLS

#endif
