/* outbox-folder-properties.c generated by valac 0.56.19, the Vala compiler
 * generated from outbox-folder-properties.vala, do not modify */

/*
 * Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later). See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <glib-object.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES (geary_outbox_folder_properties_get_type ())
#define GEARY_OUTBOX_FOLDER_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES, GearyOutboxFolderProperties))
#define GEARY_OUTBOX_FOLDER_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES, GearyOutboxFolderPropertiesClass))
#define GEARY_OUTBOX_IS_FOLDER_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES))
#define GEARY_OUTBOX_IS_FOLDER_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES))
#define GEARY_OUTBOX_FOLDER_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES, GearyOutboxFolderPropertiesClass))

typedef struct _GearyOutboxFolderProperties GearyOutboxFolderProperties;
typedef struct _GearyOutboxFolderPropertiesClass GearyOutboxFolderPropertiesClass;
typedef struct _GearyOutboxFolderPropertiesPrivate GearyOutboxFolderPropertiesPrivate;
enum  {
	GEARY_OUTBOX_FOLDER_PROPERTIES_0_PROPERTY,
	GEARY_OUTBOX_FOLDER_PROPERTIES_NUM_PROPERTIES
};
static GParamSpec* geary_outbox_folder_properties_properties[GEARY_OUTBOX_FOLDER_PROPERTIES_NUM_PROPERTIES];

struct _GearyOutboxFolderProperties {
	GearyFolderProperties parent_instance;
	GearyOutboxFolderPropertiesPrivate * priv;
};

struct _GearyOutboxFolderPropertiesClass {
	GearyFolderPropertiesClass parent_class;
};

static gpointer geary_outbox_folder_properties_parent_class = NULL;

VALA_EXTERN GType geary_outbox_folder_properties_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyOutboxFolderProperties* geary_outbox_folder_properties_new (gint total,
                                                                 gint unread);
VALA_EXTERN GearyOutboxFolderProperties* geary_outbox_folder_properties_construct (GType object_type,
                                                                       gint total,
                                                                       gint unread);
VALA_EXTERN void geary_outbox_folder_properties_set_total (GearyOutboxFolderProperties* self,
                                               gint total);
static GType geary_outbox_folder_properties_get_type_once (void);

GearyOutboxFolderProperties*
geary_outbox_folder_properties_construct (GType object_type,
                                          gint total,
                                          gint unread)
{
	GearyOutboxFolderProperties * self = NULL;
	self = (GearyOutboxFolderProperties*) geary_folder_properties_construct (object_type, total, unread, GEARY_TRILLIAN_FALSE, GEARY_TRILLIAN_FALSE, GEARY_TRILLIAN_TRUE, TRUE, FALSE, FALSE);
	return self;
}

GearyOutboxFolderProperties*
geary_outbox_folder_properties_new (gint total,
                                    gint unread)
{
	return geary_outbox_folder_properties_construct (GEARY_OUTBOX_TYPE_FOLDER_PROPERTIES, total, unread);
}

void
geary_outbox_folder_properties_set_total (GearyOutboxFolderProperties* self,
                                          gint total)
{
	g_return_if_fail (GEARY_OUTBOX_IS_FOLDER_PROPERTIES (self));
	geary_folder_properties_set_email_total (G_TYPE_CHECK_INSTANCE_CAST (self, GEARY_TYPE_FOLDER_PROPERTIES, GearyFolderProperties), total);
}

static void
geary_outbox_folder_properties_class_init (GearyOutboxFolderPropertiesClass * klass,
                                           gpointer klass_data)
{
	geary_outbox_folder_properties_parent_class = g_type_class_peek_parent (klass);
}

static void
geary_outbox_folder_properties_instance_init (GearyOutboxFolderProperties * self,
                                              gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
geary_outbox_folder_properties_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyOutboxFolderPropertiesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_outbox_folder_properties_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyOutboxFolderProperties), 0, (GInstanceInitFunc) geary_outbox_folder_properties_instance_init, NULL };
	GType geary_outbox_folder_properties_type_id;
	geary_outbox_folder_properties_type_id = g_type_register_static (GEARY_TYPE_FOLDER_PROPERTIES, "GearyOutboxFolderProperties", &g_define_type_info, 0);
	return geary_outbox_folder_properties_type_id;
}

GType
geary_outbox_folder_properties_get_type (void)
{
	static gsize geary_outbox_folder_properties_type_id__once = 0;
	if (g_once_init_enter (&geary_outbox_folder_properties_type_id__once)) {
		GType geary_outbox_folder_properties_type_id;
		geary_outbox_folder_properties_type_id = geary_outbox_folder_properties_get_type_once ();
		g_once_init_leave (&geary_outbox_folder_properties_type_id__once, geary_outbox_folder_properties_type_id);
	}
	return geary_outbox_folder_properties_type_id__once;
}

