/* main.c generated by valac 0.56.19, the Vala compiler
 * generated from main.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "geary-engine.h"
#include "geary-client-46.0.h"
#include <gio/gio.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

static gint _vala_main (gchar** args,
                 gint args_length1);
static GLogWriterOutput _geary_logging_default_log_writer_glog_writer_func (GLogLevelFlags log_level,
                                                                     GLogField* fields,
                                                                     gsize fields_length1,
                                                                     gpointer self);

static GLogWriterOutput
_geary_logging_default_log_writer_glog_writer_func (GLogLevelFlags log_level,
                                                    GLogField* fields,
                                                    gsize fields_length1,
                                                    gpointer self)
{
	GLogWriterOutput result;
	result = geary_logging_default_log_writer (log_level, fields, fields_length1);
	return result;
}

static gint
_vala_main (gchar** args,
            gint args_length1)
{
	ApplicationClient* app = NULL;
	ApplicationClient* _tmp0_;
	gint ec = 0;
	gint result;
	geary_logging_init ();
	g_log_set_writer_func (_geary_logging_default_log_writer_glog_writer_func, NULL, NULL);
	_tmp0_ = application_client_new ();
	app = _tmp0_;
	ec = g_application_run (G_TYPE_CHECK_INSTANCE_CAST (app, g_application_get_type (), GApplication), (gint) args_length1, args);
	result = ec;
	_g_object_unref0 (app);
	return result;
}

int
main (int argc,
      char ** argv)
{
	return _vala_main (argv, argc);
}

