/* client-action.c generated by valac 0.56.19, the Vala compiler
 * generated from client-action.vala, do not modify */

/*
 * Copyright 2019-2021 Michael Gratton <mike@vee.net>
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later). See the COPYING file in this distribution.
 */

#include "geary-client-46.0.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

/** Returns the given action name prefixed with the group name. */
gchar*
action_application_prefix (const gchar* action_name)
{
	gchar* _tmp0_;
	gchar* result;
	g_return_val_if_fail (action_name != NULL, NULL);
	_tmp0_ = g_strconcat (ACTION_APPLICATION_GROUP_NAME ".", action_name, NULL);
	result = _tmp0_;
	return result;
}

/** Returns the given action name prefixed with the group name. */
gchar*
action_window_prefix (const gchar* action_name)
{
	gchar* _tmp0_;
	gchar* result;
	g_return_val_if_fail (action_name != NULL, NULL);
	_tmp0_ = g_strconcat (ACTION_WINDOW_GROUP_NAME ".", action_name, NULL);
	result = _tmp0_;
	return result;
}

/** Returns the given action name prefixed with the group name. */
gchar*
action_edit_prefix (const gchar* action_name)
{
	gchar* _tmp0_;
	gchar* result;
	g_return_val_if_fail (action_name != NULL, NULL);
	_tmp0_ = g_strconcat (ACTION_EDIT_GROUP_NAME ".", action_name, NULL);
	result = _tmp0_;
	return result;
}

