/* sent-sound.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __SRC_CLIENT_PLUGIN_SENT_SOUND_SENT_SOUND_H__
#define __SRC_CLIENT_PLUGIN_SENT_SOUND_SENT_SOUND_H__

#include <glib-object.h>
#include "geary-client-46.0.h"
#include <gio/gio.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define PLUGIN_TYPE_SENT_SOUND (plugin_sent_sound_get_type ())
#define PLUGIN_SENT_SOUND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_SENT_SOUND, PluginSentSound))
#define PLUGIN_SENT_SOUND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_SENT_SOUND, PluginSentSoundClass))
#define PLUGIN_IS_SENT_SOUND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_SENT_SOUND))
#define PLUGIN_IS_SENT_SOUND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_SENT_SOUND))
#define PLUGIN_SENT_SOUND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_SENT_SOUND, PluginSentSoundClass))

typedef struct _PluginSentSound PluginSentSound;
typedef struct _PluginSentSoundClass PluginSentSoundClass;
typedef struct _PluginSentSoundPrivate PluginSentSoundPrivate;

struct _PluginSentSound {
	PluginPluginBase parent_instance;
	PluginSentSoundPrivate * priv;
};

struct _PluginSentSoundClass {
	PluginPluginBaseClass parent_class;
};

VALA_EXTERN void peas_register_types (GTypeModule* module);
GType plugin_sent_sound_register_type (GTypeModule * module);
VALA_EXTERN GType plugin_sent_sound_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginSentSound, g_object_unref)
VALA_EXTERN PluginSentSound* plugin_sent_sound_new (void);
VALA_EXTERN PluginSentSound* plugin_sent_sound_construct (GType object_type);

G_END_DECLS

#endif
