/* nonblocking-error.c generated by valac 0.56.19, the Vala compiler
 * generated from nonblocking-error.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

GQuark
geary_nonblocking_error_quark (void)
{
	return g_quark_from_static_string ("geary-nonblocking-error-quark");
}

 G_GNUC_NO_INLINE static GType
geary_nonblocking_error_get_type_once (void)
{
	static const GEnumValue values[] = {{GEARY_NONBLOCKING_ERROR_INVALID, "GEARY_NONBLOCKING_ERROR_INVALID", "invalid"}, {0, NULL, NULL}};
	GType geary_nonblocking_error_type_id;
	geary_nonblocking_error_type_id = g_enum_register_static ("GearyNonblockingError", values);
	return geary_nonblocking_error_type_id;
}

GType
geary_nonblocking_error_get_type (void)
{
	static gsize geary_nonblocking_error_type_id__once = 0;
	if (g_once_init_enter (&geary_nonblocking_error_type_id__once)) {
		GType geary_nonblocking_error_type_id;
		geary_nonblocking_error_type_id = geary_nonblocking_error_get_type_once ();
		g_once_init_leave (&geary_nonblocking_error_type_id__once, geary_nonblocking_error_type_id);
	}
	return geary_nonblocking_error_type_id__once;
}

