/* state-mapping.c generated by valac 0.56.19, the Vala compiler
 * generated from state-mapping.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <glib-object.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	GEARY_STATE_MAPPING_0_PROPERTY,
	GEARY_STATE_MAPPING_NUM_PROPERTIES
};
static GParamSpec* geary_state_mapping_properties[GEARY_STATE_MAPPING_NUM_PROPERTIES];

static gpointer geary_state_mapping_parent_class = NULL;

static void geary_state_mapping_finalize (GObject * obj);
static GType geary_state_mapping_get_type_once (void);

GearyStateMapping*
geary_state_mapping_construct (GType object_type,
                               guint state,
                               guint event,
                               GearyStateTransition transition,
                               gpointer transition_target)
{
	GearyStateMapping * self = NULL;
	self = (GearyStateMapping*) geary_base_object_construct (object_type);
	self->state = state;
	self->event = event;
	self->transition = transition;
	self->transition_target = transition_target;
	return self;
}

GearyStateMapping*
geary_state_mapping_new (guint state,
                         guint event,
                         GearyStateTransition transition,
                         gpointer transition_target)
{
	return geary_state_mapping_construct (GEARY_STATE_TYPE_MAPPING, state, event, transition, transition_target);
}

static void
geary_state_mapping_class_init (GearyStateMappingClass * klass,
                                gpointer klass_data)
{
	geary_state_mapping_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = geary_state_mapping_finalize;
}

static void
geary_state_mapping_instance_init (GearyStateMapping * self,
                                   gpointer klass)
{
}

static void
geary_state_mapping_finalize (GObject * obj)
{
	GearyStateMapping * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEARY_STATE_TYPE_MAPPING, GearyStateMapping);
	G_OBJECT_CLASS (geary_state_mapping_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
geary_state_mapping_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyStateMappingClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_state_mapping_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyStateMapping), 0, (GInstanceInitFunc) geary_state_mapping_instance_init, NULL };
	GType geary_state_mapping_type_id;
	geary_state_mapping_type_id = g_type_register_static (GEARY_TYPE_BASE_OBJECT, "GearyStateMapping", &g_define_type_info, 0);
	return geary_state_mapping_type_id;
}

GType
geary_state_mapping_get_type (void)
{
	static gsize geary_state_mapping_type_id__once = 0;
	if (g_once_init_enter (&geary_state_mapping_type_id__once)) {
		GType geary_state_mapping_type_id;
		geary_state_mapping_type_id = geary_state_mapping_get_type_once ();
		g_once_init_leave (&geary_state_mapping_type_id__once, geary_state_mapping_type_id);
	}
	return geary_state_mapping_type_id__once;
}

guint
geary_state_nop (guint state,
                 guint event)
{
	guint result;
	result = state;
	return result;
}

