/* imap-engine-copy-email.c generated by valac 0.56.19, the Vala compiler
 * generated from imap-engine-copy-email.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <gee.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION (geary_imap_engine_replay_operation_get_type ())
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperation))
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperationClass))
#define GEARY_IMAP_ENGINE_IS_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_IS_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperationClass))

typedef struct _GearyImapEngineReplayOperation GearyImapEngineReplayOperation;
typedef struct _GearyImapEngineReplayOperationClass GearyImapEngineReplayOperationClass;
typedef struct _GearyImapEngineReplayOperationPrivate GearyImapEngineReplayOperationPrivate;

#define GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER (geary_imap_db_email_identifier_get_type ())
#define GEARY_IMAP_DB_EMAIL_IDENTIFIER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, GearyImapDBEmailIdentifier))
#define GEARY_IMAP_DB_EMAIL_IDENTIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, GearyImapDBEmailIdentifierClass))
#define GEARY_IMAP_DB_IS_EMAIL_IDENTIFIER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER))
#define GEARY_IMAP_DB_IS_EMAIL_IDENTIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER))
#define GEARY_IMAP_DB_EMAIL_IDENTIFIER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, GearyImapDBEmailIdentifierClass))

typedef struct _GearyImapDBEmailIdentifier GearyImapDBEmailIdentifier;
typedef struct _GearyImapDBEmailIdentifierClass GearyImapDBEmailIdentifierClass;
typedef enum  {
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_STATUS_COMPLETED,
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_STATUS_CONTINUE
} GearyImapEngineReplayOperationStatus;

#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_TYPE_STATUS (geary_imap_engine_replay_operation_status_get_type ())

#define GEARY_IMAP_TYPE_FOLDER_SESSION (geary_imap_folder_session_get_type ())
#define GEARY_IMAP_FOLDER_SESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_TYPE_FOLDER_SESSION, GearyImapFolderSession))
#define GEARY_IMAP_FOLDER_SESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_TYPE_FOLDER_SESSION, GearyImapFolderSessionClass))
#define GEARY_IMAP_IS_FOLDER_SESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_TYPE_FOLDER_SESSION))
#define GEARY_IMAP_IS_FOLDER_SESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_TYPE_FOLDER_SESSION))
#define GEARY_IMAP_FOLDER_SESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_TYPE_FOLDER_SESSION, GearyImapFolderSessionClass))

typedef struct _GearyImapFolderSession GearyImapFolderSession;
typedef struct _GearyImapFolderSessionClass GearyImapFolderSessionClass;

#define GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION (geary_imap_engine_send_replay_operation_get_type ())
#define GEARY_IMAP_ENGINE_SEND_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, GearyImapEngineSendReplayOperation))
#define GEARY_IMAP_ENGINE_SEND_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, GearyImapEngineSendReplayOperationClass))
#define GEARY_IMAP_ENGINE_IS_SEND_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_IS_SEND_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_SEND_REPLAY_OPERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, GearyImapEngineSendReplayOperationClass))

typedef struct _GearyImapEngineSendReplayOperation GearyImapEngineSendReplayOperation;
typedef struct _GearyImapEngineSendReplayOperationClass GearyImapEngineSendReplayOperationClass;
typedef struct _GearyImapEngineSendReplayOperationPrivate GearyImapEngineSendReplayOperationPrivate;

#define GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL (geary_imap_engine_copy_email_get_type ())
#define GEARY_IMAP_ENGINE_COPY_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmail))
#define GEARY_IMAP_ENGINE_COPY_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmailClass))
#define GEARY_IMAP_ENGINE_IS_COPY_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL))
#define GEARY_IMAP_ENGINE_IS_COPY_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL))
#define GEARY_IMAP_ENGINE_COPY_EMAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmailClass))

typedef struct _GearyImapEngineCopyEmail GearyImapEngineCopyEmail;
typedef struct _GearyImapEngineCopyEmailClass GearyImapEngineCopyEmailClass;
typedef struct _GearyImapEngineCopyEmailPrivate GearyImapEngineCopyEmailPrivate;

#define GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER (geary_imap_engine_minimal_folder_get_type ())
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolder))
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolderClass))
#define GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER))
#define GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER))
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolderClass))

typedef struct _GearyImapEngineMinimalFolder GearyImapEngineMinimalFolder;
typedef struct _GearyImapEngineMinimalFolderClass GearyImapEngineMinimalFolderClass;
enum  {
	GEARY_IMAP_ENGINE_COPY_EMAIL_0_PROPERTY,
	GEARY_IMAP_ENGINE_COPY_EMAIL_NUM_PROPERTIES
};
static GParamSpec* geary_imap_engine_copy_email_properties[GEARY_IMAP_ENGINE_COPY_EMAIL_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef enum  {
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_THROW,
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_RETRY,
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_IGNORE_REMOTE
} GearyImapEngineReplayOperationOnError;

#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_TYPE_ON_ERROR (geary_imap_engine_replay_operation_on_error_get_type ())
typedef struct _GearyImapEngineCopyEmailReplayLocalAsyncData GearyImapEngineCopyEmailReplayLocalAsyncData;

#define GEARY_IMAP_DB_TYPE_FOLDER (geary_imap_db_folder_get_type ())
#define GEARY_IMAP_DB_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolder))
#define GEARY_IMAP_DB_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolderClass))
#define GEARY_IMAP_DB_IS_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_DB_TYPE_FOLDER))
#define GEARY_IMAP_DB_IS_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_DB_TYPE_FOLDER))
#define GEARY_IMAP_DB_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolderClass))

typedef struct _GearyImapDBFolder GearyImapDBFolder;
typedef struct _GearyImapDBFolderClass GearyImapDBFolderClass;
typedef enum  {
	GEARY_IMAP_DB_FOLDER_LIST_FLAGS_NONE = 0,
	GEARY_IMAP_DB_FOLDER_LIST_FLAGS_PARTIAL_OK = 1 << 0,
	GEARY_IMAP_DB_FOLDER_LIST_FLAGS_INCLUDE_MARKED_FOR_REMOVE = 1 << 1,
	GEARY_IMAP_DB_FOLDER_LIST_FLAGS_INCLUDING_ID = 1 << 2,
	GEARY_IMAP_DB_FOLDER_LIST_FLAGS_OLDEST_TO_NEWEST = 1 << 3,
	GEARY_IMAP_DB_FOLDER_LIST_FLAGS_ONLY_INCOMPLETE = 1 << 4
} GearyImapDBFolderListFlags;

#define GEARY_IMAP_DB_FOLDER_TYPE_LIST_FLAGS (geary_imap_db_folder_list_flags_get_type ())
typedef struct _GearyImapEngineCopyEmailReplayRemoteAsyncData GearyImapEngineCopyEmailReplayRemoteAsyncData;
#define _g_free0(var) (var = (g_free (var), NULL))

struct _GearyImapEngineReplayOperation {
	GearyBaseObject parent_instance;
	GearyImapEngineReplayOperationPrivate * priv;
};

struct _GearyImapEngineReplayOperationClass {
	GearyBaseObjectClass parent_class;
	void (*notify_remote_removed_position) (GearyImapEngineReplayOperation* self, GearyImapSequenceNumber* removed);
	void (*notify_remote_removed_ids) (GearyImapEngineReplayOperation* self, GeeCollection* ids);
	void (*get_ids_to_be_remote_removed) (GearyImapEngineReplayOperation* self, GeeCollection* ids);
	void (*replay_local_async) (GearyImapEngineReplayOperation* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GearyImapEngineReplayOperationStatus (*replay_local_finish) (GearyImapEngineReplayOperation* self, GAsyncResult* _res_, GError** error);
	void (*replay_remote_async) (GearyImapEngineReplayOperation* self, GearyImapFolderSession* remote, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*replay_remote_finish) (GearyImapEngineReplayOperation* self, GAsyncResult* _res_, GError** error);
	void (*backout_local_async) (GearyImapEngineReplayOperation* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*backout_local_finish) (GearyImapEngineReplayOperation* self, GAsyncResult* _res_, GError** error);
	gchar* (*describe_state) (GearyImapEngineReplayOperation* self);
};

struct _GearyImapEngineSendReplayOperation {
	GearyImapEngineReplayOperation parent_instance;
	GearyImapEngineSendReplayOperationPrivate * priv;
};

struct _GearyImapEngineSendReplayOperationClass {
	GearyImapEngineReplayOperationClass parent_class;
};

struct _GearyImapEngineCopyEmail {
	GearyImapEngineSendReplayOperation parent_instance;
	GeeSet* destination_uids;
	GearyImapEngineCopyEmailPrivate * priv;
};

struct _GearyImapEngineCopyEmailClass {
	GearyImapEngineSendReplayOperationClass parent_class;
};

struct _GearyImapEngineCopyEmailPrivate {
	GearyImapEngineMinimalFolder* engine;
	GeeHashSet* to_copy;
	GearyFolderPath* destination;
	GCancellable* cancellable;
};

struct _GearyImapEngineCopyEmailReplayLocalAsyncData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GearyImapEngineCopyEmail* self;
	GearyImapEngineReplayOperationStatus result;
	GeeHashSet* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
};

struct _GearyImapEngineCopyEmailReplayRemoteAsyncData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GearyImapEngineCopyEmail* self;
	GearyImapFolderSession* remote;
	GeeHashSet* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	GeeSet* uids;
	GearyImapEngineMinimalFolder* _tmp3_;
	GearyImapDBFolder* _tmp4_;
	GearyImapDBFolder* _tmp5_;
	GeeHashSet* _tmp6_;
	GCancellable* _tmp7_;
	GeeSet* _tmp8_;
	gboolean _tmp9_;
	GeeSet* _tmp10_;
	GeeSet* _tmp11_;
	gint _tmp12_;
	gint _tmp13_;
	GeeList* msg_sets;
	GeeSet* _tmp14_;
	GeeList* _tmp15_;
	GeeList* _msg_set_list;
	GeeList* _tmp16_;
	gint _msg_set_size;
	GeeList* _tmp17_;
	gint _tmp18_;
	gint _tmp19_;
	gint _msg_set_index;
	gint _tmp20_;
	gint _tmp21_;
	GearyImapMessageSet* msg_set;
	GeeList* _tmp22_;
	gpointer _tmp23_;
	GeeMap* src_dst_uids;
	GearyImapMessageSet* _tmp24_;
	GearyFolderPath* _tmp25_;
	GCancellable* _tmp26_;
	GeeMap* _tmp27_;
	GeeMap* _tmp28_;
	GeeSet* _tmp29_;
	GeeMap* _tmp30_;
	GeeCollection* _tmp31_;
	GeeCollection* _tmp32_;
	GeeCollection* _tmp33_;
	GError* _inner_error0_;
};

static gint GearyImapEngineCopyEmail_private_offset;
static gpointer geary_imap_engine_copy_email_parent_class = NULL;

VALA_EXTERN GType geary_imap_engine_replay_operation_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_db_email_identifier_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_replay_operation_status_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_folder_session_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_send_replay_operation_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_copy_email_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_minimal_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void geary_imap_engine_replay_operation_notify_remote_removed_ids (GearyImapEngineReplayOperation* self,
                                                                   GeeCollection* ids);
VALA_EXTERN void geary_imap_engine_replay_operation_replay_local_async (GearyImapEngineReplayOperation* self,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_);
VALA_EXTERN GearyImapEngineReplayOperationStatus geary_imap_engine_replay_operation_replay_local_finish (GearyImapEngineReplayOperation* self,
                                                                                             GAsyncResult* _res_,
                                                                                             GError** error);
VALA_EXTERN void geary_imap_engine_replay_operation_replay_remote_async (GearyImapEngineReplayOperation* self,
                                                             GearyImapFolderSession* remote,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN void geary_imap_engine_replay_operation_replay_remote_finish (GearyImapEngineReplayOperation* self,
                                                              GAsyncResult* _res_,
                                                              GError** error);
VALA_EXTERN gchar* geary_imap_engine_replay_operation_describe_state (GearyImapEngineReplayOperation* self);
VALA_EXTERN GearyImapEngineCopyEmail* geary_imap_engine_copy_email_new (GearyImapEngineMinimalFolder* engine,
                                                            GeeList* to_copy,
                                                            GearyFolderPath* destination,
                                                            GCancellable* cancellable);
VALA_EXTERN GearyImapEngineCopyEmail* geary_imap_engine_copy_email_construct (GType object_type,
                                                                  GearyImapEngineMinimalFolder* engine,
                                                                  GeeList* to_copy,
                                                                  GearyFolderPath* destination,
                                                                  GCancellable* cancellable);
VALA_EXTERN GType geary_imap_engine_replay_operation_on_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyImapEngineSendReplayOperation* geary_imap_engine_send_replay_operation_construct (GType object_type,
                                                                                       const gchar* name,
                                                                                       GearyImapEngineReplayOperationOnError on_remote_error);
static void geary_imap_engine_copy_email_real_notify_remote_removed_ids (GearyImapEngineReplayOperation* base,
                                                                  GeeCollection* ids);
static void geary_imap_engine_copy_email_real_replay_local_async_data_free (gpointer _data);
static void geary_imap_engine_copy_email_real_replay_local_async (GearyImapEngineReplayOperation* base,
                                                           GAsyncReadyCallback _callback_,
                                                           gpointer _user_data_);
static gboolean geary_imap_engine_copy_email_real_replay_local_async_co (GearyImapEngineCopyEmailReplayLocalAsyncData* _data_);
static void geary_imap_engine_copy_email_real_replay_remote_async_data_free (gpointer _data);
static void geary_imap_engine_copy_email_real_replay_remote_async (GearyImapEngineReplayOperation* base,
                                                            GearyImapFolderSession* remote,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_);
static gboolean geary_imap_engine_copy_email_real_replay_remote_async_co (GearyImapEngineCopyEmailReplayRemoteAsyncData* _data_);
VALA_EXTERN GType geary_imap_db_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyImapDBFolder* geary_imap_engine_minimal_folder_get_local_folder (GearyImapEngineMinimalFolder* self);
VALA_EXTERN GType geary_imap_db_folder_list_flags_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void geary_imap_db_folder_get_uids_async (GearyImapDBFolder* self,
                                          GeeCollection* ids,
                                          GearyImapDBFolderListFlags flags,
                                          GCancellable* cancellable,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN GeeSet* geary_imap_db_folder_get_uids_finish (GearyImapDBFolder* self,
                                              GAsyncResult* _res_,
                                              GError** error);
static void geary_imap_engine_copy_email_replay_remote_async_ready (GObject* source_object,
                                                             GAsyncResult* _res_,
                                                             gpointer _user_data_);
VALA_EXTERN void geary_imap_folder_session_copy_email_async (GearyImapFolderSession* self,
                                                 GearyImapMessageSet* msg_set,
                                                 GearyFolderPath* destination,
                                                 GCancellable* cancellable,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
VALA_EXTERN GeeMap* geary_imap_folder_session_copy_email_finish (GearyImapFolderSession* self,
                                                     GAsyncResult* _res_,
                                                     GError** error);
static gchar* geary_imap_engine_copy_email_real_describe_state (GearyImapEngineReplayOperation* base);
static void geary_imap_engine_copy_email_finalize (GObject * obj);
static GType geary_imap_engine_copy_email_get_type_once (void);

static inline gpointer
geary_imap_engine_copy_email_get_instance_private (GearyImapEngineCopyEmail* self)
{
	return G_STRUCT_MEMBER_P (self, GearyImapEngineCopyEmail_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

GearyImapEngineCopyEmail*
geary_imap_engine_copy_email_construct (GType object_type,
                                        GearyImapEngineMinimalFolder* engine,
                                        GeeList* to_copy,
                                        GearyFolderPath* destination,
                                        GCancellable* cancellable)
{
	GearyImapEngineCopyEmail * self = NULL;
	GearyImapEngineMinimalFolder* _tmp0_;
	GeeHashSet* _tmp1_;
	GearyFolderPath* _tmp2_;
	GCancellable* _tmp3_;
	g_return_val_if_fail (GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER (engine), NULL);
	g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (to_copy, GEE_TYPE_LIST), NULL);
	g_return_val_if_fail (GEARY_IS_FOLDER_PATH (destination), NULL);
	g_return_val_if_fail ((cancellable == NULL) || G_TYPE_CHECK_INSTANCE_TYPE (cancellable, g_cancellable_get_type ()), NULL);
	self = (GearyImapEngineCopyEmail*) geary_imap_engine_send_replay_operation_construct (object_type, "CopyEmail", GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_RETRY);
	_tmp0_ = _g_object_ref0 (engine);
	_g_object_unref0 (self->priv->engine);
	self->priv->engine = _tmp0_;
	_tmp1_ = self->priv->to_copy;
	gee_collection_add_all (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, GEE_TYPE_COLLECTION, GeeCollection), G_TYPE_CHECK_INSTANCE_CAST (to_copy, GEE_TYPE_COLLECTION, GeeCollection));
	_tmp2_ = _g_object_ref0 (destination);
	_g_object_unref0 (self->priv->destination);
	self->priv->destination = _tmp2_;
	_tmp3_ = _g_object_ref0 (cancellable);
	_g_object_unref0 (self->priv->cancellable);
	self->priv->cancellable = _tmp3_;
	return self;
}

GearyImapEngineCopyEmail*
geary_imap_engine_copy_email_new (GearyImapEngineMinimalFolder* engine,
                                  GeeList* to_copy,
                                  GearyFolderPath* destination,
                                  GCancellable* cancellable)
{
	return geary_imap_engine_copy_email_construct (GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, engine, to_copy, destination, cancellable);
}

static void
geary_imap_engine_copy_email_real_notify_remote_removed_ids (GearyImapEngineReplayOperation* base,
                                                             GeeCollection* ids)
{
	GearyImapEngineCopyEmail * self;
	GeeHashSet* _tmp0_;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmail);
	g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (ids, GEE_TYPE_COLLECTION));
	_tmp0_ = self->priv->to_copy;
	gee_collection_remove_all (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_COLLECTION, GeeCollection), ids);
}

static void
geary_imap_engine_copy_email_real_replay_local_async_data_free (gpointer _data)
{
	GearyImapEngineCopyEmailReplayLocalAsyncData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->self);
	g_slice_free (GearyImapEngineCopyEmailReplayLocalAsyncData, _data_);
}

static void
geary_imap_engine_copy_email_real_replay_local_async (GearyImapEngineReplayOperation* base,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_)
{
	GearyImapEngineCopyEmail * self;
	GearyImapEngineCopyEmailReplayLocalAsyncData* _data_;
	GearyImapEngineCopyEmail* _tmp0_;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmail);
	_data_ = g_slice_new0 (GearyImapEngineCopyEmailReplayLocalAsyncData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, geary_imap_engine_copy_email_real_replay_local_async_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	geary_imap_engine_copy_email_real_replay_local_async_co (_data_);
}

static GearyImapEngineReplayOperationStatus
geary_imap_engine_copy_email_replay_local_finish (GearyImapEngineReplayOperation* base,
                                                  GAsyncResult* _res_,
                                                  GError** error)
{
	GearyImapEngineReplayOperationStatus result;
	GearyImapEngineCopyEmailReplayLocalAsyncData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
	if (NULL == _data_) {
		return 0;
	}
	result = _data_->result;
	return result;
}

static gboolean
geary_imap_engine_copy_email_real_replay_local_async_co (GearyImapEngineCopyEmailReplayLocalAsyncData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->to_copy;
	_data_->_tmp1_ = gee_abstract_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp0_, GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection));
	_data_->_tmp2_ = _data_->_tmp1_;
	if (_data_->_tmp2_ == 0) {
		_data_->result = GEARY_IMAP_ENGINE_REPLAY_OPERATION_STATUS_COMPLETED;
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->result = GEARY_IMAP_ENGINE_REPLAY_OPERATION_STATUS_CONTINUE;
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
geary_imap_engine_copy_email_real_replay_remote_async_data_free (gpointer _data)
{
	GearyImapEngineCopyEmailReplayRemoteAsyncData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->remote);
	_g_object_unref0 (_data_->self);
	g_slice_free (GearyImapEngineCopyEmailReplayRemoteAsyncData, _data_);
}

static void
geary_imap_engine_copy_email_real_replay_remote_async (GearyImapEngineReplayOperation* base,
                                                       GearyImapFolderSession* remote,
                                                       GAsyncReadyCallback _callback_,
                                                       gpointer _user_data_)
{
	GearyImapEngineCopyEmail * self;
	GearyImapEngineCopyEmailReplayRemoteAsyncData* _data_;
	GearyImapEngineCopyEmail* _tmp0_;
	GearyImapFolderSession* _tmp1_;
	g_return_if_fail (GEARY_IMAP_IS_FOLDER_SESSION (remote));
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmail);
	_data_ = g_slice_new0 (GearyImapEngineCopyEmailReplayRemoteAsyncData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, geary_imap_engine_copy_email_real_replay_remote_async_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (remote);
	_g_object_unref0 (_data_->remote);
	_data_->remote = _tmp1_;
	geary_imap_engine_copy_email_real_replay_remote_async_co (_data_);
}

static void
geary_imap_engine_copy_email_replay_remote_finish (GearyImapEngineReplayOperation* base,
                                                   GAsyncResult* _res_,
                                                   GError** error)
{
	GearyImapEngineCopyEmailReplayRemoteAsyncData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
	if (NULL == _data_) {
		return;
	}
}

static void
geary_imap_engine_copy_email_replay_remote_async_ready (GObject* source_object,
                                                        GAsyncResult* _res_,
                                                        gpointer _user_data_)
{
	GearyImapEngineCopyEmailReplayRemoteAsyncData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	geary_imap_engine_copy_email_real_replay_remote_async_co (_data_);
}

static gboolean
geary_imap_engine_copy_email_real_replay_remote_async_co (GearyImapEngineCopyEmailReplayRemoteAsyncData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		case 2:
		goto _state_2;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->to_copy;
	_data_->_tmp1_ = gee_abstract_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp0_, GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection));
	_data_->_tmp2_ = _data_->_tmp1_;
	if (_data_->_tmp2_ > 0) {
		_data_->_tmp3_ = _data_->self->priv->engine;
		_data_->_tmp4_ = geary_imap_engine_minimal_folder_get_local_folder (_data_->_tmp3_);
		_data_->_tmp5_ = _data_->_tmp4_;
		_data_->_tmp6_ = _data_->self->priv->to_copy;
		_data_->_tmp7_ = _data_->self->priv->cancellable;
		_data_->_state_ = 1;
		geary_imap_db_folder_get_uids_async (_data_->_tmp5_, G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp6_, GEE_TYPE_COLLECTION, GeeCollection), GEARY_IMAP_DB_FOLDER_LIST_FLAGS_NONE, _data_->_tmp7_, geary_imap_engine_copy_email_replay_remote_async_ready, _data_);
		return FALSE;
		_state_1:
		_data_->_tmp8_ = geary_imap_db_folder_get_uids_finish (_data_->_tmp5_, _data_->_res_, &_data_->_inner_error0_);
		_data_->uids = _data_->_tmp8_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
			g_object_unref (_data_->_async_result);
			return FALSE;
		}
		_data_->_tmp10_ = _data_->uids;
		if (_data_->_tmp10_ != NULL) {
			_data_->_tmp11_ = _data_->uids;
			_data_->_tmp12_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp11_, GEE_TYPE_COLLECTION, GeeCollection));
			_data_->_tmp13_ = _data_->_tmp12_;
			_data_->_tmp9_ = _data_->_tmp13_ > 0;
		} else {
			_data_->_tmp9_ = FALSE;
		}
		if (_data_->_tmp9_) {
			_data_->_tmp14_ = _data_->uids;
			_data_->_tmp15_ = geary_imap_message_set_uid_sparse (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp14_, GEE_TYPE_COLLECTION, GeeCollection));
			_data_->msg_sets = _data_->_tmp15_;
			{
				_data_->_tmp16_ = _data_->msg_sets;
				_data_->_msg_set_list = _data_->_tmp16_;
				_data_->_tmp17_ = _data_->_msg_set_list;
				_data_->_tmp18_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp17_, GEE_TYPE_COLLECTION, GeeCollection));
				_data_->_tmp19_ = _data_->_tmp18_;
				_data_->_msg_set_size = _data_->_tmp19_;
				_data_->_msg_set_index = -1;
				while (TRUE) {
					_data_->_msg_set_index = _data_->_msg_set_index + 1;
					_data_->_tmp20_ = _data_->_msg_set_index;
					_data_->_tmp21_ = _data_->_msg_set_size;
					if (!(_data_->_tmp20_ < _data_->_tmp21_)) {
						break;
					}
					_data_->_tmp22_ = _data_->_msg_set_list;
					_data_->_tmp23_ = gee_list_get (_data_->_tmp22_, _data_->_msg_set_index);
					_data_->msg_set = (GearyImapMessageSet*) _data_->_tmp23_;
					_data_->_tmp24_ = _data_->msg_set;
					_data_->_tmp25_ = _data_->self->priv->destination;
					_data_->_tmp26_ = _data_->self->priv->cancellable;
					_data_->_state_ = 2;
					geary_imap_folder_session_copy_email_async (_data_->remote, _data_->_tmp24_, _data_->_tmp25_, _data_->_tmp26_, geary_imap_engine_copy_email_replay_remote_async_ready, _data_);
					return FALSE;
					_state_2:
					_data_->_tmp27_ = geary_imap_folder_session_copy_email_finish (_data_->remote, _data_->_res_, &_data_->_inner_error0_);
					_data_->src_dst_uids = _data_->_tmp27_;
					if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
						g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
						_g_object_unref0 (_data_->msg_set);
						_g_object_unref0 (_data_->msg_sets);
						_g_object_unref0 (_data_->uids);
						g_object_unref (_data_->_async_result);
						return FALSE;
					}
					_data_->_tmp28_ = _data_->src_dst_uids;
					if (_data_->_tmp28_ != NULL) {
						_data_->_tmp29_ = _data_->self->destination_uids;
						_data_->_tmp30_ = _data_->src_dst_uids;
						_data_->_tmp31_ = gee_map_get_values (_data_->_tmp30_);
						_data_->_tmp32_ = _data_->_tmp31_;
						_data_->_tmp33_ = _data_->_tmp32_;
						gee_collection_add_all (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp29_, GEE_TYPE_COLLECTION, GeeCollection), _data_->_tmp33_);
						_g_object_unref0 (_data_->_tmp33_);
					}
					_g_object_unref0 (_data_->src_dst_uids);
					_g_object_unref0 (_data_->msg_set);
				}
			}
			_g_object_unref0 (_data_->msg_sets);
		}
		_g_object_unref0 (_data_->uids);
	}
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static gchar*
geary_imap_engine_copy_email_real_describe_state (GearyImapEngineReplayOperation* base)
{
	GearyImapEngineCopyEmail * self;
	GeeHashSet* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	GearyFolderPath* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* result;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmail);
	_tmp0_ = self->priv->to_copy;
	_tmp1_ = gee_abstract_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection));
	_tmp2_ = _tmp1_;
	_tmp3_ = self->priv->destination;
	_tmp4_ = geary_folder_path_to_string (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = g_strdup_printf ("%d email IDs to %s", _tmp2_, _tmp5_);
	_tmp7_ = _tmp6_;
	_g_free0 (_tmp5_);
	result = _tmp7_;
	return result;
}

static void
geary_imap_engine_copy_email_class_init (GearyImapEngineCopyEmailClass * klass,
                                         gpointer klass_data)
{
	geary_imap_engine_copy_email_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GearyImapEngineCopyEmail_private_offset);
	((GearyImapEngineReplayOperationClass *) klass)->notify_remote_removed_ids = (void (*) (GearyImapEngineReplayOperation*, GeeCollection*)) geary_imap_engine_copy_email_real_notify_remote_removed_ids;
	((GearyImapEngineReplayOperationClass *) klass)->replay_local_async = (void (*) (GearyImapEngineReplayOperation*, GAsyncReadyCallback, gpointer)) geary_imap_engine_copy_email_real_replay_local_async;
	((GearyImapEngineReplayOperationClass *) klass)->replay_local_finish = (GearyImapEngineReplayOperationStatus (*) (GearyImapEngineReplayOperation*, GAsyncResult*, GError**)) geary_imap_engine_copy_email_replay_local_finish;
	((GearyImapEngineReplayOperationClass *) klass)->replay_remote_async = (void (*) (GearyImapEngineReplayOperation*, GearyImapFolderSession*, GAsyncReadyCallback, gpointer)) geary_imap_engine_copy_email_real_replay_remote_async;
	((GearyImapEngineReplayOperationClass *) klass)->replay_remote_finish = (void (*) (GearyImapEngineReplayOperation*, GAsyncResult*, GError**)) geary_imap_engine_copy_email_replay_remote_finish;
	((GearyImapEngineReplayOperationClass *) klass)->describe_state = (gchar* (*) (GearyImapEngineReplayOperation*)) geary_imap_engine_copy_email_real_describe_state;
	G_OBJECT_CLASS (klass)->finalize = geary_imap_engine_copy_email_finalize;
}

static void
geary_imap_engine_copy_email_instance_init (GearyImapEngineCopyEmail * self,
                                            gpointer klass)
{
	GeeHashSet* _tmp0_;
	GeeHashSet* _tmp1_;
	self->priv = geary_imap_engine_copy_email_get_instance_private (self);
	_tmp0_ = gee_hash_set_new (GEARY_IMAP_TYPE_UID, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL);
	self->destination_uids = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SET, GeeSet);
	_tmp1_ = gee_hash_set_new (GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL);
	self->priv->to_copy = _tmp1_;
}

static void
geary_imap_engine_copy_email_finalize (GObject * obj)
{
	GearyImapEngineCopyEmail * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEARY_IMAP_ENGINE_TYPE_COPY_EMAIL, GearyImapEngineCopyEmail);
	_g_object_unref0 (self->destination_uids);
	_g_object_unref0 (self->priv->engine);
	_g_object_unref0 (self->priv->to_copy);
	_g_object_unref0 (self->priv->destination);
	_g_object_unref0 (self->priv->cancellable);
	G_OBJECT_CLASS (geary_imap_engine_copy_email_parent_class)->finalize (obj);
}

 G_GNUC_NO_INLINE static GType
geary_imap_engine_copy_email_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyImapEngineCopyEmailClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_imap_engine_copy_email_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyImapEngineCopyEmail), 0, (GInstanceInitFunc) geary_imap_engine_copy_email_instance_init, NULL };
	GType geary_imap_engine_copy_email_type_id;
	geary_imap_engine_copy_email_type_id = g_type_register_static (GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, "GearyImapEngineCopyEmail", &g_define_type_info, 0);
	GearyImapEngineCopyEmail_private_offset = g_type_add_instance_private (geary_imap_engine_copy_email_type_id, sizeof (GearyImapEngineCopyEmailPrivate));
	return geary_imap_engine_copy_email_type_id;
}

GType
geary_imap_engine_copy_email_get_type (void)
{
	static gsize geary_imap_engine_copy_email_type_id__once = 0;
	if (g_once_init_enter (&geary_imap_engine_copy_email_type_id__once)) {
		GType geary_imap_engine_copy_email_type_id;
		geary_imap_engine_copy_email_type_id = geary_imap_engine_copy_email_get_type_once ();
		g_once_init_leave (&geary_imap_engine_copy_email_type_id__once, geary_imap_engine_copy_email_type_id);
	}
	return geary_imap_engine_copy_email_type_id__once;
}

