/* plugin-error.c generated by valac 0.56.19, the Vala compiler
 * generated from plugin-error.vala, do not modify */

/*
 * Copyright © 2020 Michael Gratton <mike@vee.net>
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later). See the COPYING file in this distribution.
 */
/**
 * The base class for objects implementing a client plugin.
 *
 * To implement a new plugin, have it derive from this type and
 * implement any additional extension interfaces (such as {@link
 * NotificationExtension}) as required.
 */

#include "geary-client-46.0.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

/**
 * Errors when plugins request resources from their contexts.
 */

GQuark
plugin_error_quark (void)
{
	return g_quark_from_static_string ("plugin-error-quark");
}

static GType
plugin_error_get_type_once (void)
{
	static const GEnumValue values[] = {{PLUGIN_ERROR_PERMISSION_DENIED, "PLUGIN_ERROR_PERMISSION_DENIED", "permission-denied"}, {PLUGIN_ERROR_NOT_FOUND, "PLUGIN_ERROR_NOT_FOUND", "not-found"}, {PLUGIN_ERROR_NOT_SUPPORTED, "PLUGIN_ERROR_NOT_SUPPORTED", "not-supported"}, {0, NULL, NULL}};
	GType plugin_error_type_id;
	plugin_error_type_id = g_enum_register_static ("PluginError", values);
	return plugin_error_type_id;
}

GType
plugin_error_get_type (void)
{
	static volatile gsize plugin_error_type_id__once = 0;
	if (g_once_init_enter (&plugin_error_type_id__once)) {
		GType plugin_error_type_id;
		plugin_error_type_id = plugin_error_get_type_once ();
		g_once_init_leave (&plugin_error_type_id__once, plugin_error_type_id);
	}
	return plugin_error_type_id__once;
}

