/* outbox-email-properties.c generated by valac 0.56.19, the Vala compiler
 * generated from outbox-email-properties.vala, do not modify */

/*
 *  Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later). See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES (geary_outbox_email_properties_get_type ())
#define GEARY_OUTBOX_EMAIL_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES, GearyOutboxEmailProperties))
#define GEARY_OUTBOX_EMAIL_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES, GearyOutboxEmailPropertiesClass))
#define GEARY_OUTBOX_IS_EMAIL_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES))
#define GEARY_OUTBOX_IS_EMAIL_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES))
#define GEARY_OUTBOX_EMAIL_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES, GearyOutboxEmailPropertiesClass))

typedef struct _GearyOutboxEmailProperties GearyOutboxEmailProperties;
typedef struct _GearyOutboxEmailPropertiesClass GearyOutboxEmailPropertiesClass;
typedef struct _GearyOutboxEmailPropertiesPrivate GearyOutboxEmailPropertiesPrivate;
enum  {
	GEARY_OUTBOX_EMAIL_PROPERTIES_0_PROPERTY,
	GEARY_OUTBOX_EMAIL_PROPERTIES_NUM_PROPERTIES
};
static GParamSpec* geary_outbox_email_properties_properties[GEARY_OUTBOX_EMAIL_PROPERTIES_NUM_PROPERTIES];

struct _GearyOutboxEmailProperties {
	GearyEmailProperties parent_instance;
	GearyOutboxEmailPropertiesPrivate * priv;
};

struct _GearyOutboxEmailPropertiesClass {
	GearyEmailPropertiesClass parent_class;
};

static gpointer geary_outbox_email_properties_parent_class = NULL;

VALA_EXTERN GType geary_outbox_email_properties_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyOutboxEmailProperties* geary_outbox_email_properties_new (GDateTime* date_received,
                                                               glong total_bytes);
VALA_EXTERN GearyOutboxEmailProperties* geary_outbox_email_properties_construct (GType object_type,
                                                                     GDateTime* date_received,
                                                                     glong total_bytes);
static gchar* geary_outbox_email_properties_real_to_string (GearyEmailProperties* base);
static GType geary_outbox_email_properties_get_type_once (void);

GearyOutboxEmailProperties*
geary_outbox_email_properties_construct (GType object_type,
                                         GDateTime* date_received,
                                         glong total_bytes)
{
	GearyOutboxEmailProperties * self = NULL;
	g_return_val_if_fail (date_received != NULL, NULL);
	self = (GearyOutboxEmailProperties*) geary_email_properties_construct (object_type, date_received, (gint64) total_bytes);
	return self;
}

GearyOutboxEmailProperties*
geary_outbox_email_properties_new (GDateTime* date_received,
                                   glong total_bytes)
{
	return geary_outbox_email_properties_construct (GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES, date_received, total_bytes);
}

static gchar*
geary_outbox_email_properties_real_to_string (GearyEmailProperties* base)
{
	GearyOutboxEmailProperties * self;
	gchar* _tmp0_;
	gchar* result;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_OUTBOX_TYPE_EMAIL_PROPERTIES, GearyOutboxEmailProperties);
	_tmp0_ = g_strdup ("Outbox.Properties");
	result = _tmp0_;
	return result;
}

static void
geary_outbox_email_properties_class_init (GearyOutboxEmailPropertiesClass * klass,
                                          gpointer klass_data)
{
	geary_outbox_email_properties_parent_class = g_type_class_peek_parent (klass);
	((GearyEmailPropertiesClass *) klass)->to_string = (gchar* (*) (GearyEmailProperties*)) geary_outbox_email_properties_real_to_string;
}

static void
geary_outbox_email_properties_instance_init (GearyOutboxEmailProperties * self,
                                             gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
geary_outbox_email_properties_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyOutboxEmailPropertiesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_outbox_email_properties_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyOutboxEmailProperties), 0, (GInstanceInitFunc) geary_outbox_email_properties_instance_init, NULL };
	GType geary_outbox_email_properties_type_id;
	geary_outbox_email_properties_type_id = g_type_register_static (GEARY_TYPE_EMAIL_PROPERTIES, "GearyOutboxEmailProperties", &g_define_type_info, 0);
	return geary_outbox_email_properties_type_id;
}

GType
geary_outbox_email_properties_get_type (void)
{
	static gsize geary_outbox_email_properties_type_id__once = 0;
	if (g_once_init_enter (&geary_outbox_email_properties_type_id__once)) {
		GType geary_outbox_email_properties_type_id;
		geary_outbox_email_properties_type_id = geary_outbox_email_properties_get_type_once ();
		g_once_init_leave (&geary_outbox_email_properties_type_id__once, geary_outbox_email_properties_type_id);
	}
	return geary_outbox_email_properties_type_id__once;
}

