/* notification-badge.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __SRC_CLIENT_PLUGIN_NOTIFICATION_BADGE_NOTIFICATION_BADGE_H__
#define __SRC_CLIENT_PLUGIN_NOTIFICATION_BADGE_NOTIFICATION_BADGE_H__

#include <glib-object.h>
#include "geary-client-46.0.h"
#include <gio/gio.h>
#include "geary-engine.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define PLUGIN_TYPE_NOTIFICATION_BADGE (plugin_notification_badge_get_type ())
#define PLUGIN_NOTIFICATION_BADGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_NOTIFICATION_BADGE, PluginNotificationBadge))
#define PLUGIN_NOTIFICATION_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_NOTIFICATION_BADGE, PluginNotificationBadgeClass))
#define PLUGIN_IS_NOTIFICATION_BADGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_NOTIFICATION_BADGE))
#define PLUGIN_IS_NOTIFICATION_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_NOTIFICATION_BADGE))
#define PLUGIN_NOTIFICATION_BADGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_NOTIFICATION_BADGE, PluginNotificationBadgeClass))

typedef struct _PluginNotificationBadge PluginNotificationBadge;
typedef struct _PluginNotificationBadgeClass PluginNotificationBadgeClass;
typedef struct _PluginNotificationBadgePrivate PluginNotificationBadgePrivate;

#define TYPE_UNITY_LAUNCHER_ENTRY (unity_launcher_entry_get_type ())
#define UNITY_LAUNCHER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNITY_LAUNCHER_ENTRY, UnityLauncherEntry))
#define UNITY_LAUNCHER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNITY_LAUNCHER_ENTRY, UnityLauncherEntryClass))
#define IS_UNITY_LAUNCHER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNITY_LAUNCHER_ENTRY))
#define IS_UNITY_LAUNCHER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNITY_LAUNCHER_ENTRY))
#define UNITY_LAUNCHER_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNITY_LAUNCHER_ENTRY, UnityLauncherEntryClass))

typedef struct _UnityLauncherEntry UnityLauncherEntry;
typedef struct _UnityLauncherEntryClass UnityLauncherEntryClass;
typedef struct _UnityLauncherEntryPrivate UnityLauncherEntryPrivate;

struct _PluginNotificationBadge {
	PluginPluginBase parent_instance;
	PluginNotificationBadgePrivate * priv;
};

struct _PluginNotificationBadgeClass {
	PluginPluginBaseClass parent_class;
};

struct _UnityLauncherEntry {
	GearyBaseObject parent_instance;
	UnityLauncherEntryPrivate * priv;
};

struct _UnityLauncherEntryClass {
	GearyBaseObjectClass parent_class;
};

VALA_EXTERN void peas_register_types (GTypeModule* module);
GType plugin_notification_badge_register_type (GTypeModule * module);
VALA_EXTERN GType plugin_notification_badge_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginNotificationBadge, g_object_unref)
VALA_EXTERN PluginNotificationBadge* plugin_notification_badge_new (void);
VALA_EXTERN PluginNotificationBadge* plugin_notification_badge_construct (GType object_type);
GType unity_launcher_entry_register_type (GTypeModule * module);
VALA_EXTERN GType unity_launcher_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UnityLauncherEntry, g_object_unref)
VALA_EXTERN UnityLauncherEntry* unity_launcher_entry_new (GDBusConnection* connection,
                                              const gchar* dbus_path,
                                              const gchar* desktop_id,
                                              GError** error);
VALA_EXTERN UnityLauncherEntry* unity_launcher_entry_construct (GType object_type,
                                                    GDBusConnection* connection,
                                                    const gchar* dbus_path,
                                                    const gchar* desktop_id,
                                                    GError** error);
VALA_EXTERN void unity_launcher_entry_set_count (UnityLauncherEntry* self,
                                     gint64 count);
VALA_EXTERN void unity_launcher_entry_clear_count (UnityLauncherEntry* self);

G_END_DECLS

#endif
