/* imap-engine-other-folder.c generated by valac 0.56.19, the Vala compiler
 * generated from imap-engine-other-folder.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <glib-object.h>
#include <gee.h>
#include <gio/gio.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER (geary_imap_engine_minimal_folder_get_type ())
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolder))
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolderClass))
#define GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER))
#define GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER))
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolderClass))

typedef struct _GearyImapEngineMinimalFolder GearyImapEngineMinimalFolder;
typedef struct _GearyImapEngineMinimalFolderClass GearyImapEngineMinimalFolderClass;
typedef struct _GearyImapEngineMinimalFolderPrivate GearyImapEngineMinimalFolderPrivate;

#define GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION (geary_imap_engine_replay_operation_get_type ())
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperation))
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperationClass))
#define GEARY_IMAP_ENGINE_IS_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_IS_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperationClass))

typedef struct _GearyImapEngineReplayOperation GearyImapEngineReplayOperation;
typedef struct _GearyImapEngineReplayOperationClass GearyImapEngineReplayOperationClass;

#define GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER (geary_imap_engine_generic_folder_get_type ())
#define GEARY_IMAP_ENGINE_GENERIC_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER, GearyImapEngineGenericFolder))
#define GEARY_IMAP_ENGINE_GENERIC_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER, GearyImapEngineGenericFolderClass))
#define GEARY_IMAP_ENGINE_IS_GENERIC_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER))
#define GEARY_IMAP_ENGINE_IS_GENERIC_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER))
#define GEARY_IMAP_ENGINE_GENERIC_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER, GearyImapEngineGenericFolderClass))

typedef struct _GearyImapEngineGenericFolder GearyImapEngineGenericFolder;
typedef struct _GearyImapEngineGenericFolderClass GearyImapEngineGenericFolderClass;
typedef struct _GearyImapEngineGenericFolderPrivate GearyImapEngineGenericFolderPrivate;

#define GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER (geary_imap_engine_other_folder_get_type ())
#define GEARY_IMAP_ENGINE_OTHER_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER, GearyImapEngineOtherFolder))
#define GEARY_IMAP_ENGINE_OTHER_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER, GearyImapEngineOtherFolderClass))
#define GEARY_IMAP_ENGINE_IS_OTHER_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER))
#define GEARY_IMAP_ENGINE_IS_OTHER_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER))
#define GEARY_IMAP_ENGINE_OTHER_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER, GearyImapEngineOtherFolderClass))

typedef struct _GearyImapEngineOtherFolder GearyImapEngineOtherFolder;
typedef struct _GearyImapEngineOtherFolderClass GearyImapEngineOtherFolderClass;
typedef struct _GearyImapEngineOtherFolderPrivate GearyImapEngineOtherFolderPrivate;
enum  {
	GEARY_IMAP_ENGINE_OTHER_FOLDER_0_PROPERTY,
	GEARY_IMAP_ENGINE_OTHER_FOLDER_NUM_PROPERTIES
};
static GParamSpec* geary_imap_engine_other_folder_properties[GEARY_IMAP_ENGINE_OTHER_FOLDER_NUM_PROPERTIES];

#define GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT (geary_imap_engine_generic_account_get_type ())
#define GEARY_IMAP_ENGINE_GENERIC_ACCOUNT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT, GearyImapEngineGenericAccount))
#define GEARY_IMAP_ENGINE_GENERIC_ACCOUNT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT, GearyImapEngineGenericAccountClass))
#define GEARY_IMAP_ENGINE_IS_GENERIC_ACCOUNT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT))
#define GEARY_IMAP_ENGINE_IS_GENERIC_ACCOUNT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT))
#define GEARY_IMAP_ENGINE_GENERIC_ACCOUNT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT, GearyImapEngineGenericAccountClass))

typedef struct _GearyImapEngineGenericAccount GearyImapEngineGenericAccount;
typedef struct _GearyImapEngineGenericAccountClass GearyImapEngineGenericAccountClass;

#define GEARY_IMAP_ENGINE_TYPE_OTHER_ACCOUNT (geary_imap_engine_other_account_get_type ())
#define GEARY_IMAP_ENGINE_OTHER_ACCOUNT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_OTHER_ACCOUNT, GearyImapEngineOtherAccount))
#define GEARY_IMAP_ENGINE_OTHER_ACCOUNT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_OTHER_ACCOUNT, GearyImapEngineOtherAccountClass))
#define GEARY_IMAP_ENGINE_IS_OTHER_ACCOUNT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_OTHER_ACCOUNT))
#define GEARY_IMAP_ENGINE_IS_OTHER_ACCOUNT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_OTHER_ACCOUNT))
#define GEARY_IMAP_ENGINE_OTHER_ACCOUNT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_OTHER_ACCOUNT, GearyImapEngineOtherAccountClass))

typedef struct _GearyImapEngineOtherAccount GearyImapEngineOtherAccount;
typedef struct _GearyImapEngineOtherAccountClass GearyImapEngineOtherAccountClass;

#define GEARY_IMAP_DB_TYPE_FOLDER (geary_imap_db_folder_get_type ())
#define GEARY_IMAP_DB_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolder))
#define GEARY_IMAP_DB_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolderClass))
#define GEARY_IMAP_DB_IS_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_DB_TYPE_FOLDER))
#define GEARY_IMAP_DB_IS_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_DB_TYPE_FOLDER))
#define GEARY_IMAP_DB_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolderClass))

typedef struct _GearyImapDBFolder GearyImapDBFolder;
typedef struct _GearyImapDBFolderClass GearyImapDBFolderClass;

struct _GearyImapEngineMinimalFolder {
	GearyFolder parent_instance;
	GearyImapEngineMinimalFolderPrivate * priv;
};

struct _GearyImapEngineMinimalFolderClass {
	GearyFolderClass parent_class;
	void (*notify_closing) (GearyImapEngineMinimalFolder* self, GeeList* final_ops);
	void (*mark_email_async) (GearyImapEngineMinimalFolder* self, GeeCollection* to_mark, GearyEmailFlags* flags_to_add, GearyEmailFlags* flags_to_remove, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*mark_email_finish) (GearyImapEngineMinimalFolder* self, GAsyncResult* _res_, GError** error);
	void (*copy_email_async) (GearyImapEngineMinimalFolder* self, GeeCollection* to_copy, GearyFolderPath* destination, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*copy_email_finish) (GearyImapEngineMinimalFolder* self, GAsyncResult* _res_, GError** error);
	void (*move_email_async) (GearyImapEngineMinimalFolder* self, GeeCollection* to_move, GearyFolderPath* destination, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GearyRevokable* (*move_email_finish) (GearyImapEngineMinimalFolder* self, GAsyncResult* _res_, GError** error);
	void (*notify_marked_email_removed) (GearyImapEngineMinimalFolder* self, GeeCollection* removed);
};

struct _GearyImapEngineGenericFolder {
	GearyImapEngineMinimalFolder parent_instance;
	GearyImapEngineGenericFolderPrivate * priv;
};

struct _GearyImapEngineGenericFolderClass {
	GearyImapEngineMinimalFolderClass parent_class;
};

struct _GearyImapEngineOtherFolder {
	GearyImapEngineGenericFolder parent_instance;
	GearyImapEngineOtherFolderPrivate * priv;
};

struct _GearyImapEngineOtherFolderClass {
	GearyImapEngineGenericFolderClass parent_class;
};

static gpointer geary_imap_engine_other_folder_parent_class = NULL;

VALA_EXTERN GType geary_imap_engine_minimal_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_replay_operation_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_generic_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_other_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_generic_account_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_other_account_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_db_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyImapEngineOtherFolder* geary_imap_engine_other_folder_new (GearyImapEngineOtherAccount* account,
                                                                GearyImapDBFolder* local_folder,
                                                                GearyFolderSpecialUse use);
VALA_EXTERN GearyImapEngineOtherFolder* geary_imap_engine_other_folder_construct (GType object_type,
                                                                      GearyImapEngineOtherAccount* account,
                                                                      GearyImapDBFolder* local_folder,
                                                                      GearyFolderSpecialUse use);
VALA_EXTERN GearyImapEngineGenericFolder* geary_imap_engine_generic_folder_new (GearyImapEngineGenericAccount* account,
                                                                    GearyImapDBFolder* local_folder,
                                                                    GearyFolderSpecialUse use);
VALA_EXTERN GearyImapEngineGenericFolder* geary_imap_engine_generic_folder_construct (GType object_type,
                                                                          GearyImapEngineGenericAccount* account,
                                                                          GearyImapDBFolder* local_folder,
                                                                          GearyFolderSpecialUse use);
static GType geary_imap_engine_other_folder_get_type_once (void);

GearyImapEngineOtherFolder*
geary_imap_engine_other_folder_construct (GType object_type,
                                          GearyImapEngineOtherAccount* account,
                                          GearyImapDBFolder* local_folder,
                                          GearyFolderSpecialUse use)
{
	GearyImapEngineOtherFolder * self = NULL;
	g_return_val_if_fail (GEARY_IMAP_ENGINE_IS_OTHER_ACCOUNT (account), NULL);
	g_return_val_if_fail (GEARY_IMAP_DB_IS_FOLDER (local_folder), NULL);
	self = (GearyImapEngineOtherFolder*) geary_imap_engine_generic_folder_construct (object_type, G_TYPE_CHECK_INSTANCE_CAST (account, GEARY_IMAP_ENGINE_TYPE_GENERIC_ACCOUNT, GearyImapEngineGenericAccount), local_folder, use);
	return self;
}

GearyImapEngineOtherFolder*
geary_imap_engine_other_folder_new (GearyImapEngineOtherAccount* account,
                                    GearyImapDBFolder* local_folder,
                                    GearyFolderSpecialUse use)
{
	return geary_imap_engine_other_folder_construct (GEARY_IMAP_ENGINE_TYPE_OTHER_FOLDER, account, local_folder, use);
}

static void
geary_imap_engine_other_folder_class_init (GearyImapEngineOtherFolderClass * klass,
                                           gpointer klass_data)
{
	geary_imap_engine_other_folder_parent_class = g_type_class_peek_parent (klass);
}

static void
geary_imap_engine_other_folder_instance_init (GearyImapEngineOtherFolder * self,
                                              gpointer klass)
{
}

 G_GNUC_NO_INLINE static GType
geary_imap_engine_other_folder_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyImapEngineOtherFolderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_imap_engine_other_folder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyImapEngineOtherFolder), 0, (GInstanceInitFunc) geary_imap_engine_other_folder_instance_init, NULL };
	GType geary_imap_engine_other_folder_type_id;
	geary_imap_engine_other_folder_type_id = g_type_register_static (GEARY_IMAP_ENGINE_TYPE_GENERIC_FOLDER, "GearyImapEngineOtherFolder", &g_define_type_info, 0);
	return geary_imap_engine_other_folder_type_id;
}

GType
geary_imap_engine_other_folder_get_type (void)
{
	static gsize geary_imap_engine_other_folder_type_id__once = 0;
	if (g_once_init_enter (&geary_imap_engine_other_folder_type_id__once)) {
		GType geary_imap_engine_other_folder_type_id;
		geary_imap_engine_other_folder_type_id = geary_imap_engine_other_folder_get_type_once ();
		g_once_init_leave (&geary_imap_engine_other_folder_type_id__once, geary_imap_engine_other_folder_type_id);
	}
	return geary_imap_engine_other_folder_type_id__once;
}

