/* smtp-error.c generated by valac 0.56.19, the Vala compiler
 * generated from smtp-error.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

/**
 * Thrown when an error occurs communicating with a SMTP server.
 */

GQuark
geary_smtp_error_quark (void)
{
	return g_quark_from_static_string ("geary-smtp-error-quark");
}

 G_GNUC_NO_INLINE static GType
geary_smtp_error_get_type_once (void)
{
	static const GEnumValue values[] = {{GEARY_SMTP_ERROR_ALREADY_CONNECTED, "GEARY_SMTP_ERROR_ALREADY_CONNECTED", "already-connected"}, {GEARY_SMTP_ERROR_AUTHENTICATION_FAILED, "GEARY_SMTP_ERROR_AUTHENTICATION_FAILED", "authentication-failed"}, {GEARY_SMTP_ERROR_NOT_CONNECTED, "GEARY_SMTP_ERROR_NOT_CONNECTED", "not-connected"}, {GEARY_SMTP_ERROR_NOT_SUPPORTED, "GEARY_SMTP_ERROR_NOT_SUPPORTED", "not-supported"}, {GEARY_SMTP_ERROR_PARSE_ERROR, "GEARY_SMTP_ERROR_PARSE_ERROR", "parse-error"}, {GEARY_SMTP_ERROR_REQUIRED_FIELD, "GEARY_SMTP_ERROR_REQUIRED_FIELD", "required-field"}, {GEARY_SMTP_ERROR_SERVER_ERROR, "GEARY_SMTP_ERROR_SERVER_ERROR", "server-error"}, {GEARY_SMTP_ERROR_STARTTLS_FAILED, "GEARY_SMTP_ERROR_STARTTLS_FAILED", "starttls-failed"}, {0, NULL, NULL}};
	GType geary_smtp_error_type_id;
	geary_smtp_error_type_id = g_enum_register_static ("GearySmtpError", values);
	return geary_smtp_error_type_id;
}

GType
geary_smtp_error_get_type (void)
{
	static gsize geary_smtp_error_type_id__once = 0;
	if (g_once_init_enter (&geary_smtp_error_type_id__once)) {
		GType geary_smtp_error_type_id;
		geary_smtp_error_type_id = geary_smtp_error_get_type_once ();
		g_once_init_leave (&geary_smtp_error_type_id__once, geary_smtp_error_type_id);
	}
	return geary_smtp_error_type_id__once;
}

