/* imap-engine-move-email-commit.c generated by valac 0.56.19, the Vala compiler
 * generated from imap-engine-move-email-commit.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

#include "geary-engine.h"
#include <gee.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION (geary_imap_engine_replay_operation_get_type ())
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperation))
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperationClass))
#define GEARY_IMAP_ENGINE_IS_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_IS_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_REPLAY_OPERATION, GearyImapEngineReplayOperationClass))

typedef struct _GearyImapEngineReplayOperation GearyImapEngineReplayOperation;
typedef struct _GearyImapEngineReplayOperationClass GearyImapEngineReplayOperationClass;
typedef struct _GearyImapEngineReplayOperationPrivate GearyImapEngineReplayOperationPrivate;

#define GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER (geary_imap_db_email_identifier_get_type ())
#define GEARY_IMAP_DB_EMAIL_IDENTIFIER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, GearyImapDBEmailIdentifier))
#define GEARY_IMAP_DB_EMAIL_IDENTIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, GearyImapDBEmailIdentifierClass))
#define GEARY_IMAP_DB_IS_EMAIL_IDENTIFIER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER))
#define GEARY_IMAP_DB_IS_EMAIL_IDENTIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER))
#define GEARY_IMAP_DB_EMAIL_IDENTIFIER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, GearyImapDBEmailIdentifierClass))

typedef struct _GearyImapDBEmailIdentifier GearyImapDBEmailIdentifier;
typedef struct _GearyImapDBEmailIdentifierClass GearyImapDBEmailIdentifierClass;
typedef enum  {
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_STATUS_COMPLETED,
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_STATUS_CONTINUE
} GearyImapEngineReplayOperationStatus;

#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_TYPE_STATUS (geary_imap_engine_replay_operation_status_get_type ())

#define GEARY_IMAP_TYPE_FOLDER_SESSION (geary_imap_folder_session_get_type ())
#define GEARY_IMAP_FOLDER_SESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_TYPE_FOLDER_SESSION, GearyImapFolderSession))
#define GEARY_IMAP_FOLDER_SESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_TYPE_FOLDER_SESSION, GearyImapFolderSessionClass))
#define GEARY_IMAP_IS_FOLDER_SESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_TYPE_FOLDER_SESSION))
#define GEARY_IMAP_IS_FOLDER_SESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_TYPE_FOLDER_SESSION))
#define GEARY_IMAP_FOLDER_SESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_TYPE_FOLDER_SESSION, GearyImapFolderSessionClass))

typedef struct _GearyImapFolderSession GearyImapFolderSession;
typedef struct _GearyImapFolderSessionClass GearyImapFolderSessionClass;

#define GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION (geary_imap_engine_send_replay_operation_get_type ())
#define GEARY_IMAP_ENGINE_SEND_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, GearyImapEngineSendReplayOperation))
#define GEARY_IMAP_ENGINE_SEND_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, GearyImapEngineSendReplayOperationClass))
#define GEARY_IMAP_ENGINE_IS_SEND_REPLAY_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_IS_SEND_REPLAY_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION))
#define GEARY_IMAP_ENGINE_SEND_REPLAY_OPERATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, GearyImapEngineSendReplayOperationClass))

typedef struct _GearyImapEngineSendReplayOperation GearyImapEngineSendReplayOperation;
typedef struct _GearyImapEngineSendReplayOperationClass GearyImapEngineSendReplayOperationClass;
typedef struct _GearyImapEngineSendReplayOperationPrivate GearyImapEngineSendReplayOperationPrivate;

#define GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT (geary_imap_engine_move_email_commit_get_type ())
#define GEARY_IMAP_ENGINE_MOVE_EMAIL_COMMIT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit))
#define GEARY_IMAP_ENGINE_MOVE_EMAIL_COMMIT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommitClass))
#define GEARY_IMAP_ENGINE_IS_MOVE_EMAIL_COMMIT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT))
#define GEARY_IMAP_ENGINE_IS_MOVE_EMAIL_COMMIT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT))
#define GEARY_IMAP_ENGINE_MOVE_EMAIL_COMMIT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommitClass))

typedef struct _GearyImapEngineMoveEmailCommit GearyImapEngineMoveEmailCommit;
typedef struct _GearyImapEngineMoveEmailCommitClass GearyImapEngineMoveEmailCommitClass;
typedef struct _GearyImapEngineMoveEmailCommitPrivate GearyImapEngineMoveEmailCommitPrivate;

#define GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER (geary_imap_engine_minimal_folder_get_type ())
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolder))
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolderClass))
#define GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER))
#define GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER))
#define GEARY_IMAP_ENGINE_MINIMAL_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_ENGINE_TYPE_MINIMAL_FOLDER, GearyImapEngineMinimalFolderClass))

typedef struct _GearyImapEngineMinimalFolder GearyImapEngineMinimalFolder;
typedef struct _GearyImapEngineMinimalFolderClass GearyImapEngineMinimalFolderClass;
enum  {
	GEARY_IMAP_ENGINE_MOVE_EMAIL_COMMIT_0_PROPERTY,
	GEARY_IMAP_ENGINE_MOVE_EMAIL_COMMIT_NUM_PROPERTIES
};
static GParamSpec* geary_imap_engine_move_email_commit_properties[GEARY_IMAP_ENGINE_MOVE_EMAIL_COMMIT_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef enum  {
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_THROW,
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_RETRY,
	GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_IGNORE_REMOTE
} GearyImapEngineReplayOperationOnError;

#define GEARY_IMAP_ENGINE_REPLAY_OPERATION_TYPE_ON_ERROR (geary_imap_engine_replay_operation_on_error_get_type ())
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _GearyImapEngineMoveEmailCommitReplayRemoteAsyncData GearyImapEngineMoveEmailCommitReplayRemoteAsyncData;

#define GEARY_IMAP_DB_TYPE_FOLDER (geary_imap_db_folder_get_type ())
#define GEARY_IMAP_DB_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolder))
#define GEARY_IMAP_DB_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolderClass))
#define GEARY_IMAP_DB_IS_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEARY_IMAP_DB_TYPE_FOLDER))
#define GEARY_IMAP_DB_IS_FOLDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEARY_IMAP_DB_TYPE_FOLDER))
#define GEARY_IMAP_DB_FOLDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEARY_IMAP_DB_TYPE_FOLDER, GearyImapDBFolderClass))

typedef struct _GearyImapDBFolder GearyImapDBFolder;
typedef struct _GearyImapDBFolderClass GearyImapDBFolderClass;
typedef struct _GearyImapEngineMoveEmailCommitBackoutLocalAsyncData GearyImapEngineMoveEmailCommitBackoutLocalAsyncData;

struct _GearyImapEngineReplayOperation {
	GearyBaseObject parent_instance;
	GearyImapEngineReplayOperationPrivate * priv;
};

struct _GearyImapEngineReplayOperationClass {
	GearyBaseObjectClass parent_class;
	void (*notify_remote_removed_position) (GearyImapEngineReplayOperation* self, GearyImapSequenceNumber* removed);
	void (*notify_remote_removed_ids) (GearyImapEngineReplayOperation* self, GeeCollection* ids);
	void (*get_ids_to_be_remote_removed) (GearyImapEngineReplayOperation* self, GeeCollection* ids);
	void (*replay_local_async) (GearyImapEngineReplayOperation* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GearyImapEngineReplayOperationStatus (*replay_local_finish) (GearyImapEngineReplayOperation* self, GAsyncResult* _res_, GError** error);
	void (*replay_remote_async) (GearyImapEngineReplayOperation* self, GearyImapFolderSession* remote, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*replay_remote_finish) (GearyImapEngineReplayOperation* self, GAsyncResult* _res_, GError** error);
	void (*backout_local_async) (GearyImapEngineReplayOperation* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*backout_local_finish) (GearyImapEngineReplayOperation* self, GAsyncResult* _res_, GError** error);
	gchar* (*describe_state) (GearyImapEngineReplayOperation* self);
};

struct _GearyImapEngineSendReplayOperation {
	GearyImapEngineReplayOperation parent_instance;
	GearyImapEngineSendReplayOperationPrivate * priv;
};

struct _GearyImapEngineSendReplayOperationClass {
	GearyImapEngineReplayOperationClass parent_class;
};

struct _GearyImapEngineMoveEmailCommit {
	GearyImapEngineSendReplayOperation parent_instance;
	GeeSet* destination_uids;
	GearyImapEngineMoveEmailCommitPrivate * priv;
};

struct _GearyImapEngineMoveEmailCommitClass {
	GearyImapEngineSendReplayOperationClass parent_class;
};

struct _GearyImapEngineMoveEmailCommitPrivate {
	GearyImapEngineMinimalFolder* engine;
	GeeList* to_move;
	GearyFolderPath* destination;
	GCancellable* cancellable;
	GeeList* remaining_msg_sets;
};

struct _GearyImapEngineMoveEmailCommitReplayRemoteAsyncData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GearyImapEngineMoveEmailCommit* self;
	GearyImapFolderSession* remote;
	GeeList* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	GeeList* _tmp3_;
	GeeList* _tmp4_;
	GeeSet* _tmp5_;
	GeeSet* _tmp6_;
	GeeList* _tmp7_;
	gboolean _tmp8_;
	GeeList* _tmp9_;
	GeeList* _tmp10_;
	gint _tmp11_;
	gint _tmp12_;
	GeeIterator* iter;
	GeeList* _tmp13_;
	GeeIterator* _tmp14_;
	GeeIterator* _tmp15_;
	gboolean _tmp16_;
	GCancellable* _tmp17_;
	GCancellable* _tmp18_;
	GearyFolderPath* _tmp19_;
	gchar* _tmp20_;
	gchar* _tmp21_;
	GError* _tmp22_;
	GError* _tmp23_;
	GearyImapMessageSet* msg_set;
	GeeIterator* _tmp24_;
	gpointer _tmp25_;
	GeeMap* map;
	GearyImapMessageSet* _tmp26_;
	GearyFolderPath* _tmp27_;
	GeeMap* _tmp28_;
	GeeMap* _tmp29_;
	GeeSet* _tmp30_;
	GeeMap* _tmp31_;
	GeeCollection* _tmp32_;
	GeeCollection* _tmp33_;
	GeeCollection* _tmp34_;
	GearyImapMessageSet* _tmp35_;
	GeeList* _tmp36_;
	GeeList* _tmp37_;
	GeeIterator* _tmp38_;
	GError* _inner_error0_;
};

struct _GearyImapEngineMoveEmailCommitBackoutLocalAsyncData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GearyImapEngineMoveEmailCommit* self;
	GeeList* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	GearyImapEngineMinimalFolder* _tmp3_;
	GearyImapDBFolder* _tmp4_;
	GearyImapDBFolder* _tmp5_;
	GeeList* _tmp6_;
	GCancellable* _tmp7_;
	GeeSet* _tmp8_;
	GeeSet* _tmp9_;
	gint count;
	GearyImapEngineMinimalFolder* _tmp10_;
	GearyFolderProperties* _tmp11_;
	GearyFolderProperties* _tmp12_;
	gint _tmp13_;
	gint _tmp14_;
	GearyImapEngineMinimalFolder* _tmp15_;
	GeeList* _tmp16_;
	GearyImapEngineMinimalFolder* _tmp17_;
	GeeList* _tmp18_;
	gint _tmp19_;
	gint _tmp20_;
	GError* _inner_error0_;
};

static gint GearyImapEngineMoveEmailCommit_private_offset;
static gpointer geary_imap_engine_move_email_commit_parent_class = NULL;

VALA_EXTERN GType geary_imap_engine_replay_operation_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_db_email_identifier_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_replay_operation_status_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_folder_session_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_send_replay_operation_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_move_email_commit_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType geary_imap_engine_minimal_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void geary_imap_engine_replay_operation_notify_remote_removed_ids (GearyImapEngineReplayOperation* self,
                                                                   GeeCollection* ids);
VALA_EXTERN void geary_imap_engine_replay_operation_get_ids_to_be_remote_removed (GearyImapEngineReplayOperation* self,
                                                                      GeeCollection* ids);
VALA_EXTERN void geary_imap_engine_replay_operation_replay_remote_async (GearyImapEngineReplayOperation* self,
                                                             GearyImapFolderSession* remote,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN void geary_imap_engine_replay_operation_replay_remote_finish (GearyImapEngineReplayOperation* self,
                                                              GAsyncResult* _res_,
                                                              GError** error);
VALA_EXTERN void geary_imap_engine_replay_operation_backout_local_async (GearyImapEngineReplayOperation* self,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN void geary_imap_engine_replay_operation_backout_local_finish (GearyImapEngineReplayOperation* self,
                                                              GAsyncResult* _res_,
                                                              GError** error);
VALA_EXTERN gchar* geary_imap_engine_replay_operation_describe_state (GearyImapEngineReplayOperation* self);
VALA_EXTERN GearyImapEngineMoveEmailCommit* geary_imap_engine_move_email_commit_new (GearyImapEngineMinimalFolder* engine,
                                                                         GeeCollection* to_move,
                                                                         GearyFolderPath* destination,
                                                                         GCancellable* cancellable);
VALA_EXTERN GearyImapEngineMoveEmailCommit* geary_imap_engine_move_email_commit_construct (GType object_type,
                                                                               GearyImapEngineMinimalFolder* engine,
                                                                               GeeCollection* to_move,
                                                                               GearyFolderPath* destination,
                                                                               GCancellable* cancellable);
VALA_EXTERN GType geary_imap_engine_replay_operation_on_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyImapEngineSendReplayOperation* geary_imap_engine_send_replay_operation_construct_only_remote (GType object_type,
                                                                                                   const gchar* name,
                                                                                                   GearyImapEngineReplayOperationOnError on_remote_error);
static void geary_imap_engine_move_email_commit_real_notify_remote_removed_ids (GearyImapEngineReplayOperation* base,
                                                                         GeeCollection* ids);
static void geary_imap_engine_move_email_commit_real_get_ids_to_be_remote_removed (GearyImapEngineReplayOperation* base,
                                                                            GeeCollection* ids);
static void geary_imap_engine_move_email_commit_real_replay_remote_async_data_free (gpointer _data);
static void geary_imap_engine_move_email_commit_real_replay_remote_async (GearyImapEngineReplayOperation* base,
                                                                   GearyImapFolderSession* remote,
                                                                   GAsyncReadyCallback _callback_,
                                                                   gpointer _user_data_);
static gboolean geary_imap_engine_move_email_commit_real_replay_remote_async_co (GearyImapEngineMoveEmailCommitReplayRemoteAsyncData* _data_);
VALA_EXTERN GeeSet* geary_imap_db_email_identifier_to_uids (GeeCollection* ids);
VALA_EXTERN void geary_imap_folder_session_copy_email_async (GearyImapFolderSession* self,
                                                 GearyImapMessageSet* msg_set,
                                                 GearyFolderPath* destination,
                                                 GCancellable* cancellable,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
VALA_EXTERN GeeMap* geary_imap_folder_session_copy_email_finish (GearyImapFolderSession* self,
                                                     GAsyncResult* _res_,
                                                     GError** error);
static void geary_imap_engine_move_email_commit_replay_remote_async_ready (GObject* source_object,
                                                                    GAsyncResult* _res_,
                                                                    gpointer _user_data_);
VALA_EXTERN void geary_imap_folder_session_remove_email_async (GearyImapFolderSession* self,
                                                   GeeList* msg_sets,
                                                   GCancellable* cancellable,
                                                   GAsyncReadyCallback _callback_,
                                                   gpointer _user_data_);
VALA_EXTERN void geary_imap_folder_session_remove_email_finish (GearyImapFolderSession* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
static void geary_imap_engine_move_email_commit_real_backout_local_async_data_free (gpointer _data);
static void geary_imap_engine_move_email_commit_real_backout_local_async (GearyImapEngineReplayOperation* base,
                                                                   GAsyncReadyCallback _callback_,
                                                                   gpointer _user_data_);
static gboolean geary_imap_engine_move_email_commit_real_backout_local_async_co (GearyImapEngineMoveEmailCommitBackoutLocalAsyncData* _data_);
VALA_EXTERN GType geary_imap_db_folder_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyImapDBFolder* geary_imap_engine_minimal_folder_get_local_folder (GearyImapEngineMinimalFolder* self);
VALA_EXTERN void geary_imap_db_folder_mark_removed_async (GearyImapDBFolder* self,
                                              GeeCollection* ids,
                                              gboolean mark_removed,
                                              GCancellable* cancellable,
                                              GAsyncReadyCallback _callback_,
                                              gpointer _user_data_);
VALA_EXTERN GeeSet* geary_imap_db_folder_mark_removed_finish (GearyImapDBFolder* self,
                                                  GAsyncResult* _res_,
                                                  GError** error);
static void geary_imap_engine_move_email_commit_backout_local_async_ready (GObject* source_object,
                                                                    GAsyncResult* _res_,
                                                                    gpointer _user_data_);
VALA_EXTERN void geary_imap_engine_minimal_folder_replay_notify_email_inserted (GearyImapEngineMinimalFolder* self,
                                                                    GeeCollection* ids);
VALA_EXTERN void geary_imap_engine_minimal_folder_replay_notify_email_count_changed (GearyImapEngineMinimalFolder* self,
                                                                         gint new_count,
                                                                         GearyFolderCountChangeReason reason);
static gchar* geary_imap_engine_move_email_commit_real_describe_state (GearyImapEngineReplayOperation* base);
static void geary_imap_engine_move_email_commit_finalize (GObject * obj);
static GType geary_imap_engine_move_email_commit_get_type_once (void);

static inline gpointer
geary_imap_engine_move_email_commit_get_instance_private (GearyImapEngineMoveEmailCommit* self)
{
	return G_STRUCT_MEMBER_P (self, GearyImapEngineMoveEmailCommit_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

GearyImapEngineMoveEmailCommit*
geary_imap_engine_move_email_commit_construct (GType object_type,
                                               GearyImapEngineMinimalFolder* engine,
                                               GeeCollection* to_move,
                                               GearyFolderPath* destination,
                                               GCancellable* cancellable)
{
	GearyImapEngineMoveEmailCommit * self = NULL;
	GearyImapEngineMinimalFolder* _tmp0_;
	GeeList* _tmp1_;
	GearyFolderPath* _tmp2_;
	GCancellable* _tmp3_;
	g_return_val_if_fail (GEARY_IMAP_ENGINE_IS_MINIMAL_FOLDER (engine), NULL);
	g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (to_move, GEE_TYPE_COLLECTION), NULL);
	g_return_val_if_fail (GEARY_IS_FOLDER_PATH (destination), NULL);
	g_return_val_if_fail ((cancellable == NULL) || G_TYPE_CHECK_INSTANCE_TYPE (cancellable, g_cancellable_get_type ()), NULL);
	self = (GearyImapEngineMoveEmailCommit*) geary_imap_engine_send_replay_operation_construct_only_remote (object_type, "MoveEmailCommit", GEARY_IMAP_ENGINE_REPLAY_OPERATION_ON_ERROR_RETRY);
	_tmp0_ = _g_object_ref0 (engine);
	_g_object_unref0 (self->priv->engine);
	self->priv->engine = _tmp0_;
	_tmp1_ = self->priv->to_move;
	gee_collection_add_all (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, GEE_TYPE_COLLECTION, GeeCollection), to_move);
	_tmp2_ = _g_object_ref0 (destination);
	_g_object_unref0 (self->priv->destination);
	self->priv->destination = _tmp2_;
	_tmp3_ = _g_object_ref0 (cancellable);
	_g_object_unref0 (self->priv->cancellable);
	self->priv->cancellable = _tmp3_;
	return self;
}

GearyImapEngineMoveEmailCommit*
geary_imap_engine_move_email_commit_new (GearyImapEngineMinimalFolder* engine,
                                         GeeCollection* to_move,
                                         GearyFolderPath* destination,
                                         GCancellable* cancellable)
{
	return geary_imap_engine_move_email_commit_construct (GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, engine, to_move, destination, cancellable);
}

static void
geary_imap_engine_move_email_commit_real_notify_remote_removed_ids (GearyImapEngineReplayOperation* base,
                                                                    GeeCollection* ids)
{
	GearyImapEngineMoveEmailCommit * self;
	GeeList* _tmp0_;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit);
	g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (ids, GEE_TYPE_COLLECTION));
	_tmp0_ = self->priv->to_move;
	gee_collection_remove_all (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_COLLECTION, GeeCollection), ids);
}

static void
geary_imap_engine_move_email_commit_real_get_ids_to_be_remote_removed (GearyImapEngineReplayOperation* base,
                                                                       GeeCollection* ids)
{
	GearyImapEngineMoveEmailCommit * self;
	GeeList* _tmp0_;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit);
	g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (ids, GEE_TYPE_COLLECTION));
	_tmp0_ = self->priv->to_move;
	gee_collection_add_all (ids, G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_COLLECTION, GeeCollection));
}

static void
geary_imap_engine_move_email_commit_real_replay_remote_async_data_free (gpointer _data)
{
	GearyImapEngineMoveEmailCommitReplayRemoteAsyncData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->remote);
	_g_object_unref0 (_data_->self);
	g_slice_free (GearyImapEngineMoveEmailCommitReplayRemoteAsyncData, _data_);
}

static void
geary_imap_engine_move_email_commit_real_replay_remote_async (GearyImapEngineReplayOperation* base,
                                                              GearyImapFolderSession* remote,
                                                              GAsyncReadyCallback _callback_,
                                                              gpointer _user_data_)
{
	GearyImapEngineMoveEmailCommit * self;
	GearyImapEngineMoveEmailCommitReplayRemoteAsyncData* _data_;
	GearyImapEngineMoveEmailCommit* _tmp0_;
	GearyImapFolderSession* _tmp1_;
	g_return_if_fail (GEARY_IMAP_IS_FOLDER_SESSION (remote));
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit);
	_data_ = g_slice_new0 (GearyImapEngineMoveEmailCommitReplayRemoteAsyncData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, geary_imap_engine_move_email_commit_real_replay_remote_async_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (remote);
	_g_object_unref0 (_data_->remote);
	_data_->remote = _tmp1_;
	geary_imap_engine_move_email_commit_real_replay_remote_async_co (_data_);
}

static void
geary_imap_engine_move_email_commit_replay_remote_finish (GearyImapEngineReplayOperation* base,
                                                          GAsyncResult* _res_,
                                                          GError** error)
{
	GearyImapEngineMoveEmailCommitReplayRemoteAsyncData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
	if (NULL == _data_) {
		return;
	}
}

static void
geary_imap_engine_move_email_commit_replay_remote_async_ready (GObject* source_object,
                                                               GAsyncResult* _res_,
                                                               gpointer _user_data_)
{
	GearyImapEngineMoveEmailCommitReplayRemoteAsyncData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	geary_imap_engine_move_email_commit_real_replay_remote_async_co (_data_);
}

static gboolean
geary_imap_engine_move_email_commit_real_replay_remote_async_co (GearyImapEngineMoveEmailCommitReplayRemoteAsyncData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		case 2:
		goto _state_2;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->to_move;
	_data_->_tmp1_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp0_, GEE_TYPE_COLLECTION, GeeCollection));
	_data_->_tmp2_ = _data_->_tmp1_;
	if (_data_->_tmp2_ > 0) {
		_data_->_tmp3_ = _data_->self->priv->remaining_msg_sets;
		if (_data_->_tmp3_ == NULL) {
			_data_->_tmp4_ = _data_->self->priv->to_move;
			_data_->_tmp5_ = geary_imap_db_email_identifier_to_uids (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp4_, GEE_TYPE_COLLECTION, GeeCollection));
			_data_->_tmp6_ = _data_->_tmp5_;
			_data_->_tmp7_ = geary_imap_message_set_uid_sparse (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp6_, GEE_TYPE_COLLECTION, GeeCollection));
			_g_object_unref0 (_data_->self->priv->remaining_msg_sets);
			_data_->self->priv->remaining_msg_sets = _data_->_tmp7_;
			_g_object_unref0 (_data_->_tmp6_);
		}
		_data_->_tmp9_ = _data_->self->priv->remaining_msg_sets;
		if (_data_->_tmp9_ == NULL) {
			_data_->_tmp8_ = TRUE;
		} else {
			_data_->_tmp10_ = _data_->self->priv->remaining_msg_sets;
			_data_->_tmp11_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp10_, GEE_TYPE_COLLECTION, GeeCollection));
			_data_->_tmp12_ = _data_->_tmp11_;
			_data_->_tmp8_ = _data_->_tmp12_ == 0;
		}
		if (_data_->_tmp8_) {
			g_task_return_pointer (_data_->_async_result, _data_, NULL);
			if (_data_->_state_ != 0) {
				while (!g_task_get_completed (_data_->_async_result)) {
					g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
				}
			}
			g_object_unref (_data_->_async_result);
			return FALSE;
		}
		_data_->_tmp13_ = _data_->self->priv->remaining_msg_sets;
		_data_->_tmp14_ = gee_iterable_iterator (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp13_, GEE_TYPE_ITERABLE, GeeIterable));
		_data_->iter = _data_->_tmp14_;
		while (TRUE) {
			_data_->_tmp15_ = _data_->iter;
			if (!gee_iterator_next (_data_->_tmp15_)) {
				break;
			}
			_data_->_tmp17_ = _data_->self->priv->cancellable;
			if (_data_->_tmp17_ != NULL) {
				_data_->_tmp18_ = _data_->self->priv->cancellable;
				_data_->_tmp16_ = g_cancellable_is_cancelled (_data_->_tmp18_);
			} else {
				_data_->_tmp16_ = FALSE;
			}
			if (_data_->_tmp16_) {
				_data_->_tmp19_ = _data_->self->priv->destination;
				_data_->_tmp20_ = geary_folder_path_to_string (_data_->_tmp19_);
				_data_->_tmp21_ = _data_->_tmp20_;
				_data_->_tmp22_ = g_error_new (G_IO_ERROR, G_IO_ERROR_CANCELLED, "Move email to %s cancelled", _data_->_tmp21_);
				_data_->_tmp23_ = _data_->_tmp22_;
				_g_free0 (_data_->_tmp21_);
				_data_->_inner_error0_ = _data_->_tmp23_;
				g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
				_g_object_unref0 (_data_->iter);
				g_object_unref (_data_->_async_result);
				return FALSE;
			}
			_data_->_tmp24_ = _data_->iter;
			_data_->_tmp25_ = gee_iterator_get (_data_->_tmp24_);
			_data_->msg_set = (GearyImapMessageSet*) _data_->_tmp25_;
			_data_->_tmp26_ = _data_->msg_set;
			_data_->_tmp27_ = _data_->self->priv->destination;
			_data_->_state_ = 1;
			geary_imap_folder_session_copy_email_async (_data_->remote, _data_->_tmp26_, _data_->_tmp27_, NULL, geary_imap_engine_move_email_commit_replay_remote_async_ready, _data_);
			return FALSE;
			_state_1:
			_data_->_tmp28_ = geary_imap_folder_session_copy_email_finish (_data_->remote, _data_->_res_, &_data_->_inner_error0_);
			_data_->map = _data_->_tmp28_;
			if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
				g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
				_g_object_unref0 (_data_->msg_set);
				_g_object_unref0 (_data_->iter);
				g_object_unref (_data_->_async_result);
				return FALSE;
			}
			_data_->_tmp29_ = _data_->map;
			if (_data_->_tmp29_ != NULL) {
				_data_->_tmp30_ = _data_->self->destination_uids;
				_data_->_tmp31_ = _data_->map;
				_data_->_tmp32_ = gee_map_get_values (_data_->_tmp31_);
				_data_->_tmp33_ = _data_->_tmp32_;
				_data_->_tmp34_ = _data_->_tmp33_;
				gee_collection_add_all (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp30_, GEE_TYPE_COLLECTION, GeeCollection), _data_->_tmp34_);
				_g_object_unref0 (_data_->_tmp34_);
			}
			_data_->_tmp35_ = _data_->msg_set;
			_data_->_tmp36_ = geary_imap_message_set_to_list (_data_->_tmp35_);
			_data_->_tmp37_ = _data_->_tmp36_;
			_data_->_state_ = 2;
			geary_imap_folder_session_remove_email_async (_data_->remote, _data_->_tmp37_, NULL, geary_imap_engine_move_email_commit_replay_remote_async_ready, _data_);
			return FALSE;
			_state_2:
			geary_imap_folder_session_remove_email_finish (_data_->remote, _data_->_res_, &_data_->_inner_error0_);
			_g_object_unref0 (_data_->_tmp37_);
			if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
				g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
				_g_object_unref0 (_data_->map);
				_g_object_unref0 (_data_->msg_set);
				_g_object_unref0 (_data_->iter);
				g_object_unref (_data_->_async_result);
				return FALSE;
			}
			_data_->_tmp38_ = _data_->iter;
			gee_iterator_remove (_data_->_tmp38_);
			_g_object_unref0 (_data_->map);
			_g_object_unref0 (_data_->msg_set);
		}
		_g_object_unref0 (_data_->iter);
	}
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
geary_imap_engine_move_email_commit_real_backout_local_async_data_free (gpointer _data)
{
	GearyImapEngineMoveEmailCommitBackoutLocalAsyncData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->self);
	g_slice_free (GearyImapEngineMoveEmailCommitBackoutLocalAsyncData, _data_);
}

static void
geary_imap_engine_move_email_commit_real_backout_local_async (GearyImapEngineReplayOperation* base,
                                                              GAsyncReadyCallback _callback_,
                                                              gpointer _user_data_)
{
	GearyImapEngineMoveEmailCommit * self;
	GearyImapEngineMoveEmailCommitBackoutLocalAsyncData* _data_;
	GearyImapEngineMoveEmailCommit* _tmp0_;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit);
	_data_ = g_slice_new0 (GearyImapEngineMoveEmailCommitBackoutLocalAsyncData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, geary_imap_engine_move_email_commit_real_backout_local_async_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	geary_imap_engine_move_email_commit_real_backout_local_async_co (_data_);
}

static void
geary_imap_engine_move_email_commit_backout_local_finish (GearyImapEngineReplayOperation* base,
                                                          GAsyncResult* _res_,
                                                          GError** error)
{
	GearyImapEngineMoveEmailCommitBackoutLocalAsyncData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
	if (NULL == _data_) {
		return;
	}
}

static void
geary_imap_engine_move_email_commit_backout_local_async_ready (GObject* source_object,
                                                               GAsyncResult* _res_,
                                                               gpointer _user_data_)
{
	GearyImapEngineMoveEmailCommitBackoutLocalAsyncData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	geary_imap_engine_move_email_commit_real_backout_local_async_co (_data_);
}

static gboolean
geary_imap_engine_move_email_commit_real_backout_local_async_co (GearyImapEngineMoveEmailCommitBackoutLocalAsyncData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->to_move;
	_data_->_tmp1_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp0_, GEE_TYPE_COLLECTION, GeeCollection));
	_data_->_tmp2_ = _data_->_tmp1_;
	if (_data_->_tmp2_ == 0) {
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp3_ = _data_->self->priv->engine;
	_data_->_tmp4_ = geary_imap_engine_minimal_folder_get_local_folder (_data_->_tmp3_);
	_data_->_tmp5_ = _data_->_tmp4_;
	_data_->_tmp6_ = _data_->self->priv->to_move;
	_data_->_tmp7_ = _data_->self->priv->cancellable;
	_data_->_state_ = 1;
	geary_imap_db_folder_mark_removed_async (_data_->_tmp5_, G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp6_, GEE_TYPE_COLLECTION, GeeCollection), FALSE, _data_->_tmp7_, geary_imap_engine_move_email_commit_backout_local_async_ready, _data_);
	return FALSE;
	_state_1:
	_data_->_tmp8_ = geary_imap_db_folder_mark_removed_finish (_data_->_tmp5_, _data_->_res_, &_data_->_inner_error0_);
	_data_->_tmp9_ = _data_->_tmp8_;
	_g_object_unref0 (_data_->_tmp9_);
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp10_ = _data_->self->priv->engine;
	_data_->_tmp11_ = geary_folder_get_properties (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp10_, GEARY_TYPE_FOLDER, GearyFolder));
	_data_->_tmp12_ = _data_->_tmp11_;
	_data_->_tmp13_ = geary_folder_properties_get_email_total (_data_->_tmp12_);
	_data_->_tmp14_ = _data_->_tmp13_;
	_data_->count = _data_->_tmp14_;
	if (_data_->count < 0) {
		_data_->count = 0;
	}
	_data_->_tmp15_ = _data_->self->priv->engine;
	_data_->_tmp16_ = _data_->self->priv->to_move;
	geary_imap_engine_minimal_folder_replay_notify_email_inserted (_data_->_tmp15_, G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp16_, GEE_TYPE_COLLECTION, GeeCollection));
	_data_->_tmp17_ = _data_->self->priv->engine;
	_data_->_tmp18_ = _data_->self->priv->to_move;
	_data_->_tmp19_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_data_->_tmp18_, GEE_TYPE_COLLECTION, GeeCollection));
	_data_->_tmp20_ = _data_->_tmp19_;
	geary_imap_engine_minimal_folder_replay_notify_email_count_changed (_data_->_tmp17_, _data_->count + _data_->_tmp20_, GEARY_FOLDER_COUNT_CHANGE_REASON_INSERTED);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static gchar*
geary_imap_engine_move_email_commit_real_describe_state (GearyImapEngineReplayOperation* base)
{
	GearyImapEngineMoveEmailCommit * self;
	GeeList* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	GearyFolderPath* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* result;
	self = G_TYPE_CHECK_INSTANCE_CAST (base, GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit);
	_tmp0_ = self->priv->to_move;
	_tmp1_ = gee_collection_get_size (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_COLLECTION, GeeCollection));
	_tmp2_ = _tmp1_;
	_tmp3_ = self->priv->destination;
	_tmp4_ = geary_folder_path_to_string (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = g_strdup_printf ("%d email IDs to %s", _tmp2_, _tmp5_);
	_tmp7_ = _tmp6_;
	_g_free0 (_tmp5_);
	result = _tmp7_;
	return result;
}

static void
geary_imap_engine_move_email_commit_class_init (GearyImapEngineMoveEmailCommitClass * klass,
                                                gpointer klass_data)
{
	geary_imap_engine_move_email_commit_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GearyImapEngineMoveEmailCommit_private_offset);
	((GearyImapEngineReplayOperationClass *) klass)->notify_remote_removed_ids = (void (*) (GearyImapEngineReplayOperation*, GeeCollection*)) geary_imap_engine_move_email_commit_real_notify_remote_removed_ids;
	((GearyImapEngineReplayOperationClass *) klass)->get_ids_to_be_remote_removed = (void (*) (GearyImapEngineReplayOperation*, GeeCollection*)) geary_imap_engine_move_email_commit_real_get_ids_to_be_remote_removed;
	((GearyImapEngineReplayOperationClass *) klass)->replay_remote_async = (void (*) (GearyImapEngineReplayOperation*, GearyImapFolderSession*, GAsyncReadyCallback, gpointer)) geary_imap_engine_move_email_commit_real_replay_remote_async;
	((GearyImapEngineReplayOperationClass *) klass)->replay_remote_finish = (void (*) (GearyImapEngineReplayOperation*, GAsyncResult*, GError**)) geary_imap_engine_move_email_commit_replay_remote_finish;
	((GearyImapEngineReplayOperationClass *) klass)->backout_local_async = (void (*) (GearyImapEngineReplayOperation*, GAsyncReadyCallback, gpointer)) geary_imap_engine_move_email_commit_real_backout_local_async;
	((GearyImapEngineReplayOperationClass *) klass)->backout_local_finish = (void (*) (GearyImapEngineReplayOperation*, GAsyncResult*, GError**)) geary_imap_engine_move_email_commit_backout_local_finish;
	((GearyImapEngineReplayOperationClass *) klass)->describe_state = (gchar* (*) (GearyImapEngineReplayOperation*)) geary_imap_engine_move_email_commit_real_describe_state;
	G_OBJECT_CLASS (klass)->finalize = geary_imap_engine_move_email_commit_finalize;
}

static void
geary_imap_engine_move_email_commit_instance_init (GearyImapEngineMoveEmailCommit * self,
                                                   gpointer klass)
{
	GeeHashSet* _tmp0_;
	GeeArrayList* _tmp1_;
	self->priv = geary_imap_engine_move_email_commit_get_instance_private (self);
	_tmp0_ = gee_hash_set_new (GEARY_IMAP_TYPE_UID, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL);
	self->destination_uids = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SET, GeeSet);
	_tmp1_ = gee_array_list_new (GEARY_IMAP_DB_TYPE_EMAIL_IDENTIFIER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
	self->priv->to_move = G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, GEE_TYPE_LIST, GeeList);
	self->priv->remaining_msg_sets = NULL;
}

static void
geary_imap_engine_move_email_commit_finalize (GObject * obj)
{
	GearyImapEngineMoveEmailCommit * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEARY_IMAP_ENGINE_TYPE_MOVE_EMAIL_COMMIT, GearyImapEngineMoveEmailCommit);
	_g_object_unref0 (self->destination_uids);
	_g_object_unref0 (self->priv->engine);
	_g_object_unref0 (self->priv->to_move);
	_g_object_unref0 (self->priv->destination);
	_g_object_unref0 (self->priv->cancellable);
	_g_object_unref0 (self->priv->remaining_msg_sets);
	G_OBJECT_CLASS (geary_imap_engine_move_email_commit_parent_class)->finalize (obj);
}

/**
 * Stage two of a {@link RevokableMove}: move messages from folder to destination.
 */
 G_GNUC_NO_INLINE static GType
geary_imap_engine_move_email_commit_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GearyImapEngineMoveEmailCommitClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) geary_imap_engine_move_email_commit_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GearyImapEngineMoveEmailCommit), 0, (GInstanceInitFunc) geary_imap_engine_move_email_commit_instance_init, NULL };
	GType geary_imap_engine_move_email_commit_type_id;
	geary_imap_engine_move_email_commit_type_id = g_type_register_static (GEARY_IMAP_ENGINE_TYPE_SEND_REPLAY_OPERATION, "GearyImapEngineMoveEmailCommit", &g_define_type_info, 0);
	GearyImapEngineMoveEmailCommit_private_offset = g_type_add_instance_private (geary_imap_engine_move_email_commit_type_id, sizeof (GearyImapEngineMoveEmailCommitPrivate));
	return geary_imap_engine_move_email_commit_type_id;
}

GType
geary_imap_engine_move_email_commit_get_type (void)
{
	static gsize geary_imap_engine_move_email_commit_type_id__once = 0;
	if (g_once_init_enter (&geary_imap_engine_move_email_commit_type_id__once)) {
		GType geary_imap_engine_move_email_commit_type_id;
		geary_imap_engine_move_email_commit_type_id = geary_imap_engine_move_email_commit_get_type_once ();
		g_once_init_leave (&geary_imap_engine_move_email_commit_type_id__once, geary_imap_engine_move_email_commit_type_id);
	}
	return geary_imap_engine_move_email_commit_type_id__once;
}

