/* geary-client-46.0.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __SRC_CLIENT_GEARY_CLIENT_46_0_H__
#define __SRC_CLIENT_GEARY_CLIENT_46_0_H__

#include "geary-engine.h"
#include <glib-object.h>
#include <gio/gio.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <handy.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>
#include <folks/folks.h>
#include <libpeas.h>
#include <goa/goa.h>
#include <webkit2/webkit2.h>
#include <gdk/gdk.h>
#include <cairo-gobject.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <jsc/jsc.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif
#define APPLICATION_CLIENT_NAME "Geary" _NAME_SUFFIX
#define APPLICATION_CLIENT_APP_ID _APP_ID
#define APPLICATION_CLIENT_RESOURCE_BASE_PATH "/org/gnome/Geary"
#define APPLICATION_CLIENT_SCHEMA_ID "org.gnome.Geary"
#define APPLICATION_CLIENT_DESCRIPTION _ ("Send and receive email")
#define APPLICATION_CLIENT_COPYRIGHT_1 _ ("Copyright © 2016 Software Freedom Conservancy Inc.")
#define APPLICATION_CLIENT_COPYRIGHT_2 _ ("Copyright © 2016-2021 Geary Development Team.")
#define APPLICATION_CLIENT_WEBSITE "https://wiki.gnome.org/Apps/Geary"
#define APPLICATION_CLIENT_WEBSITE_LABEL _ ("Visit the Geary web site")
#define APPLICATION_CLIENT_BUGREPORT "https://wiki.gnome.org/Apps/Geary/ReportingABug"
#define APPLICATION_CLIENT_VERSION _VERSION
#define APPLICATION_CLIENT_INSTALL_PREFIX _INSTALL_PREFIX
#define APPLICATION_CLIENT_GSETTINGS_DIR _GSETTINGS_DIR
#define APPLICATION_CLIENT_SOURCE_ROOT_DIR _SOURCE_ROOT_DIR
#define APPLICATION_CLIENT_BUILD_ROOT_DIR _BUILD_ROOT_DIR
#define APPLICATION_CLIENT_PROFILE_RELEASE "release"
#define APPLICATION_CLIENT_PROFILE_BETA "beta"
#define APPLICATION_CLIENT_PROFILE_DEVEL "development"
#define APPLICATION_CLIENT_AVATAR_SIZE_PIXELS 48
#define APPLICATION_CONFIGURATION_ASK_OPEN_ATTACHMENT_KEY "ask-open-attachment"
#define APPLICATION_CONFIGURATION_AUTOSELECT_KEY "autoselect"
#define APPLICATION_CONFIGURATION_BRIEF_NOTIFICATION_DURATION "brief-notification-duration"
#define APPLICATION_CONFIGURATION_COMPOSER_WINDOW_SIZE_KEY "composer-window-size"
#define APPLICATION_CONFIGURATION_COMPOSE_AS_HTML_KEY "compose-as-html"
#define APPLICATION_CONFIGURATION_CONVERSATION_VIEWER_ZOOM_KEY "conversation-viewer-zoom"
#define APPLICATION_CONFIGURATION_DISPLAY_PREVIEW_KEY "display-preview"
#define APPLICATION_CONFIGURATION_FORMATTING_TOOLBAR_VISIBLE "formatting-toolbar-visible"
#define APPLICATION_CONFIGURATION_OPTIONAL_PLUGINS "optional-plugins"
#define APPLICATION_CONFIGURATION_SEARCH_STRATEGY_KEY "search-strategy"
#define APPLICATION_CONFIGURATION_SINGLE_KEY_SHORTCUTS "single-key-shortcuts"
#define APPLICATION_CONFIGURATION_SPELL_CHECK_LANGUAGES "spell-check-languages"
#define APPLICATION_CONFIGURATION_SPELL_CHECK_VISIBLE_LANGUAGES "spell-check-visible-languages"
#define APPLICATION_CONFIGURATION_RUN_IN_BACKGROUND_KEY "run-in-background"
#define APPLICATION_CONFIGURATION_UNDO_SEND_DELAY "undo-send-delay"
#define APPLICATION_CONFIGURATION_WINDOW_HEIGHT_KEY "window-height"
#define APPLICATION_CONFIGURATION_WINDOW_MAXIMIZE_KEY "window-maximize"
#define APPLICATION_CONFIGURATION_WINDOW_WIDTH_KEY "window-width"
#define APPLICATION_CONFIGURATION_IMAGES_TRUSTED_DOMAINS "images-trusted-domains"
#define APPLICATION_MAIN_WINDOW_ACTION_ARCHIVE_CONVERSATION "archive-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_CONVERSATION_DOWN "down-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_CONVERSATION_UP "up-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_DELETE_CONVERSATION "delete-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_FIND_IN_CONVERSATION "find-in-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_FORWARD_CONVERSATION "forward-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_MARK_AS_READ "mark-conversation-read"
#define APPLICATION_MAIN_WINDOW_ACTION_MARK_AS_STARRED "mark-conversation-starred"
#define APPLICATION_MAIN_WINDOW_ACTION_MARK_AS_UNREAD "mark-conversation-unread"
#define APPLICATION_MAIN_WINDOW_ACTION_MARK_AS_UNSTARRED "mark-conversation-unstarred"
#define APPLICATION_MAIN_WINDOW_ACTION_REPLY_ALL_CONVERSATION "reply-all-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_REPLY_CONVERSATION "reply-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_SEARCH "search"
#define APPLICATION_MAIN_WINDOW_ACTION_SELECT_INBOX "select-inbox"
#define APPLICATION_MAIN_WINDOW_ACTION_SHOW_COPY_MENU "show-copy-menu"
#define APPLICATION_MAIN_WINDOW_ACTION_TOGGLE_JUNK "toggle-conversation-junk"
#define APPLICATION_MAIN_WINDOW_ACTION_TRASH_CONVERSATION "trash-conversation"
#define APPLICATION_MAIN_WINDOW_ACTION_ZOOM "zoom"
#define APPLICATION_MAIN_WINDOW_ACTION_NAVIGATION_BACK "navigation-back"
#define ACCOUNTS_MANAGER_SETTINGS_FILENAME "geary.ini"
#define ACTION_APPLICATION_GROUP_NAME "app"
#define ACTION_APPLICATION_ABOUT "about"
#define ACTION_APPLICATION_ACCOUNTS "accounts"
#define ACTION_APPLICATION_COMPOSE "compose"
#define ACTION_APPLICATION_INSPECT "inspect"
#define ACTION_APPLICATION_HELP "help"
#define ACTION_APPLICATION_MAILTO "mailto"
#define ACTION_APPLICATION_NEW_WINDOW "new-window"
#define ACTION_APPLICATION_PREFERENCES "preferences"
#define ACTION_APPLICATION_SHOW_EMAIL "show-email"
#define ACTION_APPLICATION_SHOW_FOLDER "show-folder"
#define ACTION_APPLICATION_QUIT "quit"
#define ACTION_WINDOW_GROUP_NAME "win"
#define ACTION_WINDOW_CLOSE "close"
#define ACTION_WINDOW_SHORTCUT_HELP "show-help-overlay"
#define ACTION_WINDOW_SHOW_HELP_OVERLAY "show-help-overlay"
#define ACTION_WINDOW_SHOW_MENU "show-menu"
#define ACTION_EDIT_GROUP_NAME "edt"
#define ACTION_EDIT_COPY "copy"
#define ACTION_EDIT_REDO "redo"
#define ACTION_EDIT_UNDO "undo"
#define COMPONENTS_INFO_BAR_STACK_PRIORITY_QUEUE_KEY "Components.InfoBarStack.PRIORITY_QUEUE_KEY"
#define COMPONENTS_IN_APP_NOTIFICATION_DEFAULT_DURATION ((guint) 5)
#define COMPONENTS_PLACEHOLDER_PANE_CLASS_HAS_TEXT "geary-has-text"
#define COMPONENTS_WEB_VIEW_INTERNAL_URL_PREFIX "geary:"
#define COMPONENTS_WEB_VIEW_INTERNAL_URL_BODY COMPONENTS_WEB_VIEW_INTERNAL_URL_PREFIX "body"
#define COMPONENTS_WEB_VIEW_CID_URL_PREFIX "cid:"
#define COUNT_BADGE_UNREAD_BG_COLOR "#888888"
#define COUNT_BADGE_SPACING 6
#define ICON_FACTORY_ICON_TOOLBAR GTK_ICON_SIZE_LARGE_TOOLBAR
#define ICON_FACTORY_ICON_SIDEBAR GTK_ICON_SIZE_MENU
#define ICON_FACTORY_UNREAD_ICON_SIZE 16
#define ICON_FACTORY_STAR_ICON_SIZE 16
#define STOCK__OK _ ("_OK")
#define STOCK__CANCEL _ ("_Cancel")
#define STOCK__ABOUT _ ("_About")
#define STOCK__ADD _ ("_Add")
#define STOCK__CLOSE _ ("_Close")
#define STOCK__DISCARD _ ("_Discard")
#define STOCK__HELP _ ("_Help")
#define STOCK__OPEN_BUTTON _ ("_Open")
#define STOCK__PREFERENCES _ ("_Preferences")
#define STOCK__PRINT_MENU _ ("_Print…")
#define STOCK__QUIT _ ("_Quit")
#define STOCK__REMOVE _ ("_Remove")
#define STOCK__SAVE _ ("_Save")
#define STOCK__KEEP _ ("_Keep")
#define COMPOSER_WEB_VIEW_BODY_HTML_ID "geary-body"
#define COMPOSER_WEB_VIEW_SIGNATURE_HTML_ID "geary-signature"
#define COMPOSER_WEB_VIEW_QUOTE_HTML_ID "geary-quote"
#define COMPOSER_WIDGET_REQUIRED_FIELDS ((GEARY_EMAIL_FIELD_ENVELOPE | GEARY_EMAIL_FIELD_HEADER) | GEARY_EMAIL_FIELD_BODY)
#define CONVERSATION_LIST_VIEW_REQUIRED_FIELDS ((GEARY_EMAIL_FIELD_ENVELOPE | GEARY_EMAIL_FIELD_FLAGS) | GEARY_EMAIL_FIELD_PROPERTIES)
#define CONVERSATION_LIST_BOX_REQUIRED_FIELDS ((GEARY_EMAIL_FIELD_DATE | GEARY_EMAIL_FIELD_FLAGS) | GEARY_EMAIL_FIELD_ORIGINATORS)
#define SIDEBAR_TREE_ICON_SIZE 16
#define FILES_KILOBYTE ((gint64) 1024)
#define FILES_MEGABYTE (FILES_KILOBYTE * 1024)
#define FILES_GIGABYTE (FILES_MEGABYTE * 1024)
#define FILES_TERABYTE (FILES_GIGABYTE * 1024)
#define GIO_UTIL_GEARY_RESOURCE_PREFIX "/org/gnome/Geary/"
#define UTIL_GTK_SHOW_PROGRESS_TIMEOUT_MSEC 1000
#define UTIL_GTK_HIDE_PROGRESS_TIMEOUT_MSEC 1000
#define UTIL_GTK_PROGRESS_PULSE_TIMEOUT_MSEC 250
#define UTIL_I18N_SYSTEM_LOCALE ""
#define UTIL_MIGRATE_OLD_APP_ID "org.yorba.geary"

#define APPLICATION_TYPE_ACCOUNT_CONTEXT (application_account_context_get_type ())
#define APPLICATION_ACCOUNT_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_ACCOUNT_CONTEXT, ApplicationAccountContext))
#define APPLICATION_ACCOUNT_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_ACCOUNT_CONTEXT, ApplicationAccountContextClass))
#define APPLICATION_IS_ACCOUNT_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_ACCOUNT_CONTEXT))
#define APPLICATION_IS_ACCOUNT_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_ACCOUNT_CONTEXT))
#define APPLICATION_ACCOUNT_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_ACCOUNT_CONTEXT, ApplicationAccountContextClass))

typedef struct _ApplicationAccountContext ApplicationAccountContext;
typedef struct _ApplicationAccountContextClass ApplicationAccountContextClass;
typedef struct _ApplicationAccountContextPrivate ApplicationAccountContextPrivate;

#define APPLICATION_TYPE_CONTACT_STORE (application_contact_store_get_type ())
#define APPLICATION_CONTACT_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_CONTACT_STORE, ApplicationContactStore))
#define APPLICATION_CONTACT_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_CONTACT_STORE, ApplicationContactStoreClass))
#define APPLICATION_IS_CONTACT_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_CONTACT_STORE))
#define APPLICATION_IS_CONTACT_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_CONTACT_STORE))
#define APPLICATION_CONTACT_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_CONTACT_STORE, ApplicationContactStoreClass))

typedef struct _ApplicationContactStore ApplicationContactStore;
typedef struct _ApplicationContactStoreClass ApplicationContactStoreClass;

#define APPLICATION_TYPE_COMMAND_STACK (application_command_stack_get_type ())
#define APPLICATION_COMMAND_STACK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_COMMAND_STACK, ApplicationCommandStack))
#define APPLICATION_COMMAND_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_COMMAND_STACK, ApplicationCommandStackClass))
#define APPLICATION_IS_COMMAND_STACK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_COMMAND_STACK))
#define APPLICATION_IS_COMMAND_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_COMMAND_STACK))
#define APPLICATION_COMMAND_STACK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_COMMAND_STACK, ApplicationCommandStackClass))

typedef struct _ApplicationCommandStack ApplicationCommandStack;
typedef struct _ApplicationCommandStackClass ApplicationCommandStackClass;

#define APPLICATION_TYPE_FOLDER_CONTEXT (application_folder_context_get_type ())
#define APPLICATION_FOLDER_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_FOLDER_CONTEXT, ApplicationFolderContext))
#define APPLICATION_FOLDER_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_FOLDER_CONTEXT, ApplicationFolderContextClass))
#define APPLICATION_IS_FOLDER_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_FOLDER_CONTEXT))
#define APPLICATION_IS_FOLDER_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_FOLDER_CONTEXT))
#define APPLICATION_FOLDER_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_FOLDER_CONTEXT, ApplicationFolderContextClass))

typedef struct _ApplicationFolderContext ApplicationFolderContext;
typedef struct _ApplicationFolderContextClass ApplicationFolderContextClass;

#define APPLICATION_TYPE_ATTACHMENT_MANAGER (application_attachment_manager_get_type ())
#define APPLICATION_ATTACHMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_ATTACHMENT_MANAGER, ApplicationAttachmentManager))
#define APPLICATION_ATTACHMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_ATTACHMENT_MANAGER, ApplicationAttachmentManagerClass))
#define APPLICATION_IS_ATTACHMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_ATTACHMENT_MANAGER))
#define APPLICATION_IS_ATTACHMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_ATTACHMENT_MANAGER))
#define APPLICATION_ATTACHMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_ATTACHMENT_MANAGER, ApplicationAttachmentManagerClass))

typedef struct _ApplicationAttachmentManager ApplicationAttachmentManager;
typedef struct _ApplicationAttachmentManagerClass ApplicationAttachmentManagerClass;
typedef struct _ApplicationAttachmentManagerPrivate ApplicationAttachmentManagerPrivate;

#define APPLICATION_TYPE_MAIN_WINDOW (application_main_window_get_type ())
#define APPLICATION_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_MAIN_WINDOW, ApplicationMainWindow))
#define APPLICATION_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_MAIN_WINDOW, ApplicationMainWindowClass))
#define APPLICATION_IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_MAIN_WINDOW))
#define APPLICATION_IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_MAIN_WINDOW))
#define APPLICATION_MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_MAIN_WINDOW, ApplicationMainWindowClass))

typedef struct _ApplicationMainWindow ApplicationMainWindow;
typedef struct _ApplicationMainWindowClass ApplicationMainWindowClass;

#define APPLICATION_TYPE_CERTIFICATE_MANAGER_ERROR (application_certificate_manager_error_get_type ())

#define APPLICATION_TYPE_CERTIFICATE_MANAGER (application_certificate_manager_get_type ())
#define APPLICATION_CERTIFICATE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_CERTIFICATE_MANAGER, ApplicationCertificateManager))
#define APPLICATION_CERTIFICATE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_CERTIFICATE_MANAGER, ApplicationCertificateManagerClass))
#define APPLICATION_IS_CERTIFICATE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_CERTIFICATE_MANAGER))
#define APPLICATION_IS_CERTIFICATE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_CERTIFICATE_MANAGER))
#define APPLICATION_CERTIFICATE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_CERTIFICATE_MANAGER, ApplicationCertificateManagerClass))

typedef struct _ApplicationCertificateManager ApplicationCertificateManager;
typedef struct _ApplicationCertificateManagerClass ApplicationCertificateManagerClass;
typedef struct _ApplicationCertificateManagerPrivate ApplicationCertificateManagerPrivate;

#define APPLICATION_TYPE_CLIENT (application_client_get_type ())
#define APPLICATION_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_CLIENT, ApplicationClient))
#define APPLICATION_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_CLIENT, ApplicationClientClass))
#define APPLICATION_IS_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_CLIENT))
#define APPLICATION_IS_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_CLIENT))
#define APPLICATION_CLIENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_CLIENT, ApplicationClientClass))

typedef struct _ApplicationClient ApplicationClient;
typedef struct _ApplicationClientClass ApplicationClientClass;
typedef struct _ApplicationClientPrivate ApplicationClientPrivate;

#define APPLICATION_TYPE_CONFIGURATION (application_configuration_get_type ())
#define APPLICATION_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_CONFIGURATION, ApplicationConfiguration))
#define APPLICATION_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_CONFIGURATION, ApplicationConfigurationClass))
#define APPLICATION_IS_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_CONFIGURATION))
#define APPLICATION_IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_CONFIGURATION))
#define APPLICATION_CONFIGURATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_CONFIGURATION, ApplicationConfigurationClass))

typedef struct _ApplicationConfiguration ApplicationConfiguration;
typedef struct _ApplicationConfigurationClass ApplicationConfigurationClass;

#define APPLICATION_TYPE_STARTUP_MANAGER (application_startup_manager_get_type ())
#define APPLICATION_STARTUP_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_STARTUP_MANAGER, ApplicationStartupManager))
#define APPLICATION_STARTUP_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_STARTUP_MANAGER, ApplicationStartupManagerClass))
#define APPLICATION_IS_STARTUP_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_STARTUP_MANAGER))
#define APPLICATION_IS_STARTUP_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_STARTUP_MANAGER))
#define APPLICATION_STARTUP_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_STARTUP_MANAGER, ApplicationStartupManagerClass))

typedef struct _ApplicationStartupManager ApplicationStartupManager;
typedef struct _ApplicationStartupManagerClass ApplicationStartupManagerClass;

#define APPLICATION_CLIENT_TYPE_RUNTIME_DETAIL (application_client_runtime_detail_get_type ())
typedef struct _ApplicationClientRuntimeDetail ApplicationClientRuntimeDetail;

#define APPLICATION_TYPE_COMMAND (application_command_get_type ())
#define APPLICATION_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_COMMAND, ApplicationCommand))
#define APPLICATION_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_COMMAND, ApplicationCommandClass))
#define APPLICATION_IS_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_COMMAND))
#define APPLICATION_IS_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_COMMAND))
#define APPLICATION_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_COMMAND, ApplicationCommandClass))

typedef struct _ApplicationCommand ApplicationCommand;
typedef struct _ApplicationCommandClass ApplicationCommandClass;
typedef struct _ApplicationCommandPrivate ApplicationCommandPrivate;

#define APPLICATION_TYPE_COMMAND_SEQUENCE (application_command_sequence_get_type ())
#define APPLICATION_COMMAND_SEQUENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_COMMAND_SEQUENCE, ApplicationCommandSequence))
#define APPLICATION_COMMAND_SEQUENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_COMMAND_SEQUENCE, ApplicationCommandSequenceClass))
#define APPLICATION_IS_COMMAND_SEQUENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_COMMAND_SEQUENCE))
#define APPLICATION_IS_COMMAND_SEQUENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_COMMAND_SEQUENCE))
#define APPLICATION_COMMAND_SEQUENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_COMMAND_SEQUENCE, ApplicationCommandSequenceClass))

typedef struct _ApplicationCommandSequence ApplicationCommandSequence;
typedef struct _ApplicationCommandSequenceClass ApplicationCommandSequenceClass;
typedef struct _ApplicationCommandSequencePrivate ApplicationCommandSequencePrivate;

#define APPLICATION_TYPE_PROPERTY_COMMAND (application_property_command_get_type ())
#define APPLICATION_PROPERTY_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_PROPERTY_COMMAND, ApplicationPropertyCommand))
#define APPLICATION_PROPERTY_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_PROPERTY_COMMAND, ApplicationPropertyCommandClass))
#define APPLICATION_IS_PROPERTY_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_PROPERTY_COMMAND))
#define APPLICATION_IS_PROPERTY_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_PROPERTY_COMMAND))
#define APPLICATION_PROPERTY_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_PROPERTY_COMMAND, ApplicationPropertyCommandClass))

typedef struct _ApplicationPropertyCommand ApplicationPropertyCommand;
typedef struct _ApplicationPropertyCommandClass ApplicationPropertyCommandClass;
typedef struct _ApplicationPropertyCommandPrivate ApplicationPropertyCommandPrivate;
typedef struct _ApplicationCommandStackPrivate ApplicationCommandStackPrivate;
typedef struct _ApplicationConfigurationPrivate ApplicationConfigurationPrivate;
typedef enum  {
	APPLICATION_CONFIGURATION_DESKTOP_ENVIRONMENT_UNKNOWN = 0,
	APPLICATION_CONFIGURATION_DESKTOP_ENVIRONMENT_UNITY
} ApplicationConfigurationDesktopEnvironment;

#define APPLICATION_CONFIGURATION_TYPE_DESKTOP_ENVIRONMENT (application_configuration_desktop_environment_get_type ())
typedef enum  {
	UTIL_DATE_CLOCK_FORMAT_TWELVE_HOURS,
	UTIL_DATE_CLOCK_FORMAT_TWENTY_FOUR_HOURS,
	UTIL_DATE_CLOCK_FORMAT_LOCALE_DEFAULT,
	UTIL_DATE_CLOCK_FORMAT_TOTAL
} UtilDateClockFormat;

#define UTIL_DATE_TYPE_CLOCK_FORMAT (util_date_clock_format_get_type ())
typedef struct _ApplicationContactStorePrivate ApplicationContactStorePrivate;

#define APPLICATION_TYPE_CONTACT (application_contact_get_type ())
#define APPLICATION_CONTACT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_CONTACT, ApplicationContact))
#define APPLICATION_CONTACT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_CONTACT, ApplicationContactClass))
#define APPLICATION_IS_CONTACT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_CONTACT))
#define APPLICATION_IS_CONTACT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_CONTACT))
#define APPLICATION_CONTACT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_CONTACT, ApplicationContactClass))

typedef struct _ApplicationContact ApplicationContact;
typedef struct _ApplicationContactClass ApplicationContactClass;
typedef struct _ApplicationContactPrivate ApplicationContactPrivate;

#define APPLICATION_TYPE_EMAIL_COMMAND (application_email_command_get_type ())
#define APPLICATION_EMAIL_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_EMAIL_COMMAND, ApplicationEmailCommand))
#define APPLICATION_EMAIL_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_EMAIL_COMMAND, ApplicationEmailCommandClass))
#define APPLICATION_IS_EMAIL_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_EMAIL_COMMAND))
#define APPLICATION_IS_EMAIL_COMMAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_EMAIL_COMMAND))
#define APPLICATION_EMAIL_COMMAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_EMAIL_COMMAND, ApplicationEmailCommandClass))

typedef struct _ApplicationEmailCommand ApplicationEmailCommand;
typedef struct _ApplicationEmailCommandClass ApplicationEmailCommandClass;
typedef struct _ApplicationEmailCommandPrivate ApplicationEmailCommandPrivate;
typedef enum  {
	APPLICATION_EMAIL_COMMAND_STATE_CHANGE_POLICY_IGNORE,
	APPLICATION_EMAIL_COMMAND_STATE_CHANGE_POLICY_REMOVE
} ApplicationEmailCommandStateChangePolicy;

#define APPLICATION_EMAIL_COMMAND_TYPE_STATE_CHANGE_POLICY (application_email_command_state_change_policy_get_type ())

#define APPLICATION_TYPE_TRIVIAL_COMMAND (application_trivial_command_get_type ())
#define APPLICATION_TRIVIAL_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_TRIVIAL_COMMAND, ApplicationTrivialCommand))
#define APPLICATION_IS_TRIVIAL_COMMAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_TRIVIAL_COMMAND))
#define APPLICATION_TRIVIAL_COMMAND_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), APPLICATION_TYPE_TRIVIAL_COMMAND, ApplicationTrivialCommandIface))

typedef struct _ApplicationTrivialCommand ApplicationTrivialCommand;
typedef struct _ApplicationTrivialCommandIface ApplicationTrivialCommandIface;
typedef struct _ApplicationFolderContextPrivate ApplicationFolderContextPrivate;
typedef enum  {
	APPLICATION_FOLDER_CONTEXT_EMAIL_COUNT_NONE,
	APPLICATION_FOLDER_CONTEXT_EMAIL_COUNT_UNREAD,
	APPLICATION_FOLDER_CONTEXT_EMAIL_COUNT_TOTAL
} ApplicationFolderContextEmailCount;

#define APPLICATION_FOLDER_CONTEXT_TYPE_EMAIL_COUNT (application_folder_context_email_count_get_type ())
typedef struct _ApplicationMainWindowPrivate ApplicationMainWindowPrivate;

#define COMPONENTS_TYPE_CONVERSATION_HEADER_BAR (components_conversation_header_bar_get_type ())
#define COMPONENTS_CONVERSATION_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_CONVERSATION_HEADER_BAR, ComponentsConversationHeaderBar))
#define COMPONENTS_CONVERSATION_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_CONVERSATION_HEADER_BAR, ComponentsConversationHeaderBarClass))
#define COMPONENTS_IS_CONVERSATION_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_CONVERSATION_HEADER_BAR))
#define COMPONENTS_IS_CONVERSATION_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_CONVERSATION_HEADER_BAR))
#define COMPONENTS_CONVERSATION_HEADER_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_CONVERSATION_HEADER_BAR, ComponentsConversationHeaderBarClass))

typedef struct _ComponentsConversationHeaderBar ComponentsConversationHeaderBar;
typedef struct _ComponentsConversationHeaderBarClass ComponentsConversationHeaderBarClass;

#define SIDEBAR_TYPE_TREE (sidebar_tree_get_type ())
#define SIDEBAR_TREE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_TREE, SidebarTree))
#define SIDEBAR_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SIDEBAR_TYPE_TREE, SidebarTreeClass))
#define SIDEBAR_IS_TREE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_TREE))
#define SIDEBAR_IS_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SIDEBAR_TYPE_TREE))
#define SIDEBAR_TREE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SIDEBAR_TYPE_TREE, SidebarTreeClass))

typedef struct _SidebarTree SidebarTree;
typedef struct _SidebarTreeClass SidebarTreeClass;

#define FOLDER_LIST_TYPE_TREE (folder_list_tree_get_type ())
#define FOLDER_LIST_TREE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_TREE, FolderListTree))
#define FOLDER_LIST_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_TREE, FolderListTreeClass))
#define FOLDER_LIST_IS_TREE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_TREE))
#define FOLDER_LIST_IS_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_TREE))
#define FOLDER_LIST_TREE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_TREE, FolderListTreeClass))

typedef struct _FolderListTree FolderListTree;
typedef struct _FolderListTreeClass FolderListTreeClass;

#define TYPE_SEARCH_BAR (search_bar_get_type ())
#define SEARCH_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SEARCH_BAR, SearchBar))
#define SEARCH_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SEARCH_BAR, SearchBarClass))
#define IS_SEARCH_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SEARCH_BAR))
#define IS_SEARCH_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SEARCH_BAR))
#define SEARCH_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SEARCH_BAR, SearchBarClass))

typedef struct _SearchBar SearchBar;
typedef struct _SearchBarClass SearchBarClass;

#define CONVERSATION_LIST_TYPE_VIEW (conversation_list_view_get_type ())
#define CONVERSATION_LIST_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONVERSATION_LIST_TYPE_VIEW, ConversationListView))
#define CONVERSATION_LIST_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONVERSATION_LIST_TYPE_VIEW, ConversationListViewClass))
#define CONVERSATION_LIST_IS_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONVERSATION_LIST_TYPE_VIEW))
#define CONVERSATION_LIST_IS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONVERSATION_LIST_TYPE_VIEW))
#define CONVERSATION_LIST_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONVERSATION_LIST_TYPE_VIEW, ConversationListViewClass))

typedef struct _ConversationListView ConversationListView;
typedef struct _ConversationListViewClass ConversationListViewClass;

#define TYPE_CONVERSATION_VIEWER (conversation_viewer_get_type ())
#define CONVERSATION_VIEWER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONVERSATION_VIEWER, ConversationViewer))
#define CONVERSATION_VIEWER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONVERSATION_VIEWER, ConversationViewerClass))
#define IS_CONVERSATION_VIEWER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONVERSATION_VIEWER))
#define IS_CONVERSATION_VIEWER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONVERSATION_VIEWER))
#define CONVERSATION_VIEWER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONVERSATION_VIEWER, ConversationViewerClass))

typedef struct _ConversationViewer ConversationViewer;
typedef struct _ConversationViewerClass ConversationViewerClass;

#define COMPONENTS_TYPE_INFO_BAR_STACK (components_info_bar_stack_get_type ())
#define COMPONENTS_INFO_BAR_STACK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_INFO_BAR_STACK, ComponentsInfoBarStack))
#define COMPONENTS_INFO_BAR_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_INFO_BAR_STACK, ComponentsInfoBarStackClass))
#define COMPONENTS_IS_INFO_BAR_STACK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_INFO_BAR_STACK))
#define COMPONENTS_IS_INFO_BAR_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_INFO_BAR_STACK))
#define COMPONENTS_INFO_BAR_STACK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_INFO_BAR_STACK, ComponentsInfoBarStackClass))

typedef struct _ComponentsInfoBarStack ComponentsInfoBarStack;
typedef struct _ComponentsInfoBarStackClass ComponentsInfoBarStackClass;

#define COMPONENTS_TYPE_INFO_BAR (components_info_bar_get_type ())
#define COMPONENTS_INFO_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_INFO_BAR, ComponentsInfoBar))
#define COMPONENTS_INFO_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_INFO_BAR, ComponentsInfoBarClass))
#define COMPONENTS_IS_INFO_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_INFO_BAR))
#define COMPONENTS_IS_INFO_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_INFO_BAR))
#define COMPONENTS_INFO_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_INFO_BAR, ComponentsInfoBarClass))

typedef struct _ComponentsInfoBar ComponentsInfoBar;
typedef struct _ComponentsInfoBarClass ComponentsInfoBarClass;

#define COMPONENTS_TYPE_IN_APP_NOTIFICATION (components_in_app_notification_get_type ())
#define COMPONENTS_IN_APP_NOTIFICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_IN_APP_NOTIFICATION, ComponentsInAppNotification))
#define COMPONENTS_IN_APP_NOTIFICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_IN_APP_NOTIFICATION, ComponentsInAppNotificationClass))
#define COMPONENTS_IS_IN_APP_NOTIFICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_IN_APP_NOTIFICATION))
#define COMPONENTS_IS_IN_APP_NOTIFICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_IN_APP_NOTIFICATION))
#define COMPONENTS_IN_APP_NOTIFICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_IN_APP_NOTIFICATION, ComponentsInAppNotificationClass))

typedef struct _ComponentsInAppNotification ComponentsInAppNotification;
typedef struct _ComponentsInAppNotificationClass ComponentsInAppNotificationClass;

#define APPLICATION_TYPE_PLUGIN_MANAGER (application_plugin_manager_get_type ())
#define APPLICATION_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPLICATION_TYPE_PLUGIN_MANAGER, ApplicationPluginManager))
#define APPLICATION_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPLICATION_TYPE_PLUGIN_MANAGER, ApplicationPluginManagerClass))
#define APPLICATION_IS_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPLICATION_TYPE_PLUGIN_MANAGER))
#define APPLICATION_IS_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPLICATION_TYPE_PLUGIN_MANAGER))
#define APPLICATION_PLUGIN_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPLICATION_TYPE_PLUGIN_MANAGER, ApplicationPluginManagerClass))

typedef struct _ApplicationPluginManager ApplicationPluginManager;
typedef struct _ApplicationPluginManagerClass ApplicationPluginManagerClass;
typedef struct _ApplicationPluginManagerPrivate ApplicationPluginManagerPrivate;

#define PLUGIN_TYPE_ACCOUNT (plugin_account_get_type ())
#define PLUGIN_ACCOUNT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_ACCOUNT, PluginAccount))
#define PLUGIN_IS_ACCOUNT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_ACCOUNT))
#define PLUGIN_ACCOUNT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_ACCOUNT, PluginAccountIface))

typedef struct _PluginAccount PluginAccount;
typedef struct _PluginAccountIface PluginAccountIface;

#define PLUGIN_TYPE_FOLDER (plugin_folder_get_type ())
#define PLUGIN_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_FOLDER, PluginFolder))
#define PLUGIN_IS_FOLDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_FOLDER))
#define PLUGIN_FOLDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_FOLDER, PluginFolderIface))

typedef struct _PluginFolder PluginFolder;
typedef struct _PluginFolderIface PluginFolderIface;

#define PLUGIN_TYPE_EMAIL (plugin_email_get_type ())
#define PLUGIN_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_EMAIL, PluginEmail))
#define PLUGIN_IS_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_EMAIL))
#define PLUGIN_EMAIL_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_EMAIL, PluginEmailIface))

typedef struct _PluginEmail PluginEmail;
typedef struct _PluginEmailIface PluginEmailIface;

#define PLUGIN_TYPE_EMAIL_IDENTIFIER (plugin_email_identifier_get_type ())
#define PLUGIN_EMAIL_IDENTIFIER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_EMAIL_IDENTIFIER, PluginEmailIdentifier))
#define PLUGIN_IS_EMAIL_IDENTIFIER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_EMAIL_IDENTIFIER))
#define PLUGIN_EMAIL_IDENTIFIER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_EMAIL_IDENTIFIER, PluginEmailIdentifierIface))

typedef struct _PluginEmailIdentifier PluginEmailIdentifier;
typedef struct _PluginEmailIdentifierIface PluginEmailIdentifierIface;
typedef enum  {
	PLUGIN_EMAIL_BODY_TYPE_PLAIN,
	PLUGIN_EMAIL_BODY_TYPE_HTML
} PluginEmailBodyType;

#define PLUGIN_EMAIL_TYPE_BODY_TYPE (plugin_email_body_type_get_type ())

#define PORTAL_TYPE_REQUEST (portal_request_get_type ())
#define PORTAL_REQUEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PORTAL_TYPE_REQUEST, portalRequest))
#define PORTAL_IS_REQUEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PORTAL_TYPE_REQUEST))
#define PORTAL_REQUEST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PORTAL_TYPE_REQUEST, portalRequestIface))

typedef struct _portalRequest portalRequest;
typedef struct _portalRequestIface portalRequestIface;

#define PORTAL_TYPE_REQUEST_PROXY (portal_request_proxy_get_type ())

#define PORTAL_TYPE_BACKGROUND (portal_background_get_type ())
#define PORTAL_BACKGROUND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PORTAL_TYPE_BACKGROUND, portalBackground))
#define PORTAL_IS_BACKGROUND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PORTAL_TYPE_BACKGROUND))
#define PORTAL_BACKGROUND_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PORTAL_TYPE_BACKGROUND, portalBackgroundIface))

typedef struct _portalBackground portalBackground;
typedef struct _portalBackgroundIface portalBackgroundIface;

#define PORTAL_TYPE_BACKGROUND_PROXY (portal_background_proxy_get_type ())
typedef struct _ApplicationStartupManagerPrivate ApplicationStartupManagerPrivate;

#define TYPE_GOA_MEDIATOR (goa_mediator_get_type ())
#define GOA_MEDIATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GOA_MEDIATOR, GoaMediator))
#define GOA_MEDIATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GOA_MEDIATOR, GoaMediatorClass))
#define IS_GOA_MEDIATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GOA_MEDIATOR))
#define IS_GOA_MEDIATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GOA_MEDIATOR))
#define GOA_MEDIATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GOA_MEDIATOR, GoaMediatorClass))

typedef struct _GoaMediator GoaMediator;
typedef struct _GoaMediatorClass GoaMediatorClass;
typedef struct _GoaMediatorPrivate GoaMediatorPrivate;

#define TYPE_SECRET_MEDIATOR (secret_mediator_get_type ())
#define SECRET_MEDIATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SECRET_MEDIATOR, SecretMediator))
#define SECRET_MEDIATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SECRET_MEDIATOR, SecretMediatorClass))
#define IS_SECRET_MEDIATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SECRET_MEDIATOR))
#define IS_SECRET_MEDIATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SECRET_MEDIATOR))
#define SECRET_MEDIATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SECRET_MEDIATOR, SecretMediatorClass))

typedef struct _SecretMediator SecretMediator;
typedef struct _SecretMediatorClass SecretMediatorClass;
typedef struct _SecretMediatorPrivate SecretMediatorPrivate;

#define ACCOUNTS_TYPE_EDITOR (accounts_editor_get_type ())
#define ACCOUNTS_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_EDITOR, AccountsEditor))
#define ACCOUNTS_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_EDITOR, AccountsEditorClass))
#define ACCOUNTS_IS_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_EDITOR))
#define ACCOUNTS_IS_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_EDITOR))
#define ACCOUNTS_EDITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_EDITOR, AccountsEditorClass))

typedef struct _AccountsEditor AccountsEditor;
typedef struct _AccountsEditorClass AccountsEditorClass;
typedef struct _AccountsEditorPrivate AccountsEditorPrivate;

#define COMPONENTS_TYPE_WEB_VIEW (components_web_view_get_type ())
#define COMPONENTS_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_WEB_VIEW, ComponentsWebView))
#define COMPONENTS_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_WEB_VIEW, ComponentsWebViewClass))
#define COMPONENTS_IS_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_WEB_VIEW))
#define COMPONENTS_IS_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_WEB_VIEW))
#define COMPONENTS_WEB_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_WEB_VIEW, ComponentsWebViewClass))

typedef struct _ComponentsWebView ComponentsWebView;
typedef struct _ComponentsWebViewClass ComponentsWebViewClass;
typedef struct _ComponentsWebViewPrivate ComponentsWebViewPrivate;

#define ACCOUNTS_TYPE_SIGNATURE_WEB_VIEW (accounts_signature_web_view_get_type ())
#define ACCOUNTS_SIGNATURE_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_SIGNATURE_WEB_VIEW, AccountsSignatureWebView))
#define ACCOUNTS_SIGNATURE_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_SIGNATURE_WEB_VIEW, AccountsSignatureWebViewClass))
#define ACCOUNTS_IS_SIGNATURE_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_SIGNATURE_WEB_VIEW))
#define ACCOUNTS_IS_SIGNATURE_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_SIGNATURE_WEB_VIEW))
#define ACCOUNTS_SIGNATURE_WEB_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_SIGNATURE_WEB_VIEW, AccountsSignatureWebViewClass))

typedef struct _AccountsSignatureWebView AccountsSignatureWebView;
typedef struct _AccountsSignatureWebViewClass AccountsSignatureWebViewClass;
typedef struct _AccountsSignatureWebViewPrivate AccountsSignatureWebViewPrivate;

#define ACCOUNTS_TYPE_MANAGER (accounts_manager_get_type ())
#define ACCOUNTS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_MANAGER, AccountsManager))
#define ACCOUNTS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_MANAGER, AccountsManagerClass))
#define ACCOUNTS_IS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_MANAGER))
#define ACCOUNTS_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_MANAGER))
#define ACCOUNTS_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_MANAGER, AccountsManagerClass))

typedef struct _AccountsManager AccountsManager;
typedef struct _AccountsManagerClass AccountsManagerClass;
typedef struct _AccountsManagerPrivate AccountsManagerPrivate;
typedef enum  {
	ACCOUNTS_MANAGER_STATUS_ENABLED,
	ACCOUNTS_MANAGER_STATUS_DISABLED,
	ACCOUNTS_MANAGER_STATUS_UNAVAILABLE,
	ACCOUNTS_MANAGER_STATUS_REMOVED
} AccountsManagerStatus;

#define ACCOUNTS_MANAGER_TYPE_STATUS (accounts_manager_status_get_type ())

#define ACCOUNTS_TYPE_ACCOUNT_CONFIG (accounts_account_config_get_type ())
#define ACCOUNTS_ACCOUNT_CONFIG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG, AccountsAccountConfig))
#define ACCOUNTS_IS_ACCOUNT_CONFIG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG))
#define ACCOUNTS_ACCOUNT_CONFIG_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG, AccountsAccountConfigIface))

typedef struct _AccountsAccountConfig AccountsAccountConfig;
typedef struct _AccountsAccountConfigIface AccountsAccountConfigIface;

#define ACCOUNTS_TYPE_CONFIG_ERROR (accounts_config_error_get_type ())

#define ACCOUNTS_TYPE_SERVICE_CONFIG (accounts_service_config_get_type ())
#define ACCOUNTS_SERVICE_CONFIG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG, AccountsServiceConfig))
#define ACCOUNTS_IS_SERVICE_CONFIG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG))
#define ACCOUNTS_SERVICE_CONFIG_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG, AccountsServiceConfigIface))

typedef struct _AccountsServiceConfig AccountsServiceConfig;
typedef struct _AccountsServiceConfigIface AccountsServiceConfigIface;

#define ACCOUNTS_TYPE_ACCOUNT_CONFIG_V1 (accounts_account_config_v1_get_type ())
#define ACCOUNTS_ACCOUNT_CONFIG_V1(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG_V1, AccountsAccountConfigV1))
#define ACCOUNTS_ACCOUNT_CONFIG_V1_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_ACCOUNT_CONFIG_V1, AccountsAccountConfigV1Class))
#define ACCOUNTS_IS_ACCOUNT_CONFIG_V1(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG_V1))
#define ACCOUNTS_IS_ACCOUNT_CONFIG_V1_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_ACCOUNT_CONFIG_V1))
#define ACCOUNTS_ACCOUNT_CONFIG_V1_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG_V1, AccountsAccountConfigV1Class))

typedef struct _AccountsAccountConfigV1 AccountsAccountConfigV1;
typedef struct _AccountsAccountConfigV1Class AccountsAccountConfigV1Class;
typedef struct _AccountsAccountConfigV1Private AccountsAccountConfigV1Private;

#define ACCOUNTS_TYPE_ACCOUNT_CONFIG_LEGACY (accounts_account_config_legacy_get_type ())
#define ACCOUNTS_ACCOUNT_CONFIG_LEGACY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG_LEGACY, AccountsAccountConfigLegacy))
#define ACCOUNTS_ACCOUNT_CONFIG_LEGACY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_ACCOUNT_CONFIG_LEGACY, AccountsAccountConfigLegacyClass))
#define ACCOUNTS_IS_ACCOUNT_CONFIG_LEGACY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG_LEGACY))
#define ACCOUNTS_IS_ACCOUNT_CONFIG_LEGACY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_ACCOUNT_CONFIG_LEGACY))
#define ACCOUNTS_ACCOUNT_CONFIG_LEGACY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_ACCOUNT_CONFIG_LEGACY, AccountsAccountConfigLegacyClass))

typedef struct _AccountsAccountConfigLegacy AccountsAccountConfigLegacy;
typedef struct _AccountsAccountConfigLegacyClass AccountsAccountConfigLegacyClass;
typedef struct _AccountsAccountConfigLegacyPrivate AccountsAccountConfigLegacyPrivate;

#define ACCOUNTS_TYPE_SERVICE_CONFIG_V1 (accounts_service_config_v1_get_type ())
#define ACCOUNTS_SERVICE_CONFIG_V1(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG_V1, AccountsServiceConfigV1))
#define ACCOUNTS_SERVICE_CONFIG_V1_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_SERVICE_CONFIG_V1, AccountsServiceConfigV1Class))
#define ACCOUNTS_IS_SERVICE_CONFIG_V1(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG_V1))
#define ACCOUNTS_IS_SERVICE_CONFIG_V1_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_SERVICE_CONFIG_V1))
#define ACCOUNTS_SERVICE_CONFIG_V1_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG_V1, AccountsServiceConfigV1Class))

typedef struct _AccountsServiceConfigV1 AccountsServiceConfigV1;
typedef struct _AccountsServiceConfigV1Class AccountsServiceConfigV1Class;
typedef struct _AccountsServiceConfigV1Private AccountsServiceConfigV1Private;

#define ACCOUNTS_TYPE_SERVICE_CONFIG_LEGACY (accounts_service_config_legacy_get_type ())
#define ACCOUNTS_SERVICE_CONFIG_LEGACY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG_LEGACY, AccountsServiceConfigLegacy))
#define ACCOUNTS_SERVICE_CONFIG_LEGACY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ACCOUNTS_TYPE_SERVICE_CONFIG_LEGACY, AccountsServiceConfigLegacyClass))
#define ACCOUNTS_IS_SERVICE_CONFIG_LEGACY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG_LEGACY))
#define ACCOUNTS_IS_SERVICE_CONFIG_LEGACY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ACCOUNTS_TYPE_SERVICE_CONFIG_LEGACY))
#define ACCOUNTS_SERVICE_CONFIG_LEGACY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ACCOUNTS_TYPE_SERVICE_CONFIG_LEGACY, AccountsServiceConfigLegacyClass))

typedef struct _AccountsServiceConfigLegacy AccountsServiceConfigLegacy;
typedef struct _AccountsServiceConfigLegacyClass AccountsServiceConfigLegacyClass;
typedef struct _AccountsServiceConfigLegacyPrivate AccountsServiceConfigLegacyPrivate;

#define COMPONENTS_TYPE_ATTACHMENT_PANE (components_attachment_pane_get_type ())
#define COMPONENTS_ATTACHMENT_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_ATTACHMENT_PANE, ComponentsAttachmentPane))
#define COMPONENTS_ATTACHMENT_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_ATTACHMENT_PANE, ComponentsAttachmentPaneClass))
#define COMPONENTS_IS_ATTACHMENT_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_ATTACHMENT_PANE))
#define COMPONENTS_IS_ATTACHMENT_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_ATTACHMENT_PANE))
#define COMPONENTS_ATTACHMENT_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_ATTACHMENT_PANE, ComponentsAttachmentPaneClass))

typedef struct _ComponentsAttachmentPane ComponentsAttachmentPane;
typedef struct _ComponentsAttachmentPaneClass ComponentsAttachmentPaneClass;
typedef struct _ComponentsAttachmentPanePrivate ComponentsAttachmentPanePrivate;

#define COMPONENTS_TYPE_CONVERSATION_ACTIONS (components_conversation_actions_get_type ())
#define COMPONENTS_CONVERSATION_ACTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_CONVERSATION_ACTIONS, ComponentsConversationActions))
#define COMPONENTS_CONVERSATION_ACTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_CONVERSATION_ACTIONS, ComponentsConversationActionsClass))
#define COMPONENTS_IS_CONVERSATION_ACTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_CONVERSATION_ACTIONS))
#define COMPONENTS_IS_CONVERSATION_ACTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_CONVERSATION_ACTIONS))
#define COMPONENTS_CONVERSATION_ACTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_CONVERSATION_ACTIONS, ComponentsConversationActionsClass))

typedef struct _ComponentsConversationActions ComponentsConversationActions;
typedef struct _ComponentsConversationActionsClass ComponentsConversationActionsClass;
typedef struct _ComponentsConversationActionsPrivate ComponentsConversationActionsPrivate;

#define TYPE_FOLDER_POPOVER (folder_popover_get_type ())
#define FOLDER_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FOLDER_POPOVER, FolderPopover))
#define FOLDER_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FOLDER_POPOVER, FolderPopoverClass))
#define IS_FOLDER_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FOLDER_POPOVER))
#define IS_FOLDER_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FOLDER_POPOVER))
#define FOLDER_POPOVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FOLDER_POPOVER, FolderPopoverClass))

typedef struct _FolderPopover FolderPopover;
typedef struct _FolderPopoverClass FolderPopoverClass;

#define COMPONENTS_TYPE_ENTRY_UNDO (components_entry_undo_get_type ())
#define COMPONENTS_ENTRY_UNDO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_ENTRY_UNDO, ComponentsEntryUndo))
#define COMPONENTS_ENTRY_UNDO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_ENTRY_UNDO, ComponentsEntryUndoClass))
#define COMPONENTS_IS_ENTRY_UNDO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_ENTRY_UNDO))
#define COMPONENTS_IS_ENTRY_UNDO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_ENTRY_UNDO))
#define COMPONENTS_ENTRY_UNDO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_ENTRY_UNDO, ComponentsEntryUndoClass))

typedef struct _ComponentsEntryUndo ComponentsEntryUndo;
typedef struct _ComponentsEntryUndoClass ComponentsEntryUndoClass;
typedef struct _ComponentsEntryUndoPrivate ComponentsEntryUndoPrivate;

#define COMPONENTS_TYPE_APPLICATION_HEADER_BAR (components_application_header_bar_get_type ())
#define COMPONENTS_APPLICATION_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_APPLICATION_HEADER_BAR, ComponentsApplicationHeaderBar))
#define COMPONENTS_APPLICATION_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_APPLICATION_HEADER_BAR, ComponentsApplicationHeaderBarClass))
#define COMPONENTS_IS_APPLICATION_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_APPLICATION_HEADER_BAR))
#define COMPONENTS_IS_APPLICATION_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_APPLICATION_HEADER_BAR))
#define COMPONENTS_APPLICATION_HEADER_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_APPLICATION_HEADER_BAR, ComponentsApplicationHeaderBarClass))

typedef struct _ComponentsApplicationHeaderBar ComponentsApplicationHeaderBar;
typedef struct _ComponentsApplicationHeaderBarClass ComponentsApplicationHeaderBarClass;
typedef struct _ComponentsApplicationHeaderBarPrivate ComponentsApplicationHeaderBarPrivate;

#define TYPE_MONITORED_SPINNER (monitored_spinner_get_type ())
#define MONITORED_SPINNER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MONITORED_SPINNER, MonitoredSpinner))
#define MONITORED_SPINNER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MONITORED_SPINNER, MonitoredSpinnerClass))
#define IS_MONITORED_SPINNER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MONITORED_SPINNER))
#define IS_MONITORED_SPINNER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MONITORED_SPINNER))
#define MONITORED_SPINNER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MONITORED_SPINNER, MonitoredSpinnerClass))

typedef struct _MonitoredSpinner MonitoredSpinner;
typedef struct _MonitoredSpinnerClass MonitoredSpinnerClass;

#define COMPONENTS_TYPE_CONVERSATION_LIST_HEADER_BAR (components_conversation_list_header_bar_get_type ())
#define COMPONENTS_CONVERSATION_LIST_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_CONVERSATION_LIST_HEADER_BAR, ComponentsConversationListHeaderBar))
#define COMPONENTS_CONVERSATION_LIST_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_CONVERSATION_LIST_HEADER_BAR, ComponentsConversationListHeaderBarClass))
#define COMPONENTS_IS_CONVERSATION_LIST_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_CONVERSATION_LIST_HEADER_BAR))
#define COMPONENTS_IS_CONVERSATION_LIST_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_CONVERSATION_LIST_HEADER_BAR))
#define COMPONENTS_CONVERSATION_LIST_HEADER_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_CONVERSATION_LIST_HEADER_BAR, ComponentsConversationListHeaderBarClass))

typedef struct _ComponentsConversationListHeaderBar ComponentsConversationListHeaderBar;
typedef struct _ComponentsConversationListHeaderBarClass ComponentsConversationListHeaderBarClass;
typedef struct _ComponentsConversationListHeaderBarPrivate ComponentsConversationListHeaderBarPrivate;
typedef struct _ComponentsConversationHeaderBarPrivate ComponentsConversationHeaderBarPrivate;
typedef struct _ComponentsInfoBarStackPrivate ComponentsInfoBarStackPrivate;
typedef enum  {
	COMPONENTS_INFO_BAR_STACK_STACK_TYPE_SINGLE,
	COMPONENTS_INFO_BAR_STACK_STACK_TYPE_PRIORITY_QUEUE
} ComponentsInfoBarStackStackType;

#define COMPONENTS_INFO_BAR_STACK_TYPE_STACK_TYPE (components_info_bar_stack_stack_type_get_type ())
typedef struct _ComponentsInfoBarPrivate ComponentsInfoBarPrivate;

#define PLUGIN_TYPE_INFO_BAR (plugin_info_bar_get_type ())
#define PLUGIN_INFO_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_INFO_BAR, PluginInfoBar))
#define PLUGIN_INFO_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_INFO_BAR, PluginInfoBarClass))
#define PLUGIN_IS_INFO_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_INFO_BAR))
#define PLUGIN_IS_INFO_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_INFO_BAR))
#define PLUGIN_INFO_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_INFO_BAR, PluginInfoBarClass))

typedef struct _PluginInfoBar PluginInfoBar;
typedef struct _PluginInfoBarClass PluginInfoBarClass;

#define COMPONENTS_TYPE_INSPECTOR (components_inspector_get_type ())
#define COMPONENTS_INSPECTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_INSPECTOR, ComponentsInspector))
#define COMPONENTS_INSPECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_INSPECTOR, ComponentsInspectorClass))
#define COMPONENTS_IS_INSPECTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_INSPECTOR))
#define COMPONENTS_IS_INSPECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_INSPECTOR))
#define COMPONENTS_INSPECTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_INSPECTOR, ComponentsInspectorClass))

typedef struct _ComponentsInspector ComponentsInspector;
typedef struct _ComponentsInspectorClass ComponentsInspectorClass;
typedef struct _ComponentsInspectorPrivate ComponentsInspectorPrivate;
typedef enum  {
	COMPONENTS_INSPECTOR_TEXT_FORMAT_PLAIN,
	COMPONENTS_INSPECTOR_TEXT_FORMAT_MARKDOWN
} ComponentsInspectorTextFormat;

#define COMPONENTS_INSPECTOR_TYPE_TEXT_FORMAT (components_inspector_text_format_get_type ())
typedef struct _ComponentsInAppNotificationPrivate ComponentsInAppNotificationPrivate;

#define COMPONENTS_TYPE_INSPECTOR_ERROR_VIEW (components_inspector_error_view_get_type ())
#define COMPONENTS_INSPECTOR_ERROR_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_INSPECTOR_ERROR_VIEW, ComponentsInspectorErrorView))
#define COMPONENTS_INSPECTOR_ERROR_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_INSPECTOR_ERROR_VIEW, ComponentsInspectorErrorViewClass))
#define COMPONENTS_IS_INSPECTOR_ERROR_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_INSPECTOR_ERROR_VIEW))
#define COMPONENTS_IS_INSPECTOR_ERROR_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_INSPECTOR_ERROR_VIEW))
#define COMPONENTS_INSPECTOR_ERROR_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_INSPECTOR_ERROR_VIEW, ComponentsInspectorErrorViewClass))

typedef struct _ComponentsInspectorErrorView ComponentsInspectorErrorView;
typedef struct _ComponentsInspectorErrorViewClass ComponentsInspectorErrorViewClass;
typedef struct _ComponentsInspectorErrorViewPrivate ComponentsInspectorErrorViewPrivate;

#define COMPONENTS_TYPE_INSPECTOR_LOG_VIEW (components_inspector_log_view_get_type ())
#define COMPONENTS_INSPECTOR_LOG_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_INSPECTOR_LOG_VIEW, ComponentsInspectorLogView))
#define COMPONENTS_INSPECTOR_LOG_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_INSPECTOR_LOG_VIEW, ComponentsInspectorLogViewClass))
#define COMPONENTS_IS_INSPECTOR_LOG_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_INSPECTOR_LOG_VIEW))
#define COMPONENTS_IS_INSPECTOR_LOG_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_INSPECTOR_LOG_VIEW))
#define COMPONENTS_INSPECTOR_LOG_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_INSPECTOR_LOG_VIEW, ComponentsInspectorLogViewClass))

typedef struct _ComponentsInspectorLogView ComponentsInspectorLogView;
typedef struct _ComponentsInspectorLogViewClass ComponentsInspectorLogViewClass;
typedef struct _ComponentsInspectorLogViewPrivate ComponentsInspectorLogViewPrivate;

#define COMPONENTS_TYPE_INSPECTOR_SYSTEM_VIEW (components_inspector_system_view_get_type ())
#define COMPONENTS_INSPECTOR_SYSTEM_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_INSPECTOR_SYSTEM_VIEW, ComponentsInspectorSystemView))
#define COMPONENTS_INSPECTOR_SYSTEM_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_INSPECTOR_SYSTEM_VIEW, ComponentsInspectorSystemViewClass))
#define COMPONENTS_IS_INSPECTOR_SYSTEM_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_INSPECTOR_SYSTEM_VIEW))
#define COMPONENTS_IS_INSPECTOR_SYSTEM_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_INSPECTOR_SYSTEM_VIEW))
#define COMPONENTS_INSPECTOR_SYSTEM_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_INSPECTOR_SYSTEM_VIEW, ComponentsInspectorSystemViewClass))

typedef struct _ComponentsInspectorSystemView ComponentsInspectorSystemView;
typedef struct _ComponentsInspectorSystemViewClass ComponentsInspectorSystemViewClass;
typedef struct _ComponentsInspectorSystemViewPrivate ComponentsInspectorSystemViewPrivate;

#define COMPONENTS_TYPE_PLACEHOLDER_PANE (components_placeholder_pane_get_type ())
#define COMPONENTS_PLACEHOLDER_PANE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_PLACEHOLDER_PANE, ComponentsPlaceholderPane))
#define COMPONENTS_PLACEHOLDER_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_PLACEHOLDER_PANE, ComponentsPlaceholderPaneClass))
#define COMPONENTS_IS_PLACEHOLDER_PANE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_PLACEHOLDER_PANE))
#define COMPONENTS_IS_PLACEHOLDER_PANE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_PLACEHOLDER_PANE))
#define COMPONENTS_PLACEHOLDER_PANE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_PLACEHOLDER_PANE, ComponentsPlaceholderPaneClass))

typedef struct _ComponentsPlaceholderPane ComponentsPlaceholderPane;
typedef struct _ComponentsPlaceholderPaneClass ComponentsPlaceholderPaneClass;
typedef struct _ComponentsPlaceholderPanePrivate ComponentsPlaceholderPanePrivate;

#define COMPONENTS_TYPE_PREFERENCES_WINDOW (components_preferences_window_get_type ())
#define COMPONENTS_PREFERENCES_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_PREFERENCES_WINDOW, ComponentsPreferencesWindow))
#define COMPONENTS_PREFERENCES_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_PREFERENCES_WINDOW, ComponentsPreferencesWindowClass))
#define COMPONENTS_IS_PREFERENCES_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_PREFERENCES_WINDOW))
#define COMPONENTS_IS_PREFERENCES_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_PREFERENCES_WINDOW))
#define COMPONENTS_PREFERENCES_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_PREFERENCES_WINDOW, ComponentsPreferencesWindowClass))

typedef struct _ComponentsPreferencesWindow ComponentsPreferencesWindow;
typedef struct _ComponentsPreferencesWindowClass ComponentsPreferencesWindowClass;
typedef struct _ComponentsPreferencesWindowPrivate ComponentsPreferencesWindowPrivate;

#define COMPONENTS_TYPE_PROBLEM_REPORT_INFO_BAR (components_problem_report_info_bar_get_type ())
#define COMPONENTS_PROBLEM_REPORT_INFO_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_PROBLEM_REPORT_INFO_BAR, ComponentsProblemReportInfoBar))
#define COMPONENTS_PROBLEM_REPORT_INFO_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_PROBLEM_REPORT_INFO_BAR, ComponentsProblemReportInfoBarClass))
#define COMPONENTS_IS_PROBLEM_REPORT_INFO_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_PROBLEM_REPORT_INFO_BAR))
#define COMPONENTS_IS_PROBLEM_REPORT_INFO_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_PROBLEM_REPORT_INFO_BAR))
#define COMPONENTS_PROBLEM_REPORT_INFO_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_PROBLEM_REPORT_INFO_BAR, ComponentsProblemReportInfoBarClass))

typedef struct _ComponentsProblemReportInfoBar ComponentsProblemReportInfoBar;
typedef struct _ComponentsProblemReportInfoBarClass ComponentsProblemReportInfoBarClass;
typedef struct _ComponentsProblemReportInfoBarPrivate ComponentsProblemReportInfoBarPrivate;
typedef struct _SearchBarPrivate SearchBarPrivate;

#define COMPONENTS_TYPE_VALIDATOR (components_validator_get_type ())
#define COMPONENTS_VALIDATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_VALIDATOR, ComponentsValidator))
#define COMPONENTS_VALIDATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_VALIDATOR, ComponentsValidatorClass))
#define COMPONENTS_IS_VALIDATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_VALIDATOR))
#define COMPONENTS_IS_VALIDATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_VALIDATOR))
#define COMPONENTS_VALIDATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_VALIDATOR, ComponentsValidatorClass))

typedef struct _ComponentsValidator ComponentsValidator;
typedef struct _ComponentsValidatorClass ComponentsValidatorClass;
typedef struct _ComponentsValidatorPrivate ComponentsValidatorPrivate;

#define COMPONENTS_VALIDATOR_TYPE_UI_STATE (components_validator_ui_state_get_type ())
typedef struct _ComponentsValidatorUiState ComponentsValidatorUiState;
typedef enum  {
	COMPONENTS_VALIDATOR_TRIGGER_MANUAL,
	COMPONENTS_VALIDATOR_TRIGGER_CHANGED,
	COMPONENTS_VALIDATOR_TRIGGER_LOST_FOCUS,
	COMPONENTS_VALIDATOR_TRIGGER_ACTIVATED
} ComponentsValidatorTrigger;

#define COMPONENTS_VALIDATOR_TYPE_TRIGGER (components_validator_trigger_get_type ())
typedef enum  {
	COMPONENTS_VALIDATOR_VALIDITY_INDETERMINATE,
	COMPONENTS_VALIDATOR_VALIDITY_VALID,
	COMPONENTS_VALIDATOR_VALIDITY_IN_PROGRESS,
	COMPONENTS_VALIDATOR_VALIDITY_EMPTY,
	COMPONENTS_VALIDATOR_VALIDITY_INVALID
} ComponentsValidatorValidity;

#define COMPONENTS_VALIDATOR_TYPE_VALIDITY (components_validator_validity_get_type ())

#define COMPONENTS_TYPE_EMAIL_VALIDATOR (components_email_validator_get_type ())
#define COMPONENTS_EMAIL_VALIDATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_EMAIL_VALIDATOR, ComponentsEmailValidator))
#define COMPONENTS_EMAIL_VALIDATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_EMAIL_VALIDATOR, ComponentsEmailValidatorClass))
#define COMPONENTS_IS_EMAIL_VALIDATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_EMAIL_VALIDATOR))
#define COMPONENTS_IS_EMAIL_VALIDATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_EMAIL_VALIDATOR))
#define COMPONENTS_EMAIL_VALIDATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_EMAIL_VALIDATOR, ComponentsEmailValidatorClass))

typedef struct _ComponentsEmailValidator ComponentsEmailValidator;
typedef struct _ComponentsEmailValidatorClass ComponentsEmailValidatorClass;
typedef struct _ComponentsEmailValidatorPrivate ComponentsEmailValidatorPrivate;

#define COMPONENTS_TYPE_NETWORK_ADDRESS_VALIDATOR (components_network_address_validator_get_type ())
#define COMPONENTS_NETWORK_ADDRESS_VALIDATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPONENTS_TYPE_NETWORK_ADDRESS_VALIDATOR, ComponentsNetworkAddressValidator))
#define COMPONENTS_NETWORK_ADDRESS_VALIDATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPONENTS_TYPE_NETWORK_ADDRESS_VALIDATOR, ComponentsNetworkAddressValidatorClass))
#define COMPONENTS_IS_NETWORK_ADDRESS_VALIDATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPONENTS_TYPE_NETWORK_ADDRESS_VALIDATOR))
#define COMPONENTS_IS_NETWORK_ADDRESS_VALIDATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPONENTS_TYPE_NETWORK_ADDRESS_VALIDATOR))
#define COMPONENTS_NETWORK_ADDRESS_VALIDATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPONENTS_TYPE_NETWORK_ADDRESS_VALIDATOR, ComponentsNetworkAddressValidatorClass))

typedef struct _ComponentsNetworkAddressValidator ComponentsNetworkAddressValidator;
typedef struct _ComponentsNetworkAddressValidatorClass ComponentsNetworkAddressValidatorClass;
typedef struct _ComponentsNetworkAddressValidatorPrivate ComponentsNetworkAddressValidatorPrivate;

#define UTIL_JS_TYPE_CALLABLE (util_js_callable_get_type ())
#define UTIL_JS_CALLABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), UTIL_JS_TYPE_CALLABLE, UtilJSCallable))
#define UTIL_JS_CALLABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), UTIL_JS_TYPE_CALLABLE, UtilJSCallableClass))
#define UTIL_JS_IS_CALLABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), UTIL_JS_TYPE_CALLABLE))
#define UTIL_JS_IS_CALLABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), UTIL_JS_TYPE_CALLABLE))
#define UTIL_JS_CALLABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), UTIL_JS_TYPE_CALLABLE, UtilJSCallableClass))

typedef struct _UtilJSCallable UtilJSCallable;
typedef struct _UtilJSCallableClass UtilJSCallableClass;
typedef void (*ComponentsWebViewMessageCallback) (GVariant* parameters, gpointer user_data);

#define TYPE_COUNT_BADGE (count_badge_get_type ())
#define COUNT_BADGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COUNT_BADGE, CountBadge))
#define COUNT_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COUNT_BADGE, CountBadgeClass))
#define IS_COUNT_BADGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COUNT_BADGE))
#define IS_COUNT_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COUNT_BADGE))
#define COUNT_BADGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COUNT_BADGE, CountBadgeClass))

typedef struct _CountBadge CountBadge;
typedef struct _CountBadgeClass CountBadgeClass;
typedef struct _CountBadgePrivate CountBadgePrivate;
typedef struct _FolderPopoverPrivate FolderPopoverPrivate;

#define TYPE_FOLDER_POPOVER_ROW (folder_popover_row_get_type ())
#define FOLDER_POPOVER_ROW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FOLDER_POPOVER_ROW, FolderPopoverRow))
#define FOLDER_POPOVER_ROW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FOLDER_POPOVER_ROW, FolderPopoverRowClass))
#define IS_FOLDER_POPOVER_ROW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FOLDER_POPOVER_ROW))
#define IS_FOLDER_POPOVER_ROW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FOLDER_POPOVER_ROW))
#define FOLDER_POPOVER_ROW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FOLDER_POPOVER_ROW, FolderPopoverRowClass))

typedef struct _FolderPopoverRow FolderPopoverRow;
typedef struct _FolderPopoverRowClass FolderPopoverRowClass;
typedef struct _FolderPopoverRowPrivate FolderPopoverRowPrivate;

#define TYPE_ICON_FACTORY (icon_factory_get_type ())
#define ICON_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ICON_FACTORY, IconFactory))
#define ICON_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ICON_FACTORY, IconFactoryClass))
#define IS_ICON_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ICON_FACTORY))
#define IS_ICON_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ICON_FACTORY))
#define ICON_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ICON_FACTORY, IconFactoryClass))

typedef struct _IconFactory IconFactory;
typedef struct _IconFactoryClass IconFactoryClass;
typedef struct _IconFactoryPrivate IconFactoryPrivate;

#define TYPE_MONITORED_PROGRESS_BAR (monitored_progress_bar_get_type ())
#define MONITORED_PROGRESS_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MONITORED_PROGRESS_BAR, MonitoredProgressBar))
#define MONITORED_PROGRESS_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MONITORED_PROGRESS_BAR, MonitoredProgressBarClass))
#define IS_MONITORED_PROGRESS_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MONITORED_PROGRESS_BAR))
#define IS_MONITORED_PROGRESS_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MONITORED_PROGRESS_BAR))
#define MONITORED_PROGRESS_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MONITORED_PROGRESS_BAR, MonitoredProgressBarClass))

typedef struct _MonitoredProgressBar MonitoredProgressBar;
typedef struct _MonitoredProgressBarClass MonitoredProgressBarClass;
typedef struct _MonitoredProgressBarPrivate MonitoredProgressBarPrivate;
typedef struct _MonitoredSpinnerPrivate MonitoredSpinnerPrivate;

#define COMPOSER_TYPE_CONTAINER (composer_container_get_type ())
#define COMPOSER_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_CONTAINER, ComposerContainer))
#define COMPOSER_IS_CONTAINER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_CONTAINER))
#define COMPOSER_CONTAINER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), COMPOSER_TYPE_CONTAINER, ComposerContainerIface))

typedef struct _ComposerContainer ComposerContainer;
typedef struct _ComposerContainerIface ComposerContainerIface;

#define COMPOSER_TYPE_WIDGET (composer_widget_get_type ())
#define COMPOSER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_WIDGET, ComposerWidget))
#define COMPOSER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_WIDGET, ComposerWidgetClass))
#define COMPOSER_IS_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_WIDGET))
#define COMPOSER_IS_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_WIDGET))
#define COMPOSER_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_WIDGET, ComposerWidgetClass))

typedef struct _ComposerWidget ComposerWidget;
typedef struct _ComposerWidgetClass ComposerWidgetClass;

#define COMPOSER_TYPE_BOX (composer_box_get_type ())
#define COMPOSER_BOX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_BOX, ComposerBox))
#define COMPOSER_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_BOX, ComposerBoxClass))
#define COMPOSER_IS_BOX(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_BOX))
#define COMPOSER_IS_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_BOX))
#define COMPOSER_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_BOX, ComposerBoxClass))

typedef struct _ComposerBox ComposerBox;
typedef struct _ComposerBoxClass ComposerBoxClass;
typedef struct _ComposerBoxPrivate ComposerBoxPrivate;

#define COMPOSER_TYPE_EDITOR (composer_editor_get_type ())
#define COMPOSER_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_EDITOR, ComposerEditor))
#define COMPOSER_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_EDITOR, ComposerEditorClass))
#define COMPOSER_IS_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_EDITOR))
#define COMPOSER_IS_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_EDITOR))
#define COMPOSER_EDITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_EDITOR, ComposerEditorClass))

typedef struct _ComposerEditor ComposerEditor;
typedef struct _ComposerEditorClass ComposerEditorClass;
typedef struct _ComposerEditorPrivate ComposerEditorPrivate;

#define COMPOSER_TYPE_WEB_VIEW (composer_web_view_get_type ())
#define COMPOSER_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_WEB_VIEW, ComposerWebView))
#define COMPOSER_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_WEB_VIEW, ComposerWebViewClass))
#define COMPOSER_IS_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_WEB_VIEW))
#define COMPOSER_IS_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_WEB_VIEW))
#define COMPOSER_WEB_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_WEB_VIEW, ComposerWebViewClass))

typedef struct _ComposerWebView ComposerWebView;
typedef struct _ComposerWebViewClass ComposerWebViewClass;

#define COMPOSER_TYPE_EMAIL_ENTRY (composer_email_entry_get_type ())
#define COMPOSER_EMAIL_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_EMAIL_ENTRY, ComposerEmailEntry))
#define COMPOSER_EMAIL_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_EMAIL_ENTRY, ComposerEmailEntryClass))
#define COMPOSER_IS_EMAIL_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_EMAIL_ENTRY))
#define COMPOSER_IS_EMAIL_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_EMAIL_ENTRY))
#define COMPOSER_EMAIL_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_EMAIL_ENTRY, ComposerEmailEntryClass))

typedef struct _ComposerEmailEntry ComposerEmailEntry;
typedef struct _ComposerEmailEntryClass ComposerEmailEntryClass;
typedef struct _ComposerEmailEntryPrivate ComposerEmailEntryPrivate;

#define COMPOSER_TYPE_EMBED (composer_embed_get_type ())
#define COMPOSER_EMBED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_EMBED, ComposerEmbed))
#define COMPOSER_EMBED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_EMBED, ComposerEmbedClass))
#define COMPOSER_IS_EMBED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_EMBED))
#define COMPOSER_IS_EMBED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_EMBED))
#define COMPOSER_EMBED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_EMBED, ComposerEmbedClass))

typedef struct _ComposerEmbed ComposerEmbed;
typedef struct _ComposerEmbedClass ComposerEmbedClass;
typedef struct _ComposerEmbedPrivate ComposerEmbedPrivate;

#define COMPOSER_TYPE_HEADERBAR (composer_headerbar_get_type ())
#define COMPOSER_HEADERBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_HEADERBAR, ComposerHeaderbar))
#define COMPOSER_HEADERBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_HEADERBAR, ComposerHeaderbarClass))
#define COMPOSER_IS_HEADERBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_HEADERBAR))
#define COMPOSER_IS_HEADERBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_HEADERBAR))
#define COMPOSER_HEADERBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_HEADERBAR, ComposerHeaderbarClass))

typedef struct _ComposerHeaderbar ComposerHeaderbar;
typedef struct _ComposerHeaderbarClass ComposerHeaderbarClass;
typedef struct _ComposerHeaderbarPrivate ComposerHeaderbarPrivate;

#define COMPOSER_TYPE_LINK_POPOVER (composer_link_popover_get_type ())
#define COMPOSER_LINK_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_LINK_POPOVER, ComposerLinkPopover))
#define COMPOSER_LINK_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_LINK_POPOVER, ComposerLinkPopoverClass))
#define COMPOSER_IS_LINK_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_LINK_POPOVER))
#define COMPOSER_IS_LINK_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_LINK_POPOVER))
#define COMPOSER_LINK_POPOVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_LINK_POPOVER, ComposerLinkPopoverClass))

typedef struct _ComposerLinkPopover ComposerLinkPopover;
typedef struct _ComposerLinkPopoverClass ComposerLinkPopoverClass;
typedef struct _ComposerLinkPopoverPrivate ComposerLinkPopoverPrivate;
typedef enum  {
	COMPOSER_LINK_POPOVER_TYPE_NEW_LINK,
	COMPOSER_LINK_POPOVER_TYPE_EXISTING_LINK
} ComposerLinkPopoverType;

#define COMPOSER_LINK_POPOVER_TYPE_TYPE (composer_link_popover_type_get_type ())
typedef struct _ComposerWebViewPrivate ComposerWebViewPrivate;

#define COMPOSER_WEB_VIEW_TYPE_EDIT_CONTEXT (composer_web_view_edit_context_get_type ())
#define COMPOSER_WEB_VIEW_EDIT_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_WEB_VIEW_TYPE_EDIT_CONTEXT, ComposerWebViewEditContext))
#define COMPOSER_WEB_VIEW_EDIT_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_WEB_VIEW_TYPE_EDIT_CONTEXT, ComposerWebViewEditContextClass))
#define COMPOSER_WEB_VIEW_IS_EDIT_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_WEB_VIEW_TYPE_EDIT_CONTEXT))
#define COMPOSER_WEB_VIEW_IS_EDIT_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_WEB_VIEW_TYPE_EDIT_CONTEXT))
#define COMPOSER_WEB_VIEW_EDIT_CONTEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_WEB_VIEW_TYPE_EDIT_CONTEXT, ComposerWebViewEditContextClass))

typedef struct _ComposerWebViewEditContext ComposerWebViewEditContext;
typedef struct _ComposerWebViewEditContextClass ComposerWebViewEditContextClass;
typedef struct _ComposerWebViewEditContextPrivate ComposerWebViewEditContextPrivate;
typedef struct _ComposerWidgetPrivate ComposerWidgetPrivate;
typedef enum  {
	COMPOSER_WIDGET_CONTEXT_TYPE_NONE,
	COMPOSER_WIDGET_CONTEXT_TYPE_EDIT,
	COMPOSER_WIDGET_CONTEXT_TYPE_REPLY_SENDER,
	COMPOSER_WIDGET_CONTEXT_TYPE_REPLY_ALL,
	COMPOSER_WIDGET_CONTEXT_TYPE_FORWARD
} ComposerWidgetContextType;

#define COMPOSER_WIDGET_TYPE_CONTEXT_TYPE (composer_widget_context_type_get_type ())
typedef enum  {
	COMPOSER_WIDGET_CLOSE_STATUS_CLOSED,
	COMPOSER_WIDGET_CLOSE_STATUS_READY,
	COMPOSER_WIDGET_CLOSE_STATUS_CANCELLED
} ComposerWidgetCloseStatus;

#define COMPOSER_WIDGET_TYPE_CLOSE_STATUS (composer_widget_close_status_get_type ())
typedef enum  {
	COMPOSER_WIDGET_PRESENTATION_MODE_CLOSED,
	COMPOSER_WIDGET_PRESENTATION_MODE_NONE,
	COMPOSER_WIDGET_PRESENTATION_MODE_DETACHED,
	COMPOSER_WIDGET_PRESENTATION_MODE_PANED,
	COMPOSER_WIDGET_PRESENTATION_MODE_INLINE,
	COMPOSER_WIDGET_PRESENTATION_MODE_INLINE_COMPACT
} ComposerWidgetPresentationMode;

#define COMPOSER_WIDGET_TYPE_PRESENTATION_MODE (composer_widget_presentation_mode_get_type ())

#define COMPOSER_TYPE_WINDOW (composer_window_get_type ())
#define COMPOSER_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPOSER_TYPE_WINDOW, ComposerWindow))
#define COMPOSER_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPOSER_TYPE_WINDOW, ComposerWindowClass))
#define COMPOSER_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPOSER_TYPE_WINDOW))
#define COMPOSER_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPOSER_TYPE_WINDOW))
#define COMPOSER_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPOSER_TYPE_WINDOW, ComposerWindowClass))

typedef struct _ComposerWindow ComposerWindow;
typedef struct _ComposerWindowClass ComposerWindowClass;
typedef struct _ComposerWindowPrivate ComposerWindowPrivate;

#define TYPE_CONTACT_ENTRY_COMPLETION (contact_entry_completion_get_type ())
#define CONTACT_ENTRY_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONTACT_ENTRY_COMPLETION, ContactEntryCompletion))
#define CONTACT_ENTRY_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONTACT_ENTRY_COMPLETION, ContactEntryCompletionClass))
#define IS_CONTACT_ENTRY_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONTACT_ENTRY_COMPLETION))
#define IS_CONTACT_ENTRY_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONTACT_ENTRY_COMPLETION))
#define CONTACT_ENTRY_COMPLETION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONTACT_ENTRY_COMPLETION, ContactEntryCompletionClass))

typedef struct _ContactEntryCompletion ContactEntryCompletion;
typedef struct _ContactEntryCompletionClass ContactEntryCompletionClass;
typedef struct _ContactEntryCompletionPrivate ContactEntryCompletionPrivate;
typedef enum  {
	CONTACT_ENTRY_COMPLETION_COLUMN_CONTACT,
	CONTACT_ENTRY_COMPLETION_COLUMN_MAILBOX
} ContactEntryCompletionColumn;

#define CONTACT_ENTRY_COMPLETION_TYPE_COLUMN (contact_entry_completion_column_get_type ())

#define TYPE_SPELL_CHECK_POPOVER (spell_check_popover_get_type ())
#define SPELL_CHECK_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SPELL_CHECK_POPOVER, SpellCheckPopover))
#define SPELL_CHECK_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SPELL_CHECK_POPOVER, SpellCheckPopoverClass))
#define IS_SPELL_CHECK_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SPELL_CHECK_POPOVER))
#define IS_SPELL_CHECK_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SPELL_CHECK_POPOVER))
#define SPELL_CHECK_POPOVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SPELL_CHECK_POPOVER, SpellCheckPopoverClass))

typedef struct _SpellCheckPopover SpellCheckPopover;
typedef struct _SpellCheckPopoverClass SpellCheckPopoverClass;
typedef struct _SpellCheckPopoverPrivate SpellCheckPopoverPrivate;

#define CONVERSATION_LIST_TYPE_MODEL (conversation_list_model_get_type ())
#define CONVERSATION_LIST_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONVERSATION_LIST_TYPE_MODEL, ConversationListModel))
#define CONVERSATION_LIST_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONVERSATION_LIST_TYPE_MODEL, ConversationListModelClass))
#define CONVERSATION_LIST_IS_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONVERSATION_LIST_TYPE_MODEL))
#define CONVERSATION_LIST_IS_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONVERSATION_LIST_TYPE_MODEL))
#define CONVERSATION_LIST_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONVERSATION_LIST_TYPE_MODEL, ConversationListModelClass))

typedef struct _ConversationListModel ConversationListModel;
typedef struct _ConversationListModelClass ConversationListModelClass;
typedef struct _ConversationListModelPrivate ConversationListModelPrivate;
typedef struct _ConversationListViewPrivate ConversationListViewPrivate;

#define CONVERSATION_TYPE_CONTACT_POPOVER (conversation_contact_popover_get_type ())
#define CONVERSATION_CONTACT_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONVERSATION_TYPE_CONTACT_POPOVER, ConversationContactPopover))
#define CONVERSATION_CONTACT_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONVERSATION_TYPE_CONTACT_POPOVER, ConversationContactPopoverClass))
#define CONVERSATION_IS_CONTACT_POPOVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONVERSATION_TYPE_CONTACT_POPOVER))
#define CONVERSATION_IS_CONTACT_POPOVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONVERSATION_TYPE_CONTACT_POPOVER))
#define CONVERSATION_CONTACT_POPOVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONVERSATION_TYPE_CONTACT_POPOVER, ConversationContactPopoverClass))

typedef struct _ConversationContactPopover ConversationContactPopover;
typedef struct _ConversationContactPopoverClass ConversationContactPopoverClass;
typedef struct _ConversationContactPopoverPrivate ConversationContactPopoverPrivate;

#define TYPE_CONVERSATION_EMAIL (conversation_email_get_type ())
#define CONVERSATION_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONVERSATION_EMAIL, ConversationEmail))
#define CONVERSATION_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONVERSATION_EMAIL, ConversationEmailClass))
#define IS_CONVERSATION_EMAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONVERSATION_EMAIL))
#define IS_CONVERSATION_EMAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONVERSATION_EMAIL))
#define CONVERSATION_EMAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONVERSATION_EMAIL, ConversationEmailClass))

typedef struct _ConversationEmail ConversationEmail;
typedef struct _ConversationEmailClass ConversationEmailClass;
typedef struct _ConversationEmailPrivate ConversationEmailPrivate;
typedef enum  {
	CONVERSATION_EMAIL_LOAD_STATE_NOT_STARTED,
	CONVERSATION_EMAIL_LOAD_STATE_STARTED,
	CONVERSATION_EMAIL_LOAD_STATE_COMPLETED,
	CONVERSATION_EMAIL_LOAD_STATE_FAILED
} ConversationEmailLoadState;

#define CONVERSATION_EMAIL_TYPE_LOAD_STATE (conversation_email_load_state_get_type ())

#define TYPE_CONVERSATION_MESSAGE (conversation_message_get_type ())
#define CONVERSATION_MESSAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONVERSATION_MESSAGE, ConversationMessage))
#define CONVERSATION_MESSAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONVERSATION_MESSAGE, ConversationMessageClass))
#define IS_CONVERSATION_MESSAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONVERSATION_MESSAGE))
#define IS_CONVERSATION_MESSAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONVERSATION_MESSAGE))
#define CONVERSATION_MESSAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONVERSATION_MESSAGE, ConversationMessageClass))

typedef struct _ConversationMessage ConversationMessage;
typedef struct _ConversationMessageClass ConversationMessageClass;

#define TYPE_CONVERSATION_LIST_BOX (conversation_list_box_get_type ())
#define CONVERSATION_LIST_BOX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONVERSATION_LIST_BOX, ConversationListBox))
#define CONVERSATION_LIST_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONVERSATION_LIST_BOX, ConversationListBoxClass))
#define IS_CONVERSATION_LIST_BOX(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONVERSATION_LIST_BOX))
#define IS_CONVERSATION_LIST_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONVERSATION_LIST_BOX))
#define CONVERSATION_LIST_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONVERSATION_LIST_BOX, ConversationListBoxClass))

typedef struct _ConversationListBox ConversationListBox;
typedef struct _ConversationListBoxClass ConversationListBoxClass;
typedef struct _ConversationListBoxPrivate ConversationListBoxPrivate;

#define CONVERSATION_LIST_BOX_TYPE_SEARCH_MANAGER (conversation_list_box_search_manager_get_type ())
#define CONVERSATION_LIST_BOX_SEARCH_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONVERSATION_LIST_BOX_TYPE_SEARCH_MANAGER, ConversationListBoxSearchManager))
#define CONVERSATION_LIST_BOX_SEARCH_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONVERSATION_LIST_BOX_TYPE_SEARCH_MANAGER, ConversationListBoxSearchManagerClass))
#define CONVERSATION_LIST_BOX_IS_SEARCH_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONVERSATION_LIST_BOX_TYPE_SEARCH_MANAGER))
#define CONVERSATION_LIST_BOX_IS_SEARCH_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONVERSATION_LIST_BOX_TYPE_SEARCH_MANAGER))
#define CONVERSATION_LIST_BOX_SEARCH_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONVERSATION_LIST_BOX_TYPE_SEARCH_MANAGER, ConversationListBoxSearchManagerClass))

typedef struct _ConversationListBoxSearchManager ConversationListBoxSearchManager;
typedef struct _ConversationListBoxSearchManagerClass ConversationListBoxSearchManagerClass;
typedef struct _ConversationListBoxSearchManagerPrivate ConversationListBoxSearchManagerPrivate;
typedef struct _ConversationMessagePrivate ConversationMessagePrivate;

#define CONVERSATION_MESSAGE_TYPE_CONTACT_LIST (conversation_message_contact_list_get_type ())
#define CONVERSATION_MESSAGE_CONTACT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CONVERSATION_MESSAGE_TYPE_CONTACT_LIST, ConversationMessageContactList))
#define CONVERSATION_MESSAGE_CONTACT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CONVERSATION_MESSAGE_TYPE_CONTACT_LIST, ConversationMessageContactListClass))
#define CONVERSATION_MESSAGE_IS_CONTACT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CONVERSATION_MESSAGE_TYPE_CONTACT_LIST))
#define CONVERSATION_MESSAGE_IS_CONTACT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CONVERSATION_MESSAGE_TYPE_CONTACT_LIST))
#define CONVERSATION_MESSAGE_CONTACT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CONVERSATION_MESSAGE_TYPE_CONTACT_LIST, ConversationMessageContactListClass))

typedef struct _ConversationMessageContactList ConversationMessageContactList;
typedef struct _ConversationMessageContactListClass ConversationMessageContactListClass;
typedef struct _ConversationMessageContactListPrivate ConversationMessageContactListPrivate;
typedef struct _ConversationViewerPrivate ConversationViewerPrivate;

#define TYPE_CONVERSATION_WEB_VIEW (conversation_web_view_get_type ())
#define CONVERSATION_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONVERSATION_WEB_VIEW, ConversationWebView))
#define CONVERSATION_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONVERSATION_WEB_VIEW, ConversationWebViewClass))
#define IS_CONVERSATION_WEB_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONVERSATION_WEB_VIEW))
#define IS_CONVERSATION_WEB_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONVERSATION_WEB_VIEW))
#define CONVERSATION_WEB_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONVERSATION_WEB_VIEW, ConversationWebViewClass))

typedef struct _ConversationWebView ConversationWebView;
typedef struct _ConversationWebViewClass ConversationWebViewClass;
typedef struct _ConversationWebViewPrivate ConversationWebViewPrivate;
typedef enum  {
	CONVERSATION_WEB_VIEW_DECEPTIVE_TEXT_NOT_DECEPTIVE = 0,
	CONVERSATION_WEB_VIEW_DECEPTIVE_TEXT_DECEPTIVE_HREF = 1,
	CONVERSATION_WEB_VIEW_DECEPTIVE_TEXT_DECEPTIVE_DOMAIN = 2
} ConversationWebViewDeceptiveText;

#define CONVERSATION_WEB_VIEW_TYPE_DECEPTIVE_TEXT (conversation_web_view_deceptive_text_get_type ())

#define TYPE_ATTACHMENT_DIALOG (attachment_dialog_get_type ())
#define ATTACHMENT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ATTACHMENT_DIALOG, AttachmentDialog))
#define ATTACHMENT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ATTACHMENT_DIALOG, AttachmentDialogClass))
#define IS_ATTACHMENT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ATTACHMENT_DIALOG))
#define IS_ATTACHMENT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ATTACHMENT_DIALOG))
#define ATTACHMENT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ATTACHMENT_DIALOG, AttachmentDialogClass))

typedef struct _AttachmentDialog AttachmentDialog;
typedef struct _AttachmentDialogClass AttachmentDialogClass;
typedef struct _AttachmentDialogPrivate AttachmentDialogPrivate;
typedef gboolean (*AttachmentDialogAttacher) (GFile* attachment_file, gboolean alert_errors, gpointer user_data);

#define TYPE_CERTIFICATE_WARNING_DIALOG (certificate_warning_dialog_get_type ())
#define CERTIFICATE_WARNING_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CERTIFICATE_WARNING_DIALOG, CertificateWarningDialog))
#define CERTIFICATE_WARNING_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CERTIFICATE_WARNING_DIALOG, CertificateWarningDialogClass))
#define IS_CERTIFICATE_WARNING_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CERTIFICATE_WARNING_DIALOG))
#define IS_CERTIFICATE_WARNING_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CERTIFICATE_WARNING_DIALOG))
#define CERTIFICATE_WARNING_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CERTIFICATE_WARNING_DIALOG, CertificateWarningDialogClass))

typedef struct _CertificateWarningDialog CertificateWarningDialog;
typedef struct _CertificateWarningDialogClass CertificateWarningDialogClass;
typedef struct _CertificateWarningDialogPrivate CertificateWarningDialogPrivate;
typedef enum  {
	CERTIFICATE_WARNING_DIALOG_RESULT_DONT_TRUST,
	CERTIFICATE_WARNING_DIALOG_RESULT_TRUST,
	CERTIFICATE_WARNING_DIALOG_RESULT_ALWAYS_TRUST
} CertificateWarningDialogResult;

#define CERTIFICATE_WARNING_DIALOG_TYPE_RESULT (certificate_warning_dialog_result_get_type ())

#define DIALOGS_TYPE_PROBLEM_DETAILS_DIALOG (dialogs_problem_details_dialog_get_type ())
#define DIALOGS_PROBLEM_DETAILS_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DIALOGS_TYPE_PROBLEM_DETAILS_DIALOG, DialogsProblemDetailsDialog))
#define DIALOGS_PROBLEM_DETAILS_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DIALOGS_TYPE_PROBLEM_DETAILS_DIALOG, DialogsProblemDetailsDialogClass))
#define DIALOGS_IS_PROBLEM_DETAILS_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DIALOGS_TYPE_PROBLEM_DETAILS_DIALOG))
#define DIALOGS_IS_PROBLEM_DETAILS_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DIALOGS_TYPE_PROBLEM_DETAILS_DIALOG))
#define DIALOGS_PROBLEM_DETAILS_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DIALOGS_TYPE_PROBLEM_DETAILS_DIALOG, DialogsProblemDetailsDialogClass))

typedef struct _DialogsProblemDetailsDialog DialogsProblemDetailsDialog;
typedef struct _DialogsProblemDetailsDialogClass DialogsProblemDetailsDialogClass;
typedef struct _DialogsProblemDetailsDialogPrivate DialogsProblemDetailsDialogPrivate;

#define TYPE_PASSWORD_DIALOG (password_dialog_get_type ())
#define PASSWORD_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PASSWORD_DIALOG, PasswordDialog))
#define PASSWORD_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PASSWORD_DIALOG, PasswordDialogClass))
#define IS_PASSWORD_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PASSWORD_DIALOG))
#define IS_PASSWORD_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PASSWORD_DIALOG))
#define PASSWORD_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PASSWORD_DIALOG, PasswordDialogClass))

typedef struct _PasswordDialog PasswordDialog;
typedef struct _PasswordDialogClass PasswordDialogClass;
typedef struct _PasswordDialogPrivate PasswordDialogPrivate;

#define SIDEBAR_TYPE_ENTRY (sidebar_entry_get_type ())
#define SIDEBAR_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_ENTRY, SidebarEntry))
#define SIDEBAR_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_ENTRY))
#define SIDEBAR_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_ENTRY, SidebarEntryIface))

typedef struct _SidebarEntry SidebarEntry;
typedef struct _SidebarEntryIface SidebarEntryIface;

#define SIDEBAR_TYPE_SELECTABLE_ENTRY (sidebar_selectable_entry_get_type ())
#define SIDEBAR_SELECTABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_SELECTABLE_ENTRY, SidebarSelectableEntry))
#define SIDEBAR_IS_SELECTABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_SELECTABLE_ENTRY))
#define SIDEBAR_SELECTABLE_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_SELECTABLE_ENTRY, SidebarSelectableEntryIface))

typedef struct _SidebarSelectableEntry SidebarSelectableEntry;
typedef struct _SidebarSelectableEntryIface SidebarSelectableEntryIface;

#define FOLDER_LIST_TYPE_ABSTRACT_FOLDER_ENTRY (folder_list_abstract_folder_entry_get_type ())
#define FOLDER_LIST_ABSTRACT_FOLDER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_ABSTRACT_FOLDER_ENTRY, FolderListAbstractFolderEntry))
#define FOLDER_LIST_ABSTRACT_FOLDER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_ABSTRACT_FOLDER_ENTRY, FolderListAbstractFolderEntryClass))
#define FOLDER_LIST_IS_ABSTRACT_FOLDER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_ABSTRACT_FOLDER_ENTRY))
#define FOLDER_LIST_IS_ABSTRACT_FOLDER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_ABSTRACT_FOLDER_ENTRY))
#define FOLDER_LIST_ABSTRACT_FOLDER_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_ABSTRACT_FOLDER_ENTRY, FolderListAbstractFolderEntryClass))

typedef struct _FolderListAbstractFolderEntry FolderListAbstractFolderEntry;
typedef struct _FolderListAbstractFolderEntryClass FolderListAbstractFolderEntryClass;
typedef struct _FolderListAbstractFolderEntryPrivate FolderListAbstractFolderEntryPrivate;

#define SIDEBAR_TYPE_BRANCH (sidebar_branch_get_type ())
#define SIDEBAR_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_BRANCH, SidebarBranch))
#define SIDEBAR_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SIDEBAR_TYPE_BRANCH, SidebarBranchClass))
#define SIDEBAR_IS_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_BRANCH))
#define SIDEBAR_IS_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SIDEBAR_TYPE_BRANCH))
#define SIDEBAR_BRANCH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SIDEBAR_TYPE_BRANCH, SidebarBranchClass))

typedef struct _SidebarBranch SidebarBranch;
typedef struct _SidebarBranchClass SidebarBranchClass;
typedef struct _SidebarBranchPrivate SidebarBranchPrivate;

#define FOLDER_LIST_TYPE_ACCOUNT_BRANCH (folder_list_account_branch_get_type ())
#define FOLDER_LIST_ACCOUNT_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_ACCOUNT_BRANCH, FolderListAccountBranch))
#define FOLDER_LIST_ACCOUNT_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_ACCOUNT_BRANCH, FolderListAccountBranchClass))
#define FOLDER_LIST_IS_ACCOUNT_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_ACCOUNT_BRANCH))
#define FOLDER_LIST_IS_ACCOUNT_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_ACCOUNT_BRANCH))
#define FOLDER_LIST_ACCOUNT_BRANCH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_ACCOUNT_BRANCH, FolderListAccountBranchClass))

typedef struct _FolderListAccountBranch FolderListAccountBranch;
typedef struct _FolderListAccountBranchClass FolderListAccountBranchClass;
typedef struct _FolderListAccountBranchPrivate FolderListAccountBranchPrivate;

#define SIDEBAR_TYPE_GROUPING (sidebar_grouping_get_type ())
#define SIDEBAR_GROUPING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_GROUPING, SidebarGrouping))
#define SIDEBAR_GROUPING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SIDEBAR_TYPE_GROUPING, SidebarGroupingClass))
#define SIDEBAR_IS_GROUPING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_GROUPING))
#define SIDEBAR_IS_GROUPING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SIDEBAR_TYPE_GROUPING))
#define SIDEBAR_GROUPING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SIDEBAR_TYPE_GROUPING, SidebarGroupingClass))

typedef struct _SidebarGrouping SidebarGrouping;
typedef struct _SidebarGroupingClass SidebarGroupingClass;

#define FOLDER_LIST_TYPE_SPECIAL_GROUPING (folder_list_special_grouping_get_type ())
#define FOLDER_LIST_SPECIAL_GROUPING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_SPECIAL_GROUPING, FolderListSpecialGrouping))
#define FOLDER_LIST_SPECIAL_GROUPING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_SPECIAL_GROUPING, FolderListSpecialGroupingClass))
#define FOLDER_LIST_IS_SPECIAL_GROUPING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_SPECIAL_GROUPING))
#define FOLDER_LIST_IS_SPECIAL_GROUPING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_SPECIAL_GROUPING))
#define FOLDER_LIST_SPECIAL_GROUPING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_SPECIAL_GROUPING, FolderListSpecialGroupingClass))

typedef struct _FolderListSpecialGrouping FolderListSpecialGrouping;
typedef struct _FolderListSpecialGroupingClass FolderListSpecialGroupingClass;

#define FOLDER_LIST_TYPE_FOLDER_ENTRY (folder_list_folder_entry_get_type ())
#define FOLDER_LIST_FOLDER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_FOLDER_ENTRY, FolderListFolderEntry))
#define FOLDER_LIST_FOLDER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_FOLDER_ENTRY, FolderListFolderEntryClass))
#define FOLDER_LIST_IS_FOLDER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_FOLDER_ENTRY))
#define FOLDER_LIST_IS_FOLDER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_FOLDER_ENTRY))
#define FOLDER_LIST_FOLDER_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_FOLDER_ENTRY, FolderListFolderEntryClass))

typedef struct _FolderListFolderEntry FolderListFolderEntry;
typedef struct _FolderListFolderEntryClass FolderListFolderEntryClass;

#define SIDEBAR_TYPE_INTERNAL_DROP_TARGET_ENTRY (sidebar_internal_drop_target_entry_get_type ())
#define SIDEBAR_INTERNAL_DROP_TARGET_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_INTERNAL_DROP_TARGET_ENTRY, SidebarInternalDropTargetEntry))
#define SIDEBAR_IS_INTERNAL_DROP_TARGET_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_INTERNAL_DROP_TARGET_ENTRY))
#define SIDEBAR_INTERNAL_DROP_TARGET_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_INTERNAL_DROP_TARGET_ENTRY, SidebarInternalDropTargetEntryIface))

typedef struct _SidebarInternalDropTargetEntry SidebarInternalDropTargetEntry;
typedef struct _SidebarInternalDropTargetEntryIface SidebarInternalDropTargetEntryIface;

#define SIDEBAR_TYPE_EMPHASIZABLE_ENTRY (sidebar_emphasizable_entry_get_type ())
#define SIDEBAR_EMPHASIZABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_EMPHASIZABLE_ENTRY, SidebarEmphasizableEntry))
#define SIDEBAR_IS_EMPHASIZABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_EMPHASIZABLE_ENTRY))
#define SIDEBAR_EMPHASIZABLE_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_EMPHASIZABLE_ENTRY, SidebarEmphasizableEntryIface))

typedef struct _SidebarEmphasizableEntry SidebarEmphasizableEntry;
typedef struct _SidebarEmphasizableEntryIface SidebarEmphasizableEntryIface;
typedef struct _FolderListFolderEntryPrivate FolderListFolderEntryPrivate;
typedef struct _SidebarTreePrivate SidebarTreePrivate;
typedef struct _FolderListTreePrivate FolderListTreePrivate;

#define FOLDER_LIST_TYPE_INBOXES_BRANCH (folder_list_inboxes_branch_get_type ())
#define FOLDER_LIST_INBOXES_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_INBOXES_BRANCH, FolderListInboxesBranch))
#define FOLDER_LIST_INBOXES_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_INBOXES_BRANCH, FolderListInboxesBranchClass))
#define FOLDER_LIST_IS_INBOXES_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_INBOXES_BRANCH))
#define FOLDER_LIST_IS_INBOXES_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_INBOXES_BRANCH))
#define FOLDER_LIST_INBOXES_BRANCH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_INBOXES_BRANCH, FolderListInboxesBranchClass))

typedef struct _FolderListInboxesBranch FolderListInboxesBranch;
typedef struct _FolderListInboxesBranchClass FolderListInboxesBranchClass;
typedef struct _FolderListInboxesBranchPrivate FolderListInboxesBranchPrivate;

#define FOLDER_LIST_TYPE_INBOX_FOLDER_ENTRY (folder_list_inbox_folder_entry_get_type ())
#define FOLDER_LIST_INBOX_FOLDER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_INBOX_FOLDER_ENTRY, FolderListInboxFolderEntry))
#define FOLDER_LIST_INBOX_FOLDER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_INBOX_FOLDER_ENTRY, FolderListInboxFolderEntryClass))
#define FOLDER_LIST_IS_INBOX_FOLDER_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_INBOX_FOLDER_ENTRY))
#define FOLDER_LIST_IS_INBOX_FOLDER_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_INBOX_FOLDER_ENTRY))
#define FOLDER_LIST_INBOX_FOLDER_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_INBOX_FOLDER_ENTRY, FolderListInboxFolderEntryClass))

typedef struct _FolderListInboxFolderEntry FolderListInboxFolderEntry;
typedef struct _FolderListInboxFolderEntryClass FolderListInboxFolderEntryClass;
typedef struct _FolderListInboxFolderEntryPrivate FolderListInboxFolderEntryPrivate;

#define SIDEBAR_TYPE_ROOT_ONLY_BRANCH (sidebar_root_only_branch_get_type ())
#define SIDEBAR_ROOT_ONLY_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_ROOT_ONLY_BRANCH, SidebarRootOnlyBranch))
#define SIDEBAR_ROOT_ONLY_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SIDEBAR_TYPE_ROOT_ONLY_BRANCH, SidebarRootOnlyBranchClass))
#define SIDEBAR_IS_ROOT_ONLY_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_ROOT_ONLY_BRANCH))
#define SIDEBAR_IS_ROOT_ONLY_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SIDEBAR_TYPE_ROOT_ONLY_BRANCH))
#define SIDEBAR_ROOT_ONLY_BRANCH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SIDEBAR_TYPE_ROOT_ONLY_BRANCH, SidebarRootOnlyBranchClass))

typedef struct _SidebarRootOnlyBranch SidebarRootOnlyBranch;
typedef struct _SidebarRootOnlyBranchClass SidebarRootOnlyBranchClass;
typedef struct _SidebarRootOnlyBranchPrivate SidebarRootOnlyBranchPrivate;

#define FOLDER_LIST_TYPE_SEARCH_BRANCH (folder_list_search_branch_get_type ())
#define FOLDER_LIST_SEARCH_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_SEARCH_BRANCH, FolderListSearchBranch))
#define FOLDER_LIST_SEARCH_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_SEARCH_BRANCH, FolderListSearchBranchClass))
#define FOLDER_LIST_IS_SEARCH_BRANCH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_SEARCH_BRANCH))
#define FOLDER_LIST_IS_SEARCH_BRANCH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_SEARCH_BRANCH))
#define FOLDER_LIST_SEARCH_BRANCH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_SEARCH_BRANCH, FolderListSearchBranchClass))

typedef struct _FolderListSearchBranch FolderListSearchBranch;
typedef struct _FolderListSearchBranchClass FolderListSearchBranchClass;
typedef struct _FolderListSearchBranchPrivate FolderListSearchBranchPrivate;

#define FOLDER_LIST_TYPE_SEARCH_ENTRY (folder_list_search_entry_get_type ())
#define FOLDER_LIST_SEARCH_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLDER_LIST_TYPE_SEARCH_ENTRY, FolderListSearchEntry))
#define FOLDER_LIST_SEARCH_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLDER_LIST_TYPE_SEARCH_ENTRY, FolderListSearchEntryClass))
#define FOLDER_LIST_IS_SEARCH_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLDER_LIST_TYPE_SEARCH_ENTRY))
#define FOLDER_LIST_IS_SEARCH_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLDER_LIST_TYPE_SEARCH_ENTRY))
#define FOLDER_LIST_SEARCH_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLDER_LIST_TYPE_SEARCH_ENTRY, FolderListSearchEntryClass))

typedef struct _FolderListSearchEntry FolderListSearchEntry;
typedef struct _FolderListSearchEntryClass FolderListSearchEntryClass;
typedef struct _FolderListSearchEntryPrivate FolderListSearchEntryPrivate;

#define SIDEBAR_TYPE_EXPANDABLE_ENTRY (sidebar_expandable_entry_get_type ())
#define SIDEBAR_EXPANDABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_EXPANDABLE_ENTRY, SidebarExpandableEntry))
#define SIDEBAR_IS_EXPANDABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_EXPANDABLE_ENTRY))
#define SIDEBAR_EXPANDABLE_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_EXPANDABLE_ENTRY, SidebarExpandableEntryIface))

typedef struct _SidebarExpandableEntry SidebarExpandableEntry;
typedef struct _SidebarExpandableEntryIface SidebarExpandableEntryIface;

#define SIDEBAR_TYPE_RENAMEABLE_ENTRY (sidebar_renameable_entry_get_type ())
#define SIDEBAR_RENAMEABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_RENAMEABLE_ENTRY, SidebarRenameableEntry))
#define SIDEBAR_IS_RENAMEABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_RENAMEABLE_ENTRY))
#define SIDEBAR_RENAMEABLE_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_RENAMEABLE_ENTRY, SidebarRenameableEntryIface))

typedef struct _SidebarRenameableEntry SidebarRenameableEntry;
typedef struct _SidebarRenameableEntryIface SidebarRenameableEntryIface;
typedef struct _SidebarGroupingPrivate SidebarGroupingPrivate;
typedef struct _FolderListSpecialGroupingPrivate FolderListSpecialGroupingPrivate;

#define PLUGIN_TYPE_ACTION_BAR (plugin_action_bar_get_type ())
#define PLUGIN_ACTION_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_ACTION_BAR, PluginActionBar))
#define PLUGIN_ACTION_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_ACTION_BAR, PluginActionBarClass))
#define PLUGIN_IS_ACTION_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_ACTION_BAR))
#define PLUGIN_IS_ACTION_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_ACTION_BAR))
#define PLUGIN_ACTION_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_ACTION_BAR, PluginActionBarClass))

typedef struct _PluginActionBar PluginActionBar;
typedef struct _PluginActionBarClass PluginActionBarClass;
typedef struct _PluginActionBarPrivate PluginActionBarPrivate;
typedef enum  {
	PLUGIN_ACTION_BAR_POSITION_START,
	PLUGIN_ACTION_BAR_POSITION_CENTRE,
	PLUGIN_ACTION_BAR_POSITION_END
} PluginActionBarPosition;

#define PLUGIN_ACTION_BAR_TYPE_POSITION (plugin_action_bar_position_get_type ())

#define PLUGIN_ACTION_BAR_TYPE_ITEM (plugin_action_bar_item_get_type ())
#define PLUGIN_ACTION_BAR_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_ACTION_BAR_TYPE_ITEM, PluginActionBarItem))
#define PLUGIN_ACTION_BAR_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_ACTION_BAR_TYPE_ITEM))
#define PLUGIN_ACTION_BAR_ITEM_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_ACTION_BAR_TYPE_ITEM, PluginActionBarItemIface))

typedef struct _PluginActionBarItem PluginActionBarItem;
typedef struct _PluginActionBarItemIface PluginActionBarItemIface;

#define PLUGIN_ACTION_BAR_TYPE_LABEL_ITEM (plugin_action_bar_label_item_get_type ())
#define PLUGIN_ACTION_BAR_LABEL_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_ACTION_BAR_TYPE_LABEL_ITEM, PluginActionBarLabelItem))
#define PLUGIN_ACTION_BAR_LABEL_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_ACTION_BAR_TYPE_LABEL_ITEM, PluginActionBarLabelItemClass))
#define PLUGIN_ACTION_BAR_IS_LABEL_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_ACTION_BAR_TYPE_LABEL_ITEM))
#define PLUGIN_ACTION_BAR_IS_LABEL_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_ACTION_BAR_TYPE_LABEL_ITEM))
#define PLUGIN_ACTION_BAR_LABEL_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_ACTION_BAR_TYPE_LABEL_ITEM, PluginActionBarLabelItemClass))

typedef struct _PluginActionBarLabelItem PluginActionBarLabelItem;
typedef struct _PluginActionBarLabelItemClass PluginActionBarLabelItemClass;
typedef struct _PluginActionBarLabelItemPrivate PluginActionBarLabelItemPrivate;

#define PLUGIN_ACTION_BAR_TYPE_BUTTON_ITEM (plugin_action_bar_button_item_get_type ())
#define PLUGIN_ACTION_BAR_BUTTON_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_ACTION_BAR_TYPE_BUTTON_ITEM, PluginActionBarButtonItem))
#define PLUGIN_ACTION_BAR_BUTTON_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_ACTION_BAR_TYPE_BUTTON_ITEM, PluginActionBarButtonItemClass))
#define PLUGIN_ACTION_BAR_IS_BUTTON_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_ACTION_BAR_TYPE_BUTTON_ITEM))
#define PLUGIN_ACTION_BAR_IS_BUTTON_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_ACTION_BAR_TYPE_BUTTON_ITEM))
#define PLUGIN_ACTION_BAR_BUTTON_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_ACTION_BAR_TYPE_BUTTON_ITEM, PluginActionBarButtonItemClass))

typedef struct _PluginActionBarButtonItem PluginActionBarButtonItem;
typedef struct _PluginActionBarButtonItemClass PluginActionBarButtonItemClass;
typedef struct _PluginActionBarButtonItemPrivate PluginActionBarButtonItemPrivate;

#define PLUGIN_TYPE_ACTIONABLE (plugin_actionable_get_type ())
#define PLUGIN_ACTIONABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_ACTIONABLE, PluginActionable))
#define PLUGIN_ACTIONABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_ACTIONABLE, PluginActionableClass))
#define PLUGIN_IS_ACTIONABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_ACTIONABLE))
#define PLUGIN_IS_ACTIONABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_ACTIONABLE))
#define PLUGIN_ACTIONABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_ACTIONABLE, PluginActionableClass))

typedef struct _PluginActionable PluginActionable;
typedef struct _PluginActionableClass PluginActionableClass;

#define PLUGIN_ACTION_BAR_TYPE_MENU_ITEM (plugin_action_bar_menu_item_get_type ())
#define PLUGIN_ACTION_BAR_MENU_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_ACTION_BAR_TYPE_MENU_ITEM, PluginActionBarMenuItem))
#define PLUGIN_ACTION_BAR_MENU_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_ACTION_BAR_TYPE_MENU_ITEM, PluginActionBarMenuItemClass))
#define PLUGIN_ACTION_BAR_IS_MENU_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_ACTION_BAR_TYPE_MENU_ITEM))
#define PLUGIN_ACTION_BAR_IS_MENU_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_ACTION_BAR_TYPE_MENU_ITEM))
#define PLUGIN_ACTION_BAR_MENU_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_ACTION_BAR_TYPE_MENU_ITEM, PluginActionBarMenuItemClass))

typedef struct _PluginActionBarMenuItem PluginActionBarMenuItem;
typedef struct _PluginActionBarMenuItemClass PluginActionBarMenuItemClass;
typedef struct _PluginActionBarMenuItemPrivate PluginActionBarMenuItemPrivate;

#define PLUGIN_ACTION_BAR_TYPE_GROUP_ITEM (plugin_action_bar_group_item_get_type ())
#define PLUGIN_ACTION_BAR_GROUP_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_ACTION_BAR_TYPE_GROUP_ITEM, PluginActionBarGroupItem))
#define PLUGIN_ACTION_BAR_GROUP_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_ACTION_BAR_TYPE_GROUP_ITEM, PluginActionBarGroupItemClass))
#define PLUGIN_ACTION_BAR_IS_GROUP_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_ACTION_BAR_TYPE_GROUP_ITEM))
#define PLUGIN_ACTION_BAR_IS_GROUP_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_ACTION_BAR_TYPE_GROUP_ITEM))
#define PLUGIN_ACTION_BAR_GROUP_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_ACTION_BAR_TYPE_GROUP_ITEM, PluginActionBarGroupItemClass))

typedef struct _PluginActionBarGroupItem PluginActionBarGroupItem;
typedef struct _PluginActionBarGroupItemClass PluginActionBarGroupItemClass;
typedef struct _PluginActionBarGroupItemPrivate PluginActionBarGroupItemPrivate;
typedef struct _PluginActionablePrivate PluginActionablePrivate;

#define PLUGIN_TYPE_APPLICATION (plugin_application_get_type ())
#define PLUGIN_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_APPLICATION, PluginApplication))
#define PLUGIN_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_APPLICATION))
#define PLUGIN_APPLICATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_APPLICATION, PluginApplicationIface))

typedef struct _PluginApplication PluginApplication;
typedef struct _PluginApplicationIface PluginApplicationIface;

#define PLUGIN_TYPE_COMPOSER (plugin_composer_get_type ())
#define PLUGIN_COMPOSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_COMPOSER, PluginComposer))
#define PLUGIN_IS_COMPOSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_COMPOSER))
#define PLUGIN_COMPOSER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_COMPOSER, PluginComposerIface))

typedef struct _PluginComposer PluginComposer;
typedef struct _PluginComposerIface PluginComposerIface;

#define PLUGIN_TYPE_ERROR (plugin_error_get_type ())
typedef enum  {
	PLUGIN_COMPOSER_CONTEXT_TYPE_NONE,
	PLUGIN_COMPOSER_CONTEXT_TYPE_EDIT,
	PLUGIN_COMPOSER_CONTEXT_TYPE_REPLY_SENDER,
	PLUGIN_COMPOSER_CONTEXT_TYPE_REPLY_ALL,
	PLUGIN_COMPOSER_CONTEXT_TYPE_FORWARD
} PluginComposerContextType;

#define PLUGIN_COMPOSER_TYPE_CONTEXT_TYPE (plugin_composer_context_type_get_type ())

#define PLUGIN_TYPE_CONTACT_STORE (plugin_contact_store_get_type ())
#define PLUGIN_CONTACT_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_CONTACT_STORE, PluginContactStore))
#define PLUGIN_IS_CONTACT_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_CONTACT_STORE))
#define PLUGIN_CONTACT_STORE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_CONTACT_STORE, PluginContactStoreIface))

typedef struct _PluginContactStore PluginContactStore;
typedef struct _PluginContactStoreIface PluginContactStoreIface;

#define PLUGIN_TYPE_EMAIL_EXTENSION (plugin_email_extension_get_type ())
#define PLUGIN_EMAIL_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_EMAIL_EXTENSION, PluginEmailExtension))
#define PLUGIN_IS_EMAIL_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_EMAIL_EXTENSION))
#define PLUGIN_EMAIL_EXTENSION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_EMAIL_EXTENSION, PluginEmailExtensionIface))

typedef struct _PluginEmailExtension PluginEmailExtension;
typedef struct _PluginEmailExtensionIface PluginEmailExtensionIface;

#define PLUGIN_TYPE_PLUGIN_BASE (plugin_plugin_base_get_type ())
#define PLUGIN_PLUGIN_BASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_PLUGIN_BASE, PluginPluginBase))
#define PLUGIN_PLUGIN_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PLUGIN_TYPE_PLUGIN_BASE, PluginPluginBaseClass))
#define PLUGIN_IS_PLUGIN_BASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_PLUGIN_BASE))
#define PLUGIN_IS_PLUGIN_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PLUGIN_TYPE_PLUGIN_BASE))
#define PLUGIN_PLUGIN_BASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PLUGIN_TYPE_PLUGIN_BASE, PluginPluginBaseClass))

typedef struct _PluginPluginBase PluginPluginBase;
typedef struct _PluginPluginBaseClass PluginPluginBaseClass;

#define PLUGIN_TYPE_EMAIL_CONTEXT (plugin_email_context_get_type ())
#define PLUGIN_EMAIL_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_EMAIL_CONTEXT, PluginEmailContext))
#define PLUGIN_IS_EMAIL_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_EMAIL_CONTEXT))
#define PLUGIN_EMAIL_CONTEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_EMAIL_CONTEXT, PluginEmailContextIface))

typedef struct _PluginEmailContext PluginEmailContext;
typedef struct _PluginEmailContextIface PluginEmailContextIface;

#define PLUGIN_TYPE_EMAIL_STORE (plugin_email_store_get_type ())
#define PLUGIN_EMAIL_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_EMAIL_STORE, PluginEmailStore))
#define PLUGIN_IS_EMAIL_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_EMAIL_STORE))
#define PLUGIN_EMAIL_STORE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_EMAIL_STORE, PluginEmailStoreIface))

typedef struct _PluginEmailStore PluginEmailStore;
typedef struct _PluginEmailStoreIface PluginEmailStoreIface;

#define PLUGIN_TYPE_FOLDER_EXTENSION (plugin_folder_extension_get_type ())
#define PLUGIN_FOLDER_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_FOLDER_EXTENSION, PluginFolderExtension))
#define PLUGIN_IS_FOLDER_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_FOLDER_EXTENSION))
#define PLUGIN_FOLDER_EXTENSION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_FOLDER_EXTENSION, PluginFolderExtensionIface))

typedef struct _PluginFolderExtension PluginFolderExtension;
typedef struct _PluginFolderExtensionIface PluginFolderExtensionIface;

#define PLUGIN_TYPE_FOLDER_CONTEXT (plugin_folder_context_get_type ())
#define PLUGIN_FOLDER_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_FOLDER_CONTEXT, PluginFolderContext))
#define PLUGIN_IS_FOLDER_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_FOLDER_CONTEXT))
#define PLUGIN_FOLDER_CONTEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_FOLDER_CONTEXT, PluginFolderContextIface))

typedef struct _PluginFolderContext PluginFolderContext;
typedef struct _PluginFolderContextIface PluginFolderContextIface;

#define PLUGIN_TYPE_FOLDER_STORE (plugin_folder_store_get_type ())
#define PLUGIN_FOLDER_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_FOLDER_STORE, PluginFolderStore))
#define PLUGIN_IS_FOLDER_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_FOLDER_STORE))
#define PLUGIN_FOLDER_STORE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_FOLDER_STORE, PluginFolderStoreIface))

typedef struct _PluginFolderStore PluginFolderStore;
typedef struct _PluginFolderStoreIface PluginFolderStoreIface;
typedef struct _PluginInfoBarPrivate PluginInfoBarPrivate;

#define PLUGIN_TYPE_NOTIFICATION_EXTENSION (plugin_notification_extension_get_type ())
#define PLUGIN_NOTIFICATION_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_NOTIFICATION_EXTENSION, PluginNotificationExtension))
#define PLUGIN_IS_NOTIFICATION_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_NOTIFICATION_EXTENSION))
#define PLUGIN_NOTIFICATION_EXTENSION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_NOTIFICATION_EXTENSION, PluginNotificationExtensionIface))

typedef struct _PluginNotificationExtension PluginNotificationExtension;
typedef struct _PluginNotificationExtensionIface PluginNotificationExtensionIface;

#define PLUGIN_TYPE_NOTIFICATION_CONTEXT (plugin_notification_context_get_type ())
#define PLUGIN_NOTIFICATION_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_NOTIFICATION_CONTEXT, PluginNotificationContext))
#define PLUGIN_IS_NOTIFICATION_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_NOTIFICATION_CONTEXT))
#define PLUGIN_NOTIFICATION_CONTEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_NOTIFICATION_CONTEXT, PluginNotificationContextIface))

typedef struct _PluginNotificationContext PluginNotificationContext;
typedef struct _PluginNotificationContextIface PluginNotificationContextIface;
typedef struct _PluginPluginBasePrivate PluginPluginBasePrivate;

#define PLUGIN_TYPE_TRUSTED_EXTENSION (plugin_trusted_extension_get_type ())
#define PLUGIN_TRUSTED_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLUGIN_TYPE_TRUSTED_EXTENSION, PluginTrustedExtension))
#define PLUGIN_IS_TRUSTED_EXTENSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLUGIN_TYPE_TRUSTED_EXTENSION))
#define PLUGIN_TRUSTED_EXTENSION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PLUGIN_TYPE_TRUSTED_EXTENSION, PluginTrustedExtensionIface))

typedef struct _PluginTrustedExtension PluginTrustedExtension;
typedef struct _PluginTrustedExtensionIface PluginTrustedExtensionIface;
typedef gboolean (*Locator) (gconstpointer item, gpointer user_data);
typedef enum  {
	SIDEBAR_BRANCH_OPTIONS_NONE = 0,
	SIDEBAR_BRANCH_OPTIONS_HIDE_IF_EMPTY = 1 << 0,
	SIDEBAR_BRANCH_OPTIONS_AUTO_OPEN_ON_NEW_CHILD = 1 << 1,
	SIDEBAR_BRANCH_OPTIONS_STARTUP_EXPAND_TO_FIRST_CHILD = 1 << 2,
	SIDEBAR_BRANCH_OPTIONS_STARTUP_OPEN_GROUPING = 1 << 3
} SidebarBranchOptions;

#define SIDEBAR_BRANCH_TYPE_OPTIONS (sidebar_branch_options_get_type ())

#define SIDEBAR_TYPE_HEADER (sidebar_header_get_type ())
#define SIDEBAR_HEADER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_HEADER, SidebarHeader))
#define SIDEBAR_HEADER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SIDEBAR_TYPE_HEADER, SidebarHeaderClass))
#define SIDEBAR_IS_HEADER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_HEADER))
#define SIDEBAR_IS_HEADER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SIDEBAR_TYPE_HEADER))
#define SIDEBAR_HEADER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SIDEBAR_TYPE_HEADER, SidebarHeaderClass))

typedef struct _SidebarHeader SidebarHeader;
typedef struct _SidebarHeaderClass SidebarHeaderClass;
typedef struct _SidebarHeaderPrivate SidebarHeaderPrivate;

#define SIDEBAR_TYPE_CONTEXTABLE (sidebar_contextable_get_type ())
#define SIDEBAR_CONTEXTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_CONTEXTABLE, SidebarContextable))
#define SIDEBAR_IS_CONTEXTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_CONTEXTABLE))
#define SIDEBAR_CONTEXTABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_CONTEXTABLE, SidebarContextableIface))

typedef struct _SidebarContextable SidebarContextable;
typedef struct _SidebarContextableIface SidebarContextableIface;

#define TYPE_SIDEBAR_COUNT_CELL_RENDERER (sidebar_count_cell_renderer_get_type ())
#define SIDEBAR_COUNT_CELL_RENDERER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIDEBAR_COUNT_CELL_RENDERER, SidebarCountCellRenderer))
#define SIDEBAR_COUNT_CELL_RENDERER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIDEBAR_COUNT_CELL_RENDERER, SidebarCountCellRendererClass))
#define IS_SIDEBAR_COUNT_CELL_RENDERER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIDEBAR_COUNT_CELL_RENDERER))
#define IS_SIDEBAR_COUNT_CELL_RENDERER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIDEBAR_COUNT_CELL_RENDERER))
#define SIDEBAR_COUNT_CELL_RENDERER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIDEBAR_COUNT_CELL_RENDERER, SidebarCountCellRendererClass))

typedef struct _SidebarCountCellRenderer SidebarCountCellRenderer;
typedef struct _SidebarCountCellRendererClass SidebarCountCellRendererClass;
typedef struct _SidebarCountCellRendererPrivate SidebarCountCellRendererPrivate;

#define SIDEBAR_TYPE_DESTROYABLE_ENTRY (sidebar_destroyable_entry_get_type ())
#define SIDEBAR_DESTROYABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_DESTROYABLE_ENTRY, SidebarDestroyableEntry))
#define SIDEBAR_IS_DESTROYABLE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_DESTROYABLE_ENTRY))
#define SIDEBAR_DESTROYABLE_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_DESTROYABLE_ENTRY, SidebarDestroyableEntryIface))

typedef struct _SidebarDestroyableEntry SidebarDestroyableEntry;
typedef struct _SidebarDestroyableEntryIface SidebarDestroyableEntryIface;

#define SIDEBAR_TYPE_INTERNAL_DRAG_SOURCE_ENTRY (sidebar_internal_drag_source_entry_get_type ())
#define SIDEBAR_INTERNAL_DRAG_SOURCE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIDEBAR_TYPE_INTERNAL_DRAG_SOURCE_ENTRY, SidebarInternalDragSourceEntry))
#define SIDEBAR_IS_INTERNAL_DRAG_SOURCE_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIDEBAR_TYPE_INTERNAL_DRAG_SOURCE_ENTRY))
#define SIDEBAR_INTERNAL_DRAG_SOURCE_ENTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), SIDEBAR_TYPE_INTERNAL_DRAG_SOURCE_ENTRY, SidebarInternalDragSourceEntryIface))

typedef struct _SidebarInternalDragSourceEntry SidebarInternalDragSourceEntry;
typedef struct _SidebarInternalDragSourceEntryIface SidebarInternalDragSourceEntryIface;
typedef void (*SidebarTreeExternalDropHandler) (GdkDragContext* context, SidebarEntry* entry, GtkSelectionData* data, guint info, guint time, gpointer user_data);

#define UTIL_CACHE_TYPE_LRU (util_cache_lru_get_type ())
#define UTIL_CACHE_LRU(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), UTIL_CACHE_TYPE_LRU, UtilCacheLru))
#define UTIL_CACHE_LRU_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), UTIL_CACHE_TYPE_LRU, UtilCacheLruClass))
#define UTIL_CACHE_IS_LRU(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), UTIL_CACHE_TYPE_LRU))
#define UTIL_CACHE_IS_LRU_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), UTIL_CACHE_TYPE_LRU))
#define UTIL_CACHE_LRU_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), UTIL_CACHE_TYPE_LRU, UtilCacheLruClass))

typedef struct _UtilCacheLru UtilCacheLru;
typedef struct _UtilCacheLruClass UtilCacheLruClass;
typedef struct _UtilCacheLruPrivate UtilCacheLruPrivate;
typedef enum  {
	UTIL_DATE_COARSE_DATE_NOW,
	UTIL_DATE_COARSE_DATE_MINUTES,
	UTIL_DATE_COARSE_DATE_HOURS,
	UTIL_DATE_COARSE_DATE_TODAY,
	UTIL_DATE_COARSE_DATE_YESTERDAY,
	UTIL_DATE_COARSE_DATE_THIS_WEEK,
	UTIL_DATE_COARSE_DATE_THIS_YEAR,
	UTIL_DATE_COARSE_DATE_YEARS,
	UTIL_DATE_COARSE_DATE_FUTURE
} UtilDateCoarseDate;

#define UTIL_DATE_TYPE_COARSE_DATE (util_date_coarse_date_get_type ())

#define UTIL_EMAIL_TYPE_SEARCH_EXPRESSION_FACTORY (util_email_search_expression_factory_get_type ())
#define UTIL_EMAIL_SEARCH_EXPRESSION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), UTIL_EMAIL_TYPE_SEARCH_EXPRESSION_FACTORY, UtilEmailSearchExpressionFactory))
#define UTIL_EMAIL_SEARCH_EXPRESSION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), UTIL_EMAIL_TYPE_SEARCH_EXPRESSION_FACTORY, UtilEmailSearchExpressionFactoryClass))
#define UTIL_EMAIL_IS_SEARCH_EXPRESSION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), UTIL_EMAIL_TYPE_SEARCH_EXPRESSION_FACTORY))
#define UTIL_EMAIL_IS_SEARCH_EXPRESSION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), UTIL_EMAIL_TYPE_SEARCH_EXPRESSION_FACTORY))
#define UTIL_EMAIL_SEARCH_EXPRESSION_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), UTIL_EMAIL_TYPE_SEARCH_EXPRESSION_FACTORY, UtilEmailSearchExpressionFactoryClass))

typedef struct _UtilEmailSearchExpressionFactory UtilEmailSearchExpressionFactory;
typedef struct _UtilEmailSearchExpressionFactoryClass UtilEmailSearchExpressionFactoryClass;
typedef struct _UtilEmailSearchExpressionFactoryPrivate UtilEmailSearchExpressionFactoryPrivate;
typedef gboolean (*UtilGtkMenuVisitor) (GMenuModel* existing_menu, GMenuModel* existing_child_menu, const gchar* existing_action, GMenuItem* new_item, gpointer user_data);

#define UTIL_JS_TYPE_ERROR (util_js_error_get_type ())
typedef enum  {
	UTIL_JS_JSC_TYPE_UNKNOWN,
	UTIL_JS_JSC_TYPE_UNDEFINED,
	UTIL_JS_JSC_TYPE_NULL,
	UTIL_JS_JSC_TYPE_FUNCTION,
	UTIL_JS_JSC_TYPE_STRING,
	UTIL_JS_JSC_TYPE_NUMBER,
	UTIL_JS_JSC_TYPE_BOOLEAN,
	UTIL_JS_JSC_TYPE_ARRAY,
	UTIL_JS_JSC_TYPE_CONSTRUCTOR,
	UTIL_JS_JSC_TYPE_OBJECT
} UtilJSJscType;

#define UTIL_JS_TYPE_JSC_TYPE (util_js_jsc_type_get_type ())
typedef struct _UtilJSCallablePrivate UtilJSCallablePrivate;

struct _ApplicationAccountContext {
	GearyBaseObject parent_instance;
	GearyFolder* inbox;
	GearyAppSearchFolder* search;
	ApplicationAccountContextPrivate * priv;
};

struct _ApplicationAccountContextClass {
	GearyBaseObjectClass parent_class;
};

struct _ApplicationAttachmentManager {
	GObject parent_instance;
	ApplicationAttachmentManagerPrivate * priv;
};

struct _ApplicationAttachmentManagerClass {
	GObjectClass parent_class;
};

typedef enum  {
	APPLICATION_CERTIFICATE_MANAGER_ERROR_UNTRUSTED,
	APPLICATION_CERTIFICATE_MANAGER_ERROR_STORE_FAILED
} ApplicationCertificateManagerError;
#define APPLICATION_CERTIFICATE_MANAGER_ERROR application_certificate_manager_error_quark ()

struct _ApplicationCertificateManager {
	GObject parent_instance;
	ApplicationCertificateManagerPrivate * priv;
};

struct _ApplicationCertificateManagerClass {
	GObjectClass parent_class;
};

struct _ApplicationClient {
	GtkApplication parent_instance;
	ApplicationClientPrivate * priv;
};

struct _ApplicationClientClass {
	GtkApplicationClass parent_class;
};

struct _ApplicationClientRuntimeDetail {
	gchar* name;
	gchar* value;
};

struct _ApplicationCommand {
	GObject parent_instance;
	ApplicationCommandPrivate * priv;
};

struct _ApplicationCommandClass {
	GObjectClass parent_class;
	gboolean (*get_can_undo) (ApplicationCommand* self);
	gboolean (*get_can_redo) (ApplicationCommand* self);
	void (*executed) (ApplicationCommand* self);
	void (*undone) (ApplicationCommand* self);
	void (*redone) (ApplicationCommand* self);
	void (*execute) (ApplicationCommand* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*execute_finish) (ApplicationCommand* self, GAsyncResult* _res_, GError** error);
	void (*undo) (ApplicationCommand* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*undo_finish) (ApplicationCommand* self, GAsyncResult* _res_, GError** error);
	void (*redo) (ApplicationCommand* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*redo_finish) (ApplicationCommand* self, GAsyncResult* _res_, GError** error);
	gboolean (*equal_to) (ApplicationCommand* self, ApplicationCommand* other);
	gchar* (*to_string) (ApplicationCommand* self);
};

struct _ApplicationCommandSequence {
	ApplicationCommand parent_instance;
	ApplicationCommandSequencePrivate * priv;
};

struct _ApplicationCommandSequenceClass {
	ApplicationCommandClass parent_class;
};

struct _ApplicationPropertyCommand {
	ApplicationCommand parent_instance;
	ApplicationPropertyCommandPrivate * priv;
};

struct _ApplicationPropertyCommandClass {
	ApplicationCommandClass parent_class;
};

struct _ApplicationCommandStack {
	GObject parent_instance;
	GeeDeque* undo_stack;
	GeeDeque* redo_stack;
	ApplicationCommandStackPrivate * priv;
};

struct _ApplicationCommandStackClass {
	GObjectClass parent_class;
	void (*execute) (ApplicationCommandStack* self, ApplicationCommand* target, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*execute_finish) (ApplicationCommandStack* self, GAsyncResult* _res_, GError** error);
	void (*undo) (ApplicationCommandStack* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*undo_finish) (ApplicationCommandStack* self, GAsyncResult* _res_, GError** error);
	void (*redo) (ApplicationCommandStack* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*redo_finish) (ApplicationCommandStack* self, GAsyncResult* _res_, GError** error);
	void (*update_undo_stack) (ApplicationCommandStack* self, ApplicationCommand* target);
	void (*update_redo_stack) (ApplicationCommandStack* self, ApplicationCommand* target);
};

struct _ApplicationConfiguration {
	GearyBaseObject parent_instance;
	ApplicationConfigurationPrivate * priv;
};

struct _ApplicationConfigurationClass {
	GearyBaseObjectClass parent_class;
};

struct _ApplicationContactStore {
	GearyBaseObject parent_instance;
	FolksIndividualAggregator* individuals;
	ApplicationContactStorePrivate * priv;
};

struct _ApplicationContactStoreClass {
	GearyBaseObjectClass parent_class;
};

struct _ApplicationContact {
	GearyBaseObject parent_instance;
	ApplicationContactPrivate * priv;
};

struct _ApplicationContactClass {
	GearyBaseObjectClass parent_class;
};

struct _ApplicationEmailCommand {
	ApplicationCommand parent_instance;
	ApplicationEmailCommandPrivate * priv;
};

struct _ApplicationEmailCommandClass {
	ApplicationCommandClass parent_class;
	ApplicationEmailCommandStateChangePolicy (*folders_removed) (ApplicationEmailCommand* self, GeeCollection* removed);
	ApplicationEmailCommandStateChangePolicy (*email_removed) (ApplicationEmailCommand* self, GearyFolder* location, GeeCollection* targets);
};

struct _ApplicationTrivialCommandIface {
	GTypeInterface parent_iface;
};

struct _ApplicationFolderContext {
	GearyBaseObject parent_instance;
	ApplicationFolderContextPrivate * priv;
};

struct _ApplicationFolderContextClass {
	GearyBaseObjectClass parent_class;
};

struct _ApplicationMainWindow {
	HdyApplicationWindow parent_instance;
	ComponentsConversationHeaderBar* conversation_headerbar;
	ApplicationMainWindowPrivate * priv;
};

struct _ApplicationMainWindowClass {
	HdyApplicationWindowClass parent_class;
	void (*reply_conversation_sender) (ApplicationMainWindow* self);
	void (*reply_conversation_all) (ApplicationMainWindow* self);
	void (*forward_conversation) (ApplicationMainWindow* self);
	void (*mark_conversations_read) (ApplicationMainWindow* self, gboolean prefer_read);
	void (*mark_conversations_starred) (ApplicationMainWindow* self, gboolean prefer_starred);
	void (*show_copy_menu) (ApplicationMainWindow* self);
	void (*archive_conversations) (ApplicationMainWindow* self);
	void (*junk_conversations) (ApplicationMainWindow* self);
	void (*trash_conversations) (ApplicationMainWindow* self);
	void (*delete_conversations) (ApplicationMainWindow* self);
	void (*search) (ApplicationMainWindow* self);
	void (*find) (ApplicationMainWindow* self);
	void (*escape) (ApplicationMainWindow* self);
	void (*select_all) (ApplicationMainWindow* self);
	void (*navigate) (ApplicationMainWindow* self, GtkScrollType type);
};

struct _ApplicationPluginManager {
	GObject parent_instance;
	ApplicationPluginManagerPrivate * priv;
};

struct _ApplicationPluginManagerClass {
	GObjectClass parent_class;
};

struct _PluginAccountIface {
	GTypeInterface parent_iface;
	const gchar* (*get_display_name) (PluginAccount* self);
};

struct _PluginFolderIface {
	GTypeInterface parent_iface;
	const gchar* (*get_persistent_id) (PluginFolder* self);
	const gchar* (*get_display_name) (PluginFolder* self);
	GearyFolderSpecialUse (*get_used_as) (PluginFolder* self);
	PluginAccount* (*get_account) (PluginFolder* self);
	GVariant* (*to_variant) (PluginFolder* self);
};

struct _PluginEmailIdentifierIface {
	GTypeInterface parent_iface;
	PluginAccount* (*get_account) (PluginEmailIdentifier* self);
	GVariant* (*to_variant) (PluginEmailIdentifier* self);
};

struct _PluginEmailIface {
	GTypeInterface parent_iface;
	PluginEmailIdentifier* (*get_identifier) (PluginEmail* self);
	GearyEmailFlags* (*get_flags) (PluginEmail* self);
	GearyRFC822MailboxAddress* (*get_primary_originator) (PluginEmail* self);
	void (*load_body_as) (PluginEmail* self, PluginEmailBodyType type, gboolean convert, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar* (*load_body_as_finish) (PluginEmail* self, GAsyncResult* _res_, GError** error);
};

struct _portalRequestIface {
	GTypeInterface parent_iface;
};

struct _portalBackgroundIface {
	GTypeInterface parent_iface;
	char* (*request_background) (portalBackground* self, const gchar* parent_window, GHashTable* options, GError** error);
};

struct _ApplicationStartupManager {
	GObject parent_instance;
	ApplicationStartupManagerPrivate * priv;
};

struct _ApplicationStartupManagerClass {
	GObjectClass parent_class;
};

struct _GoaMediator {
	GObject parent_instance;
	GoaMediatorPrivate * priv;
};

struct _GoaMediatorClass {
	GObjectClass parent_class;
	void (*load_token) (GoaMediator* self, GearyAccountInformation* account, GearyServiceInformation* service, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gboolean (*load_token_finish) (GoaMediator* self, GAsyncResult* _res_, GError** error);
};

struct _SecretMediator {
	GObject parent_instance;
	SecretMediatorPrivate * priv;
};

struct _SecretMediatorClass {
	GObjectClass parent_class;
	void (*load_token) (SecretMediator* self, GearyAccountInformation* account, GearyServiceInformation* service, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gboolean (*load_token_finish) (SecretMediator* self, GAsyncResult* _res_, GError** error);
};

struct _AccountsEditor {
	GtkDialog parent_instance;
	AccountsEditorPrivate * priv;
};

struct _AccountsEditorClass {
	GtkDialogClass parent_class;
};

struct _ComponentsWebView {
	WebKitWebView parent_instance;
	ComponentsWebViewPrivate * priv;
};

struct _ComponentsWebViewClass {
	WebKitWebViewClass parent_class;
};

struct _AccountsSignatureWebView {
	ComponentsWebView parent_instance;
	AccountsSignatureWebViewPrivate * priv;
};

struct _AccountsSignatureWebViewClass {
	ComponentsWebViewClass parent_class;
};

struct _AccountsManager {
	GObject parent_instance;
	AccountsManagerPrivate * priv;
};

struct _AccountsManagerClass {
	GObjectClass parent_class;
};

typedef enum  {
	ACCOUNTS_CONFIG_ERROR_IO,
	ACCOUNTS_CONFIG_ERROR_MANAGEMENT,
	ACCOUNTS_CONFIG_ERROR_SYNTAX,
	ACCOUNTS_CONFIG_ERROR_VERSION,
	ACCOUNTS_CONFIG_ERROR_UNAVAILABLE,
	ACCOUNTS_CONFIG_ERROR_REMOVED
} AccountsConfigError;
#define ACCOUNTS_CONFIG_ERROR accounts_config_error_quark ()

struct _AccountsAccountConfigIface {
	GTypeInterface parent_iface;
	GearyAccountInformation* (*load) (AccountsAccountConfig* self, GearyConfigFile* config, const gchar* id, GearyCredentialsMediator* mediator, GearyServiceProvider* default_provider, const gchar* default_name, GError** error);
	void (*save) (AccountsAccountConfig* self, GearyAccountInformation* account, GearyConfigFile* config);
};

struct _AccountsServiceConfigIface {
	GTypeInterface parent_iface;
	void (*load) (AccountsServiceConfig* self, GearyConfigFile* config, GearyAccountInformation* account, GearyServiceInformation* service, GError** error);
	void (*save) (AccountsServiceConfig* self, GearyAccountInformation* account, GearyServiceInformation* service, GearyConfigFile* config);
};

struct _AccountsAccountConfigV1 {
	GObject parent_instance;
	AccountsAccountConfigV1Private * priv;
};

struct _AccountsAccountConfigV1Class {
	GObjectClass parent_class;
};

struct _AccountsAccountConfigLegacy {
	GObject parent_instance;
	AccountsAccountConfigLegacyPrivate * priv;
};

struct _AccountsAccountConfigLegacyClass {
	GObjectClass parent_class;
};

struct _AccountsServiceConfigV1 {
	GObject parent_instance;
	AccountsServiceConfigV1Private * priv;
};

struct _AccountsServiceConfigV1Class {
	GObjectClass parent_class;
};

struct _AccountsServiceConfigLegacy {
	GObject parent_instance;
	AccountsServiceConfigLegacyPrivate * priv;
};

struct _AccountsServiceConfigLegacyClass {
	GObjectClass parent_class;
};

struct _ComponentsAttachmentPane {
	GtkGrid parent_instance;
	ComponentsAttachmentPanePrivate * priv;
};

struct _ComponentsAttachmentPaneClass {
	GtkGridClass parent_class;
};

struct _ComponentsConversationActions {
	GtkBox parent_instance;
	ComponentsConversationActionsPrivate * priv;
};

struct _ComponentsConversationActionsClass {
	GtkBoxClass parent_class;
};

struct _ComponentsEntryUndo {
	GearyBaseObject parent_instance;
	ComponentsEntryUndoPrivate * priv;
};

struct _ComponentsEntryUndoClass {
	GearyBaseObjectClass parent_class;
};

struct _ComponentsApplicationHeaderBar {
	HdyHeaderBar parent_instance;
	MonitoredSpinner* spinner;
	ComponentsApplicationHeaderBarPrivate * priv;
};

struct _ComponentsApplicationHeaderBarClass {
	HdyHeaderBarClass parent_class;
};

struct _ComponentsConversationListHeaderBar {
	HdyHeaderBar parent_instance;
	GtkButton* back_button;
	ComponentsConversationListHeaderBarPrivate * priv;
};

struct _ComponentsConversationListHeaderBarClass {
	HdyHeaderBarClass parent_class;
};

struct _ComponentsConversationHeaderBar {
	GtkBin parent_instance;
	ComponentsConversationActions* full_actions;
	ComponentsConversationActions* compact_actions;
	GtkButton* back_button;
	ComponentsConversationHeaderBarPrivate * priv;
};

struct _ComponentsConversationHeaderBarClass {
	GtkBinClass parent_class;
};

struct _ComponentsInfoBarStack {
	GtkFrame parent_instance;
	ComponentsInfoBarStackPrivate * priv;
};

struct _ComponentsInfoBarStackClass {
	GtkFrameClass parent_class;
};

struct _ComponentsInfoBar {
	GtkBox parent_instance;
	ComponentsInfoBarPrivate * priv;
};

struct _ComponentsInfoBarClass {
	GtkBoxClass parent_class;
};

struct _ComponentsInspector {
	GtkApplicationWindow parent_instance;
	ComponentsInspectorPrivate * priv;
};

struct _ComponentsInspectorClass {
	GtkApplicationWindowClass parent_class;
};

struct _ComponentsInAppNotification {
	GtkRevealer parent_instance;
	ComponentsInAppNotificationPrivate * priv;
};

struct _ComponentsInAppNotificationClass {
	GtkRevealerClass parent_class;
};

struct _ComponentsInspectorErrorView {
	GtkGrid parent_instance;
	ComponentsInspectorErrorViewPrivate * priv;
};

struct _ComponentsInspectorErrorViewClass {
	GtkGridClass parent_class;
};

struct _ComponentsInspectorLogView {
	GtkGrid parent_instance;
	ComponentsInspectorLogViewPrivate * priv;
};

struct _ComponentsInspectorLogViewClass {
	GtkGridClass parent_class;
};

struct _ComponentsInspectorSystemView {
	GtkGrid parent_instance;
	ComponentsInspectorSystemViewPrivate * priv;
};

struct _ComponentsInspectorSystemViewClass {
	GtkGridClass parent_class;
};

struct _ComponentsPlaceholderPane {
	GtkGrid parent_instance;
	ComponentsPlaceholderPanePrivate * priv;
};

struct _ComponentsPlaceholderPaneClass {
	GtkGridClass parent_class;
};

struct _ComponentsPreferencesWindow {
	HdyPreferencesWindow parent_instance;
	ComponentsPreferencesWindowPrivate * priv;
};

struct _ComponentsPreferencesWindowClass {
	HdyPreferencesWindowClass parent_class;
};

struct _ComponentsProblemReportInfoBar {
	ComponentsInfoBar parent_instance;
	ComponentsProblemReportInfoBarPrivate * priv;
};

struct _ComponentsProblemReportInfoBarClass {
	ComponentsInfoBarClass parent_class;
};

struct _SearchBar {
	HdySearchBar parent_instance;
	SearchBarPrivate * priv;
};

struct _SearchBarClass {
	HdySearchBarClass parent_class;
};

struct _ComponentsValidatorUiState {
	gchar* icon_name;
	gchar* icon_tooltip_text;
};

struct _ComponentsValidator {
	GObject parent_instance;
	ComponentsValidatorUiState indeterminate_state;
	ComponentsValidatorUiState valid_state;
	ComponentsValidatorUiState in_progress_state;
	ComponentsValidatorUiState empty_state;
	ComponentsValidatorUiState invalid_state;
	ComponentsValidatorPrivate * priv;
};

struct _ComponentsValidatorClass {
	GObjectClass parent_class;
	ComponentsValidatorValidity (*do_validate) (ComponentsValidator* self, const gchar* value, ComponentsValidatorTrigger reason);
};

struct _ComponentsEmailValidator {
	ComponentsValidator parent_instance;
	ComponentsEmailValidatorPrivate * priv;
};

struct _ComponentsEmailValidatorClass {
	ComponentsValidatorClass parent_class;
};

struct _ComponentsNetworkAddressValidator {
	ComponentsValidator parent_instance;
	ComponentsNetworkAddressValidatorPrivate * priv;
};

struct _ComponentsNetworkAddressValidatorClass {
	ComponentsValidatorClass parent_class;
};

struct _CountBadge {
	GearyBaseObject parent_instance;
	CountBadgePrivate * priv;
};

struct _CountBadgeClass {
	GearyBaseObjectClass parent_class;
};

struct _FolderPopover {
	GtkPopover parent_instance;
	FolderPopoverPrivate * priv;
};

struct _FolderPopoverClass {
	GtkPopoverClass parent_class;
};

struct _FolderPopoverRow {
	GtkListBoxRow parent_instance;
	FolderPopoverRowPrivate * priv;
};

struct _FolderPopoverRowClass {
	GtkListBoxRowClass parent_class;
};

struct _IconFactory {
	GTypeInstance parent_instance;
	volatile int ref_count;
	IconFactoryPrivate * priv;
};

struct _IconFactoryClass {
	GTypeClass parent_class;
	void (*finalize) (IconFactory *self);
};

struct _MonitoredProgressBar {
	GtkProgressBar parent_instance;
	MonitoredProgressBarPrivate * priv;
};

struct _MonitoredProgressBarClass {
	GtkProgressBarClass parent_class;
};

struct _MonitoredSpinner {
	GtkSpinner parent_instance;
	MonitoredSpinnerPrivate * priv;
};

struct _MonitoredSpinnerClass {
	GtkSpinnerClass parent_class;
};

struct _ComposerContainerIface {
	GTypeInterface parent_iface;
	GtkApplicationWindow* (*get_top_window) (ComposerContainer* self);
	ComposerWidget* (*get_composer) (ComposerContainer* self);
	void (*set_composer) (ComposerContainer* self, ComposerWidget* value);
	void (*present) (ComposerContainer* self);
	GtkWidget* (*get_focus) (ComposerContainer* self);
	void (*close) (ComposerContainer* self);
};

struct _ComposerBox {
	GtkFrame parent_instance;
	ComposerBoxPrivate * priv;
};

struct _ComposerBoxClass {
	GtkFrameClass parent_class;
};

struct _ComposerEditor {
	GtkGrid parent_instance;
	GSimpleActionGroup* actions;
	GtkButton* new_message_attach_button;
	GtkBox* conversation_attach_buttons;
	ComposerEditorPrivate * priv;
};

struct _ComposerEditorClass {
	GtkGridClass parent_class;
};

struct _ComposerEmailEntry {
	GtkEntry parent_instance;
	ComposerEmailEntryPrivate * priv;
};

struct _ComposerEmailEntryClass {
	GtkEntryClass parent_class;
};

struct _ComposerEmbed {
	GtkEventBox parent_instance;
	ComposerEmbedPrivate * priv;
};

struct _ComposerEmbedClass {
	GtkEventBoxClass parent_class;
};

struct _ComposerHeaderbar {
	HdyHeaderBar parent_instance;
	ComposerHeaderbarPrivate * priv;
};

struct _ComposerHeaderbarClass {
	HdyHeaderBarClass parent_class;
};

struct _ComposerLinkPopover {
	GtkPopover parent_instance;
	ComposerLinkPopoverPrivate * priv;
};

struct _ComposerLinkPopoverClass {
	GtkPopoverClass parent_class;
};

struct _ComposerWebView {
	ComponentsWebView parent_instance;
	ComposerWebViewPrivate * priv;
};

struct _ComposerWebViewClass {
	ComponentsWebViewClass parent_class;
};

struct _ComposerWebViewEditContext {
	GObject parent_instance;
	ComposerWebViewEditContextPrivate * priv;
};

struct _ComposerWebViewEditContextClass {
	GObjectClass parent_class;
};

struct _ComposerWidget {
	GtkEventBox parent_instance;
	ComposerWidgetPrivate * priv;
};

struct _ComposerWidgetClass {
	GtkEventBoxClass parent_class;
};

struct _ComposerWindow {
	GtkApplicationWindow parent_instance;
	ComposerWindowPrivate * priv;
};

struct _ComposerWindowClass {
	GtkApplicationWindowClass parent_class;
};

struct _ContactEntryCompletion {
	GtkEntryCompletion parent_instance;
	ContactEntryCompletionPrivate * priv;
};

struct _ContactEntryCompletionClass {
	GtkEntryCompletionClass parent_class;
};

struct _SpellCheckPopover {
	GTypeInstance parent_instance;
	volatile int ref_count;
	SpellCheckPopoverPrivate * priv;
};

struct _SpellCheckPopoverClass {
	GTypeClass parent_class;
	void (*finalize) (SpellCheckPopover *self);
};

struct _ConversationListModel {
	GObject parent_instance;
	GPtrArray* items;
	ConversationListModelPrivate * priv;
};

struct _ConversationListModelClass {
	GObjectClass parent_class;
};

struct _ConversationListView {
	GtkScrolledWindow parent_instance;
	ConversationListViewPrivate * priv;
};

struct _ConversationListViewClass {
	GtkScrolledWindowClass parent_class;
};

struct _ConversationContactPopover {
	GtkPopover parent_instance;
	ConversationContactPopoverPrivate * priv;
};

struct _ConversationContactPopoverClass {
	GtkPopoverClass parent_class;
};

struct _ConversationEmail {
	GtkBox parent_instance;
	gboolean is_collapsed;
	GearyAppConversation* conversation;
	ConversationEmailPrivate * priv;
};

struct _ConversationEmailClass {
	GtkBoxClass parent_class;
};

struct _ConversationListBox {
	GtkListBox parent_instance;
	ConversationListBoxPrivate * priv;
};

struct _ConversationListBoxClass {
	GtkListBoxClass parent_class;
	void (*scroll) (ConversationListBox* self, GtkScrollType type);
	void (*focus_next) (ConversationListBox* self);
	void (*focus_prev) (ConversationListBox* self);
};

struct _ConversationListBoxSearchManager {
	GearyBaseObject parent_instance;
	ConversationListBoxSearchManagerPrivate * priv;
};

struct _ConversationListBoxSearchManagerClass {
	GearyBaseObjectClass parent_class;
};

struct _ConversationMessage {
	GtkGrid parent_instance;
	ConversationMessagePrivate * priv;
};

struct _ConversationMessageClass {
	GtkGridClass parent_class;
};

struct _ConversationMessageContactList {
	GtkFlowBox parent_instance;
	ConversationMessageContactListPrivate * priv;
};

struct _ConversationMessageContactListClass {
	GtkFlowBoxClass parent_class;
};

struct _ConversationViewer {
	GtkStack parent_instance;
	GtkSearchBar* conversation_find_bar;
	GtkSearchEntry* conversation_find_entry;
	ConversationViewerPrivate * priv;
};

struct _ConversationViewerClass {
	GtkStackClass parent_class;
};

struct _ConversationWebView {
	ComponentsWebView parent_instance;
	ConversationWebViewPrivate * priv;
};

struct _ConversationWebViewClass {
	ComponentsWebViewClass parent_class;
};

struct _AttachmentDialog {
	GObject parent_instance;
	AttachmentDialogPrivate * priv;
};

struct _AttachmentDialogClass {
	GObjectClass parent_class;
};

struct _CertificateWarningDialog {
	GTypeInstance parent_instance;
	volatile int ref_count;
	CertificateWarningDialogPrivate * priv;
};

struct _CertificateWarningDialogClass {
	GTypeClass parent_class;
	void (*finalize) (CertificateWarningDialog *self);
};

struct _DialogsProblemDetailsDialog {
	GtkDialog parent_instance;
	DialogsProblemDetailsDialogPrivate * priv;
};

struct _DialogsProblemDetailsDialogClass {
	GtkDialogClass parent_class;
};

struct _PasswordDialog {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PasswordDialogPrivate * priv;
};

struct _PasswordDialogClass {
	GTypeClass parent_class;
	void (*finalize) (PasswordDialog *self);
};

struct _SidebarEntryIface {
	GTypeInterface parent_iface;
	gchar* (*get_sidebar_name) (SidebarEntry* self);
	gchar* (*get_sidebar_tooltip) (SidebarEntry* self);
	gchar* (*get_sidebar_icon) (SidebarEntry* self);
	gint (*get_count) (SidebarEntry* self);
	gchar* (*to_string) (SidebarEntry* self);
	void (*grafted) (SidebarEntry* self, SidebarTree* tree);
	void (*pruned) (SidebarEntry* self, SidebarTree* tree);
};

struct _SidebarSelectableEntryIface {
	GTypeInterface parent_iface;
};

struct _FolderListAbstractFolderEntry {
	GearyBaseObject parent_instance;
	FolderListAbstractFolderEntryPrivate * priv;
};

struct _FolderListAbstractFolderEntryClass {
	GearyBaseObjectClass parent_class;
	gchar* (*get_sidebar_name) (FolderListAbstractFolderEntry* self);
	gchar* (*get_sidebar_tooltip) (FolderListAbstractFolderEntry* self);
	gchar* (*get_sidebar_icon) (FolderListAbstractFolderEntry* self);
	gint (*get_count) (FolderListAbstractFolderEntry* self);
	gchar* (*to_string) (FolderListAbstractFolderEntry* self);
};

struct _SidebarBranch {
	GearyBaseObject parent_instance;
	SidebarBranchPrivate * priv;
};

struct _SidebarBranchClass {
	GearyBaseObjectClass parent_class;
};

struct _FolderListAccountBranch {
	SidebarBranch parent_instance;
	FolderListAccountBranchPrivate * priv;
};

struct _FolderListAccountBranchClass {
	SidebarBranchClass parent_class;
};

struct _SidebarInternalDropTargetEntryIface {
	GTypeInterface parent_iface;
	gboolean (*internal_drop_received) (SidebarInternalDropTargetEntry* self, SidebarTree* parent, GdkDragContext* context, GtkSelectionData* data);
};

struct _SidebarEmphasizableEntryIface {
	GTypeInterface parent_iface;
	gboolean (*is_emphasized) (SidebarEmphasizableEntry* self);
};

struct _FolderListFolderEntry {
	FolderListAbstractFolderEntry parent_instance;
	FolderListFolderEntryPrivate * priv;
};

struct _FolderListFolderEntryClass {
	FolderListAbstractFolderEntryClass parent_class;
};

struct _SidebarTree {
	GtkTreeView parent_instance;
	SidebarTreePrivate * priv;
};

struct _SidebarTreeClass {
	GtkTreeViewClass parent_class;
	gboolean (*accept_cursor_changed) (SidebarTree* self);
};

struct _FolderListTree {
	SidebarTree parent_instance;
	FolderListTreePrivate * priv;
};

struct _FolderListTreeClass {
	SidebarTreeClass parent_class;
};

struct _FolderListInboxesBranch {
	SidebarBranch parent_instance;
	FolderListInboxesBranchPrivate * priv;
};

struct _FolderListInboxesBranchClass {
	SidebarBranchClass parent_class;
};

struct _FolderListInboxFolderEntry {
	FolderListFolderEntry parent_instance;
	FolderListInboxFolderEntryPrivate * priv;
};

struct _FolderListInboxFolderEntryClass {
	FolderListFolderEntryClass parent_class;
};

struct _SidebarRootOnlyBranch {
	SidebarBranch parent_instance;
	SidebarRootOnlyBranchPrivate * priv;
};

struct _SidebarRootOnlyBranchClass {
	SidebarBranchClass parent_class;
};

struct _FolderListSearchBranch {
	SidebarRootOnlyBranch parent_instance;
	FolderListSearchBranchPrivate * priv;
};

struct _FolderListSearchBranchClass {
	SidebarRootOnlyBranchClass parent_class;
};

struct _FolderListSearchEntry {
	FolderListAbstractFolderEntry parent_instance;
	FolderListSearchEntryPrivate * priv;
};

struct _FolderListSearchEntryClass {
	FolderListAbstractFolderEntryClass parent_class;
};

struct _SidebarExpandableEntryIface {
	GTypeInterface parent_iface;
	gboolean (*expand_on_select) (SidebarExpandableEntry* self);
};

struct _SidebarRenameableEntryIface {
	GTypeInterface parent_iface;
	void (*rename) (SidebarRenameableEntry* self, const gchar* new_name);
	gboolean (*is_user_renameable) (SidebarRenameableEntry* self);
};

struct _SidebarGrouping {
	GearyBaseObject parent_instance;
	SidebarGroupingPrivate * priv;
};

struct _SidebarGroupingClass {
	GearyBaseObjectClass parent_class;
};

struct _FolderListSpecialGrouping {
	SidebarGrouping parent_instance;
	FolderListSpecialGroupingPrivate * priv;
};

struct _FolderListSpecialGroupingClass {
	SidebarGroupingClass parent_class;
};

struct _PluginActionBar {
	GearyBaseObject parent_instance;
	PluginActionBarPrivate * priv;
};

struct _PluginActionBarClass {
	GearyBaseObjectClass parent_class;
};

struct _PluginActionBarItemIface {
	GTypeInterface parent_iface;
};

struct _PluginActionBarLabelItem {
	GObject parent_instance;
	PluginActionBarLabelItemPrivate * priv;
};

struct _PluginActionBarLabelItemClass {
	GObjectClass parent_class;
};

struct _PluginActionBarButtonItem {
	GObject parent_instance;
	PluginActionBarButtonItemPrivate * priv;
};

struct _PluginActionBarButtonItemClass {
	GObjectClass parent_class;
};

struct _PluginActionBarMenuItem {
	GObject parent_instance;
	PluginActionBarMenuItemPrivate * priv;
};

struct _PluginActionBarMenuItemClass {
	GObjectClass parent_class;
};

struct _PluginActionBarGroupItem {
	GObject parent_instance;
	PluginActionBarGroupItemPrivate * priv;
};

struct _PluginActionBarGroupItemClass {
	GObjectClass parent_class;
};

struct _PluginActionable {
	GearyBaseObject parent_instance;
	PluginActionablePrivate * priv;
};

struct _PluginActionableClass {
	GearyBaseObjectClass parent_class;
};

struct _PluginComposerIface {
	GTypeInterface parent_iface;
	PluginAccount* (*get_sender_context) (PluginComposer* self);
	gboolean (*get_can_send) (PluginComposer* self);
	void (*set_can_send) (PluginComposer* self, gboolean value);
	const gchar* (*get_action_group_name) (PluginComposer* self);
	PluginFolder* (*get_save_to) (PluginComposer* self);
	void (*present) (PluginComposer* self);
	void (*insert_text) (PluginComposer* self, const gchar* plain_text);
	void (*save_to_folder) (PluginComposer* self, PluginFolder* location);
	void (*register_action) (PluginComposer* self, GAction* action);
	void (*deregister_action) (PluginComposer* self, GAction* action);
	void (*append_menu_item) (PluginComposer* self, PluginActionable* menu_item);
	void (*set_action_bar) (PluginComposer* self, PluginActionBar* action_bar);
};

typedef enum  {
	PLUGIN_ERROR_PERMISSION_DENIED,
	PLUGIN_ERROR_NOT_FOUND,
	PLUGIN_ERROR_NOT_SUPPORTED
} PluginError;
#define PLUGIN_ERROR plugin_error_quark ()

struct _PluginApplicationIface {
	GTypeInterface parent_iface;
	void (*compose_blank) (PluginApplication* self, PluginAccount* send_from, GAsyncReadyCallback _callback_, gpointer _user_data_);
	PluginComposer* (*compose_blank_finish) (PluginApplication* self, GAsyncResult* _res_, GError** error);
	void (*compose_with_context) (PluginApplication* self, PluginAccount* send_from, PluginComposerContextType type, PluginEmailIdentifier* context, const gchar* quote, GAsyncReadyCallback _callback_, gpointer _user_data_);
	PluginComposer* (*compose_with_context_finish) (PluginApplication* self, GAsyncResult* _res_, GError** error);
	void (*register_action) (PluginApplication* self, GAction* action);
	void (*deregister_action) (PluginApplication* self, GAction* action);
	void (*show_folder) (PluginApplication* self, PluginFolder* folder);
	void (*empty_folder) (PluginApplication* self, PluginFolder* folder, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*empty_folder_finish) (PluginApplication* self, GAsyncResult* _res_, GError** error);
	void (*report_problem) (PluginApplication* self, GearyProblemReport* problem);
};

struct _PluginContactStoreIface {
	GTypeInterface parent_iface;
	void (*search) (PluginContactStore* self, const gchar* query, guint min_importance, guint limit, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GeeCollection* (*search_finish) (PluginContactStore* self, GAsyncResult* _res_, GError** error);
	void (*load) (PluginContactStore* self, GearyRFC822MailboxAddress* mailbox, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	ApplicationContact* (*load_finish) (PluginContactStore* self, GAsyncResult* _res_, GError** error);
};

struct _PluginEmailStoreIface {
	GTypeInterface parent_iface;
	const GVariantType* (*get_email_identifier_variant_type) (PluginEmailStore* self);
	void (*get_email) (PluginEmailStore* self, GeeCollection* ids, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GeeCollection* (*get_email_finish) (PluginEmailStore* self, GAsyncResult* _res_, GError** error);
	PluginEmailIdentifier* (*get_email_identifier_for_variant) (PluginEmailStore* self, GVariant* id);
};

struct _PluginEmailContextIface {
	GTypeInterface parent_iface;
	void (*get_email_store) (PluginEmailContext* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	PluginEmailStore* (*get_email_store_finish) (PluginEmailContext* self, GAsyncResult* _res_, GError** error);
	void (*add_email_info_bar) (PluginEmailContext* self, PluginEmailIdentifier* displayed, PluginInfoBar* info_bar, guint priority);
	void (*remove_email_info_bar) (PluginEmailContext* self, PluginEmailIdentifier* displayed, PluginInfoBar* info_bar);
};

struct _PluginEmailExtensionIface {
	GTypeInterface parent_iface;
	PluginEmailContext* (*get_email) (PluginEmailExtension* self);
	void (*set_email) (PluginEmailExtension* self, PluginEmailContext* value);
};

struct _PluginFolderStoreIface {
	GTypeInterface parent_iface;
	const GVariantType* (*get_folder_variant_type) (PluginFolderStore* self);
	GeeCollection* (*get_folders) (PluginFolderStore* self);
	void (*list_containing_folders) (PluginFolderStore* self, PluginEmailIdentifier* target, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GeeCollection* (*list_containing_folders_finish) (PluginFolderStore* self, GAsyncResult* _res_, GError** error);
	void (*create_personal_folder) (PluginFolderStore* self, PluginAccount* target, const gchar* name, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	PluginFolder* (*create_personal_folder_finish) (PluginFolderStore* self, GAsyncResult* _res_, GError** error);
	PluginFolder* (*get_folder_for_variant) (PluginFolderStore* self, GVariant* id);
};

struct _PluginFolderContextIface {
	GTypeInterface parent_iface;
	void (*get_folder_store) (PluginFolderContext* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	PluginFolderStore* (*get_folder_store_finish) (PluginFolderContext* self, GAsyncResult* _res_, GError** error);
	void (*add_folder_info_bar) (PluginFolderContext* self, PluginFolder* selected, PluginInfoBar* infobar, guint priority);
	void (*remove_folder_info_bar) (PluginFolderContext* self, PluginFolder* selected, PluginInfoBar* infobar);
	void (*register_folder_used_as) (PluginFolderContext* self, PluginFolder* target, const gchar* name, const gchar* icon_name, GError** error);
	void (*unregister_folder_used_as) (PluginFolderContext* self, PluginFolder* target, GError** error);
};

struct _PluginFolderExtensionIface {
	GTypeInterface parent_iface;
	PluginFolderContext* (*get_folders) (PluginFolderExtension* self);
	void (*set_folders) (PluginFolderExtension* self, PluginFolderContext* value);
};

struct _PluginInfoBar {
	GearyBaseObject parent_instance;
	PluginInfoBarPrivate * priv;
};

struct _PluginInfoBarClass {
	GearyBaseObjectClass parent_class;
};

struct _PluginNotificationContextIface {
	GTypeInterface parent_iface;
	gint (*get_total_new_messages) (PluginNotificationContext* self);
	void (*get_contacts_for_folder) (PluginNotificationContext* self, PluginFolder* source, GAsyncReadyCallback _callback_, gpointer _user_data_);
	PluginContactStore* (*get_contacts_for_folder_finish) (PluginNotificationContext* self, GAsyncResult* _res_, GError** error);
	gboolean (*should_notify_new_messages) (PluginNotificationContext* self, PluginFolder* target);
	gint (*get_new_message_count) (PluginNotificationContext* self, PluginFolder* target, GError** error);
	void (*start_monitoring_folder) (PluginNotificationContext* self, PluginFolder* target);
	void (*stop_monitoring_folder) (PluginNotificationContext* self, PluginFolder* target);
	gboolean (*is_monitoring_folder) (PluginNotificationContext* self, PluginFolder* target);
};

struct _PluginNotificationExtensionIface {
	GTypeInterface parent_iface;
	PluginNotificationContext* (*get_notifications) (PluginNotificationExtension* self);
	void (*set_notifications) (PluginNotificationExtension* self, PluginNotificationContext* value);
};

struct _PluginPluginBase {
	GearyBaseObject parent_instance;
	PluginPluginBasePrivate * priv;
};

struct _PluginPluginBaseClass {
	GearyBaseObjectClass parent_class;
	void (*activate) (PluginPluginBase* self, gboolean is_startup, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*activate_finish) (PluginPluginBase* self, GAsyncResult* _res_, GError** error);
	void (*deactivate) (PluginPluginBase* self, gboolean is_shutdown, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*deactivate_finish) (PluginPluginBase* self, GAsyncResult* _res_, GError** error);
};

struct _PluginTrustedExtensionIface {
	GTypeInterface parent_iface;
	ApplicationClient* (*get_client_application) (PluginTrustedExtension* self);
	void (*set_client_application) (PluginTrustedExtension* self, ApplicationClient* value);
	ApplicationPluginManager* (*get_client_plugins) (PluginTrustedExtension* self);
	void (*set_client_plugins) (PluginTrustedExtension* self, ApplicationPluginManager* value);
};

struct _SidebarHeader {
	SidebarGrouping parent_instance;
	SidebarHeaderPrivate * priv;
};

struct _SidebarHeaderClass {
	SidebarGroupingClass parent_class;
};

struct _SidebarContextableIface {
	GTypeInterface parent_iface;
	GtkMenu* (*get_sidebar_context_menu) (SidebarContextable* self, GdkEventButton* event);
};

struct _SidebarCountCellRenderer {
	GtkCellRenderer parent_instance;
	SidebarCountCellRendererPrivate * priv;
};

struct _SidebarCountCellRendererClass {
	GtkCellRendererClass parent_class;
};

struct _SidebarDestroyableEntryIface {
	GTypeInterface parent_iface;
	void (*destroy_source) (SidebarDestroyableEntry* self);
};

struct _SidebarInternalDragSourceEntryIface {
	GTypeInterface parent_iface;
	void (*prepare_selection_data) (SidebarInternalDragSourceEntry* self, GtkSelectionData* data);
};

struct _UtilCacheLru {
	GearyBaseObject parent_instance;
	UtilCacheLruPrivate * priv;
};

struct _UtilCacheLruClass {
	GearyBaseObjectClass parent_class;
};

struct _UtilEmailSearchExpressionFactory {
	GearyBaseObject parent_instance;
	UtilEmailSearchExpressionFactoryPrivate * priv;
};

struct _UtilEmailSearchExpressionFactoryClass {
	GearyBaseObjectClass parent_class;
};

typedef enum  {
	UTIL_JS_ERROR_EXCEPTION,
	UTIL_JS_ERROR_TYPE
} UtilJSError;
#define UTIL_JS_ERROR util_js_error_quark ()

struct _UtilJSCallable {
	GTypeInstance parent_instance;
	volatile int ref_count;
	UtilJSCallablePrivate * priv;
};

struct _UtilJSCallableClass {
	GTypeClass parent_class;
	void (*finalize) (UtilJSCallable *self);
};

VALA_EXTERN GType application_account_context_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationAccountContext, g_object_unref)
VALA_EXTERN GearyAccount* application_account_context_get_account (ApplicationAccountContext* self);
VALA_EXTERN GearyAppEmailStore* application_account_context_get_emails (ApplicationAccountContext* self);
VALA_EXTERN GType application_contact_store_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationContactStore, g_object_unref)
VALA_EXTERN ApplicationContactStore* application_account_context_get_contacts (ApplicationAccountContext* self);
VALA_EXTERN GType application_command_stack_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationCommandStack, g_object_unref)
VALA_EXTERN ApplicationCommandStack* application_account_context_get_commands (ApplicationAccountContext* self);
VALA_EXTERN GCancellable* application_account_context_get_cancellable (ApplicationAccountContext* self);
VALA_EXTERN ApplicationAccountContext* application_account_context_new (GearyAccount* account,
                                                            GearyAppSearchFolder* search,
                                                            GearyAppEmailStore* emails,
                                                            ApplicationContactStore* contacts);
VALA_EXTERN ApplicationAccountContext* application_account_context_construct (GType object_type,
                                                                  GearyAccount* account,
                                                                  GearyAppSearchFolder* search,
                                                                  GearyAppEmailStore* emails,
                                                                  ApplicationContactStore* contacts);
VALA_EXTERN GearyAccountStatus application_account_context_get_effective_status (ApplicationAccountContext* self);
VALA_EXTERN GType application_folder_context_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationFolderContext, g_object_unref)
VALA_EXTERN GeeCollection* application_account_context_get_folders (ApplicationAccountContext* self);
VALA_EXTERN ApplicationFolderContext* application_account_context_get_folder (ApplicationAccountContext* self,
                                                                  GearyFolder* target);
VALA_EXTERN void application_account_context_add_folders (ApplicationAccountContext* self,
                                              GeeCollection* to_add);
VALA_EXTERN void application_account_context_remove_folders (ApplicationAccountContext* self,
                                                 GeeCollection* to_remove);
VALA_EXTERN GType application_attachment_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationAttachmentManager, g_object_unref)
VALA_EXTERN gchar* application_attachment_manager_untitled_file_name;
VALA_EXTERN GType application_main_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationMainWindow, g_object_unref)
VALA_EXTERN ApplicationAttachmentManager* application_attachment_manager_new (ApplicationMainWindow* parent);
VALA_EXTERN ApplicationAttachmentManager* application_attachment_manager_construct (GType object_type,
                                                                        ApplicationMainWindow* parent);
VALA_EXTERN void application_attachment_manager_save_attachments (ApplicationAttachmentManager* self,
                                                      GeeCollection* attachments,
                                                      GCancellable* cancellable,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_);
VALA_EXTERN gboolean application_attachment_manager_save_attachments_finish (ApplicationAttachmentManager* self,
                                                                 GAsyncResult* _res_);
VALA_EXTERN void application_attachment_manager_save_attachment (ApplicationAttachmentManager* self,
                                                     GearyAttachment* attachment,
                                                     const gchar* alt_name,
                                                     GCancellable* cancellable,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_);
VALA_EXTERN gboolean application_attachment_manager_save_attachment_finish (ApplicationAttachmentManager* self,
                                                                GAsyncResult* _res_);
VALA_EXTERN void application_attachment_manager_save_buffer (ApplicationAttachmentManager* self,
                                                 const gchar* display_name,
                                                 GearyMemoryBuffer* buffer,
                                                 GCancellable* cancellable,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
VALA_EXTERN gboolean application_attachment_manager_save_buffer_finish (ApplicationAttachmentManager* self,
                                                            GAsyncResult* _res_);
VALA_EXTERN GQuark application_certificate_manager_error_quark (void);
VALA_EXTERN GType application_certificate_manager_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType application_certificate_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationCertificateManager, g_object_unref)
VALA_EXTERN void application_certificate_manager_new (GFile* store_dir,
                                          GCancellable* cancellable,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN ApplicationCertificateManager* application_certificate_manager_new_finish (GAsyncResult* _res_);
VALA_EXTERN void application_certificate_manager_construct (GType object_type,
                                                GFile* store_dir,
                                                GCancellable* cancellable,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_);
VALA_EXTERN ApplicationCertificateManager* application_certificate_manager_construct_finish (GAsyncResult* _res_);
VALA_EXTERN void application_certificate_manager_prompt_pin_certificate (ApplicationCertificateManager* self,
                                                             GtkWindow* parent,
                                                             GearyAccountInformation* account,
                                                             GearyServiceInformation* service,
                                                             GearyEndpoint* endpoint,
                                                             gboolean is_validation,
                                                             GCancellable* cancellable,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN void application_certificate_manager_prompt_pin_certificate_finish (ApplicationCertificateManager* self,
                                                                    GAsyncResult* _res_,
                                                                    GError** error);
VALA_EXTERN GType application_client_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationClient, g_object_unref)
VALA_EXTERN GearyEngine* application_client_get_engine (ApplicationClient* self);
VALA_EXTERN GType application_configuration_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationConfiguration, g_object_unref)
VALA_EXTERN ApplicationConfiguration* application_client_get_config (ApplicationClient* self);
VALA_EXTERN ApplicationMainWindow* application_client_get_last_active_main_window (ApplicationClient* self);
VALA_EXTERN GType application_startup_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationStartupManager, g_object_unref)
VALA_EXTERN ApplicationStartupManager* application_client_get_autostart (ApplicationClient* self);
VALA_EXTERN gboolean application_client_get_is_background_service (ApplicationClient* self);
VALA_EXTERN gboolean application_client_get_is_flatpak_sandboxed (ApplicationClient* self);
VALA_EXTERN GType application_client_runtime_detail_get_type (void) G_GNUC_CONST ;
VALA_EXTERN ApplicationClientRuntimeDetail* application_client_runtime_detail_dup (const ApplicationClientRuntimeDetail* self);
VALA_EXTERN void application_client_runtime_detail_free (ApplicationClientRuntimeDetail* self);
VALA_EXTERN void application_client_runtime_detail_copy (const ApplicationClientRuntimeDetail* self,
                                             ApplicationClientRuntimeDetail* dest);
VALA_EXTERN void application_client_runtime_detail_destroy (ApplicationClientRuntimeDetail* self);
G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (ApplicationClientRuntimeDetail, application_client_runtime_detail_destroy)
VALA_EXTERN GeeCollection* application_client_get_runtime_information (ApplicationClient* self);
VALA_EXTERN ApplicationClient* application_client_new (void);
VALA_EXTERN ApplicationClient* application_client_construct (GType object_type);
VALA_EXTERN GeeCollection* application_client_get_main_windows (ApplicationClient* self);
VALA_EXTERN ApplicationMainWindow* application_client_get_active_main_window (ApplicationClient* self);
VALA_EXTERN void application_client_add_window_accelerators (ApplicationClient* self,
                                                 const gchar* action,
                                                 gchar** accelerators,
                                                 gint accelerators_length1,
                                                 GVariant* param);
VALA_EXTERN void application_client_add_edit_accelerators (ApplicationClient* self,
                                               const gchar* action,
                                               gchar** accelerators,
                                               gint accelerators_length1,
                                               GVariant* param);
VALA_EXTERN void application_client_show_about (ApplicationClient* self,
                                    GAsyncReadyCallback _callback_,
                                    gpointer _user_data_);
VALA_EXTERN void application_client_show_about_finish (ApplicationClient* self,
                                           GAsyncResult* _res_);
VALA_EXTERN void application_client_show_accounts (ApplicationClient* self,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN void application_client_show_accounts_finish (ApplicationClient* self,
                                              GAsyncResult* _res_);
VALA_EXTERN void application_client_show_email (ApplicationClient* self,
                                    GVariant* id,
                                    GAsyncReadyCallback _callback_,
                                    gpointer _user_data_);
VALA_EXTERN void application_client_show_email_finish (ApplicationClient* self,
                                           GAsyncResult* _res_);
VALA_EXTERN void application_client_show_folder (ApplicationClient* self,
                                     GVariant* id,
                                     GAsyncReadyCallback _callback_,
                                     gpointer _user_data_);
VALA_EXTERN void application_client_show_folder_finish (ApplicationClient* self,
                                            GAsyncResult* _res_);
VALA_EXTERN void application_client_show_inspector (ApplicationClient* self,
                                        GAsyncReadyCallback _callback_,
                                        gpointer _user_data_);
VALA_EXTERN void application_client_show_inspector_finish (ApplicationClient* self,
                                               GAsyncResult* _res_);
VALA_EXTERN void application_client_show_preferences (ApplicationClient* self,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN void application_client_show_preferences_finish (ApplicationClient* self,
                                                 GAsyncResult* _res_);
VALA_EXTERN void application_client_new_composer (ApplicationClient* self,
                                      GearyRFC822MailboxAddress* to,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN void application_client_new_composer_finish (ApplicationClient* self,
                                             GAsyncResult* _res_);
VALA_EXTERN void application_client_new_composer_mailto (ApplicationClient* self,
                                             const gchar* mailto,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN void application_client_new_composer_mailto_finish (ApplicationClient* self,
                                                    GAsyncResult* _res_);
VALA_EXTERN void application_client_new_window (ApplicationClient* self,
                                    GearyFolder* select_folder,
                                    GeeCollection* select_conversations,
                                    GAsyncReadyCallback _callback_,
                                    gpointer _user_data_);
VALA_EXTERN void application_client_new_window_finish (ApplicationClient* self,
                                           GAsyncResult* _res_);
VALA_EXTERN GFile* application_client_get_home_config_directory (ApplicationClient* self);
VALA_EXTERN GFile* application_client_get_home_cache_directory (ApplicationClient* self);
VALA_EXTERN GFile* application_client_get_home_data_directory (ApplicationClient* self);
VALA_EXTERN GFile* application_client_get_resource_directory (ApplicationClient* self);
VALA_EXTERN GFile* application_client_get_desktop_directory (ApplicationClient* self);
VALA_EXTERN GFile* application_client_get_web_extensions_dir (ApplicationClient* self);
VALA_EXTERN GFile* application_client_get_app_plugins_dir (ApplicationClient* self);
VALA_EXTERN void application_client_show_uri (ApplicationClient* self,
                                  const gchar* uri,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN void application_client_show_uri_finish (ApplicationClient* self,
                                         GAsyncResult* _res_);
VALA_EXTERN void application_client_quit (ApplicationClient* self);
VALA_EXTERN GType application_command_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationCommand, g_object_unref)
VALA_EXTERN gboolean application_command_get_can_undo (ApplicationCommand* self);
VALA_EXTERN gboolean application_command_get_can_redo (ApplicationCommand* self);
VALA_EXTERN const gchar* application_command_get_undo_label (ApplicationCommand* self);
VALA_EXTERN void application_command_set_undo_label (ApplicationCommand* self,
                                         const gchar* value);
VALA_EXTERN const gchar* application_command_get_redo_label (ApplicationCommand* self);
VALA_EXTERN void application_command_set_redo_label (ApplicationCommand* self,
                                         const gchar* value);
VALA_EXTERN const gchar* application_command_get_executed_label (ApplicationCommand* self);
VALA_EXTERN void application_command_set_executed_label (ApplicationCommand* self,
                                             const gchar* value);
VALA_EXTERN gboolean application_command_get_executed_notification_brief (ApplicationCommand* self);
VALA_EXTERN void application_command_set_executed_notification_brief (ApplicationCommand* self,
                                                          gboolean value);
VALA_EXTERN const gchar* application_command_get_undone_label (ApplicationCommand* self);
VALA_EXTERN void application_command_set_undone_label (ApplicationCommand* self,
                                           const gchar* value);
VALA_EXTERN void application_command_execute (ApplicationCommand* self,
                                  GCancellable* cancellable,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN void application_command_execute_finish (ApplicationCommand* self,
                                         GAsyncResult* _res_,
                                         GError** error);
VALA_EXTERN void application_command_undo (ApplicationCommand* self,
                               GCancellable* cancellable,
                               GAsyncReadyCallback _callback_,
                               gpointer _user_data_);
VALA_EXTERN void application_command_undo_finish (ApplicationCommand* self,
                                      GAsyncResult* _res_,
                                      GError** error);
VALA_EXTERN void application_command_redo (ApplicationCommand* self,
                               GCancellable* cancellable,
                               GAsyncReadyCallback _callback_,
                               gpointer _user_data_);
VALA_EXTERN void application_command_redo_finish (ApplicationCommand* self,
                                      GAsyncResult* _res_,
                                      GError** error);
VALA_EXTERN gboolean application_command_equal_to (ApplicationCommand* self,
                                       ApplicationCommand* other);
VALA_EXTERN gchar* application_command_to_string (ApplicationCommand* self);
VALA_EXTERN ApplicationCommand* application_command_construct (GType object_type);
VALA_EXTERN GType application_command_sequence_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationCommandSequence, g_object_unref)
VALA_EXTERN ApplicationCommandSequence* application_command_sequence_new (ApplicationCommand** commands,
                                                              gint commands_length1);
VALA_EXTERN ApplicationCommandSequence* application_command_sequence_construct (GType object_type,
                                                                    ApplicationCommand** commands,
                                                                    gint commands_length1);
VALA_EXTERN GType application_property_command_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationPropertyCommand, g_object_unref)
VALA_EXTERN ApplicationPropertyCommand* application_property_command_new (GType t_type,
                                                              GBoxedCopyFunc t_dup_func,
                                                              GDestroyNotify t_destroy_func,
                                                              GObject* object,
                                                              const gchar* property_name,
                                                              gconstpointer new_value,
                                                              const gchar* undo_label,
                                                              const gchar* redo_label,
                                                              const gchar* executed_label,
                                                              const gchar* undone_label);
VALA_EXTERN ApplicationPropertyCommand* application_property_command_construct (GType object_type,
                                                                    GType t_type,
                                                                    GBoxedCopyFunc t_dup_func,
                                                                    GDestroyNotify t_destroy_func,
                                                                    GObject* object,
                                                                    const gchar* property_name,
                                                                    gconstpointer new_value,
                                                                    const gchar* undo_label,
                                                                    const gchar* redo_label,
                                                                    const gchar* executed_label,
                                                                    const gchar* undone_label);
VALA_EXTERN gboolean application_command_stack_get_can_undo (ApplicationCommandStack* self);
VALA_EXTERN gboolean application_command_stack_get_can_redo (ApplicationCommandStack* self);
VALA_EXTERN void application_command_stack_execute (ApplicationCommandStack* self,
                                        ApplicationCommand* target,
                                        GCancellable* cancellable,
                                        GAsyncReadyCallback _callback_,
                                        gpointer _user_data_);
VALA_EXTERN void application_command_stack_execute_finish (ApplicationCommandStack* self,
                                               GAsyncResult* _res_,
                                               GError** error);
VALA_EXTERN void application_command_stack_undo (ApplicationCommandStack* self,
                                     GCancellable* cancellable,
                                     GAsyncReadyCallback _callback_,
                                     gpointer _user_data_);
VALA_EXTERN void application_command_stack_undo_finish (ApplicationCommandStack* self,
                                            GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN void application_command_stack_redo (ApplicationCommandStack* self,
                                     GCancellable* cancellable,
                                     GAsyncReadyCallback _callback_,
                                     gpointer _user_data_);
VALA_EXTERN void application_command_stack_redo_finish (ApplicationCommandStack* self,
                                            GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN ApplicationCommand* application_command_stack_peek_undo (ApplicationCommandStack* self);
VALA_EXTERN ApplicationCommand* application_command_stack_peek_redo (ApplicationCommandStack* self);
VALA_EXTERN void application_command_stack_clear (ApplicationCommandStack* self);
VALA_EXTERN void application_command_stack_update_undo_stack (ApplicationCommandStack* self,
                                                  ApplicationCommand* target);
VALA_EXTERN void application_command_stack_update_redo_stack (ApplicationCommandStack* self,
                                                  ApplicationCommand* target);
VALA_EXTERN ApplicationCommandStack* application_command_stack_new (void);
VALA_EXTERN ApplicationCommandStack* application_command_stack_construct (GType object_type);
VALA_EXTERN GType application_configuration_desktop_environment_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void application_configuration_init (gboolean is_installed,
                                     const gchar* schema_dir);
VALA_EXTERN GSettings* application_configuration_get_settings (ApplicationConfiguration* self);
VALA_EXTERN GSettings* application_configuration_get_gnome_interface (ApplicationConfiguration* self);
VALA_EXTERN gboolean application_configuration_get_enable_debug (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_enable_debug (ApplicationConfiguration* self,
                                                 gboolean value);
VALA_EXTERN gboolean application_configuration_get_enable_inspector (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_enable_inspector (ApplicationConfiguration* self,
                                                     gboolean value);
VALA_EXTERN gboolean application_configuration_get_revoke_certs (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_revoke_certs (ApplicationConfiguration* self,
                                                 gboolean value);
VALA_EXTERN ApplicationConfigurationDesktopEnvironment application_configuration_get_desktop_environment (ApplicationConfiguration* self);
VALA_EXTERN gint application_configuration_get_window_width (ApplicationConfiguration* self);
VALA_EXTERN gint application_configuration_get_window_height (ApplicationConfiguration* self);
VALA_EXTERN gboolean application_configuration_get_window_maximize (ApplicationConfiguration* self);
VALA_EXTERN gboolean application_configuration_get_formatting_toolbar_visible (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_formatting_toolbar_visible (ApplicationConfiguration* self,
                                                               gboolean value);
VALA_EXTERN gboolean application_configuration_get_autoselect (ApplicationConfiguration* self);
VALA_EXTERN gboolean application_configuration_get_display_preview (ApplicationConfiguration* self);
VALA_EXTERN gboolean application_configuration_get_single_key_shortcuts (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_single_key_shortcuts (ApplicationConfiguration* self,
                                                         gboolean value);
VALA_EXTERN gboolean application_configuration_get_run_in_background (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_run_in_background (ApplicationConfiguration* self,
                                                      gboolean value);
VALA_EXTERN GType util_date_clock_format_get_type (void) G_GNUC_CONST ;
VALA_EXTERN UtilDateClockFormat application_configuration_get_clock_format (ApplicationConfiguration* self);
VALA_EXTERN gboolean application_configuration_get_ask_open_attachment (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_ask_open_attachment (ApplicationConfiguration* self,
                                                        gboolean value);
VALA_EXTERN gboolean application_configuration_get_compose_as_html (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_compose_as_html (ApplicationConfiguration* self,
                                                    gboolean value);
VALA_EXTERN gdouble application_configuration_get_conversation_viewer_zoom (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_conversation_viewer_zoom (ApplicationConfiguration* self,
                                                             gdouble value);
VALA_EXTERN gint application_configuration_get_undo_send_delay (ApplicationConfiguration* self);
VALA_EXTERN gint application_configuration_get_brief_notification_duration (ApplicationConfiguration* self);
VALA_EXTERN ApplicationConfiguration* application_configuration_new (const gchar* schema_id);
VALA_EXTERN ApplicationConfiguration* application_configuration_construct (GType object_type,
                                                               const gchar* schema_id);
VALA_EXTERN void application_configuration_bind (ApplicationConfiguration* self,
                                     const gchar* key,
                                     GObject* object,
                                     const gchar* property,
                                     GSettingsBindFlags flags);
VALA_EXTERN void application_configuration_bind_with_mapping (ApplicationConfiguration* self,
                                                  const gchar* key,
                                                  GObject* object,
                                                  const gchar* property,
                                                  GSettingsBindGetMapping get_mapping,
                                                  GSettingsBindSetMapping set_mapping,
                                                  GSettingsBindFlags flags);
VALA_EXTERN gint* application_configuration_get_composer_window_size (ApplicationConfiguration* self,
                                                          gint* result_length1);
VALA_EXTERN void application_configuration_set_composer_window_size (ApplicationConfiguration* self,
                                                         gint* value,
                                                         gint value_length1);
VALA_EXTERN gchar** application_configuration_get_images_trusted_domains (ApplicationConfiguration* self,
                                                              gint* result_length1);
VALA_EXTERN void application_configuration_set_images_trusted_domains (ApplicationConfiguration* self,
                                                           gchar** value,
                                                           gint value_length1);
VALA_EXTERN void application_configuration_add_images_trusted_domain (ApplicationConfiguration* self,
                                                          const gchar* domain);
VALA_EXTERN void application_configuration_remove_images_trusted_domain (ApplicationConfiguration* self,
                                                             const gchar* domain);
VALA_EXTERN gchar** application_configuration_get_optional_plugins (ApplicationConfiguration* self,
                                                        gint* result_length1);
VALA_EXTERN void application_configuration_set_optional_plugins (ApplicationConfiguration* self,
                                                     gchar** value,
                                                     gint value_length1);
VALA_EXTERN gchar** application_configuration_get_spell_check_languages (ApplicationConfiguration* self,
                                                             gint* result_length1);
VALA_EXTERN void application_configuration_set_spell_check_languages (ApplicationConfiguration* self,
                                                          gchar** value,
                                                          gint value_length1);
VALA_EXTERN gchar** application_configuration_get_spell_check_visible_languages (ApplicationConfiguration* self,
                                                                     gint* result_length1);
VALA_EXTERN void application_configuration_set_spell_check_visible_languages (ApplicationConfiguration* self,
                                                                  gchar** value,
                                                                  gint value_length1);
VALA_EXTERN GearySearchQueryStrategy application_configuration_get_search_strategy (ApplicationConfiguration* self);
VALA_EXTERN void application_configuration_set_search_strategy (ApplicationConfiguration* self,
                                                    GearySearchQueryStrategy strategy);
VALA_EXTERN GearyAccount* application_contact_store_get_account (ApplicationContactStore* self);
VALA_EXTERN void application_contact_store_load (ApplicationContactStore* self,
                                     GearyRFC822MailboxAddress* mailbox,
                                     GCancellable* cancellable,
                                     GAsyncReadyCallback _callback_,
                                     gpointer _user_data_);
VALA_EXTERN GType application_contact_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationContact, g_object_unref)
VALA_EXTERN ApplicationContact* application_contact_store_load_finish (ApplicationContactStore* self,
                                                           GAsyncResult* _res_,
                                                           GError** error);
VALA_EXTERN void application_contact_store_search (ApplicationContactStore* self,
                                       const gchar* query,
                                       guint min_importance,
                                       guint limit,
                                       GCancellable* cancellable,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN GeeCollection* application_contact_store_search_finish (ApplicationContactStore* self,
                                                        GAsyncResult* _res_,
                                                        GError** error);
VALA_EXTERN const gchar* application_contact_get_display_name (ApplicationContact* self);
VALA_EXTERN GLoadableIcon* application_contact_get_avatar (ApplicationContact* self);
VALA_EXTERN gboolean application_contact_get_display_name_is_email (ApplicationContact* self);
VALA_EXTERN gboolean application_contact_get_is_desktop_contact (ApplicationContact* self);
VALA_EXTERN gboolean application_contact_get_is_trusted (ApplicationContact* self);
VALA_EXTERN gboolean application_contact_get_is_favourite (ApplicationContact* self);
VALA_EXTERN gboolean application_contact_get_load_remote_resources (ApplicationContact* self);
VALA_EXTERN GeeCollection* application_contact_get_email_addresses (ApplicationContact* self);
VALA_EXTERN gboolean application_contact_equal_to (ApplicationContact* self,
                                       ApplicationContact* other);
VALA_EXTERN void application_contact_save_to_desktop (ApplicationContact* self,
                                          GCancellable* cancellable,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN void application_contact_save_to_desktop_finish (ApplicationContact* self,
                                                 GAsyncResult* _res_,
                                                 GError** error);
VALA_EXTERN void application_contact_open_on_desktop (ApplicationContact* self,
                                          GCancellable* cancellable,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN void application_contact_open_on_desktop_finish (ApplicationContact* self,
                                                 GAsyncResult* _res_,
                                                 GError** error);
VALA_EXTERN void application_contact_set_remote_resource_loading (ApplicationContact* self,
                                                      gboolean enabled,
                                                      GCancellable* cancellable,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_);
VALA_EXTERN void application_contact_set_remote_resource_loading_finish (ApplicationContact* self,
                                                             GAsyncResult* _res_,
                                                             GError** error);
VALA_EXTERN void application_contact_set_favourite (ApplicationContact* self,
                                        gboolean is_favourite,
                                        GCancellable* cancellable,
                                        GAsyncReadyCallback _callback_,
                                        gpointer _user_data_);
VALA_EXTERN void application_contact_set_favourite_finish (ApplicationContact* self,
                                               GAsyncResult* _res_,
                                               GError** error);
VALA_EXTERN gchar* application_contact_to_string (ApplicationContact* self);
VALA_EXTERN GType application_email_command_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationEmailCommand, g_object_unref)
VALA_EXTERN GType application_email_command_state_change_policy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyFolder* application_email_command_get_location (ApplicationEmailCommand* self);
VALA_EXTERN void application_email_command_set_location (ApplicationEmailCommand* self,
                                             GearyFolder* value);
VALA_EXTERN GeeCollection* application_email_command_get_conversations (ApplicationEmailCommand* self);
VALA_EXTERN GeeCollection* application_email_command_get_email (ApplicationEmailCommand* self);
VALA_EXTERN ApplicationEmailCommand* application_email_command_construct (GType object_type,
                                                              GearyFolder* location,
                                                              GeeCollection* conversations,
                                                              GeeCollection* email);
VALA_EXTERN GType application_trivial_command_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationTrivialCommand, g_object_unref)
VALA_EXTERN GType application_folder_context_email_count_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyFolder* application_folder_context_get_folder (ApplicationFolderContext* self);
VALA_EXTERN const gchar* application_folder_context_get_display_name (ApplicationFolderContext* self);
VALA_EXTERN void application_folder_context_set_display_name (ApplicationFolderContext* self,
                                                  const gchar* value);
VALA_EXTERN const gchar* application_folder_context_get_icon_name (ApplicationFolderContext* self);
VALA_EXTERN void application_folder_context_set_icon_name (ApplicationFolderContext* self,
                                               const gchar* value);
VALA_EXTERN ApplicationFolderContextEmailCount application_folder_context_get_displayed_count (ApplicationFolderContext* self);
VALA_EXTERN void application_folder_context_set_displayed_count (ApplicationFolderContext* self,
                                                     ApplicationFolderContextEmailCount value);
VALA_EXTERN ApplicationFolderContext* application_folder_context_new (GearyFolder* folder);
VALA_EXTERN ApplicationFolderContext* application_folder_context_construct (GType object_type,
                                                                GearyFolder* folder);
VALA_EXTERN GType components_conversation_header_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsConversationHeaderBar, g_object_unref)
VALA_EXTERN void application_main_window_add_accelerators (ApplicationClient* owner);
VALA_EXTERN ApplicationClient* application_main_window_get_application (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_set_application (ApplicationMainWindow* self,
                                              ApplicationClient* value);
VALA_EXTERN GearyAccount* application_main_window_get_selected_account (ApplicationMainWindow* self);
VALA_EXTERN GearyFolder* application_main_window_get_selected_folder (ApplicationMainWindow* self);
VALA_EXTERN GearyAppConversationMonitor* application_main_window_get_conversations (ApplicationMainWindow* self);
VALA_EXTERN gboolean application_main_window_get_is_folder_list_shown (ApplicationMainWindow* self);
VALA_EXTERN gboolean application_main_window_get_is_conversation_list_shown (ApplicationMainWindow* self);
VALA_EXTERN gboolean application_main_window_get_is_conversation_viewer_shown (ApplicationMainWindow* self);
VALA_EXTERN ApplicationAttachmentManager* application_main_window_get_attachments (ApplicationMainWindow* self);
VALA_EXTERN gboolean application_main_window_get_selected_folder_supports_trash (ApplicationMainWindow* self);
VALA_EXTERN gboolean application_main_window_get_has_composer (ApplicationMainWindow* self);
VALA_EXTERN gboolean application_main_window_get_is_shift_down (ApplicationMainWindow* self);
VALA_EXTERN gint application_main_window_get_window_width (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_set_window_width (ApplicationMainWindow* self,
                                               gint value);
VALA_EXTERN gint application_main_window_get_window_height (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_set_window_height (ApplicationMainWindow* self,
                                                gint value);
VALA_EXTERN gboolean application_main_window_get_window_maximized (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_set_window_maximized (ApplicationMainWindow* self,
                                                   gboolean value);
VALA_EXTERN GType sidebar_tree_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarTree, g_object_unref)
VALA_EXTERN GType folder_list_tree_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListTree, g_object_unref)
VALA_EXTERN FolderListTree* application_main_window_get_folder_list (ApplicationMainWindow* self);
VALA_EXTERN GType search_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SearchBar, g_object_unref)
VALA_EXTERN SearchBar* application_main_window_get_search_bar (ApplicationMainWindow* self);
VALA_EXTERN GType conversation_list_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationListView, g_object_unref)
VALA_EXTERN ConversationListView* application_main_window_get_conversation_list_view (ApplicationMainWindow* self);
VALA_EXTERN GType conversation_viewer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationViewer, g_object_unref)
VALA_EXTERN ConversationViewer* application_main_window_get_conversation_viewer (ApplicationMainWindow* self);
VALA_EXTERN GType components_info_bar_stack_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInfoBarStack, g_object_unref)
VALA_EXTERN ComponentsInfoBarStack* application_main_window_get_conversation_list_info_bars (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_update_title (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_update_account_status (ApplicationMainWindow* self,
                                                    GearyAccountStatus status,
                                                    gboolean has_auth_error,
                                                    gboolean has_cert_error,
                                                    GearyAccount* problem_source);
VALA_EXTERN void application_main_window_select_folder (ApplicationMainWindow* self,
                                            GearyFolder* to_select,
                                            gboolean is_interactive,
                                            gboolean inhibit_autoselect,
                                            GAsyncReadyCallback _callback_,
                                            gpointer _user_data_);
VALA_EXTERN void application_main_window_select_folder_finish (ApplicationMainWindow* self,
                                                   GAsyncResult* _res_);
VALA_EXTERN void application_main_window_show_conversations (ApplicationMainWindow* self,
                                                 GearyFolder* location,
                                                 GeeCollection* to_show,
                                                 gboolean is_interactive,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
VALA_EXTERN void application_main_window_show_conversations_finish (ApplicationMainWindow* self,
                                                        GAsyncResult* _res_);
VALA_EXTERN void application_main_window_show_email (ApplicationMainWindow* self,
                                         GearyFolder* location,
                                         GeeCollection* to_show,
                                         gboolean is_interactive,
                                         GAsyncReadyCallback _callback_,
                                         gpointer _user_data_);
VALA_EXTERN void application_main_window_show_email_finish (ApplicationMainWindow* self,
                                                GAsyncResult* _res_);
VALA_EXTERN void application_main_window_show_window_menu (ApplicationMainWindow* self);
VALA_EXTERN void application_main_window_show_search_bar (ApplicationMainWindow* self,
                                              const gchar* text);
VALA_EXTERN GType components_info_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInfoBar, g_object_unref)
VALA_EXTERN void application_main_window_show_info_bar (ApplicationMainWindow* self,
                                            ComponentsInfoBar* info_bar);
VALA_EXTERN GType components_in_app_notification_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInAppNotification, g_object_unref)
VALA_EXTERN void application_main_window_add_notification (ApplicationMainWindow* self,
                                               ComponentsInAppNotification* notification);
VALA_EXTERN GType application_plugin_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ApplicationPluginManager, g_object_unref)
VALA_EXTERN GType plugin_account_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginAccount, g_object_unref)
VALA_EXTERN ApplicationAccountContext* application_plugin_manager_to_client_account (ApplicationPluginManager* self,
                                                                         PluginAccount* plugin);
VALA_EXTERN GearyAccount* application_plugin_manager_to_engine_account (ApplicationPluginManager* self,
                                                            PluginAccount* plugin);
VALA_EXTERN GType plugin_folder_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginFolder, g_object_unref)
VALA_EXTERN GearyFolder* application_plugin_manager_to_engine_folder (ApplicationPluginManager* self,
                                                          PluginFolder* plugin);
VALA_EXTERN GType plugin_email_identifier_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType plugin_email_body_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType plugin_email_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyEmail* application_plugin_manager_to_engine_email (ApplicationPluginManager* self,
                                                        PluginEmail* plugin);
VALA_EXTERN GeeCollection* application_plugin_manager_get_optional_plugins (ApplicationPluginManager* self);
VALA_EXTERN gboolean application_plugin_manager_load_optional (ApplicationPluginManager* self,
                                                   PeasPluginInfo* plugin,
                                                   GError** error);
VALA_EXTERN gboolean application_plugin_manager_unload_optional (ApplicationPluginManager* self,
                                                     PeasPluginInfo* plugin,
                                                     GError** error);
VALA_EXTERN GType portal_request_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (portalRequest, g_object_unref)
VALA_EXTERN GType portal_request_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint portal_request_register_object (void* object,
                                      GDBusConnection* connection,
                                      const gchar* path,
                                      GError** error);
VALA_EXTERN GType portal_background_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint portal_background_register_object (void* object,
                                         GDBusConnection* connection,
                                         const gchar* path,
                                         GError** error);
VALA_EXTERN GType portal_background_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (portalBackground, g_object_unref)
VALA_EXTERN char* portal_background_request_background (portalBackground* self,
                                            const gchar* parent_window,
                                            GHashTable* options,
                                            GError** error);
VALA_EXTERN ApplicationStartupManager* application_startup_manager_new (ApplicationClient* app);
VALA_EXTERN ApplicationStartupManager* application_startup_manager_construct (GType object_type,
                                                                  ApplicationClient* app);
VALA_EXTERN GType goa_mediator_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GoaMediator, g_object_unref)
VALA_EXTERN GoaMediator* goa_mediator_new (GoaObject* handle);
VALA_EXTERN GoaMediator* goa_mediator_construct (GType object_type,
                                     GoaObject* handle);
VALA_EXTERN GearyServiceProvider goa_mediator_get_service_provider (GoaMediator* self);
VALA_EXTERN gchar* goa_mediator_get_service_label (GoaMediator* self);
VALA_EXTERN void goa_mediator_update (GoaMediator* self,
                          GearyAccountInformation* geary_account,
                          GCancellable* cancellable,
                          GAsyncReadyCallback _callback_,
                          gpointer _user_data_);
VALA_EXTERN void goa_mediator_update_finish (GoaMediator* self,
                                 GAsyncResult* _res_,
                                 GError** error);
VALA_EXTERN void goa_mediator_load_token (GoaMediator* self,
                              GearyAccountInformation* account,
                              GearyServiceInformation* service,
                              GCancellable* cancellable,
                              GAsyncReadyCallback _callback_,
                              gpointer _user_data_);
VALA_EXTERN gboolean goa_mediator_load_token_finish (GoaMediator* self,
                                         GAsyncResult* _res_,
                                         GError** error);
VALA_EXTERN GType secret_mediator_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SecretMediator, g_object_unref)
VALA_EXTERN void secret_mediator_new (ApplicationClient* application,
                          GCancellable* cancellable,
                          GAsyncReadyCallback _callback_,
                          gpointer _user_data_);
VALA_EXTERN SecretMediator* secret_mediator_new_finish (GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN void secret_mediator_construct (GType object_type,
                                ApplicationClient* application,
                                GCancellable* cancellable,
                                GAsyncReadyCallback _callback_,
                                gpointer _user_data_);
VALA_EXTERN SecretMediator* secret_mediator_construct_finish (GAsyncResult* _res_,
                                                  GError** error);
VALA_EXTERN void secret_mediator_load_token (SecretMediator* self,
                                 GearyAccountInformation* account,
                                 GearyServiceInformation* service,
                                 GCancellable* cancellable,
                                 GAsyncReadyCallback _callback_,
                                 gpointer _user_data_);
VALA_EXTERN gboolean secret_mediator_load_token_finish (SecretMediator* self,
                                            GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN void secret_mediator_update_token (SecretMediator* self,
                                   GearyAccountInformation* account,
                                   GearyServiceInformation* service,
                                   GCancellable* cancellable,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN void secret_mediator_update_token_finish (SecretMediator* self,
                                          GAsyncResult* _res_,
                                          GError** error);
VALA_EXTERN void secret_mediator_clear_token (SecretMediator* self,
                                  GearyAccountInformation* account,
                                  GearyServiceInformation* service,
                                  GCancellable* cancellable,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN void secret_mediator_clear_token_finish (SecretMediator* self,
                                         GAsyncResult* _res_,
                                         GError** error);
VALA_EXTERN GType accounts_editor_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsEditor, g_object_unref)
VALA_EXTERN ApplicationClient* accounts_editor_get_application (AccountsEditor* self);
VALA_EXTERN void accounts_editor_set_application (AccountsEditor* self,
                                      ApplicationClient* value);
VALA_EXTERN AccountsEditor* accounts_editor_new (ApplicationClient* application,
                                     GtkWindow* parent);
VALA_EXTERN AccountsEditor* accounts_editor_construct (GType object_type,
                                           ApplicationClient* application,
                                           GtkWindow* parent);
VALA_EXTERN GType components_web_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsWebView, g_object_unref)
VALA_EXTERN GType accounts_signature_web_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsSignatureWebView, g_object_unref)
VALA_EXTERN void accounts_signature_web_view_load_resources (GError** error);
VALA_EXTERN AccountsSignatureWebView* accounts_signature_web_view_new (ApplicationConfiguration* config);
VALA_EXTERN AccountsSignatureWebView* accounts_signature_web_view_construct (GType object_type,
                                                                 ApplicationConfiguration* config);
VALA_EXTERN GType accounts_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsManager, g_object_unref)
VALA_EXTERN GType accounts_manager_status_get_type (void) G_GNUC_CONST ;
VALA_EXTERN AccountsManagerStatus accounts_manager_status_for_value (const gchar* value,
                                                         GError** error);
VALA_EXTERN gchar* accounts_manager_status_to_value (AccountsManagerStatus self);
VALA_EXTERN gint accounts_manager_get_size (AccountsManager* self);
VALA_EXTERN GFile* accounts_manager_get_config_dir (AccountsManager* self);
VALA_EXTERN GFile* accounts_manager_get_data_dir (AccountsManager* self);
VALA_EXTERN AccountsManager* accounts_manager_new (GearyCredentialsMediator* local_mediator,
                                       GFile* config_dir,
                                       GFile* data_dir);
VALA_EXTERN AccountsManager* accounts_manager_construct (GType object_type,
                                             GearyCredentialsMediator* local_mediator,
                                             GFile* config_dir,
                                             GFile* data_dir);
VALA_EXTERN void accounts_manager_connect_goa (AccountsManager* self,
                                   GCancellable* cancellable,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN void accounts_manager_connect_goa_finish (AccountsManager* self,
                                          GAsyncResult* _res_,
                                          GError** error);
VALA_EXTERN GearyAccountInformation* accounts_manager_get_account (AccountsManager* self,
                                                       const gchar* id);
VALA_EXTERN AccountsManagerStatus accounts_manager_get_status (AccountsManager* self,
                                                   GearyAccountInformation* account);
VALA_EXTERN GearyIterable* accounts_manager_iterable (AccountsManager* self);
VALA_EXTERN gchar* accounts_manager_get_account_name (AccountsManager* self);
VALA_EXTERN void accounts_manager_new_orphan_account (AccountsManager* self,
                                          GearyServiceProvider provider,
                                          GearyRFC822MailboxAddress* primary_mailbox,
                                          GCancellable* cancellable,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN GearyAccountInformation* accounts_manager_new_orphan_account_finish (AccountsManager* self,
                                                                     GAsyncResult* _res_);
VALA_EXTERN void accounts_manager_create_account (AccountsManager* self,
                                      GearyAccountInformation* account,
                                      GCancellable* cancellable,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN void accounts_manager_create_account_finish (AccountsManager* self,
                                             GAsyncResult* _res_,
                                             GError** error);
VALA_EXTERN void accounts_manager_load_accounts (AccountsManager* self,
                                     GCancellable* cancellable,
                                     GAsyncReadyCallback _callback_,
                                     gpointer _user_data_);
VALA_EXTERN void accounts_manager_load_accounts_finish (AccountsManager* self,
                                            GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN void accounts_manager_disable_account (AccountsManager* self,
                                       GearyAccountInformation* account);
VALA_EXTERN void accounts_manager_remove_account (AccountsManager* self,
                                      GearyAccountInformation* account,
                                      GCancellable* cancellable,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN void accounts_manager_remove_account_finish (AccountsManager* self,
                                             GAsyncResult* _res_,
                                             GError** error);
VALA_EXTERN void accounts_manager_restore_account (AccountsManager* self,
                                       GearyAccountInformation* account,
                                       GCancellable* cancellable,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN void accounts_manager_restore_account_finish (AccountsManager* self,
                                              GAsyncResult* _res_,
                                              GError** error);
VALA_EXTERN void accounts_manager_expunge_accounts (AccountsManager* self,
                                        GCancellable* cancellable,
                                        GAsyncReadyCallback _callback_,
                                        gpointer _user_data_);
VALA_EXTERN void accounts_manager_expunge_accounts_finish (AccountsManager* self,
                                               GAsyncResult* _res_,
                                               GError** error);
VALA_EXTERN void accounts_manager_save_account (AccountsManager* self,
                                    GearyAccountInformation* info,
                                    GCancellable* cancellable,
                                    GAsyncReadyCallback _callback_,
                                    gpointer _user_data_);
VALA_EXTERN void accounts_manager_save_account_finish (AccountsManager* self,
                                           GAsyncResult* _res_,
                                           GError** error);
VALA_EXTERN void accounts_manager_update_local_credentials (AccountsManager* self,
                                                GearyAccountInformation* account,
                                                GearyServiceInformation* old_service,
                                                GearyServiceInformation* new_service,
                                                GCancellable* cancellable,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_);
VALA_EXTERN void accounts_manager_update_local_credentials_finish (AccountsManager* self,
                                                       GAsyncResult* _res_,
                                                       GError** error);
VALA_EXTERN gboolean accounts_manager_is_goa_account (AccountsManager* self,
                                          GearyAccountInformation* account);
VALA_EXTERN void accounts_manager_add_goa_account (AccountsManager* self,
                                       GearyServiceProvider type,
                                       GCancellable* cancellable,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN void accounts_manager_add_goa_account_finish (AccountsManager* self,
                                              GAsyncResult* _res_,
                                              GError** error);
VALA_EXTERN void accounts_manager_show_goa_account (AccountsManager* self,
                                        GearyAccountInformation* account,
                                        GCancellable* cancellable,
                                        GAsyncReadyCallback _callback_,
                                        gpointer _user_data_);
VALA_EXTERN void accounts_manager_show_goa_account_finish (AccountsManager* self,
                                               GAsyncResult* _res_,
                                               GError** error);
VALA_EXTERN GQuark accounts_config_error_quark (void);
VALA_EXTERN GType accounts_config_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType accounts_account_config_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsAccountConfig, g_object_unref)
VALA_EXTERN GearyAccountInformation* accounts_account_config_load (AccountsAccountConfig* self,
                                                       GearyConfigFile* config,
                                                       const gchar* id,
                                                       GearyCredentialsMediator* mediator,
                                                       GearyServiceProvider* default_provider,
                                                       const gchar* default_name,
                                                       GError** error);
VALA_EXTERN void accounts_account_config_save (AccountsAccountConfig* self,
                                   GearyAccountInformation* account,
                                   GearyConfigFile* config);
VALA_EXTERN GType accounts_service_config_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsServiceConfig, g_object_unref)
VALA_EXTERN void accounts_service_config_load (AccountsServiceConfig* self,
                                   GearyConfigFile* config,
                                   GearyAccountInformation* account,
                                   GearyServiceInformation* service,
                                   GError** error);
VALA_EXTERN void accounts_service_config_save (AccountsServiceConfig* self,
                                   GearyAccountInformation* account,
                                   GearyServiceInformation* service,
                                   GearyConfigFile* config);
VALA_EXTERN GType accounts_account_config_v1_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsAccountConfigV1, g_object_unref)
VALA_EXTERN AccountsAccountConfigV1* accounts_account_config_v1_new (gboolean is_managed);
VALA_EXTERN AccountsAccountConfigV1* accounts_account_config_v1_construct (GType object_type,
                                                               gboolean is_managed);
VALA_EXTERN GType accounts_account_config_legacy_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsAccountConfigLegacy, g_object_unref)
VALA_EXTERN AccountsAccountConfigLegacy* accounts_account_config_legacy_new (void);
VALA_EXTERN AccountsAccountConfigLegacy* accounts_account_config_legacy_construct (GType object_type);
VALA_EXTERN GType accounts_service_config_v1_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsServiceConfigV1, g_object_unref)
VALA_EXTERN AccountsServiceConfigV1* accounts_service_config_v1_new (void);
VALA_EXTERN AccountsServiceConfigV1* accounts_service_config_v1_construct (GType object_type);
VALA_EXTERN GType accounts_service_config_legacy_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AccountsServiceConfigLegacy, g_object_unref)
VALA_EXTERN AccountsServiceConfigLegacy* accounts_service_config_legacy_new (void);
VALA_EXTERN AccountsServiceConfigLegacy* accounts_service_config_legacy_construct (GType object_type);
VALA_EXTERN gchar* action_application_prefix (const gchar* action_name);
VALA_EXTERN gchar* action_window_prefix (const gchar* action_name);
VALA_EXTERN gchar* action_edit_prefix (const gchar* action_name);
VALA_EXTERN GType components_attachment_pane_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsAttachmentPane, g_object_unref)
VALA_EXTERN gboolean components_attachment_pane_get_edit_mode (ComponentsAttachmentPane* self);
VALA_EXTERN ComponentsAttachmentPane* components_attachment_pane_new (gboolean edit_mode,
                                                          ApplicationAttachmentManager* manager);
VALA_EXTERN ComponentsAttachmentPane* components_attachment_pane_construct (GType object_type,
                                                                gboolean edit_mode,
                                                                ApplicationAttachmentManager* manager);
VALA_EXTERN void components_attachment_pane_add_attachment (ComponentsAttachmentPane* self,
                                                GearyAttachment* attachment,
                                                GCancellable* cancellable);
VALA_EXTERN void components_attachment_pane_open_attachment (ComponentsAttachmentPane* self,
                                                 GearyAttachment* attachment);
VALA_EXTERN void components_attachment_pane_save_attachment (ComponentsAttachmentPane* self,
                                                 GearyAttachment* attachment);
VALA_EXTERN void components_attachment_pane_remove_attachment (ComponentsAttachmentPane* self,
                                                   GearyAttachment* attachment);
VALA_EXTERN gboolean components_attachment_pane_save_all (ComponentsAttachmentPane* self);
VALA_EXTERN GType components_conversation_actions_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsConversationActions, g_object_unref)
VALA_EXTERN gboolean components_conversation_actions_get_show_conversation_actions (ComponentsConversationActions* self);
VALA_EXTERN gboolean components_conversation_actions_get_show_response_actions (ComponentsConversationActions* self);
VALA_EXTERN gboolean components_conversation_actions_get_pack_justified (ComponentsConversationActions* self);
VALA_EXTERN GType folder_popover_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderPopover, g_object_unref)
VALA_EXTERN FolderPopover* components_conversation_actions_get_copy_move_popover (ComponentsConversationActions* self);
VALA_EXTERN gint components_conversation_actions_get_selected_conversations (ComponentsConversationActions* self);
VALA_EXTERN void components_conversation_actions_set_selected_conversations (ComponentsConversationActions* self,
                                                                 gint value);
VALA_EXTERN GearyAccount* components_conversation_actions_get_account (ComponentsConversationActions* self);
VALA_EXTERN void components_conversation_actions_set_account (ComponentsConversationActions* self,
                                                  GearyAccount* value);
VALA_EXTERN void components_conversation_actions_init (ComponentsConversationActions* self,
                                           ApplicationConfiguration* config);
VALA_EXTERN void components_conversation_actions_set_copy_sensitive (ComponentsConversationActions* self,
                                                         gboolean is_sensitive);
VALA_EXTERN void components_conversation_actions_set_mark_sensitive (ComponentsConversationActions* self,
                                                         gboolean is_sensitive);
VALA_EXTERN void components_conversation_actions_show_copy_menu (ComponentsConversationActions* self);
VALA_EXTERN void components_conversation_actions_set_mark_inverted (ComponentsConversationActions* self);
VALA_EXTERN void components_conversation_actions_update_trash_button (ComponentsConversationActions* self,
                                                          gboolean show_trash);
VALA_EXTERN ComponentsConversationActions* components_conversation_actions_new (void);
VALA_EXTERN ComponentsConversationActions* components_conversation_actions_construct (GType object_type);
VALA_EXTERN GType components_entry_undo_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsEntryUndo, g_object_unref)
VALA_EXTERN GtkEntry* components_entry_undo_get_target (ComponentsEntryUndo* self);
VALA_EXTERN ComponentsEntryUndo* components_entry_undo_new (GtkEntry* target);
VALA_EXTERN ComponentsEntryUndo* components_entry_undo_construct (GType object_type,
                                                      GtkEntry* target);
VALA_EXTERN void components_entry_undo_reset (ComponentsEntryUndo* self);
VALA_EXTERN GType components_application_header_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsApplicationHeaderBar, g_object_unref)
VALA_EXTERN GType monitored_spinner_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MonitoredSpinner, g_object_unref)
VALA_EXTERN void components_application_header_bar_show_app_menu (ComponentsApplicationHeaderBar* self);
VALA_EXTERN ComponentsApplicationHeaderBar* components_application_header_bar_new (void);
VALA_EXTERN ComponentsApplicationHeaderBar* components_application_header_bar_construct (GType object_type);
VALA_EXTERN GType components_conversation_list_header_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsConversationListHeaderBar, g_object_unref)
VALA_EXTERN const gchar* components_conversation_list_header_bar_get_account (ComponentsConversationListHeaderBar* self);
VALA_EXTERN void components_conversation_list_header_bar_set_account (ComponentsConversationListHeaderBar* self,
                                                          const gchar* value);
VALA_EXTERN const gchar* components_conversation_list_header_bar_get_folder (ComponentsConversationListHeaderBar* self);
VALA_EXTERN void components_conversation_list_header_bar_set_folder (ComponentsConversationListHeaderBar* self,
                                                         const gchar* value);
VALA_EXTERN gboolean components_conversation_list_header_bar_get_search_open (ComponentsConversationListHeaderBar* self);
VALA_EXTERN void components_conversation_list_header_bar_set_search_open (ComponentsConversationListHeaderBar* self,
                                                              gboolean value);
VALA_EXTERN gboolean components_conversation_list_header_bar_get_selection_open (ComponentsConversationListHeaderBar* self);
VALA_EXTERN void components_conversation_list_header_bar_set_selection_open (ComponentsConversationListHeaderBar* self,
                                                                 gboolean value);
VALA_EXTERN ComponentsConversationListHeaderBar* components_conversation_list_header_bar_new (void);
VALA_EXTERN ComponentsConversationListHeaderBar* components_conversation_list_header_bar_construct (GType object_type);
VALA_EXTERN gboolean components_conversation_header_bar_get_find_open (ComponentsConversationHeaderBar* self);
VALA_EXTERN void components_conversation_header_bar_set_find_open (ComponentsConversationHeaderBar* self,
                                                       gboolean value);
VALA_EXTERN ComponentsConversationActions* components_conversation_header_bar_get_shown_actions (ComponentsConversationHeaderBar* self);
VALA_EXTERN gboolean components_conversation_header_bar_get_show_close_button (ComponentsConversationHeaderBar* self);
VALA_EXTERN void components_conversation_header_bar_set_show_close_button (ComponentsConversationHeaderBar* self,
                                                               gboolean value);
VALA_EXTERN void components_conversation_header_bar_set_conversation_header (ComponentsConversationHeaderBar* self,
                                                                 HdyHeaderBar* header);
VALA_EXTERN void components_conversation_header_bar_remove_conversation_header (ComponentsConversationHeaderBar* self,
                                                                    HdyHeaderBar* header);
VALA_EXTERN void components_conversation_header_bar_set_find_sensitive (ComponentsConversationHeaderBar* self,
                                                            gboolean is_sensitive);
VALA_EXTERN ComponentsConversationHeaderBar* components_conversation_header_bar_new (void);
VALA_EXTERN ComponentsConversationHeaderBar* components_conversation_header_bar_construct (GType object_type);
VALA_EXTERN GType components_info_bar_stack_stack_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gint components_info_bar_stack_priority_queue_comparator (ComponentsInfoBar* a,
                                                          ComponentsInfoBar* b);
VALA_EXTERN ComponentsInfoBarStackStackType components_info_bar_stack_get_algorithm (ComponentsInfoBarStack* self);
VALA_EXTERN void components_info_bar_stack_set_algorithm (ComponentsInfoBarStack* self,
                                              ComponentsInfoBarStackStackType value);
VALA_EXTERN gboolean components_info_bar_stack_get_has_current (ComponentsInfoBarStack* self);
VALA_EXTERN ComponentsInfoBar* components_info_bar_stack_get_current_info_bar (ComponentsInfoBarStack* self);
VALA_EXTERN ComponentsInfoBarStack* components_info_bar_stack_new (ComponentsInfoBarStackStackType algorithm);
VALA_EXTERN ComponentsInfoBarStack* components_info_bar_stack_construct (GType object_type,
                                                             ComponentsInfoBarStackStackType algorithm);
VALA_EXTERN void components_info_bar_stack_add (ComponentsInfoBarStack* self,
                                    ComponentsInfoBar* to_add);
VALA_EXTERN void components_info_bar_stack_remove (ComponentsInfoBarStack* self,
                                       ComponentsInfoBar* to_remove);
VALA_EXTERN void components_info_bar_stack_remove_all (ComponentsInfoBarStack* self);
VALA_EXTERN GtkLabel* components_info_bar_get_status (ComponentsInfoBar* self);
VALA_EXTERN GtkLabel* components_info_bar_get_description (ComponentsInfoBar* self);
VALA_EXTERN gboolean components_info_bar_get_show_close_button (ComponentsInfoBar* self);
VALA_EXTERN void components_info_bar_set_show_close_button (ComponentsInfoBar* self,
                                                gboolean value);
VALA_EXTERN gboolean components_info_bar_get_revealed (ComponentsInfoBar* self);
VALA_EXTERN void components_info_bar_set_revealed (ComponentsInfoBar* self,
                                       gboolean value);
VALA_EXTERN GtkMessageType components_info_bar_get_message_type (ComponentsInfoBar* self);
VALA_EXTERN void components_info_bar_set_message_type (ComponentsInfoBar* self,
                                           GtkMessageType value);
VALA_EXTERN ComponentsInfoBar* components_info_bar_new (const gchar* status,
                                            const gchar* description);
VALA_EXTERN ComponentsInfoBar* components_info_bar_construct (GType object_type,
                                                  const gchar* status,
                                                  const gchar* description);
VALA_EXTERN GType plugin_info_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginInfoBar, g_object_unref)
VALA_EXTERN ComponentsInfoBar* components_info_bar_new_for_plugin (PluginInfoBar* plugin,
                                                       const gchar* action_group_name,
                                                       gint priority);
VALA_EXTERN ComponentsInfoBar* components_info_bar_construct_for_plugin (GType object_type,
                                                             PluginInfoBar* plugin,
                                                             const gchar* action_group_name,
                                                             gint priority);
VALA_EXTERN void components_info_bar_on_close_button_clicked (ComponentsInfoBar* self);
VALA_EXTERN GtkBox* components_info_bar_get_action_area (ComponentsInfoBar* self);
VALA_EXTERN GtkBox* components_info_bar_get_content_area (ComponentsInfoBar* self);
VALA_EXTERN GtkButton* components_info_bar_add_button (ComponentsInfoBar* self,
                                           const gchar* button_text,
                                           gint response_id);
VALA_EXTERN GType components_inspector_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInspector, g_object_unref)
VALA_EXTERN GType components_inspector_text_format_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gchar* components_inspector_text_format_get_line_separator (ComponentsInspectorTextFormat self);
VALA_EXTERN void components_inspector_add_accelerators (ApplicationClient* app);
VALA_EXTERN ComponentsInspector* components_inspector_new (ApplicationClient* application);
VALA_EXTERN ComponentsInspector* components_inspector_construct (GType object_type,
                                                     ApplicationClient* application);
VALA_EXTERN ComponentsInAppNotification* components_in_app_notification_new (const gchar* message,
                                                                 guint duration);
VALA_EXTERN ComponentsInAppNotification* components_in_app_notification_construct (GType object_type,
                                                                       const gchar* message,
                                                                       guint duration);
VALA_EXTERN void components_in_app_notification_set_button (ComponentsInAppNotification* self,
                                                const gchar* label,
                                                const gchar* action_name);
VALA_EXTERN void components_in_app_notification_close (ComponentsInAppNotification* self);
VALA_EXTERN GType components_inspector_error_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInspectorErrorView, g_object_unref)
VALA_EXTERN ComponentsInspectorErrorView* components_inspector_error_view_new (GearyErrorContext* _error_,
                                                                   GearyAccountInformation* account,
                                                                   GearyServiceInformation* service);
VALA_EXTERN ComponentsInspectorErrorView* components_inspector_error_view_construct (GType object_type,
                                                                         GearyErrorContext* _error_,
                                                                         GearyAccountInformation* account,
                                                                         GearyServiceInformation* service);
VALA_EXTERN void components_inspector_error_view_save (ComponentsInspectorErrorView* self,
                                           GDataOutputStream* out,
                                           ComponentsInspectorTextFormat format,
                                           GCancellable* cancellable,
                                           GError** error);
VALA_EXTERN GType components_inspector_log_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInspectorLogView, g_object_unref)
VALA_EXTERN gboolean components_inspector_log_view_get_search_mode_enabled (ComponentsInspectorLogView* self);
VALA_EXTERN void components_inspector_log_view_set_search_mode_enabled (ComponentsInspectorLogView* self,
                                                            gboolean value);
VALA_EXTERN ComponentsInspectorLogView* components_inspector_log_view_new (ApplicationConfiguration* config,
                                                               GearyAccountInformation* filter_by);
VALA_EXTERN ComponentsInspectorLogView* components_inspector_log_view_construct (GType object_type,
                                                                     ApplicationConfiguration* config,
                                                                     GearyAccountInformation* filter_by);
VALA_EXTERN void components_inspector_log_view_load (ComponentsInspectorLogView* self,
                                         GearyLoggingRecord* first,
                                         GearyLoggingRecord* last);
VALA_EXTERN void components_inspector_log_view_clear (ComponentsInspectorLogView* self);
VALA_EXTERN gboolean components_inspector_log_view_handle_key_press (ComponentsInspectorLogView* self,
                                                         GdkEventKey* event);
VALA_EXTERN gint components_inspector_log_view_count_selected_records (ComponentsInspectorLogView* self);
VALA_EXTERN void components_inspector_log_view_enable_log_updates (ComponentsInspectorLogView* self,
                                                       gboolean enabled);
VALA_EXTERN void components_inspector_log_view_save (ComponentsInspectorLogView* self,
                                         GDataOutputStream* out,
                                         ComponentsInspectorTextFormat format,
                                         gboolean save_all,
                                         GCancellable* cancellable,
                                         GError** error);
VALA_EXTERN GType components_inspector_system_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsInspectorSystemView, g_object_unref)
VALA_EXTERN ComponentsInspectorSystemView* components_inspector_system_view_new (ApplicationClient* application);
VALA_EXTERN ComponentsInspectorSystemView* components_inspector_system_view_construct (GType object_type,
                                                                           ApplicationClient* application);
VALA_EXTERN void components_inspector_system_view_save (ComponentsInspectorSystemView* self,
                                            GDataOutputStream* out,
                                            ComponentsInspectorTextFormat format,
                                            GCancellable* cancellable,
                                            GError** error);
VALA_EXTERN GType components_placeholder_pane_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsPlaceholderPane, g_object_unref)
VALA_EXTERN gchar* components_placeholder_pane_get_icon_name (ComponentsPlaceholderPane* self);
VALA_EXTERN void components_placeholder_pane_set_icon_name (ComponentsPlaceholderPane* self,
                                                const gchar* value);
VALA_EXTERN const gchar* components_placeholder_pane_get_title (ComponentsPlaceholderPane* self);
VALA_EXTERN void components_placeholder_pane_set_title (ComponentsPlaceholderPane* self,
                                            const gchar* value);
VALA_EXTERN const gchar* components_placeholder_pane_get_subtitle (ComponentsPlaceholderPane* self);
VALA_EXTERN void components_placeholder_pane_set_subtitle (ComponentsPlaceholderPane* self,
                                               const gchar* value);
VALA_EXTERN ComponentsPlaceholderPane* components_placeholder_pane_new (void);
VALA_EXTERN ComponentsPlaceholderPane* components_placeholder_pane_construct (GType object_type);
VALA_EXTERN GType components_preferences_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsPreferencesWindow, g_object_unref)
VALA_EXTERN void components_preferences_window_add_accelerators (ApplicationClient* app);
VALA_EXTERN ApplicationClient* components_preferences_window_get_application (ComponentsPreferencesWindow* self);
VALA_EXTERN void components_preferences_window_set_application (ComponentsPreferencesWindow* self,
                                                    ApplicationClient* value);
VALA_EXTERN ComponentsPreferencesWindow* components_preferences_window_new (ApplicationMainWindow* parent,
                                                                ApplicationPluginManager* plugins);
VALA_EXTERN ComponentsPreferencesWindow* components_preferences_window_construct (GType object_type,
                                                                      ApplicationMainWindow* parent,
                                                                      ApplicationPluginManager* plugins);
VALA_EXTERN GType components_problem_report_info_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsProblemReportInfoBar, g_object_unref)
VALA_EXTERN GearyProblemReport* components_problem_report_info_bar_get_report (ComponentsProblemReportInfoBar* self);
VALA_EXTERN ComponentsProblemReportInfoBar* components_problem_report_info_bar_new (GearyProblemReport* report);
VALA_EXTERN ComponentsProblemReportInfoBar* components_problem_report_info_bar_construct (GType object_type,
                                                                              GearyProblemReport* report);
VALA_EXTERN GtkSearchEntry* search_bar_get_entry (SearchBar* self);
VALA_EXTERN SearchBar* search_bar_new (GearyEngine* engine);
VALA_EXTERN SearchBar* search_bar_construct (GType object_type,
                                 GearyEngine* engine);
VALA_EXTERN void search_bar_set_account (SearchBar* self,
                             GearyAccount* account);
VALA_EXTERN GType components_validator_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsValidator, g_object_unref)
VALA_EXTERN GType components_validator_ui_state_get_type (void) G_GNUC_CONST ;
VALA_EXTERN ComponentsValidatorUiState* components_validator_ui_state_dup (const ComponentsValidatorUiState* self);
VALA_EXTERN void components_validator_ui_state_free (ComponentsValidatorUiState* self);
VALA_EXTERN void components_validator_ui_state_copy (const ComponentsValidatorUiState* self,
                                         ComponentsValidatorUiState* dest);
VALA_EXTERN void components_validator_ui_state_destroy (ComponentsValidatorUiState* self);
G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (ComponentsValidatorUiState, components_validator_ui_state_destroy)
VALA_EXTERN GType components_validator_trigger_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType components_validator_validity_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GtkEntry* components_validator_get_target (ComponentsValidator* self);
VALA_EXTERN gboolean components_validator_get_is_valid (ComponentsValidator* self);
VALA_EXTERN gboolean components_validator_get_is_required (ComponentsValidator* self);
VALA_EXTERN void components_validator_set_is_required (ComponentsValidator* self,
                                           gboolean value);
VALA_EXTERN ComponentsValidatorValidity components_validator_get_state (ComponentsValidator* self);
VALA_EXTERN ComponentsValidator* components_validator_new (GtkEntry* target);
VALA_EXTERN ComponentsValidator* components_validator_construct (GType object_type,
                                                     GtkEntry* target);
VALA_EXTERN void components_validator_validate (ComponentsValidator* self);
VALA_EXTERN ComponentsValidatorValidity components_validator_do_validate (ComponentsValidator* self,
                                                              const gchar* value,
                                                              ComponentsValidatorTrigger reason);
VALA_EXTERN void components_validator_update_state (ComponentsValidator* self,
                                        ComponentsValidatorValidity new_state,
                                        ComponentsValidatorTrigger reason);
VALA_EXTERN GType components_email_validator_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsEmailValidator, g_object_unref)
VALA_EXTERN ComponentsEmailValidator* components_email_validator_new (GtkEntry* target);
VALA_EXTERN ComponentsEmailValidator* components_email_validator_construct (GType object_type,
                                                                GtkEntry* target);
VALA_EXTERN GType components_network_address_validator_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComponentsNetworkAddressValidator, g_object_unref)
VALA_EXTERN GNetworkAddress* components_network_address_validator_get_validated_address (ComponentsNetworkAddressValidator* self);
VALA_EXTERN guint16 components_network_address_validator_get_default_port (ComponentsNetworkAddressValidator* self);
VALA_EXTERN ComponentsNetworkAddressValidator* components_network_address_validator_new (GtkEntry* target,
                                                                             guint16 default_port);
VALA_EXTERN ComponentsNetworkAddressValidator* components_network_address_validator_construct (GType object_type,
                                                                                   GtkEntry* target,
                                                                                   guint16 default_port);
VALA_EXTERN void components_web_view_init_web_context (ApplicationConfiguration* config,
                                           GFile* web_extension_dir,
                                           GFile* cache_dir,
                                           gboolean sandboxed);
VALA_EXTERN void components_web_view_load_resources (GFile* user_dir,
                                         GError** error);
VALA_EXTERN WebKitUserStyleSheet* components_web_view_load_app_stylesheet (const gchar* name,
                                                               GError** error);
VALA_EXTERN WebKitUserStyleSheet* components_web_view_load_user_stylesheet (GFile* name,
                                                                GError** error);
VALA_EXTERN WebKitUserScript* components_web_view_load_app_script (const gchar* name,
                                                       GError** error);
VALA_EXTERN gboolean components_web_view_get_is_content_loaded (ComponentsWebView* self);
VALA_EXTERN gboolean components_web_view_get_has_selection (ComponentsWebView* self);
VALA_EXTERN gint components_web_view_get_preferred_height (ComponentsWebView* self);
VALA_EXTERN gboolean components_web_view_get_is_load_remote_resources_enabled (ComponentsWebView* self);
VALA_EXTERN const gchar* components_web_view_get_document_font (ComponentsWebView* self);
VALA_EXTERN void components_web_view_set_document_font (ComponentsWebView* self,
                                            const gchar* value);
VALA_EXTERN const gchar* components_web_view_get_monospace_font (ComponentsWebView* self);
VALA_EXTERN void components_web_view_set_monospace_font (ComponentsWebView* self,
                                             const gchar* value);
VALA_EXTERN ComponentsWebView* components_web_view_construct (GType object_type,
                                                  ApplicationConfiguration* config,
                                                  WebKitUserContentManager* custom_manager,
                                                  ComponentsWebView* related);
VALA_EXTERN ComponentsWebView* components_web_view_construct_with_related_view (GType object_type,
                                                                    ApplicationConfiguration* config,
                                                                    ComponentsWebView* related);
VALA_EXTERN void components_web_view_load_html (ComponentsWebView* self,
                                    const gchar* body,
                                    const gchar* base_uri);
VALA_EXTERN void components_web_view_load_html_headless (ComponentsWebView* self,
                                             const gchar* body,
                                             const gchar* base_uri);
VALA_EXTERN void components_web_view_get_html (ComponentsWebView* self,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN gchar* components_web_view_get_html_finish (ComponentsWebView* self,
                                            GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN void components_web_view_add_internal_resource (ComponentsWebView* self,
                                                const gchar* id,
                                                GearyMemoryBuffer* buf);
VALA_EXTERN void components_web_view_add_internal_resources (ComponentsWebView* self,
                                                 GeeMap* res);
VALA_EXTERN void components_web_view_load_remote_resources (ComponentsWebView* self,
                                                GCancellable* cancellable,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_);
VALA_EXTERN void components_web_view_load_remote_resources_finish (ComponentsWebView* self,
                                                       GAsyncResult* _res_,
                                                       GError** error);
VALA_EXTERN void components_web_view_select_all (ComponentsWebView* self);
VALA_EXTERN void components_web_view_copy_clipboard (ComponentsWebView* self);
VALA_EXTERN void components_web_view_zoom_reset (ComponentsWebView* self);
VALA_EXTERN void components_web_view_zoom_in (ComponentsWebView* self);
VALA_EXTERN void components_web_view_zoom_out (ComponentsWebView* self);
VALA_EXTERN void components_web_view_set_editable (ComponentsWebView* self,
                                       gboolean enabled,
                                       GCancellable* cancellable,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN void components_web_view_set_editable_finish (ComponentsWebView* self,
                                              GAsyncResult* _res_,
                                              GError** error);
VALA_EXTERN gpointer util_js_callable_ref (gpointer instance);
VALA_EXTERN void util_js_callable_unref (gpointer instance);
VALA_EXTERN GParamSpec* util_js_param_spec_callable (const gchar* name,
                                         const gchar* nick,
                                         const gchar* blurb,
                                         GType object_type,
                                         GParamFlags flags);
VALA_EXTERN void util_js_value_set_callable (GValue* value,
                                 gpointer v_object);
VALA_EXTERN void util_js_value_take_callable (GValue* value,
                                  gpointer v_object);
VALA_EXTERN gpointer util_js_value_get_callable (const GValue* value);
VALA_EXTERN GType util_js_callable_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UtilJSCallable, util_js_callable_unref)
VALA_EXTERN void components_web_view_call_void (ComponentsWebView* self,
                                    UtilJSCallable* target,
                                    GCancellable* cancellable,
                                    GAsyncReadyCallback _callback_,
                                    gpointer _user_data_);
VALA_EXTERN void components_web_view_call_void_finish (ComponentsWebView* self,
                                           GAsyncResult* _res_,
                                           GError** error);
VALA_EXTERN void components_web_view_call_returning (ComponentsWebView* self,
                                         GType t_type,
                                         GBoxedCopyFunc t_dup_func,
                                         GDestroyNotify t_destroy_func,
                                         UtilJSCallable* target,
                                         GCancellable* cancellable,
                                         GAsyncReadyCallback _callback_,
                                         gpointer _user_data_);
VALA_EXTERN gpointer components_web_view_call_returning_finish (ComponentsWebView* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
VALA_EXTERN void components_web_view_register_message_callback (ComponentsWebView* self,
                                                    const gchar* name,
                                                    ComponentsWebViewMessageCallback handler,
                                                    gpointer handler_target);
VALA_EXTERN GType count_badge_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CountBadge, g_object_unref)
VALA_EXTERN gint count_badge_get_count (CountBadge* self);
VALA_EXTERN void count_badge_set_count (CountBadge* self,
                            gint value);
VALA_EXTERN CountBadge* count_badge_new (gint min);
VALA_EXTERN CountBadge* count_badge_construct (GType object_type,
                                   gint min);
VALA_EXTERN gint count_badge_get_width (CountBadge* self,
                            GtkWidget* widget);
VALA_EXTERN gint count_badge_get_height (CountBadge* self,
                             GtkWidget* widget);
VALA_EXTERN void count_badge_render (CountBadge* self,
                         GtkWidget* widget,
                         cairo_t* ctx,
                         gint x,
                         gint y,
                         gboolean selected);
VALA_EXTERN GearyAccount* folder_popover_get_account (FolderPopover* self);
VALA_EXTERN void folder_popover_set_account (FolderPopover* self,
                                 GearyAccount* value);
VALA_EXTERN FolderPopover* folder_popover_new (ApplicationConfiguration* config);
VALA_EXTERN FolderPopover* folder_popover_construct (GType object_type,
                                         ApplicationConfiguration* config);
VALA_EXTERN GType folder_popover_row_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderPopoverRow, g_object_unref)
VALA_EXTERN FolderPopoverRow* folder_popover_row_new (ApplicationFolderContext* context,
                                          GeeHashMap* map);
VALA_EXTERN FolderPopoverRow* folder_popover_row_construct (GType object_type,
                                                ApplicationFolderContext* context,
                                                GeeHashMap* map);
VALA_EXTERN gpointer icon_factory_ref (gpointer instance);
VALA_EXTERN void icon_factory_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_icon_factory (const gchar* name,
                                     const gchar* nick,
                                     const gchar* blurb,
                                     GType object_type,
                                     GParamFlags flags);
VALA_EXTERN void value_set_icon_factory (GValue* value,
                             gpointer v_object);
VALA_EXTERN void value_take_icon_factory (GValue* value,
                              gpointer v_object);
VALA_EXTERN gpointer value_get_icon_factory (const GValue* value);
VALA_EXTERN GType icon_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IconFactory, icon_factory_unref)
VALA_EXTERN IconFactory* icon_factory_get_instance (void);
VALA_EXTERN void icon_factory_init (GFile* resource_directory);
VALA_EXTERN GIcon* icon_factory_get_theme_icon (IconFactory* self,
                                    const gchar* name);
VALA_EXTERN GIcon* icon_factory_get_custom_icon (IconFactory* self,
                                     const gchar* name,
                                     GtkIconSize size);
VALA_EXTERN GtkIconInfo* icon_factory_lookup_icon (IconFactory* self,
                                       const gchar* icon_name,
                                       gint size,
                                       GtkIconLookupFlags flags);
VALA_EXTERN GdkPixbuf* icon_factory_load_symbolic (IconFactory* self,
                                       const gchar* icon_name,
                                       gint size,
                                       GtkStyleContext* style,
                                       GtkIconLookupFlags flags);
VALA_EXTERN GdkPixbuf* icon_factory_load_symbolic_colored (IconFactory* self,
                                               const gchar* icon_name,
                                               gint size,
                                               GdkRGBA* color,
                                               GtkIconLookupFlags flags);
VALA_EXTERN GType monitored_progress_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MonitoredProgressBar, g_object_unref)
VALA_EXTERN void monitored_progress_bar_set_progress_monitor (MonitoredProgressBar* self,
                                                  GearyProgressMonitor* monitor);
VALA_EXTERN MonitoredProgressBar* monitored_progress_bar_new (void);
VALA_EXTERN MonitoredProgressBar* monitored_progress_bar_construct (GType object_type);
VALA_EXTERN void monitored_spinner_set_progress_monitor (MonitoredSpinner* self,
                                             GearyProgressMonitor* monitor);
VALA_EXTERN MonitoredSpinner* monitored_spinner_new (void);
VALA_EXTERN MonitoredSpinner* monitored_spinner_construct (GType object_type);
VALA_EXTERN GType composer_widget_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerWidget, g_object_unref)
VALA_EXTERN GType composer_container_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType composer_box_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerBox, g_object_unref)
VALA_EXTERN ComposerBox* composer_box_new (ComposerWidget* composer,
                               ComponentsConversationHeaderBar* headerbar);
VALA_EXTERN ComposerBox* composer_box_construct (GType object_type,
                                     ComposerWidget* composer,
                                     ComponentsConversationHeaderBar* headerbar);
VALA_EXTERN GtkApplicationWindow* composer_container_get_top_window (ComposerContainer* self);
VALA_EXTERN void composer_container_present (ComposerContainer* self);
VALA_EXTERN GtkWidget* composer_container_get_focus (ComposerContainer* self);
VALA_EXTERN void composer_container_close (ComposerContainer* self);
VALA_EXTERN GType composer_editor_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerEditor, g_object_unref)
VALA_EXTERN void composer_editor_add_accelerators (ApplicationClient* application);
VALA_EXTERN GType composer_web_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerWebView, g_object_unref)
VALA_EXTERN ComposerWebView* composer_editor_get_body (ComposerEditor* self);
VALA_EXTERN void composer_editor_add_action_bar (ComposerEditor* self,
                                     GtkActionBar* to_add);
VALA_EXTERN void composer_editor_insert_menu_section (ComposerEditor* self,
                                          GMenuModel* section);
VALA_EXTERN GType composer_email_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerEmailEntry, g_object_unref)
VALA_EXTERN GearyRFC822MailboxAddresses* composer_email_entry_get_addresses (ComposerEmailEntry* self);
VALA_EXTERN void composer_email_entry_set_addresses (ComposerEmailEntry* self,
                                         GearyRFC822MailboxAddresses* value);
VALA_EXTERN gboolean composer_email_entry_get_is_valid (ComposerEmailEntry* self);
VALA_EXTERN gboolean composer_email_entry_get_is_empty (ComposerEmailEntry* self);
VALA_EXTERN gboolean composer_email_entry_get_is_modified (ComposerEmailEntry* self);
VALA_EXTERN ComposerEmailEntry* composer_email_entry_new (ComposerWidget* composer);
VALA_EXTERN ComposerEmailEntry* composer_email_entry_construct (GType object_type,
                                                    ComposerWidget* composer);
VALA_EXTERN void composer_email_entry_set_modified (ComposerEmailEntry* self);
VALA_EXTERN GType composer_embed_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerEmbed, g_object_unref)
VALA_EXTERN GearyEmail* composer_embed_get_referred (ComposerEmbed* self);
VALA_EXTERN ComposerEmbed* composer_embed_new (GearyEmail* referred,
                                   ComposerWidget* composer,
                                   GtkScrolledWindow* outer_scroller);
VALA_EXTERN ComposerEmbed* composer_embed_construct (GType object_type,
                                         GearyEmail* referred,
                                         ComposerWidget* composer,
                                         GtkScrolledWindow* outer_scroller);
VALA_EXTERN GType composer_headerbar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerHeaderbar, g_object_unref)
VALA_EXTERN gboolean composer_headerbar_get_show_save_and_close (ComposerHeaderbar* self);
VALA_EXTERN void composer_headerbar_set_show_save_and_close (ComposerHeaderbar* self,
                                                 gboolean value);
VALA_EXTERN gboolean composer_headerbar_get_show_send (ComposerHeaderbar* self);
VALA_EXTERN void composer_headerbar_set_show_send (ComposerHeaderbar* self,
                                       gboolean value);
VALA_EXTERN ComposerHeaderbar* composer_headerbar_new (ApplicationConfiguration* config);
VALA_EXTERN ComposerHeaderbar* composer_headerbar_construct (GType object_type,
                                                 ApplicationConfiguration* config);
VALA_EXTERN void composer_headerbar_set_recipients (ComposerHeaderbar* self,
                                        const gchar* label,
                                        const gchar* tooltip);
VALA_EXTERN GType composer_link_popover_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerLinkPopover, g_object_unref)
VALA_EXTERN GType composer_link_popover_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN const gchar* composer_link_popover_get_link_uri (ComposerLinkPopover* self);
VALA_EXTERN ComposerLinkPopover* composer_link_popover_new (ComposerLinkPopoverType type);
VALA_EXTERN ComposerLinkPopover* composer_link_popover_construct (GType object_type,
                                                      ComposerLinkPopoverType type);
VALA_EXTERN void composer_link_popover_set_link_url (ComposerLinkPopover* self,
                                         const gchar* url);
VALA_EXTERN void composer_web_view_load_resources (GError** error);
VALA_EXTERN gboolean composer_web_view_get_is_empty (ComposerWebView* self);
VALA_EXTERN gboolean composer_web_view_get_is_rich_text (ComposerWebView* self);
VALA_EXTERN ComposerWebView* composer_web_view_new (ApplicationConfiguration* config);
VALA_EXTERN ComposerWebView* composer_web_view_construct (GType object_type,
                                              ApplicationConfiguration* config);
VALA_EXTERN void composer_web_view_load_html (ComposerWebView* self,
                                  const gchar* body,
                                  const gchar* quote,
                                  gboolean top_posting,
                                  gboolean body_complete);
VALA_EXTERN void composer_web_view_load_html_headless (ComposerWebView* self,
                                           const gchar* body,
                                           const gchar* quote,
                                           gboolean top_posting,
                                           gboolean body_complete);
VALA_EXTERN void composer_web_view_get_html_for_draft (ComposerWebView* self,
                                           GAsyncReadyCallback _callback_,
                                           gpointer _user_data_);
VALA_EXTERN gchar* composer_web_view_get_html_for_draft_finish (ComposerWebView* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
VALA_EXTERN void composer_web_view_set_rich_text (ComposerWebView* self,
                                      gboolean enabled);
VALA_EXTERN void composer_web_view_undo (ComposerWebView* self);
VALA_EXTERN void composer_web_view_redo (ComposerWebView* self);
VALA_EXTERN void composer_web_view_save_selection (ComposerWebView* self,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN gchar* composer_web_view_save_selection_finish (ComposerWebView* self,
                                                GAsyncResult* _res_,
                                                GError** error);
VALA_EXTERN void composer_web_view_free_selection (ComposerWebView* self,
                                       const gchar* id);
VALA_EXTERN void composer_web_view_cut_clipboard (ComposerWebView* self);
VALA_EXTERN void composer_web_view_paste_plain_text (ComposerWebView* self);
VALA_EXTERN void composer_web_view_paste_rich_text (ComposerWebView* self);
VALA_EXTERN void composer_web_view_insert_text (ComposerWebView* self,
                                    const gchar* text);
VALA_EXTERN void composer_web_view_insert_html (ComposerWebView* self,
                                    const gchar* markup);
VALA_EXTERN void composer_web_view_insert_link (ComposerWebView* self,
                                    const gchar* href,
                                    const gchar* selection_id);
VALA_EXTERN void composer_web_view_delete_link (ComposerWebView* self,
                                    const gchar* selection_id);
VALA_EXTERN void composer_web_view_insert_image (ComposerWebView* self,
                                     const gchar* src);
VALA_EXTERN void composer_web_view_indent_line (ComposerWebView* self);
VALA_EXTERN void composer_web_view_insert_olist (ComposerWebView* self);
VALA_EXTERN void composer_web_view_insert_ulist (ComposerWebView* self);
VALA_EXTERN void composer_web_view_update_signature (ComposerWebView* self,
                                         const gchar* signature);
VALA_EXTERN void composer_web_view_delete_quoted_message (ComposerWebView* self);
VALA_EXTERN void composer_web_view_contains_attachment_keywords (ComposerWebView* self,
                                                     const gchar* keyword_spec,
                                                     const gchar* subject,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_);
VALA_EXTERN gboolean* composer_web_view_contains_attachment_keywords_finish (ComposerWebView* self,
                                                                 GAsyncResult* _res_);
VALA_EXTERN void composer_web_view_clean_content (ComposerWebView* self,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN void composer_web_view_clean_content_finish (ComposerWebView* self,
                                             GAsyncResult* _res_,
                                             GError** error);
VALA_EXTERN void composer_web_view_get_text (ComposerWebView* self,
                                 GAsyncReadyCallback _callback_,
                                 gpointer _user_data_);
VALA_EXTERN gchar* composer_web_view_get_text_finish (ComposerWebView* self,
                                          GAsyncResult* _res_,
                                          GError** error);
VALA_EXTERN GType composer_web_view_edit_context_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerWebViewEditContext, g_object_unref)
VALA_EXTERN gboolean composer_web_view_edit_context_get_is_link (ComposerWebViewEditContext* self);
VALA_EXTERN const gchar* composer_web_view_edit_context_get_link_url (ComposerWebViewEditContext* self);
VALA_EXTERN const gchar* composer_web_view_edit_context_get_font_family (ComposerWebViewEditContext* self);
VALA_EXTERN guint composer_web_view_edit_context_get_font_size (ComposerWebViewEditContext* self);
VALA_EXTERN void composer_web_view_edit_context_get_font_color (ComposerWebViewEditContext* self,
                                                    GdkRGBA * result);
VALA_EXTERN ComposerWebViewEditContext* composer_web_view_edit_context_new (const gchar* message);
VALA_EXTERN ComposerWebViewEditContext* composer_web_view_edit_context_construct (GType object_type,
                                                                      const gchar* message);
VALA_EXTERN GType composer_widget_context_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType composer_widget_close_status_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType composer_widget_presentation_mode_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void composer_widget_add_accelerators (ApplicationClient* application);
VALA_EXTERN ApplicationAccountContext* composer_widget_get_sender_context (ComposerWidget* self);
VALA_EXTERN GearyEmailIdentifier* composer_widget_get_saved_id (ComposerWidget* self);
VALA_EXTERN ComposerWidgetContextType composer_widget_get_context_type (ComposerWidget* self);
VALA_EXTERN ComposerWidgetPresentationMode composer_widget_get_current_mode (ComposerWidget* self);
VALA_EXTERN void composer_widget_set_current_mode (ComposerWidget* self,
                                       ComposerWidgetPresentationMode value);
VALA_EXTERN gboolean composer_widget_get_is_blank (ComposerWidget* self);
VALA_EXTERN ComposerEditor* composer_widget_get_editor (ComposerWidget* self);
VALA_EXTERN GtkWidget* composer_widget_get_focused_input_widget (ComposerWidget* self);
VALA_EXTERN gboolean composer_widget_get_can_send (ComposerWidget* self);
VALA_EXTERN void composer_widget_set_can_send (ComposerWidget* self,
                                   gboolean value);
VALA_EXTERN GearyRFC822MailboxAddresses* composer_widget_get_from (ComposerWidget* self);
VALA_EXTERN const gchar* composer_widget_get_to (ComposerWidget* self);
VALA_EXTERN const gchar* composer_widget_get_cc (ComposerWidget* self);
VALA_EXTERN const gchar* composer_widget_get_bcc (ComposerWidget* self);
VALA_EXTERN const gchar* composer_widget_get_reply_to (ComposerWidget* self);
VALA_EXTERN const gchar* composer_widget_get_subject (ComposerWidget* self);
VALA_EXTERN GearyRFC822MessageIDList* composer_widget_get_in_reply_to (ComposerWidget* self);
VALA_EXTERN GearyRFC822MessageIDList* composer_widget_get_references (ComposerWidget* self);
VALA_EXTERN void composer_widget_load_empty_body (ComposerWidget* self,
                                      GearyRFC822MailboxAddress* to,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN void composer_widget_load_empty_body_finish (ComposerWidget* self,
                                             GAsyncResult* _res_,
                                             GError** error);
VALA_EXTERN void composer_widget_load_mailto (ComposerWidget* self,
                                  const gchar* mailto,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN void composer_widget_load_mailto_finish (ComposerWidget* self,
                                         GAsyncResult* _res_,
                                         GError** error);
VALA_EXTERN void composer_widget_load_context (ComposerWidget* self,
                                   ComposerWidgetContextType type,
                                   GearyEmail* context,
                                   const gchar* quote,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN void composer_widget_load_context_finish (ComposerWidget* self,
                                          GAsyncResult* _res_,
                                          GError** error);
VALA_EXTERN GeeSet* composer_widget_get_referred_ids (ComposerWidget* self);
VALA_EXTERN void composer_widget_detach (ComposerWidget* self,
                             ApplicationClient* application);
VALA_EXTERN ComposerWidgetCloseStatus composer_widget_conditional_close (ComposerWidget* self,
                                                             gboolean should_prompt,
                                                             gboolean is_shutdown);
VALA_EXTERN void composer_widget_close (ComposerWidget* self,
                            GAsyncReadyCallback _callback_,
                            gpointer _user_data_);
VALA_EXTERN void composer_widget_close_finish (ComposerWidget* self,
                                   GAsyncResult* _res_);
VALA_EXTERN void composer_widget_set_enabled (ComposerWidget* self,
                                  gboolean enabled);
VALA_EXTERN void composer_widget_set_save_to_override (ComposerWidget* self,
                                           GearyFolder* save_to);
VALA_EXTERN void composer_widget_present (ComposerWidget* self);
VALA_EXTERN void composer_widget_set_focus (ComposerWidget* self);
VALA_EXTERN void composer_widget_to_composed_email (ComposerWidget* self,
                                        GDateTime* date_override,
                                        gboolean for_draft,
                                        GAsyncReadyCallback _callback_,
                                        gpointer _user_data_);
VALA_EXTERN GearyComposedEmail* composer_widget_to_composed_email_finish (ComposerWidget* self,
                                                              GAsyncResult* _res_);
VALA_EXTERN void composer_widget_append_to_email (ComposerWidget* self,
                                      GearyEmail* referred,
                                      const gchar* to_quote,
                                      ComposerWidgetContextType type,
                                      GError** error);
VALA_EXTERN void composer_widget_update_window_title (ComposerWidget* self);
VALA_EXTERN void composer_widget_activate_close_action (ComposerWidget* self);
VALA_EXTERN GType composer_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ComposerWindow, g_object_unref)
VALA_EXTERN ApplicationClient* composer_window_get_application (ComposerWindow* self);
VALA_EXTERN void composer_window_set_application (ComposerWindow* self,
                                      ApplicationClient* value);
VALA_EXTERN ComposerWindow* composer_window_new (ComposerWidget* composer,
                                     ApplicationClient* application);
VALA_EXTERN ComposerWindow* composer_window_construct (GType object_type,
                                           ComposerWidget* composer,
                                           ApplicationClient* application);
VALA_EXTERN GType contact_entry_completion_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ContactEntryCompletion, g_object_unref)
VALA_EXTERN GType contact_entry_completion_column_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType* contact_entry_completion_column_get_types (gint* result_length1);
VALA_EXTERN ContactEntryCompletion* contact_entry_completion_new (ApplicationContactStore* contacts);
VALA_EXTERN ContactEntryCompletion* contact_entry_completion_construct (GType object_type,
                                                            ApplicationContactStore* contacts);
VALA_EXTERN void contact_entry_completion_update_model (ContactEntryCompletion* self);
VALA_EXTERN void contact_entry_completion_trigger_selection (ContactEntryCompletion* self);
VALA_EXTERN gpointer spell_check_popover_ref (gpointer instance);
VALA_EXTERN void spell_check_popover_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_spell_check_popover (const gchar* name,
                                            const gchar* nick,
                                            const gchar* blurb,
                                            GType object_type,
                                            GParamFlags flags);
VALA_EXTERN void value_set_spell_check_popover (GValue* value,
                                    gpointer v_object);
VALA_EXTERN void value_take_spell_check_popover (GValue* value,
                                     gpointer v_object);
VALA_EXTERN gpointer value_get_spell_check_popover (const GValue* value);
VALA_EXTERN GType spell_check_popover_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SpellCheckPopover, spell_check_popover_unref)
VALA_EXTERN SpellCheckPopover* spell_check_popover_new (GtkMenuButton* button,
                                            ApplicationConfiguration* config);
VALA_EXTERN SpellCheckPopover* spell_check_popover_construct (GType object_type,
                                                  GtkMenuButton* button,
                                                  ApplicationConfiguration* config);
VALA_EXTERN GType conversation_list_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationListModel, g_object_unref)
VALA_EXTERN gboolean conversation_list_model_load_more (ConversationListModel* self,
                                            gint amount);
VALA_EXTERN gboolean conversation_list_view_get_selection_mode_enabled (ConversationListView* self);
VALA_EXTERN void conversation_list_view_set_selection_mode_enabled (ConversationListView* self,
                                                        gboolean value);
VALA_EXTERN GeeSet* conversation_list_view_get_selected (ConversationListView* self);
VALA_EXTERN void conversation_list_view_set_selected (ConversationListView* self,
                                          GeeSet* value);
VALA_EXTERN ConversationListView* conversation_list_view_new (ApplicationConfiguration* config);
VALA_EXTERN ConversationListView* conversation_list_view_construct (GType object_type,
                                                        ApplicationConfiguration* config);
VALA_EXTERN void conversation_list_view_refresh_times (ConversationListView* self);
VALA_EXTERN void conversation_list_view_set_monitor (ConversationListView* self,
                                         GearyAppConversationMonitor* monitor);
VALA_EXTERN void conversation_list_view_load_more (ConversationListView* self,
                                       gint request);
VALA_EXTERN void conversation_list_view_scroll (ConversationListView* self,
                                    GtkScrollType scroll_type);
VALA_EXTERN GeeSet* conversation_list_view_get_selected_conversations (ConversationListView* self);
VALA_EXTERN void conversation_list_view_select_conversations (ConversationListView* self,
                                                  GeeCollection* selection);
VALA_EXTERN void conversation_list_view_activate_selected (ConversationListView* self);
VALA_EXTERN void conversation_list_view_select_all (ConversationListView* self);
VALA_EXTERN void conversation_list_view_unselect_all (ConversationListView* self);
VALA_EXTERN GeeSet* conversation_list_view_get_visible_conversations (ConversationListView* self);
VALA_EXTERN void conversation_list_view_inhibit_next_autoselect (ConversationListView* self);
VALA_EXTERN GType conversation_contact_popover_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationContactPopover, g_object_unref)
VALA_EXTERN ApplicationContact* conversation_contact_popover_get_contact (ConversationContactPopover* self);
VALA_EXTERN GearyRFC822MailboxAddress* conversation_contact_popover_get_mailbox (ConversationContactPopover* self);
VALA_EXTERN ConversationContactPopover* conversation_contact_popover_new (GtkWidget* relative_to,
                                                              ApplicationContact* contact,
                                                              GearyRFC822MailboxAddress* mailbox,
                                                              ApplicationConfiguration* config);
VALA_EXTERN ConversationContactPopover* conversation_contact_popover_construct (GType object_type,
                                                                    GtkWidget* relative_to,
                                                                    ApplicationContact* contact,
                                                                    GearyRFC822MailboxAddress* mailbox,
                                                                    ApplicationConfiguration* config);
VALA_EXTERN GType conversation_email_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationEmail, g_object_unref)
VALA_EXTERN GType conversation_email_load_state_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GearyEmail* conversation_email_get_email (ConversationEmail* self);
VALA_EXTERN gboolean conversation_email_get_is_unread (ConversationEmail* self);
VALA_EXTERN gboolean conversation_email_get_is_starred (ConversationEmail* self);
VALA_EXTERN gboolean conversation_email_get_is_manually_read (ConversationEmail* self);
VALA_EXTERN void conversation_email_set_is_manually_read (ConversationEmail* self,
                                              gboolean value);
VALA_EXTERN gboolean conversation_email_get_is_draft (ConversationEmail* self);
VALA_EXTERN GType conversation_message_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationMessage, g_object_unref)
VALA_EXTERN ConversationMessage* conversation_email_get_primary_message (ConversationEmail* self);
VALA_EXTERN ComponentsAttachmentPane* conversation_email_get_attachments_pane (ConversationEmail* self);
VALA_EXTERN GeeList* conversation_email_get_attached_messages (ConversationEmail* self);
VALA_EXTERN ConversationEmailLoadState conversation_email_get_message_body_state (ConversationEmail* self);
VALA_EXTERN ConversationEmail* conversation_email_new (GearyAppConversation* conversation,
                                           GearyEmail* email,
                                           GearyAppEmailStore* email_store,
                                           ApplicationContactStore* contacts,
                                           ApplicationConfiguration* config,
                                           gboolean is_sent,
                                           gboolean is_draft,
                                           GCancellable* load_cancellable);
VALA_EXTERN ConversationEmail* conversation_email_construct (GType object_type,
                                                 GearyAppConversation* conversation,
                                                 GearyEmail* email,
                                                 GearyAppEmailStore* email_store,
                                                 ApplicationContactStore* contacts,
                                                 ApplicationConfiguration* config,
                                                 gboolean is_sent,
                                                 gboolean is_draft,
                                                 GCancellable* load_cancellable);
VALA_EXTERN void conversation_email_load_contacts (ConversationEmail* self,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN void conversation_email_load_contacts_finish (ConversationEmail* self,
                                              GAsyncResult* _res_,
                                              GError** error);
VALA_EXTERN void conversation_email_load_body (ConversationEmail* self,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN void conversation_email_load_body_finish (ConversationEmail* self,
                                          GAsyncResult* _res_,
                                          GError** error);
VALA_EXTERN void conversation_email_expand_email (ConversationEmail* self,
                                      gboolean include_transitions);
VALA_EXTERN void conversation_email_collapse_email (ConversationEmail* self);
VALA_EXTERN void conversation_email_update_flags (ConversationEmail* self,
                                      GearyEmail* email);
VALA_EXTERN void conversation_email_get_selection_for_quoting (ConversationEmail* self,
                                                   GAsyncReadyCallback _callback_,
                                                   gpointer _user_data_);
VALA_EXTERN gchar* conversation_email_get_selection_for_quoting_finish (ConversationEmail* self,
                                                            GAsyncResult* _res_);
VALA_EXTERN void conversation_email_get_selection_for_find (ConversationEmail* self,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_);
VALA_EXTERN gchar* conversation_email_get_selection_for_find_finish (ConversationEmail* self,
                                                         GAsyncResult* _res_);
VALA_EXTERN void conversation_email_view_source (ConversationEmail* self,
                                     GAsyncReadyCallback _callback_,
                                     gpointer _user_data_);
VALA_EXTERN void conversation_email_view_source_finish (ConversationEmail* self,
                                            GAsyncResult* _res_);
VALA_EXTERN void conversation_email_print (ConversationEmail* self,
                               GAsyncReadyCallback _callback_,
                               gpointer _user_data_);
VALA_EXTERN void conversation_email_print_finish (ConversationEmail* self,
                                      GAsyncResult* _res_,
                                      GError** error);
VALA_EXTERN GType conversation_list_box_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationListBox, g_object_unref)
VALA_EXTERN GearyAppConversation* conversation_list_box_get_conversation (ConversationListBox* self);
VALA_EXTERN GType conversation_list_box_search_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationListBoxSearchManager, g_object_unref)
VALA_EXTERN ConversationListBoxSearchManager* conversation_list_box_get_search (ConversationListBox* self);
VALA_EXTERN gboolean conversation_list_box_get_has_composer (ConversationListBox* self);
VALA_EXTERN ConversationListBox* conversation_list_box_new (GearyAppConversation* conversation,
                                                gboolean suppress_mark_timer,
                                                GearyAppEmailStore* email_store,
                                                ApplicationContactStore* contacts,
                                                ApplicationConfiguration* config,
                                                GtkAdjustment* adjustment);
VALA_EXTERN ConversationListBox* conversation_list_box_construct (GType object_type,
                                                      GearyAppConversation* conversation,
                                                      gboolean suppress_mark_timer,
                                                      GearyAppEmailStore* email_store,
                                                      ApplicationContactStore* contacts,
                                                      ApplicationConfiguration* config,
                                                      GtkAdjustment* adjustment);
VALA_EXTERN void conversation_list_box_insert (ConversationListBox* self,
                                   GtkWidget* child,
                                   gint position);
VALA_EXTERN void conversation_list_box_load_conversation (ConversationListBox* self,
                                              GeeCollection* scroll_to,
                                              GearySearchQuery* query,
                                              GAsyncReadyCallback _callback_,
                                              gpointer _user_data_);
VALA_EXTERN void conversation_list_box_load_conversation_finish (ConversationListBox* self,
                                                     GAsyncResult* _res_,
                                                     GError** error);
VALA_EXTERN void conversation_list_box_cancel_conversation_load (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_scroll_to_messages (ConversationListBox* self,
                                               GeeCollection* targets);
VALA_EXTERN ConversationEmail* conversation_list_box_get_reply_target (ConversationListBox* self);
VALA_EXTERN ConversationEmail* conversation_list_box_get_selection_view (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_add_embedded_composer (ConversationListBox* self,
                                                  ComposerEmbed* embed,
                                                  gboolean is_draft);
VALA_EXTERN void conversation_list_box_mark_visible_read (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_add_email_info_bar (ConversationListBox* self,
                                               GearyEmailIdentifier* id,
                                               ComponentsInfoBar* info_bar);
VALA_EXTERN void conversation_list_box_remove_email_info_bar (ConversationListBox* self,
                                                  GearyEmailIdentifier* id,
                                                  ComponentsInfoBar* info_bar);
VALA_EXTERN void conversation_list_box_zoom_in (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_zoom_out (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_zoom_reset (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_update_display (ConversationListBox* self);
VALA_EXTERN void conversation_list_box_search_manager_highlight_matching_email (ConversationListBoxSearchManager* self,
                                                                    GearySearchQuery* query,
                                                                    gboolean enable_scroll,
                                                                    GAsyncReadyCallback _callback_,
                                                                    gpointer _user_data_);
VALA_EXTERN void conversation_list_box_search_manager_highlight_matching_email_finish (ConversationListBoxSearchManager* self,
                                                                           GAsyncResult* _res_,
                                                                           GError** error);
VALA_EXTERN void conversation_list_box_search_manager_unmark_terms (ConversationListBoxSearchManager* self);
VALA_EXTERN void conversation_list_box_search_manager_cancel (ConversationListBoxSearchManager* self);
VALA_EXTERN ConversationMessage* conversation_message_new_from_email (GearyEmail* email,
                                                          gboolean load_remote_resources,
                                                          ApplicationContactStore* contacts,
                                                          ApplicationConfiguration* config);
VALA_EXTERN ConversationMessage* conversation_message_construct_from_email (GType object_type,
                                                                GearyEmail* email,
                                                                gboolean load_remote_resources,
                                                                ApplicationContactStore* contacts,
                                                                ApplicationConfiguration* config);
VALA_EXTERN ConversationMessage* conversation_message_new_from_message (GearyRFC822Message* message,
                                                            gboolean load_remote_resources,
                                                            ApplicationContactStore* contacts,
                                                            ApplicationConfiguration* config);
VALA_EXTERN ConversationMessage* conversation_message_construct_from_message (GType object_type,
                                                                  GearyRFC822Message* message,
                                                                  gboolean load_remote_resources,
                                                                  ApplicationContactStore* contacts,
                                                                  ApplicationConfiguration* config);
VALA_EXTERN void conversation_message_get_selection_for_quoting (ConversationMessage* self,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_);
VALA_EXTERN gchar* conversation_message_get_selection_for_quoting_finish (ConversationMessage* self,
                                                              GAsyncResult* _res_,
                                                              GError** error);
VALA_EXTERN void conversation_message_get_selection_for_find (ConversationMessage* self,
                                                  GAsyncReadyCallback _callback_,
                                                  gpointer _user_data_);
VALA_EXTERN gchar* conversation_message_get_selection_for_find_finish (ConversationMessage* self,
                                                           GAsyncResult* _res_,
                                                           GError** error);
VALA_EXTERN void conversation_message_add_internal_resources (ConversationMessage* self,
                                                  GeeMap* res);
VALA_EXTERN WebKitPrintOperation* conversation_message_new_print_operation (ConversationMessage* self);
VALA_EXTERN void conversation_message_evaluate_javascript (ConversationMessage* self,
                                               const gchar* script,
                                               GCancellable* cancellable,
                                               GAsyncReadyCallback _callback_,
                                               gpointer _user_data_);
VALA_EXTERN void conversation_message_evaluate_javascript_finish (ConversationMessage* self,
                                                      GAsyncResult* _res_,
                                                      GError** error);
VALA_EXTERN void conversation_message_zoom_in (ConversationMessage* self);
VALA_EXTERN void conversation_message_zoom_out (ConversationMessage* self);
VALA_EXTERN void conversation_message_zoom_reset (ConversationMessage* self);
VALA_EXTERN void conversation_message_web_view_translate_coordinates (ConversationMessage* self,
                                                          GtkWidget* widget,
                                                          gint x,
                                                          gint anchor_y,
                                                          gint* x1,
                                                          gint* y1);
VALA_EXTERN gint conversation_message_web_view_get_allocated_height (ConversationMessage* self);
VALA_EXTERN void conversation_message_show_message_body (ConversationMessage* self,
                                             gboolean include_transitions);
VALA_EXTERN void conversation_message_hide_message_body (ConversationMessage* self);
VALA_EXTERN void conversation_message_show_loading_pane (ConversationMessage* self);
VALA_EXTERN void conversation_message_show_load_error_pane (ConversationMessage* self);
VALA_EXTERN void conversation_message_show_offline_pane (ConversationMessage* self);
VALA_EXTERN void conversation_message_start_progress_loading (ConversationMessage* self);
VALA_EXTERN void conversation_message_stop_progress_loading (ConversationMessage* self);
VALA_EXTERN void conversation_message_start_progress_pulse (ConversationMessage* self);
VALA_EXTERN void conversation_message_stop_progress_pulse (ConversationMessage* self);
VALA_EXTERN void conversation_message_load_contacts (ConversationMessage* self,
                                         GCancellable* cancellable,
                                         GAsyncReadyCallback _callback_,
                                         gpointer _user_data_);
VALA_EXTERN void conversation_message_load_contacts_finish (ConversationMessage* self,
                                                GAsyncResult* _res_,
                                                GError** error);
VALA_EXTERN void conversation_message_load_message_body (ConversationMessage* self,
                                             GearyRFC822Message* message,
                                             GCancellable* load_cancelled,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN void conversation_message_load_message_body_finish (ConversationMessage* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
VALA_EXTERN void conversation_message_highlight_search_terms (ConversationMessage* self,
                                                  GeeSet* search_matches,
                                                  GCancellable* cancellable,
                                                  GAsyncReadyCallback _callback_,
                                                  gpointer _user_data_);
VALA_EXTERN guint conversation_message_highlight_search_terms_finish (ConversationMessage* self,
                                                          GAsyncResult* _res_,
                                                          GError** error);
VALA_EXTERN void conversation_message_unmark_search_terms (ConversationMessage* self);
VALA_EXTERN void conversation_message_update_display (ConversationMessage* self);
VALA_EXTERN GType conversation_message_contact_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationMessageContactList, g_object_unref)
VALA_EXTERN ConversationMessageContactList* conversation_message_contact_list_new (void);
VALA_EXTERN ConversationMessageContactList* conversation_message_contact_list_construct (GType object_type);
VALA_EXTERN ConversationListBox* conversation_viewer_get_current_list (ConversationViewer* self);
VALA_EXTERN ComposerWidget* conversation_viewer_get_current_composer (ConversationViewer* self);
VALA_EXTERN ConversationViewer* conversation_viewer_new (ApplicationConfiguration* config);
VALA_EXTERN ConversationViewer* conversation_viewer_construct (GType object_type,
                                                   ApplicationConfiguration* config);
VALA_EXTERN void conversation_viewer_do_compose (ConversationViewer* self,
                                     ComposerWidget* composer);
VALA_EXTERN void conversation_viewer_do_compose_embedded (ConversationViewer* self,
                                              ComposerWidget* composer,
                                              GearyEmail* referred);
VALA_EXTERN void conversation_viewer_show_loading (ConversationViewer* self);
VALA_EXTERN void conversation_viewer_show_none_selected (ConversationViewer* self);
VALA_EXTERN void conversation_viewer_show_multiple_selected (ConversationViewer* self);
VALA_EXTERN void conversation_viewer_show_empty_folder (ConversationViewer* self);
VALA_EXTERN void conversation_viewer_show_empty_search (ConversationViewer* self);
VALA_EXTERN void conversation_viewer_enable_find (ConversationViewer* self);
VALA_EXTERN void conversation_viewer_load_conversation (ConversationViewer* self,
                                            GearyAppConversation* conversation,
                                            GeeCollection* scroll_to,
                                            GearyAppEmailStore* store,
                                            ApplicationContactStore* contacts,
                                            gboolean start_mark_timer,
                                            GAsyncReadyCallback _callback_,
                                            gpointer _user_data_);
VALA_EXTERN void conversation_viewer_load_conversation_finish (ConversationViewer* self,
                                                   GAsyncResult* _res_,
                                                   GError** error);
VALA_EXTERN GType conversation_web_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ConversationWebView, g_object_unref)
VALA_EXTERN GType conversation_web_view_deceptive_text_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void conversation_web_view_load_resources (GError** error);
VALA_EXTERN ConversationWebView* conversation_web_view_new (ApplicationConfiguration* config);
VALA_EXTERN ConversationWebView* conversation_web_view_construct (GType object_type,
                                                      ApplicationConfiguration* config);
VALA_EXTERN void conversation_web_view_get_selection_for_find (ConversationWebView* self,
                                                   GAsyncReadyCallback _callback_,
                                                   gpointer _user_data_);
VALA_EXTERN gchar* conversation_web_view_get_selection_for_find_finish (ConversationWebView* self,
                                                            GAsyncResult* _res_,
                                                            GError** error);
VALA_EXTERN void conversation_web_view_get_selection_for_quoting (ConversationWebView* self,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_);
VALA_EXTERN gchar* conversation_web_view_get_selection_for_quoting_finish (ConversationWebView* self,
                                                               GAsyncResult* _res_,
                                                               GError** error);
VALA_EXTERN void conversation_web_view_get_anchor_target_y (ConversationWebView* self,
                                                const gchar* anchor_body,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_);
VALA_EXTERN gint* conversation_web_view_get_anchor_target_y_finish (ConversationWebView* self,
                                                        GAsyncResult* _res_,
                                                        GError** error);
VALA_EXTERN void conversation_web_view_highlight_search_terms (ConversationWebView* self,
                                                   GeeCollection* terms,
                                                   GCancellable* cancellable,
                                                   GAsyncReadyCallback _callback_,
                                                   gpointer _user_data_);
VALA_EXTERN guint conversation_web_view_highlight_search_terms_finish (ConversationWebView* self,
                                                           GAsyncResult* _res_,
                                                           GError** error);
VALA_EXTERN void conversation_web_view_unmark_search_terms (ConversationWebView* self);
VALA_EXTERN GType attachment_dialog_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (AttachmentDialog, g_object_unref)
VALA_EXTERN AttachmentDialog* attachment_dialog_new (GtkWindow* parent,
                                         ApplicationConfiguration* config);
VALA_EXTERN AttachmentDialog* attachment_dialog_construct (GType object_type,
                                               GtkWindow* parent,
                                               ApplicationConfiguration* config);
VALA_EXTERN void attachment_dialog_add_filter (AttachmentDialog* self,
                                   GtkFileFilter* filter);
VALA_EXTERN GSList* attachment_dialog_get_files (AttachmentDialog* self);
VALA_EXTERN gint attachment_dialog_run (AttachmentDialog* self);
VALA_EXTERN void attachment_dialog_hide (AttachmentDialog* self);
VALA_EXTERN void attachment_dialog_destroy (AttachmentDialog* self);
VALA_EXTERN gpointer certificate_warning_dialog_ref (gpointer instance);
VALA_EXTERN void certificate_warning_dialog_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_certificate_warning_dialog (const gchar* name,
                                                   const gchar* nick,
                                                   const gchar* blurb,
                                                   GType object_type,
                                                   GParamFlags flags);
VALA_EXTERN void value_set_certificate_warning_dialog (GValue* value,
                                           gpointer v_object);
VALA_EXTERN void value_take_certificate_warning_dialog (GValue* value,
                                            gpointer v_object);
VALA_EXTERN gpointer value_get_certificate_warning_dialog (const GValue* value);
VALA_EXTERN GType certificate_warning_dialog_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CertificateWarningDialog, certificate_warning_dialog_unref)
VALA_EXTERN GType certificate_warning_dialog_result_get_type (void) G_GNUC_CONST ;
VALA_EXTERN CertificateWarningDialog* certificate_warning_dialog_new (GtkWindow* parent,
                                                          GearyAccountInformation* account,
                                                          GearyServiceInformation* service,
                                                          GearyEndpoint* endpoint,
                                                          gboolean is_validation);
VALA_EXTERN CertificateWarningDialog* certificate_warning_dialog_construct (GType object_type,
                                                                GtkWindow* parent,
                                                                GearyAccountInformation* account,
                                                                GearyServiceInformation* service,
                                                                GearyEndpoint* endpoint,
                                                                gboolean is_validation);
VALA_EXTERN CertificateWarningDialogResult certificate_warning_dialog_run (CertificateWarningDialog* self);
VALA_EXTERN GType dialogs_problem_details_dialog_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DialogsProblemDetailsDialog, g_object_unref)
VALA_EXTERN void dialogs_problem_details_dialog_add_accelerators (ApplicationClient* app);
VALA_EXTERN DialogsProblemDetailsDialog* dialogs_problem_details_dialog_new (GtkWindow* parent,
                                                                 ApplicationClient* application,
                                                                 GearyProblemReport* report);
VALA_EXTERN DialogsProblemDetailsDialog* dialogs_problem_details_dialog_construct (GType object_type,
                                                                       GtkWindow* parent,
                                                                       ApplicationClient* application,
                                                                       GearyProblemReport* report);
VALA_EXTERN gpointer password_dialog_ref (gpointer instance);
VALA_EXTERN void password_dialog_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_password_dialog (const gchar* name,
                                        const gchar* nick,
                                        const gchar* blurb,
                                        GType object_type,
                                        GParamFlags flags);
VALA_EXTERN void value_set_password_dialog (GValue* value,
                                gpointer v_object);
VALA_EXTERN void value_take_password_dialog (GValue* value,
                                 gpointer v_object);
VALA_EXTERN gpointer value_get_password_dialog (const GValue* value);
VALA_EXTERN GType password_dialog_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PasswordDialog, password_dialog_unref)
VALA_EXTERN const gchar* password_dialog_get_password (PasswordDialog* self);
VALA_EXTERN gboolean password_dialog_get_remember_password (PasswordDialog* self);
VALA_EXTERN PasswordDialog* password_dialog_new (GtkWindow* parent,
                                     GearyAccountInformation* account,
                                     GearyServiceInformation* service,
                                     GearyCredentials* credentials);
VALA_EXTERN PasswordDialog* password_dialog_construct (GType object_type,
                                           GtkWindow* parent,
                                           GearyAccountInformation* account,
                                           GearyServiceInformation* service,
                                           GearyCredentials* credentials);
VALA_EXTERN gboolean password_dialog_run (PasswordDialog* self);
VALA_EXTERN GType sidebar_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarEntry, g_object_unref)
VALA_EXTERN GType sidebar_selectable_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType folder_list_abstract_folder_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListAbstractFolderEntry, g_object_unref)
VALA_EXTERN GearyFolder* folder_list_abstract_folder_entry_get_folder (FolderListAbstractFolderEntry* self);
VALA_EXTERN FolderListAbstractFolderEntry* folder_list_abstract_folder_entry_construct (GType object_type,
                                                                            GearyFolder* folder);
VALA_EXTERN gchar* folder_list_abstract_folder_entry_get_sidebar_name (FolderListAbstractFolderEntry* self);
VALA_EXTERN gchar* folder_list_abstract_folder_entry_get_sidebar_tooltip (FolderListAbstractFolderEntry* self);
VALA_EXTERN gchar* folder_list_abstract_folder_entry_get_sidebar_icon (FolderListAbstractFolderEntry* self);
VALA_EXTERN gint folder_list_abstract_folder_entry_get_count (FolderListAbstractFolderEntry* self);
VALA_EXTERN gchar* folder_list_abstract_folder_entry_to_string (FolderListAbstractFolderEntry* self);
VALA_EXTERN GType sidebar_branch_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarBranch, g_object_unref)
VALA_EXTERN GType folder_list_account_branch_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListAccountBranch, g_object_unref)
VALA_EXTERN GearyAccount* folder_list_account_branch_get_account (FolderListAccountBranch* self);
VALA_EXTERN GType sidebar_grouping_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarGrouping, g_object_unref)
VALA_EXTERN GType folder_list_special_grouping_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListSpecialGrouping, g_object_unref)
VALA_EXTERN FolderListSpecialGrouping* folder_list_account_branch_get_user_folder_group (FolderListAccountBranch* self);
VALA_EXTERN GType folder_list_folder_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListFolderEntry, g_object_unref)
VALA_EXTERN GeeHashMap* folder_list_account_branch_get_folder_entries (FolderListAccountBranch* self);
VALA_EXTERN FolderListAccountBranch* folder_list_account_branch_new (GearyAccount* account);
VALA_EXTERN FolderListAccountBranch* folder_list_account_branch_construct (GType object_type,
                                                               GearyAccount* account);
VALA_EXTERN FolderListFolderEntry* folder_list_account_branch_get_entry_for_path (FolderListAccountBranch* self,
                                                                      GearyFolderPath* folder_path);
VALA_EXTERN void folder_list_account_branch_add_folder (FolderListAccountBranch* self,
                                            ApplicationFolderContext* context);
VALA_EXTERN void folder_list_account_branch_remove_folder (FolderListAccountBranch* self,
                                               GearyFolderPath* path);
VALA_EXTERN GType sidebar_internal_drop_target_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType sidebar_emphasizable_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN FolderListFolderEntry* folder_list_folder_entry_new (ApplicationFolderContext* context);
VALA_EXTERN FolderListFolderEntry* folder_list_folder_entry_construct (GType object_type,
                                                           ApplicationFolderContext* context);
VALA_EXTERN void folder_list_folder_entry_set_has_new (FolderListFolderEntry* self,
                                           gboolean has_new);
VALA_EXTERN GearyFolder* folder_list_tree_get_selected (FolderListTree* self);
VALA_EXTERN FolderListTree* folder_list_tree_new (void);
VALA_EXTERN FolderListTree* folder_list_tree_construct (GType object_type);
VALA_EXTERN void folder_list_tree_set_has_new (FolderListTree* self,
                                   GearyFolder* folder,
                                   gboolean has_new);
VALA_EXTERN void folder_list_tree_set_user_folders_root_name (FolderListTree* self,
                                                  GearyAccount* account,
                                                  const gchar* name);
VALA_EXTERN void folder_list_tree_add_folder (FolderListTree* self,
                                  ApplicationFolderContext* context);
VALA_EXTERN void folder_list_tree_remove_folder (FolderListTree* self,
                                     ApplicationFolderContext* context);
VALA_EXTERN void folder_list_tree_remove_account (FolderListTree* self,
                                      GearyAccount* account);
VALA_EXTERN void folder_list_tree_select_folder (FolderListTree* self,
                                     GearyFolder* to_select);
VALA_EXTERN gboolean folder_list_tree_select_inbox (FolderListTree* self,
                                        GearyAccount* account);
VALA_EXTERN void folder_list_tree_deselect_folder (FolderListTree* self);
VALA_EXTERN void folder_list_tree_set_search (FolderListTree* self,
                                  GearyEngine* engine,
                                  GearyAppSearchFolder* search_folder);
VALA_EXTERN void folder_list_tree_remove_search (FolderListTree* self);
VALA_EXTERN GType folder_list_inboxes_branch_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListInboxesBranch, g_object_unref)
VALA_EXTERN GType folder_list_inbox_folder_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListInboxFolderEntry, g_object_unref)
VALA_EXTERN GeeHashMap* folder_list_inboxes_branch_get_folder_entries (FolderListInboxesBranch* self);
VALA_EXTERN FolderListInboxesBranch* folder_list_inboxes_branch_new (void);
VALA_EXTERN FolderListInboxesBranch* folder_list_inboxes_branch_construct (GType object_type);
VALA_EXTERN FolderListInboxFolderEntry* folder_list_inboxes_branch_get_entry_for_account (FolderListInboxesBranch* self,
                                                                              GearyAccount* account);
VALA_EXTERN void folder_list_inboxes_branch_add_inbox (FolderListInboxesBranch* self,
                                           ApplicationFolderContext* inbox);
VALA_EXTERN void folder_list_inboxes_branch_remove_inbox (FolderListInboxesBranch* self,
                                              GearyAccount* account);
VALA_EXTERN FolderListInboxFolderEntry* folder_list_inbox_folder_entry_new (ApplicationFolderContext* context);
VALA_EXTERN FolderListInboxFolderEntry* folder_list_inbox_folder_entry_construct (GType object_type,
                                                                      ApplicationFolderContext* context);
VALA_EXTERN GearyAccountInformation* folder_list_inbox_folder_entry_get_account_information (FolderListInboxFolderEntry* self);
VALA_EXTERN GType sidebar_root_only_branch_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarRootOnlyBranch, g_object_unref)
VALA_EXTERN GType folder_list_search_branch_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListSearchBranch, g_object_unref)
VALA_EXTERN FolderListSearchBranch* folder_list_search_branch_new (GearyAppSearchFolder* folder,
                                                       GearyEngine* engine);
VALA_EXTERN FolderListSearchBranch* folder_list_search_branch_construct (GType object_type,
                                                             GearyAppSearchFolder* folder,
                                                             GearyEngine* engine);
VALA_EXTERN GearyAppSearchFolder* folder_list_search_branch_get_search_folder (FolderListSearchBranch* self);
VALA_EXTERN GType folder_list_search_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolderListSearchEntry, g_object_unref)
VALA_EXTERN FolderListSearchEntry* folder_list_search_entry_new (GearyAppSearchFolder* folder,
                                                     GearyEngine* engine);
VALA_EXTERN FolderListSearchEntry* folder_list_search_entry_construct (GType object_type,
                                                           GearyAppSearchFolder* folder,
                                                           GearyEngine* engine);
VALA_EXTERN GType sidebar_expandable_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType sidebar_renameable_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gint folder_list_special_grouping_get_position (FolderListSpecialGrouping* self);
VALA_EXTERN FolderListSpecialGrouping* folder_list_special_grouping_new (gint position,
                                                             const gchar* name,
                                                             const gchar* icon,
                                                             const gchar* tooltip);
VALA_EXTERN FolderListSpecialGrouping* folder_list_special_grouping_construct (GType object_type,
                                                                   gint position,
                                                                   const gchar* name,
                                                                   const gchar* icon,
                                                                   const gchar* tooltip);
VALA_EXTERN const gchar* plugin_account_get_display_name (PluginAccount* self);
VALA_EXTERN GType plugin_action_bar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionBar, g_object_unref)
VALA_EXTERN GType plugin_action_bar_position_get_type (void) G_GNUC_CONST ;
VALA_EXTERN PluginActionBar* plugin_action_bar_new (void);
VALA_EXTERN PluginActionBar* plugin_action_bar_construct (GType object_type);
VALA_EXTERN GType plugin_action_bar_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionBarItem, g_object_unref)
VALA_EXTERN void plugin_action_bar_append_item (PluginActionBar* self,
                                    PluginActionBarItem* item,
                                    PluginActionBarPosition item_position);
VALA_EXTERN GeeList* plugin_action_bar_get_items (PluginActionBar* self,
                                      PluginActionBarPosition item_position);
VALA_EXTERN GType plugin_action_bar_label_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionBarLabelItem, g_object_unref)
VALA_EXTERN const gchar* plugin_action_bar_label_item_get_text (PluginActionBarLabelItem* self);
VALA_EXTERN PluginActionBarLabelItem* plugin_action_bar_label_item_new (const gchar* text);
VALA_EXTERN PluginActionBarLabelItem* plugin_action_bar_label_item_construct (GType object_type,
                                                                  const gchar* text);
VALA_EXTERN GType plugin_action_bar_button_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionBarButtonItem, g_object_unref)
VALA_EXTERN GType plugin_actionable_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionable, g_object_unref)
VALA_EXTERN PluginActionable* plugin_action_bar_button_item_get_action (PluginActionBarButtonItem* self);
VALA_EXTERN PluginActionBarButtonItem* plugin_action_bar_button_item_new (PluginActionable* action);
VALA_EXTERN PluginActionBarButtonItem* plugin_action_bar_button_item_construct (GType object_type,
                                                                    PluginActionable* action);
VALA_EXTERN GType plugin_action_bar_menu_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionBarMenuItem, g_object_unref)
VALA_EXTERN const gchar* plugin_action_bar_menu_item_get_label (PluginActionBarMenuItem* self);
VALA_EXTERN GMenuModel* plugin_action_bar_menu_item_get_menu (PluginActionBarMenuItem* self);
VALA_EXTERN PluginActionBarMenuItem* plugin_action_bar_menu_item_new (const gchar* label,
                                                          GMenuModel* menu);
VALA_EXTERN PluginActionBarMenuItem* plugin_action_bar_menu_item_construct (GType object_type,
                                                                const gchar* label,
                                                                GMenuModel* menu);
VALA_EXTERN GType plugin_action_bar_group_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginActionBarGroupItem, g_object_unref)
VALA_EXTERN PluginActionBarGroupItem* plugin_action_bar_group_item_new (GeeCollection* items);
VALA_EXTERN PluginActionBarGroupItem* plugin_action_bar_group_item_construct (GType object_type,
                                                                  GeeCollection* items);
VALA_EXTERN void plugin_action_bar_group_item_append_item (PluginActionBarGroupItem* self,
                                               PluginActionBarItem* item);
VALA_EXTERN GeeList* plugin_action_bar_group_item_get_items (PluginActionBarGroupItem* self);
VALA_EXTERN const gchar* plugin_actionable_get_label (PluginActionable* self);
VALA_EXTERN const gchar* plugin_actionable_get_icon_name (PluginActionable* self);
VALA_EXTERN GAction* plugin_actionable_get_action (PluginActionable* self);
VALA_EXTERN GVariant* plugin_actionable_get_action_target (PluginActionable* self);
VALA_EXTERN PluginActionable* plugin_actionable_new (const gchar* label,
                                         GAction* action,
                                         GVariant* action_target);
VALA_EXTERN PluginActionable* plugin_actionable_construct (GType object_type,
                                               const gchar* label,
                                               GAction* action,
                                               GVariant* action_target);
VALA_EXTERN PluginActionable* plugin_actionable_new_with_icon (const gchar* label,
                                                   const gchar* icon_name,
                                                   GAction* action,
                                                   GVariant* action_target);
VALA_EXTERN PluginActionable* plugin_actionable_construct_with_icon (GType object_type,
                                                         const gchar* label,
                                                         const gchar* icon_name,
                                                         GAction* action,
                                                         GVariant* action_target);
VALA_EXTERN GType plugin_composer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginComposer, g_object_unref)
VALA_EXTERN GQuark plugin_error_quark (void);
VALA_EXTERN GType plugin_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType plugin_composer_context_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType plugin_application_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginApplication, g_object_unref)
VALA_EXTERN void plugin_application_compose_blank (PluginApplication* self,
                                       PluginAccount* send_from,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN PluginComposer* plugin_application_compose_blank_finish (PluginApplication* self,
                                                         GAsyncResult* _res_,
                                                         GError** error);
VALA_EXTERN void plugin_application_compose_with_context (PluginApplication* self,
                                              PluginAccount* send_from,
                                              PluginComposerContextType type,
                                              PluginEmailIdentifier* context,
                                              const gchar* quote,
                                              GAsyncReadyCallback _callback_,
                                              gpointer _user_data_);
VALA_EXTERN PluginComposer* plugin_application_compose_with_context_finish (PluginApplication* self,
                                                                GAsyncResult* _res_,
                                                                GError** error);
VALA_EXTERN void plugin_application_register_action (PluginApplication* self,
                                         GAction* action);
VALA_EXTERN void plugin_application_deregister_action (PluginApplication* self,
                                           GAction* action);
VALA_EXTERN void plugin_application_show_folder (PluginApplication* self,
                                     PluginFolder* folder);
VALA_EXTERN void plugin_application_empty_folder (PluginApplication* self,
                                      PluginFolder* folder,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN void plugin_application_empty_folder_finish (PluginApplication* self,
                                             GAsyncResult* _res_,
                                             GError** error);
VALA_EXTERN void plugin_application_report_problem (PluginApplication* self,
                                        GearyProblemReport* problem);
VALA_EXTERN PluginAccount* plugin_composer_get_sender_context (PluginComposer* self);
VALA_EXTERN gboolean plugin_composer_get_can_send (PluginComposer* self);
VALA_EXTERN void plugin_composer_set_can_send (PluginComposer* self,
                                   gboolean value);
VALA_EXTERN const gchar* plugin_composer_get_action_group_name (PluginComposer* self);
VALA_EXTERN PluginFolder* plugin_composer_get_save_to (PluginComposer* self);
VALA_EXTERN void plugin_composer_present (PluginComposer* self);
VALA_EXTERN void plugin_composer_insert_text (PluginComposer* self,
                                  const gchar* plain_text);
VALA_EXTERN void plugin_composer_save_to_folder (PluginComposer* self,
                                     PluginFolder* location);
VALA_EXTERN void plugin_composer_register_action (PluginComposer* self,
                                      GAction* action);
VALA_EXTERN void plugin_composer_deregister_action (PluginComposer* self,
                                        GAction* action);
VALA_EXTERN void plugin_composer_append_menu_item (PluginComposer* self,
                                       PluginActionable* menu_item);
VALA_EXTERN void plugin_composer_set_action_bar (PluginComposer* self,
                                     PluginActionBar* action_bar);
VALA_EXTERN GType plugin_contact_store_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginContactStore, g_object_unref)
VALA_EXTERN void plugin_contact_store_search (PluginContactStore* self,
                                  const gchar* query,
                                  guint min_importance,
                                  guint limit,
                                  GCancellable* cancellable,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN GeeCollection* plugin_contact_store_search_finish (PluginContactStore* self,
                                                   GAsyncResult* _res_,
                                                   GError** error);
VALA_EXTERN void plugin_contact_store_load (PluginContactStore* self,
                                GearyRFC822MailboxAddress* mailbox,
                                GCancellable* cancellable,
                                GAsyncReadyCallback _callback_,
                                gpointer _user_data_);
VALA_EXTERN ApplicationContact* plugin_contact_store_load_finish (PluginContactStore* self,
                                                      GAsyncResult* _res_,
                                                      GError** error);
VALA_EXTERN GType plugin_plugin_base_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginPluginBase, g_object_unref)
VALA_EXTERN GType plugin_email_store_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginEmailStore, g_object_unref)
VALA_EXTERN GType plugin_email_context_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginEmailContext, g_object_unref)
VALA_EXTERN GType plugin_email_extension_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginEmailExtension, g_object_unref)
VALA_EXTERN PluginEmailContext* plugin_email_extension_get_email (PluginEmailExtension* self);
VALA_EXTERN void plugin_email_extension_set_email (PluginEmailExtension* self,
                                       PluginEmailContext* value);
VALA_EXTERN void plugin_email_context_get_email_store (PluginEmailContext* self,
                                           GAsyncReadyCallback _callback_,
                                           gpointer _user_data_);
VALA_EXTERN PluginEmailStore* plugin_email_context_get_email_store_finish (PluginEmailContext* self,
                                                               GAsyncResult* _res_,
                                                               GError** error);
VALA_EXTERN void plugin_email_context_add_email_info_bar (PluginEmailContext* self,
                                              PluginEmailIdentifier* displayed,
                                              PluginInfoBar* info_bar,
                                              guint priority);
VALA_EXTERN void plugin_email_context_remove_email_info_bar (PluginEmailContext* self,
                                                 PluginEmailIdentifier* displayed,
                                                 PluginInfoBar* info_bar);
VALA_EXTERN const GVariantType* plugin_email_store_get_email_identifier_variant_type (PluginEmailStore* self);
VALA_EXTERN void plugin_email_store_get_email (PluginEmailStore* self,
                                   GeeCollection* ids,
                                   GCancellable* cancellable,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN GeeCollection* plugin_email_store_get_email_finish (PluginEmailStore* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
VALA_EXTERN PluginEmailIdentifier* plugin_email_store_get_email_identifier_for_variant (PluginEmailStore* self,
                                                                            GVariant* id);
VALA_EXTERN PluginEmailIdentifier* plugin_email_get_identifier (PluginEmail* self);
VALA_EXTERN GearyEmailFlags* plugin_email_get_flags (PluginEmail* self);
VALA_EXTERN GearyRFC822MailboxAddress* plugin_email_get_primary_originator (PluginEmail* self);
VALA_EXTERN void plugin_email_load_body_as (PluginEmail* self,
                                PluginEmailBodyType type,
                                gboolean convert,
                                GCancellable* cancellable,
                                GAsyncReadyCallback _callback_,
                                gpointer _user_data_);
VALA_EXTERN gchar* plugin_email_load_body_as_finish (PluginEmail* self,
                                         GAsyncResult* _res_,
                                         GError** error);
VALA_EXTERN PluginAccount* plugin_email_identifier_get_account (PluginEmailIdentifier* self);
VALA_EXTERN GVariant* plugin_email_identifier_to_variant (PluginEmailIdentifier* self);
VALA_EXTERN GType plugin_folder_store_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginFolderStore, g_object_unref)
VALA_EXTERN GType plugin_folder_context_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginFolderContext, g_object_unref)
VALA_EXTERN GType plugin_folder_extension_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginFolderExtension, g_object_unref)
VALA_EXTERN PluginFolderContext* plugin_folder_extension_get_folders (PluginFolderExtension* self);
VALA_EXTERN void plugin_folder_extension_set_folders (PluginFolderExtension* self,
                                          PluginFolderContext* value);
VALA_EXTERN void plugin_folder_context_get_folder_store (PluginFolderContext* self,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN PluginFolderStore* plugin_folder_context_get_folder_store_finish (PluginFolderContext* self,
                                                                  GAsyncResult* _res_,
                                                                  GError** error);
VALA_EXTERN void plugin_folder_context_add_folder_info_bar (PluginFolderContext* self,
                                                PluginFolder* selected,
                                                PluginInfoBar* infobar,
                                                guint priority);
VALA_EXTERN void plugin_folder_context_remove_folder_info_bar (PluginFolderContext* self,
                                                   PluginFolder* selected,
                                                   PluginInfoBar* infobar);
VALA_EXTERN void plugin_folder_context_register_folder_used_as (PluginFolderContext* self,
                                                    PluginFolder* target,
                                                    const gchar* name,
                                                    const gchar* icon_name,
                                                    GError** error);
VALA_EXTERN void plugin_folder_context_unregister_folder_used_as (PluginFolderContext* self,
                                                      PluginFolder* target,
                                                      GError** error);
VALA_EXTERN const GVariantType* plugin_folder_store_get_folder_variant_type (PluginFolderStore* self);
VALA_EXTERN GeeCollection* plugin_folder_store_get_folders (PluginFolderStore* self);
VALA_EXTERN void plugin_folder_store_list_containing_folders (PluginFolderStore* self,
                                                  PluginEmailIdentifier* target,
                                                  GCancellable* cancellable,
                                                  GAsyncReadyCallback _callback_,
                                                  gpointer _user_data_);
VALA_EXTERN GeeCollection* plugin_folder_store_list_containing_folders_finish (PluginFolderStore* self,
                                                                   GAsyncResult* _res_,
                                                                   GError** error);
VALA_EXTERN void plugin_folder_store_create_personal_folder (PluginFolderStore* self,
                                                 PluginAccount* target,
                                                 const gchar* name,
                                                 GCancellable* cancellable,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
VALA_EXTERN PluginFolder* plugin_folder_store_create_personal_folder_finish (PluginFolderStore* self,
                                                                 GAsyncResult* _res_,
                                                                 GError** error);
VALA_EXTERN PluginFolder* plugin_folder_store_get_folder_for_variant (PluginFolderStore* self,
                                                          GVariant* id);
VALA_EXTERN const gchar* plugin_folder_get_persistent_id (PluginFolder* self);
VALA_EXTERN const gchar* plugin_folder_get_display_name (PluginFolder* self);
VALA_EXTERN GearyFolderSpecialUse plugin_folder_get_used_as (PluginFolder* self);
VALA_EXTERN PluginAccount* plugin_folder_get_account (PluginFolder* self);
VALA_EXTERN GVariant* plugin_folder_to_variant (PluginFolder* self);
VALA_EXTERN const gchar* plugin_info_bar_get_status (PluginInfoBar* self);
VALA_EXTERN void plugin_info_bar_set_status (PluginInfoBar* self,
                                 const gchar* value);
VALA_EXTERN const gchar* plugin_info_bar_get_description (PluginInfoBar* self);
VALA_EXTERN void plugin_info_bar_set_description (PluginInfoBar* self,
                                      const gchar* value);
VALA_EXTERN gboolean plugin_info_bar_get_show_close_button (PluginInfoBar* self);
VALA_EXTERN void plugin_info_bar_set_show_close_button (PluginInfoBar* self,
                                            gboolean value);
VALA_EXTERN PluginActionable* plugin_info_bar_get_primary_button (PluginInfoBar* self);
VALA_EXTERN void plugin_info_bar_set_primary_button (PluginInfoBar* self,
                                         PluginActionable* value);
VALA_EXTERN GeeBidirList* plugin_info_bar_get_secondary_buttons (PluginInfoBar* self);
VALA_EXTERN PluginInfoBar* plugin_info_bar_new (const gchar* status,
                                    const gchar* description);
VALA_EXTERN PluginInfoBar* plugin_info_bar_construct (GType object_type,
                                          const gchar* status,
                                          const gchar* description);
VALA_EXTERN GType plugin_notification_context_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginNotificationContext, g_object_unref)
VALA_EXTERN GType plugin_notification_extension_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginNotificationExtension, g_object_unref)
VALA_EXTERN PluginNotificationContext* plugin_notification_extension_get_notifications (PluginNotificationExtension* self);
VALA_EXTERN void plugin_notification_extension_set_notifications (PluginNotificationExtension* self,
                                                      PluginNotificationContext* value);
VALA_EXTERN gint plugin_notification_context_get_total_new_messages (PluginNotificationContext* self);
VALA_EXTERN void plugin_notification_context_get_contacts_for_folder (PluginNotificationContext* self,
                                                          PluginFolder* source,
                                                          GAsyncReadyCallback _callback_,
                                                          gpointer _user_data_);
VALA_EXTERN PluginContactStore* plugin_notification_context_get_contacts_for_folder_finish (PluginNotificationContext* self,
                                                                                GAsyncResult* _res_,
                                                                                GError** error);
VALA_EXTERN gboolean plugin_notification_context_should_notify_new_messages (PluginNotificationContext* self,
                                                                 PluginFolder* target);
VALA_EXTERN gint plugin_notification_context_get_new_message_count (PluginNotificationContext* self,
                                                        PluginFolder* target,
                                                        GError** error);
VALA_EXTERN void plugin_notification_context_start_monitoring_folder (PluginNotificationContext* self,
                                                          PluginFolder* target);
VALA_EXTERN void plugin_notification_context_stop_monitoring_folder (PluginNotificationContext* self,
                                                         PluginFolder* target);
VALA_EXTERN gboolean plugin_notification_context_is_monitoring_folder (PluginNotificationContext* self,
                                                           PluginFolder* target);
VALA_EXTERN PluginApplication* plugin_plugin_base_get_plugin_application (PluginPluginBase* self);
VALA_EXTERN void plugin_plugin_base_activate (PluginPluginBase* self,
                                  gboolean is_startup,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN void plugin_plugin_base_activate_finish (PluginPluginBase* self,
                                         GAsyncResult* _res_,
                                         GError** error);
VALA_EXTERN void plugin_plugin_base_deactivate (PluginPluginBase* self,
                                    gboolean is_shutdown,
                                    GAsyncReadyCallback _callback_,
                                    gpointer _user_data_);
VALA_EXTERN void plugin_plugin_base_deactivate_finish (PluginPluginBase* self,
                                           GAsyncResult* _res_,
                                           GError** error);
VALA_EXTERN PluginPluginBase* plugin_plugin_base_construct (GType object_type);
VALA_EXTERN GType plugin_trusted_extension_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PluginTrustedExtension, g_object_unref)
VALA_EXTERN ApplicationClient* plugin_trusted_extension_get_client_application (PluginTrustedExtension* self);
VALA_EXTERN void plugin_trusted_extension_set_client_application (PluginTrustedExtension* self,
                                                      ApplicationClient* value);
VALA_EXTERN ApplicationPluginManager* plugin_trusted_extension_get_client_plugins (PluginTrustedExtension* self);
VALA_EXTERN void plugin_trusted_extension_set_client_plugins (PluginTrustedExtension* self,
                                                  ApplicationPluginManager* value);
VALA_EXTERN GType sidebar_branch_options_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gboolean sidebar_branch_options_is_hide_if_empty (SidebarBranchOptions self);
VALA_EXTERN gboolean sidebar_branch_options_is_auto_open_on_new_child (SidebarBranchOptions self);
VALA_EXTERN gboolean sidebar_branch_options_is_startup_expand_to_first_child (SidebarBranchOptions self);
VALA_EXTERN gboolean sidebar_branch_options_is_startup_open_grouping (SidebarBranchOptions self);
VALA_EXTERN SidebarBranch* sidebar_branch_new (SidebarEntry* root,
                                   SidebarBranchOptions options,
                                   GCompareFunc default_comparator,
                                   GCompareFunc root_comparator);
VALA_EXTERN SidebarBranch* sidebar_branch_construct (GType object_type,
                                         SidebarEntry* root,
                                         SidebarBranchOptions options,
                                         GCompareFunc default_comparator,
                                         GCompareFunc root_comparator);
VALA_EXTERN SidebarEntry* sidebar_branch_get_root (SidebarBranch* self);
VALA_EXTERN void sidebar_branch_set_show_branch (SidebarBranch* self,
                                     gboolean shown);
VALA_EXTERN gboolean sidebar_branch_get_show_branch (SidebarBranch* self);
VALA_EXTERN gboolean sidebar_branch_is_auto_open_on_new_child (SidebarBranch* self);
VALA_EXTERN gboolean sidebar_branch_is_startup_expand_to_first_child (SidebarBranch* self);
VALA_EXTERN gboolean sidebar_branch_is_startup_open_grouping (SidebarBranch* self);
VALA_EXTERN void sidebar_branch_graft (SidebarBranch* self,
                           SidebarEntry* parent,
                           SidebarEntry* entry,
                           GCompareFunc comparator);
VALA_EXTERN void sidebar_branch_prune (SidebarBranch* self,
                           SidebarEntry* entry);
VALA_EXTERN void sidebar_branch_reparent (SidebarBranch* self,
                              SidebarEntry* new_parent,
                              SidebarEntry* entry);
VALA_EXTERN gboolean sidebar_branch_has_entry (SidebarBranch* self,
                                   SidebarEntry* entry);
VALA_EXTERN void sidebar_branch_reorder (SidebarBranch* self,
                             SidebarEntry* entry);
VALA_EXTERN void sidebar_branch_reorder_all (SidebarBranch* self);
VALA_EXTERN void sidebar_branch_reorder_children (SidebarBranch* self,
                                      SidebarEntry* entry,
                                      gboolean recursive);
VALA_EXTERN void sidebar_branch_change_all_comparators (SidebarBranch* self,
                                            GCompareFunc comparator);
VALA_EXTERN void sidebar_branch_change_comparator (SidebarBranch* self,
                                       SidebarEntry* entry,
                                       gboolean recursive,
                                       GCompareFunc comparator);
VALA_EXTERN gint sidebar_branch_get_child_count (SidebarBranch* self,
                                     SidebarEntry* parent);
VALA_EXTERN GeeList* sidebar_branch_get_children (SidebarBranch* self,
                                      SidebarEntry* parent);
VALA_EXTERN SidebarEntry* sidebar_branch_find_first_child (SidebarBranch* self,
                                               SidebarEntry* parent,
                                               Locator locator,
                                               gpointer locator_target);
VALA_EXTERN SidebarEntry* sidebar_branch_get_parent (SidebarBranch* self,
                                         SidebarEntry* entry);
VALA_EXTERN SidebarEntry* sidebar_branch_get_previous_sibling (SidebarBranch* self,
                                                   SidebarEntry* entry);
VALA_EXTERN SidebarEntry* sidebar_branch_get_next_sibling (SidebarBranch* self,
                                               SidebarEntry* entry);
VALA_EXTERN SidebarGrouping* sidebar_grouping_new (const gchar* name,
                                       const gchar* icon,
                                       const gchar* tooltip);
VALA_EXTERN SidebarGrouping* sidebar_grouping_construct (GType object_type,
                                             const gchar* name,
                                             const gchar* icon,
                                             const gchar* tooltip);
VALA_EXTERN SidebarRootOnlyBranch* sidebar_root_only_branch_new (SidebarEntry* root);
VALA_EXTERN SidebarRootOnlyBranch* sidebar_root_only_branch_construct (GType object_type,
                                                           SidebarEntry* root);
VALA_EXTERN GType sidebar_header_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarHeader, g_object_unref)
VALA_EXTERN SidebarHeader* sidebar_header_new (const gchar* name,
                                   gboolean emphasized);
VALA_EXTERN SidebarHeader* sidebar_header_construct (GType object_type,
                                         const gchar* name,
                                         gboolean emphasized);
VALA_EXTERN GType sidebar_contextable_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarContextable, g_object_unref)
VALA_EXTERN GtkMenu* sidebar_contextable_get_sidebar_context_menu (SidebarContextable* self,
                                                       GdkEventButton* event);
VALA_EXTERN GType sidebar_count_cell_renderer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (SidebarCountCellRenderer, g_object_unref)
VALA_EXTERN gint sidebar_count_cell_renderer_get_counter (SidebarCountCellRenderer* self);
VALA_EXTERN void sidebar_count_cell_renderer_set_counter (SidebarCountCellRenderer* self,
                                              gint value);
VALA_EXTERN SidebarCountCellRenderer* sidebar_count_cell_renderer_new (void);
VALA_EXTERN SidebarCountCellRenderer* sidebar_count_cell_renderer_construct (GType object_type);
VALA_EXTERN gchar* sidebar_entry_get_sidebar_name (SidebarEntry* self);
VALA_EXTERN gchar* sidebar_entry_get_sidebar_tooltip (SidebarEntry* self);
VALA_EXTERN gchar* sidebar_entry_get_sidebar_icon (SidebarEntry* self);
VALA_EXTERN gint sidebar_entry_get_count (SidebarEntry* self);
VALA_EXTERN gchar* sidebar_entry_to_string (SidebarEntry* self);
VALA_EXTERN gboolean sidebar_expandable_entry_expand_on_select (SidebarExpandableEntry* self);
VALA_EXTERN void sidebar_renameable_entry_rename (SidebarRenameableEntry* self,
                                      const gchar* new_name);
VALA_EXTERN gboolean sidebar_renameable_entry_is_user_renameable (SidebarRenameableEntry* self);
VALA_EXTERN gboolean sidebar_emphasizable_entry_is_emphasized (SidebarEmphasizableEntry* self);
VALA_EXTERN GType sidebar_destroyable_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void sidebar_destroyable_entry_destroy_source (SidebarDestroyableEntry* self);
VALA_EXTERN gboolean sidebar_internal_drop_target_entry_internal_drop_received (SidebarInternalDropTargetEntry* self,
                                                                    SidebarTree* parent,
                                                                    GdkDragContext* context,
                                                                    GtkSelectionData* data);
VALA_EXTERN GType sidebar_internal_drag_source_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void sidebar_internal_drag_source_entry_prepare_selection_data (SidebarInternalDragSourceEntry* self,
                                                                GtkSelectionData* data);
VALA_EXTERN SidebarTree* sidebar_tree_new (GtkTargetEntry* target_entries,
                               gint target_entries_length1,
                               GdkDragAction actions,
                               SidebarTreeExternalDropHandler drop_handler,
                               gpointer drop_handler_target,
                               GtkIconTheme* theme);
VALA_EXTERN SidebarTree* sidebar_tree_construct (GType object_type,
                                     GtkTargetEntry* target_entries,
                                     gint target_entries_length1,
                                     GdkDragAction actions,
                                     SidebarTreeExternalDropHandler drop_handler,
                                     gpointer drop_handler_target,
                                     GtkIconTheme* theme);
VALA_EXTERN void sidebar_tree_icon_renderer_function (SidebarTree* self,
                                          GtkCellLayout* layout,
                                          GtkCellRenderer* renderer,
                                          GtkTreeModel* model,
                                          GtkTreeIter* iter);
VALA_EXTERN void sidebar_tree_counter_renderer_function (SidebarTree* self,
                                             GtkCellLayout* layout,
                                             GtkCellRenderer* renderer,
                                             GtkTreeModel* model,
                                             GtkTreeIter* iter);
VALA_EXTERN void sidebar_tree_set_default_context_menu (SidebarTree* self,
                                            GtkMenu* context_menu);
VALA_EXTERN gboolean sidebar_tree_place_cursor (SidebarTree* self,
                                    SidebarEntry* entry,
                                    gboolean mask_signal);
VALA_EXTERN gboolean sidebar_tree_is_selected (SidebarTree* self,
                                   SidebarEntry* entry);
VALA_EXTERN gboolean sidebar_tree_is_any_selected (SidebarTree* self);
VALA_EXTERN gboolean sidebar_tree_accept_cursor_changed (SidebarTree* self);
VALA_EXTERN void sidebar_tree_disable_editing (SidebarTree* self);
VALA_EXTERN void sidebar_tree_enable_editing (SidebarTree* self);
VALA_EXTERN gboolean sidebar_tree_expand_to_entry (SidebarTree* self,
                                       SidebarEntry* entry);
VALA_EXTERN void sidebar_tree_expand_to_first_child (SidebarTree* self,
                                         SidebarEntry* entry);
VALA_EXTERN gboolean sidebar_tree_has_branch (SidebarTree* self,
                                  SidebarBranch* branch);
VALA_EXTERN void sidebar_tree_graft (SidebarTree* self,
                         SidebarBranch* branch,
                         gint position);
VALA_EXTERN gint sidebar_tree_get_position_for_branch (SidebarTree* self,
                                           SidebarBranch* branch);
VALA_EXTERN void sidebar_tree_prune_all (SidebarTree* self);
VALA_EXTERN void sidebar_tree_prune (SidebarTree* self,
                         SidebarBranch* branch);
VALA_EXTERN gboolean sidebar_tree_rename_entry_in_place (SidebarTree* self,
                                             SidebarEntry* entry);
VALA_EXTERN gboolean sidebar_tree_scroll_to_entry (SidebarTree* self,
                                       SidebarEntry* entry);
VALA_EXTERN GType util_cache_lru_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UtilCacheLru, g_object_unref)
VALA_EXTERN guint util_cache_lru_get_max_size (UtilCacheLru* self);
VALA_EXTERN void util_cache_lru_set_max_size (UtilCacheLru* self,
                                  guint value);
VALA_EXTERN gboolean util_cache_lru_get_is_empty (UtilCacheLru* self);
VALA_EXTERN guint util_cache_lru_get_size (UtilCacheLru* self);
VALA_EXTERN UtilCacheLru* util_cache_lru_new (GType t_type,
                                  GBoxedCopyFunc t_dup_func,
                                  GDestroyNotify t_destroy_func,
                                  guint max_size);
VALA_EXTERN UtilCacheLru* util_cache_lru_construct (GType object_type,
                                        GType t_type,
                                        GBoxedCopyFunc t_dup_func,
                                        GDestroyNotify t_destroy_func,
                                        guint max_size);
VALA_EXTERN gboolean util_cache_lru_has_key (UtilCacheLru* self,
                                 const gchar* key);
VALA_EXTERN void util_cache_lru_set_entry (UtilCacheLru* self,
                               const gchar* key,
                               gconstpointer value);
VALA_EXTERN gpointer util_cache_lru_get_entry (UtilCacheLru* self,
                                   const gchar* key);
VALA_EXTERN gpointer util_cache_lru_remove_entry (UtilCacheLru* self,
                                      const gchar* key);
VALA_EXTERN void util_cache_lru_clear (UtilCacheLru* self);
VALA_EXTERN gboolean util_contact_should_load_images (ApplicationContact* contact,
                                          ApplicationConfiguration* config);
VALA_EXTERN GType util_date_coarse_date_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void util_date_init (void);
VALA_EXTERN UtilDateCoarseDate util_date_as_coarse_date (GDateTime* datetime,
                                             GDateTime* now,
                                             GTimeSpan diff);
VALA_EXTERN gchar* util_date_pretty_print (GDateTime* datetime,
                               UtilDateClockFormat clock_format);
VALA_EXTERN gchar* util_date_pretty_print_verbose (GDateTime* datetime,
                                       UtilDateClockFormat clock_format);
VALA_EXTERN gchar* util_date_get_clock_format (UtilDateClockFormat clock_format);
VALA_EXTERN gint util_email_compare_conversation_ascending (GearyAppConversation* a,
                                                GearyAppConversation* b);
VALA_EXTERN gint util_email_compare_conversation_descending (GearyAppConversation* a,
                                                 GearyAppConversation* b);
VALA_EXTERN gchar* util_email_strip_subject_prefixes (GearyEmailHeaderSet* email);
VALA_EXTERN GearyRFC822MailboxAddress* util_email_get_primary_originator (GearyEmailHeaderSet* email);
VALA_EXTERN gchar* util_email_to_short_recipient_display (GearyEmailHeaderSet* headers);
VALA_EXTERN gchar* util_email_quote_email_for_reply (GearyEmail* email,
                                         const gchar* quote,
                                         GearyRFC822TextFormat format);
VALA_EXTERN gchar* util_email_quote_email_for_forward (GearyEmail* email,
                                           const gchar* quote,
                                           GearyRFC822TextFormat format);
VALA_EXTERN GType util_email_search_expression_factory_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UtilEmailSearchExpressionFactory, g_object_unref)
VALA_EXTERN GearySearchQueryStrategy util_email_search_expression_factory_get_default_strategy (UtilEmailSearchExpressionFactory* self);
VALA_EXTERN GearyAccountInformation* util_email_search_expression_factory_get_account (UtilEmailSearchExpressionFactory* self);
VALA_EXTERN UtilEmailSearchExpressionFactory* util_email_search_expression_factory_new (GearySearchQueryStrategy default_strategy,
                                                                            GearyAccountInformation* account);
VALA_EXTERN UtilEmailSearchExpressionFactory* util_email_search_expression_factory_construct (GType object_type,
                                                                                  GearySearchQueryStrategy default_strategy,
                                                                                  GearyAccountInformation* account);
VALA_EXTERN GeeList* util_email_search_expression_factory_parse_query (UtilEmailSearchExpressionFactory* self,
                                                           const gchar* query);
VALA_EXTERN gchar* files_get_filesize_as_string (gint64 filesize);
VALA_EXTERN GtkBuilder* gio_util_create_builder (const gchar* name);
VALA_EXTERN gchar* gio_util_read_resource (const gchar* name,
                               GError** error);
VALA_EXTERN void util_gtk_set_source_color_from_string (cairo_t* ctx,
                                            const gchar* spec);
VALA_EXTERN gint util_gtk_get_border_box_height (GtkWidget* widget);
VALA_EXTERN GMenu* util_gtk_construct_menu (GMenuModel* template,
                                UtilGtkMenuVisitor visitor,
                                gpointer visitor_target);
VALA_EXTERN GMenu* util_gtk_copy_menu_with_targets (GMenu* template,
                                        const gchar* group,
                                        GeeMap* targets);
VALA_EXTERN gchar* util_gtk_shorten_url (const gchar* url);
VALA_EXTERN void util_gtk_rgba (gdouble red,
                    gdouble green,
                    gdouble blue,
                    gdouble alpha,
                    GdkRGBA* result);
VALA_EXTERN gboolean util_gtk_query_tooltip_label (GtkWidget* widget,
                                       gint x,
                                       gint y,
                                       gboolean keyboard,
                                       GtkTooltip* tooltip);
VALA_EXTERN gchar** util_i18n_get_available_dictionaries (gint* result_length1);
VALA_EXTERN gchar** util_i18n_get_available_locales (gint* result_length1);
VALA_EXTERN gchar* util_i18n_strip_encoding (const gchar* locale);
VALA_EXTERN gchar** util_i18n_get_user_preferred_languages (gint* result_length1);
VALA_EXTERN gchar* util_i18n_language_name_from_locale (const gchar* locale);
VALA_EXTERN gchar* util_i18n_country_name_from_locale (const gchar* locale);
VALA_EXTERN gchar* util_i18n_to_folder_display_name (GearyFolder* folder);
VALA_EXTERN const gchar* util_i18n_to_folder_type_display_name (GearyFolderSpecialUse use);
VALA_EXTERN GQuark util_js_error_quark (void);
VALA_EXTERN GType util_js_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType util_js_jsc_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN UtilJSJscType util_js_jsc_type_to_type (JSCValue* value);
VALA_EXTERN gboolean util_js_to_bool (JSCValue* value,
                          GError** error);
VALA_EXTERN gdouble util_js_to_double (JSCValue* value,
                           GError** error);
VALA_EXTERN gint32 util_js_to_int32 (JSCValue* value,
                         GError** error);
VALA_EXTERN gchar* util_js_to_string (JSCValue* value,
                          GError** error);
VALA_EXTERN JSCValue* util_js_get_property (JSCValue* value,
                                const gchar* name,
                                GError** error);
VALA_EXTERN void util_js_check_exception (JSCContext* context,
                              GError** error);
VALA_EXTERN GVariant* util_js_value_to_variant (JSCValue* value,
                                    GError** error);
VALA_EXTERN JSCValue* util_js_variant_to_value (JSCContext* context,
                                    GVariant* variant,
                                    GError** error);
VALA_EXTERN UtilJSCallable* util_js_callable (const gchar* base_name);
VALA_EXTERN UtilJSCallable* util_js_callable_new (const gchar* name);
VALA_EXTERN UtilJSCallable* util_js_callable_construct (GType object_type,
                                            const gchar* name);
VALA_EXTERN WebKitUserMessage* util_js_callable_to_message (UtilJSCallable* self);
VALA_EXTERN gchar* util_js_callable_to_string (UtilJSCallable* self);
VALA_EXTERN UtilJSCallable* util_js_callable_string (UtilJSCallable* self,
                                         const gchar* value);
VALA_EXTERN UtilJSCallable* util_js_callable_double (UtilJSCallable* self,
                                         gdouble value);
VALA_EXTERN UtilJSCallable* util_js_callable_int (UtilJSCallable* self,
                                      gint value);
VALA_EXTERN UtilJSCallable* util_js_callable_bool (UtilJSCallable* self,
                                       gboolean value);
VALA_EXTERN void util_migrate_xdg_config_dir (GFile* user_config_dir,
                                  GFile* user_data_dir,
                                  GError** error);
VALA_EXTERN void util_migrate_release_config (GFile** search_path,
                                  gint search_path_length1,
                                  GFile* config_dir,
                                  GError** error);
VALA_EXTERN void util_migrate_old_app_config (GSettings* newSettings,
                                  const gchar* old_app_id);

VALA_EXTERN const gchar* APPLICATION_CLIENT_AUTHORS[8];
VALA_EXTERN const GtkTargetEntry FOLDER_LIST_TREE_TARGET_ENTRY_LIST[1];

G_END_DECLS

#endif
